(function() {
    'use strict';

    angular.module('i18n', []);
})();
(function() {
    'use strict';

    angular.module('i18n').directive('messageKey', messageKeyDirective);

    messageKeyDirective.$inject = ['i18nService'];

    function messageKeyDirective(i18nService) {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                var i18n;
                if(!attrs.messageParams){
                    i18n = i18nService.message(attrs.messageKey);
                } else {
                    i18n = i18nService.format(attrs.messageKey, attrs.messageParams);
                }

                if(attrs.messageAttr) {
                    // store the i18n in the specified element attr
                    element.attr(attrs.messageAttr, i18n);
                } else {
                    // set the i18n as element text
                    element.text(i18n);
                }
            }
        };
    }
})();
(function() {
    'use strict';

    angular.module('i18n').filter('translate', translateFilter);

    translateFilter.$inject = ['i18nService'];

    function translateFilter(i18nService) {
        return function(input) {
            return i18nService.message(input);
        };
    }
})();
(function() {
    'use strict';

    angular.module('i18n').service('i18nService', i18nService);

    function i18nService() {
        var _i18nMap = {};

        return {
            initMap: initMap,
            addKeyToMap: addKeyToMap,
            message: message,
            format: format,
            contains: contains
        };

        function contains(key) {
            return (_i18nMap && _i18nMap[key]);
        }

        function message(key) {
            if (_i18nMap && _i18nMap[key]) {
                return _i18nMap[key];
            } else {
                return '???' + key + '???';
            }
        }

        function format(key, params) {
            var replacer = function(params){
                return function(s, index) {
                    return params[index] ? (params[index] == '__void__' ? '' : params[index]) : '';
                };
            };

            if(params){
                if (_i18nMap && _i18nMap[key]) {
                    return _i18nMap[key].replace(/\{(\w+)\}/g, replacer(params.split('|')));
                } else {
                    return '???' + key + '???';
                }
            } else {
                return this.message(key);
            }
        }

        function addKeyToMap(newI18nMap) {
            angular.forEach(newI18nMap, function (value, key) {
                _i18nMap[key] = value;
            });
        }

        function initMap(i18nMap) {
            _i18nMap = i18nMap;
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemApp', [
        // Vendor modules
        'ngRoute',
        'ngResource',
        'ngCookies',
        'ngSanitize',

        'underscore',
        'datatables',
        'angularMoment',
        'angular-md5',
        'chart.js',

        // Shared modules
        'i18n',
        'wemApp.ui',
        'wemApp.factories',
        'wemApp.services',
        'wemApp.components'
    ]);

    angular.module('wemApp.ui',
        wemAppIsNewUI ? // UI angular material
        ['ngAnimate', 'ngMessages', 'ngMaterial', 'uiGmapgoogle-maps', 'lfNgMdFileInput']
        : // UI bootstrap
        ['ui-notification', 'ui.bootstrap', 'datatables.bootstrap', 'uiGmapgoogle-maps', 'autoCompleteModule']
    );

    angular.module('wemApp.factories', []);

    angular.module('wemApp.services', ['i18n', 'wemApp.ui']);

    angular.module('wemApp.components', ['i18n', 'wemApp.ui', 'wemApp.services', 'wemApp.factories']);
})();
(function() {
    'use strict';

    angular.module('wemApp').config(wemAppConfig);

    wemAppConfig.$inject = ['$routeProvider'];

    function wemAppConfig($routeProvider) {
        $routeProvider
            .when('/context-server-issue', {
                template: '<wem-context-server-issue></wem-context-server-issue>'
            });
    }

    angular.module('wemApp.components').config(wemAppComponentsConfig);

    wemAppComponentsConfig.$inject = ['$sceDelegateProvider', 'uiGmapGoogleMapApiProvider'];

    function wemAppComponentsConfig($sceDelegateProvider, uiGmapGoogleMapApiProvider) {
        // THIS IS FOR THE ACTIONS AND THE PROPERTIES
        // we need to normalize the URL as angular uses normalized URLs to check against the white list
        var urlParsingNode = document.createElement('a');
        urlParsingNode.setAttribute('href', ManagersContext.proxyServletUrl);
        var normalizedProxyServletUrl = urlParsingNode.href;

        $sceDelegateProvider.resourceUrlWhitelist([
            // Allow same origin resource loads.
            'self',
            // Allow loading from our assets domain.  Notice the difference between * and **.
            normalizedProxyServletUrl + '**'
        ]);

        // condition-mapCondition.controller.js and wemPersonaPanelApp
        uiGmapGoogleMapApiProvider.configure({
            key: ManagersContext.googleAPIKey,
            v: '3.17',
            libraries: 'weather,geometry,visualization'
        });
    }

    if (!wemAppIsNewUI) {
        angular.module('wemApp.ui').config(wemAppOldUIConfig);

        wemAppOldUIConfig.$inject = ['$httpProvider', 'NotificationProvider'];
    } else {
        angular.module('wemApp.ui').config(wemAppNewUIConfig);

        wemAppNewUIConfig.$inject = ['$mdThemingProvider'];
    }

    function wemAppOldUIConfig($httpProvider, NotificationProvider) {
        $httpProvider.defaults.headers.common['Accept-Language'] = ManagersContext.jahiaUILocale;

        $httpProvider.interceptors.push(function ($q, i18nService) {
            return {
                responseError: function (rejection) {
                    if (rejection.status != 404) {
                        if (rejection.data && rejection.data.error) {
                            ManagersUtils.notificationService.errorToast("Error : " + rejection.data.error);
                        } else if (rejection.data && rejection.data != null) {
                            ManagersUtils.notificationService.errorToast("Error : " + rejection.data);
                        } else {
                            ManagersUtils.notificationService.errorToast("Error : " + rejection.statusText);
                        }
                    }
                    if (rejection.config.url.match(/templates\/condition/)) {
                        rejection.data = i18nService.message("condition.unknown");
                        return rejection;
                    }

                    return $q.reject(rejection);
                }
            };
        });

        NotificationProvider.setOptions({
            startTop: 20,
            startRight: 20,
            verticalSpacing: 20,
            horizontalSpacing: 20,
            positionX: 'left',
            positionY: 'bottom'
        });
    }

    function wemAppNewUIConfig($mdThemingProvider) {
        $mdThemingProvider.theme('error-toast');

        // Add jahia blue as default 500 for blue palette
        var jahiaBlue = $mdThemingProvider.extendPalette('blue', {
            '500': '#00A0E3',
            'contrastDefaultColor': 'light'
        });

        var mfPink = $mdThemingProvider.extendPalette('pink', {
            '500': '#F21D8E',
            'contrastDefaultColor': 'light'
        });

        $mdThemingProvider.definePalette('jahia-blue', jahiaBlue);

        $mdThemingProvider.definePalette('mf-pink', mfPink);

        $mdThemingProvider.theme('wemApp')
            .primaryPalette('jahia-blue')
            .accentPalette('deep-purple')
            .warnPalette('mf-pink');

        $mdThemingProvider.setDefaultTheme('wemApp');
    }
})();
(function() {
    'use strict';

    angular.module('wemApp').run(wemAppRun);

    wemAppRun.$inject = ['$rootScope', '$window', '$location', 'notificationService', 'i18nService', 'amMoment'];

    function wemAppRun($rootScope, $window, $location, notificationService, i18nService, amMoment) {
        i18nService.initMap(wemi18n);

        amMoment.changeLocale(ManagersContext.jahiaUILocale);

        if (!wemAppIsNewUI) {
            ManagersUtils.notificationService = notificationService;
        }

        // To manage context server issue and display a page
        $rootScope.$on('$locationChangeStart', function (event, nextUrl, currentUrl) {
            if (!ManagersContext.isAllowedByLicense) {
                _redirectHandler(event, nextUrl, currentUrl);
            } else if ((!ManagersContext.contextServerSetup && $window.location.pathname.indexOf('.marketing-90-settings.html') == -1)
                || (!ManagersContext.contextServerOnline && $window.location.pathname.indexOf('.marketing-90-settings.html') == -1)) {
                _redirectHandler(event, nextUrl, currentUrl);
            }
        });

        $rootScope.$on('$locationChangeSuccess', function (event, newUrl) {
            // only executed if not in edit frame
            if ($window.parent.location.pathname.indexOf('/editframe/') == -1) {
                $window.parent.location.hash = "#" + $location.path();
            }
        });

        // Private functions under this line
        function _redirectHandler(event, nextUrl, currentUrl) {
            if (nextUrl.indexOf('#!/context-server-issue') == -1
                && currentUrl.indexOf('#!/context-server-issue') == -1) {
                // Context server is not setup and nextUrl and current URL are not going to right view
                $location.path('/context-server-issue');
                event.preventDefault();
            }

            if (nextUrl.indexOf('#!/context-server-issue') == -1
                && currentUrl.indexOf('#!/context-server-issue') != -1) {
                // context server is not setup and current URL is going to the right view
                event.preventDefault();
            }
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp').filter('unique', function() {
        return function(collection, keyname) {
            var output = [],
                keys = [];

            angular.forEach(collection, function(item) {
                var key = item[keyname];
                if(keys.indexOf(key) === -1) {
                    keys.push(key);
                    output.push(item);
                }
            });

            return output;
        };
    });
})();
var ManagersUtils = ManagersUtils || {};

ManagersUtils.cxsContext = '/cxs/';

ManagersUtils.formMappingActionStrategies = ['alwaysSet', 'setIfMissing'];

ManagersUtils.getQueryOrDefault = function (query) {
    var scopedAndSharedQuery = {
        "condition": {
            "parameterValues": {
                "subConditions": [
                    {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "metadata.scope",
                            "comparisonOperator": "equals",
                            "propertyValue": ManagersContext.dmfScope
                        }
                    },
                    {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "metadata.scope",
                            "comparisonOperator": "equals",
                            "propertyValue": "systemscope"
                        }
                    }
                ],
                "operator": "or"
            },
            "type": "booleanCondition"
        },
        "limit": 50,
        "offset": 0,
        "sortby": null,
        "forceRefresh": false
    };

    return query || scopedAndSharedQuery;
};

ManagersUtils.humanReadableMissingIfNeeded = function (value, i18nService) {
    if (value == "_missing") {
        // replace _missing by something more user friendly
        value = i18nService.message("serverSettings.internalSearches.missing");
    }

    return value;
};

ManagersUtils.AggregateDisplay = function (key, occurrences, i18nService) {
    var AggregateDetail = function (value, occurrences) {
        this.item = value;
        this.occurrences = occurrences;

        this.importance = function (totalOccurrences) {
            return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
        }
    };

    this.key = key;
    this.occurrences = occurrences;
    this.details = {};

    this.importance = function (totalOccurrences) {
        return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
    };

    this.setDetails = function (type, details) {
        if (details) {
            var detailsArray = [];
            var totalOccurrences = 0;
            for (var detail in details) {
                if (ManagersUtils.acceptProperty(details, detail)) {
                    occurrences = details[detail];

                    detail = ManagersUtils.humanReadableMissingIfNeeded(detail, i18nService);

                    totalOccurrences += occurrences;
                    detailsArray.push(new AggregateDetail(detail, occurrences));
                }
            }
            this.occurrences = totalOccurrences;

            this.details[type] = detailsArray;

            return detailsArray;
        }
    };
};

ManagersUtils.booleanAnd = function (firstCondition, secondCondition) {
    return {
        "type": "booleanCondition",
        "parameterValues": {
            "operator": "and",
            "subConditions": [
                firstCondition,
                secondCondition
            ]
        }
    }
};

ManagersUtils.booleanAndArray = function (subConditions) {
    return {
        "type": "booleanCondition",
        "parameterValues": {
            "operator": "and",
            "subConditions": subConditions
        }
    }
};

ManagersUtils.getDateRangeConditionBody = function (from, to) {
    return ManagersUtils.booleanAnd(
        {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": "scope",
                "comparisonOperator": "equals",
                "propertyValue": ManagersContext.dmfScope
            }
        },
        {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": "timeStamp",
                "comparisonOperator": "between",
                "propertyValues": [from, to]
            }
        });
};

ManagersUtils.parseErrorsInForm = function (form, parsedErrors, subFormName) {
    if (!parsedErrors) {
        parsedErrors = {};
    }

    angular.forEach(form.$error, function (value, key) {

        if (value.length) {
            angular.forEach(value, function (subForm) {
                ManagersUtils.parseErrorsInForm(subForm, parsedErrors, form.$name);
            });
        } else if (value && form.$name) {
            parsedErrors["wem_validation." + (subFormName.indexOf('_') != -1 ? subFormName.substr(0, subFormName.indexOf('_')) : subFormName) + "." + form.$name + "." + key] = true;
        }
    });

    return parsedErrors;
};

ManagersUtils.displayErrors = function (errorKeys, i18nService, notificationService) {
    angular.forEach(errorKeys, function (value, key) {
        notificationService.errorToast(i18nService.message(key));
    });
};

ManagersUtils.hasErrors = function (rootForm, form, propertyName) {
    if (rootForm) {
        if (!rootForm.showErrors) {
            return false;
        }

        if (form[propertyName]) {
            return form[propertyName].$invalid;
        }
    }
    return false;
};

ManagersUtils.goalTags = ["downloadGoal", "formGoal", "funnelGoal", "landingPageGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.campaingTags = ["downloadGoal", "formGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.optimizationTestGoalTags = ["downloadGoal", "formGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.buildTags = function (tags, i18nService) {
    var result = {};
    tags.forEach(function (tag) {
        result[tag] = {
            id: tag,
            name: i18nService.message("serverSettings.manageGoals.tag." + tag),
            description: i18nService.message("serverSettings.manageGoals.tag." + tag + ".description"),
            goals: []
        }
    });

    return result;
};

ManagersUtils.supportedTimelineIntervales = [
    {
        "key": "1M",
        "i18n": "serverSettings.manageGoals.interval.1M",
        "unitcount": 1,
        "units": "months",
        "unit": "month",
        minpoints: 1,
        maxpoints: 36,
        scaleFormat: "YYYY-MM",
        pickerFormat: "MMMM yyyy",
        aggregateFormat: "MMMM YYYY"
    },
    {
        "key": "1d",
        "i18n": "serverSettings.manageGoals.interval.1d",
        "unitcount": 1,
        "units": "days",
        "unit": "day",
        minpoints: 1,
        maxpoints: 1080,
        scaleFormat: "L",
        pickerFormat: "mediumDate",
        aggregateFormat: "LL"
    }
];

ManagersUtils.supportedTimelineIntervalesObj = {
    "1M": ManagersUtils.supportedTimelineIntervales[0],
    "1d": ManagersUtils.supportedTimelineIntervales[1]
};

ManagersUtils.generateId = function () {
    // Math.random should be unique because of its seeding algorithm.
    // Convert it to base 36 (numbers + letters), and grab the first 9 characters
    // after the decimal.
    return '_' + Math.random().toString(36).substr(2, 9);
};
/**
 * Utility method to read nested objects in a json obj without having to test all the object in the path
 * example before to access goal.metadata.type you have to check if goal and metadata are not undefined before trying access type, if not you will have
 * "cannot read property type of undefined"
 *
 * Now you can use ManagersUtils.getProperty(goal, "metadata.type") safely, can return undefined in case of undefined are found in the given path
 * @param obj
 * @param propPath
 * @returns {*}
 */
ManagersUtils.getProperty = function (obj, propPath) {
    props = propPath.split('.');
    if (props.length == 1) {
        return obj[props[0]]
    } else if (props.length > 1) {
        var prop = props.shift();
        var resolvedObj = obj[prop];
        if (resolvedObj) {
            return this.getProperty(resolvedObj, props.join("."))
        }
    }
    return undefined;
};

ManagersUtils.acceptProperty = function (data, prop) {
    return (prop && data.hasOwnProperty(prop) && prop != "_all" && prop != "_filtered");
};

ManagersUtils.getCurrentModuleResource = function (resourcePath) {
    if (ManagersContext.currentModuleURL) {
        return ManagersContext.currentModuleURL + resourcePath;
    } else {
        return "/modules/marketing-factory-core" + resourcePath;
    }
};

ManagersUtils.getPercentage = function (value) {
    return +((value * 100).toFixed(2)); // convert formatted string back to a number
};

ManagersUtils.downloadForm = function (url, data, method) {
    //url and data options required
    if (url && data) {
        //split params into form inputs
        var $query = jQuery('<input type="hidden" name="query"/>').val(data);
        //send request
        jQuery('<form action="' + url + '" method="' + (method || 'post') + '"></form>').append($query)
            .appendTo('body').submit().remove();
    }
};

ManagersUtils.maxInterestsInChart = 16;

ManagersUtils.getProfileDisplayName = function (i18nService, properties) {
    if (properties.firstName && properties.lastName) {
        return properties.firstName + ' ' + properties.lastName;
    } else if (properties.firstName) {
        return properties.firstName;
    } else if (properties.lastName) {
        return properties.lastName;
    } else {
        return i18nService.message("serverSettings.manageProfiles.profile.unnamed");
    }
};

ManagersUtils.campaignPropertiesForColName = {
    "id": "metadata.id",
    "name": "metadata.name",
    "description": "metadata.description",
    "startDate": "startDate",
    "endDate": "endDate"
};

ManagersUtils.formMappingPropertiesForColName = {
    "id": "metadata.id",
    "name": "metadata.name",
    "description": "metadata.description",
    "action": "action"
};

ManagersUtils.personaMap = {
    center: {
        latitude: 45,
        longitude: -73
    },
    zoom: 8,
    control: {},
    options:{
        draggable:true,
        styles:
            [
                {
                    "featureType": "administrative",
                    "elementType": "labels.text.fill",
                    "stylers": [
                        {
                            "color": "#444444"
                        }
                    ]
                },
                {
                    "featureType": "landscape",
                    "elementType": "all",
                    "stylers": [
                        {
                            "color": "#f2f2f2"
                        }
                    ]
                },
                {
                    "featureType": "poi",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "poi.park",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "road",
                    "elementType": "all",
                    "stylers": [
                        {
                            "saturation": -100
                        },
                        {
                            "lightness": 45
                        }
                    ]
                },
                {
                    "featureType": "road.highway",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "simplified"
                        }
                    ]
                },
                {
                    "featureType": "road.arterial",
                    "elementType": "labels.icon",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "transit",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "transit.line",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.airport",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.airport",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.bus",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.bus",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.rail",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.rail",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "water",
                    "elementType": "all",
                    "stylers": [
                        {
                            "color": "#6e769e"
                        },
                        {
                            "visibility": "on"
                        }
                    ]
                }
            ]
    }
};


ManagersUtils.dateFormats = {
    '/y':'YYYY',
    '/M':'MMMM YYYY',
    '/d':'LL',
    '/h':'LL',
    '/m':'LL H:00',
    '/s':'LLL'
};

ManagersUtils.parseDateMath = function(f) {
    var result = /now(([+-]\d+[yMwdhms])+)(\/[yMwdhms])?/.exec(f);
    if (result) {
        var offsetRegexp = /([+-])(\d+)([yMwdhms])/g;
        f = moment();
        var offset;
        while (offset = offsetRegexp.exec(result[1])) {
            if (offset[1] == '+') {
                f = f.add(offset[2], offset[3])
            } else {
                f = f.subtract(offset[2], offset[3])
            }
        }

        var format = 'LLL';
        if (result[3]) {
            format = ManagersUtils.dateFormats[result[3]];
            moment().millisecond(0);
            var reset;
            if (reset = (result[3] == '/y')) {
                moment().month(0);
            }
            if (reset |= (result[3] == '/M')) {
                moment().date(0);
            }
            if (reset |= (result[3] == '/d')) {
                moment().hour(0);
            }
            if (reset |= (result[3] == '/h')) {
                moment().minute(0);
            }
            if (reset |= (result[3] == '/m')) {
                moment().second(0);
            }
            if (reset || (result[3] == '/s')) {
                moment().millisecond(0);
            }
        }
        f = f.format(format);
    }
    return f;
};

(function() {
    'use strict';

    angular.module('wemApp.factories').factory('RecursionHelper', RecursionHelper);

    RecursionHelper.$inject = ['$compile'];

    function RecursionHelper($compile) {
        return {
            /**
             * Manually compiles the element, fixing the recursion loop.
             * @param element
             * @param [link] A post-link function, or an object with function(s) registered via pre and post properties.
             * @returns An object containing the linking functions.
             */
            compile: function (element, link) {
                // Normalize the link parameter
                if (angular.isFunction(link)) {
                    link = {post: link};
                }

                // Break the recursion loop by removing the contents
                var contents = element.contents().remove();
                var compiledContents;
                return {
                    pre: (link && link.pre) ? link.pre : null,
                    /**
                     * Compiles and re-adds the contents
                     */
                    post: function (scope, element) {
                        // Compile the contents
                        if (!compiledContents) {
                            compiledContents = $compile(contents);
                        }
                        // Re-add the compiled contents to the element
                        compiledContents(scope, function (clone) {
                            element.append(clone);
                        });

                        // Call the post-linking function, if any
                        if (link && link.post) {
                            link.post.apply(null, arguments);
                        }
                    }
                };
            }
        };
    }
})();

(function() {
    'use strict';
    angular.module('wemApp.services').service('autocompleteService', autocompleteService);
    autocompleteService.$inject = ['_','unomiProfileService', 'unomiQueryService'];


    function autocompleteService(_,unomiProfileService,unomiQueryService) {

        return {
            completePropertyValue: completePropertyValue,
            getAutocompletedProperties: getAutocompletedProperties,
            isAutocomplete:isAutocomplete
        };

        function isAutocomplete (propertyName, propertiesList){
            var propertyIndex = _.findIndex(propertiesList, function(value) {
                return value.itemId == propertyName.replace("properties.", "");
            });
            return propertyIndex > -1;
        }

        /**
         * This function calculates the candidates values for a typed term
         * @param propertyName : The property for which the term is typed
         * @param term : the input value after last keypress
         * @returns {*} a table a values which results from a Unomi query on the properties values using a startsWith operator
         */
            function completePropertyValue(propertyName,term,propertyType, multivalued){
                return unomiQueryService.query(propertyType, propertyName, getPropertyValuesQuery(propertyName, term)).then(function (response) {
                    var filteredList = _.omit(response.data, ['_all', '_filtered']);
                    if(multivalued){
                        filteredList = _.omit(filteredList,function(value,key){
                            return !(key.toUpperCase().startsWith(term));
                        });
                    }
                    return _.map(filteredList, function(value,key){
                        return {value:key,count:value};
                    });
                });
            }

            function getAutocompletedProperties(){
                return unomiProfileService.getPropertyTypesByTag("autocompleted", true);
            }

            /**
             * This function generates the query to get all the property values that starts with the "value" value
             * @param propertyName
             * @param value
             * @returns {{condition: {type: string, parameterValues: {operator: string, subConditions: *[]}}}}
             */
            function getPropertyValuesQuery(propertyName,  value) {
                return {
                    "condition": {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": propertyName,
                                        "comparisonOperator": "startsWith",
                                        "propertyValue": value
                                    }
                                }
                            ]
                        }
                    }
                };
            }

        String.prototype.startsWith = function (str)
        {
            return this.indexOf(str) == 0;
        }
    }

})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('dataTableService', dataTableService);

    dataTableService.$inject = ['_', 'DTOptionsBuilder', 'DTColumnBuilder', 'i18nService'];

    function dataTableService(_, DTOptionsBuilder, DTColumnBuilder, i18nService) {
        var _promise;

        return {
            generateDataTableOptions: generateDataTableOptions,
            generateDataTableColumns: generateDataTableColumns,
            setPromise: setPromise
        };

        function generateDataTableOptions(options) {

            var dtOptions = DTOptionsBuilder.newOptions();

            dtOptions.withOption('lengthMenu', options.lengthMenu ? options.lengthMenu : [ 25, 50, 100 ])
                .withOption('pageLength', options.pageLength ? options.pageLength : 25);

            if (options.rowCallback) {
                dtOptions = dtOptions.withOption('fnRowCallback',
                    options.rowCallback);
            }
            if (options.createdRow) {
                dtOptions = dtOptions.withOption('createdRow',
                    options.createdRow);
            }
            dtOptions = dtOptions.withOption('processing', options.processing ? options.processing : true)
                .withOption('serverSide', options.serverSide ? options.serverSide : true)
                .withOption('sDom', 'lt')
                .withFnServerData(_serverData)
                .withOption('order', options.defaultOrder);
            if (options.paging != undefined) {
                dtOptions = dtOptions.withOption('paging', false);
            }
            if (options.ordering != undefined) {
                dtOptions = dtOptions.withOption('ordering', false);
            }
            return dtOptions;

        }

        function generateDataTableColumns(columns) {
            var dtColumns = [];
            _.forEach(columns, function(column){
                var columnItem = DTColumnBuilder.newColumn(column.name).withTitle(i18nService.message(column.bundleKey));
                if (column.notSortable) {
                    columnItem = columnItem.notSortable();
                }
                if (column.actionsHtml) {
                    columnItem = columnItem.renderWith(column.actionsHtml);
                }
                if (column.options && column.options.length) {
                    _.forEach(column.options, function(columnOption){
                        columnItem = columnItem.withOption(columnOption[0], columnOption[1]);
                    });
                }
                dtColumns.push(columnItem);
            });
            return dtColumns;
        }

        function setPromise(promise) {
            _promise = promise;
        }

        // Private functions under this line
        function _serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function(x) {
                return (x.name === 'draw');
            }).value;

            var start = _.find(aoData, function(x) {
                return (x.name === 'start');
            }).value;

            var length = _.find(aoData, function(x) {
                return (x.name === 'length');
            }).value;

            var orderData = _.find(aoData, function(x) {
                return (x.name === "order");
            }).value;

            var order = _.map(orderData, function(o) {
                return {prop: o.column, dir: o.dir};
            })[0];

            _promise.then(
                function(dataMap) {

                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": dataMap.data.length,
                        "iTotalDisplayRecords": dataMap.data.length,
                        "aaData": dataMap.data
                    };
                    if (order) {
                        results.aaData = _.sortBy(dataMap.data, _.keys(dataMap.data[0])[order.prop]);
                        if (order.dir === 'desc') {
                            results.aaData = results.aaData.reverse();
                        }
                    }
                    if (dataMap.limit > 0) {
                        results.aaData.splice(dataMap.limit);
                    } else {
                        results.aaData = results.aaData.splice(start, length);
                    }

                    fnCallback(results);
                },
                function() {
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                }
            );
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('dtPagerService', dtPagerService);

    dtPagerService.$inject = ['$timeout', 'i18nService'];

    function dtPagerService($timeout, i18nService) {
        return {
            refreshPager: refreshPager,
            initPagerProperties: initPagerProperties
        };

        function refreshPager($scope) {
            $timeout(function () {
                initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                // need to call $scope.$digest to force variable link of the pager when reordering.
                if (!$scope.$$phase) {
                    $scope.$digest();
                }
            });
        }

        function initPagerProperties(DataTable, $scope, i18nService) {
            var init = function () {
                if (!DataTable.data().length) {
                    $scope.pagerActive = false;
                    return;
                }
                $scope.pageInfo = DataTable.page.info();
                $scope.pageLength = DataTable.page.len();
                $scope.pagerPages = [];

                if ($scope.pageInfo.pages < 8) {
                    for (var i = 0; i < $scope.pageInfo.pages; i++) {
                        $scope.pagerPages.push({
                            id: i,
                            label: i + 1,
                            active: $scope.pageInfo.page == i,
                            enabled: true
                        });
                    }
                } else {
                    $scope.pagerPages.push({
                        id: 0,
                        label: 1,
                        active: $scope.pageInfo.page == 0,
                        enabled: true
                    });
                    if ($scope.pageInfo.page < 4) {
                        for (var i = 1; i < 5; i++) {
                            $scope.pagerPages.push({
                                id: i,
                                label: i + 1,
                                active: $scope.pageInfo.page == i,
                                enabled: true
                            });
                        }
                    }
                    $scope.pagerPages.push({
                        id: 0,
                        label: "...",
                        active: false,
                        enabled: false
                    });
                    if ($scope.pageInfo.page >= 4 && $scope.pageInfo.page <= $scope.pageInfo.pages - 5) {
                        for (var i = $scope.pageInfo.page - 1; i <= $scope.pageInfo.page + 1; i++) {
                            $scope.pagerPages.push({
                                id: i,
                                label: i + 1,
                                active: $scope.pageInfo.page == i,
                                enabled: true
                            });
                        }
                        $scope.pagerPages.push({
                            id: 0,
                            label: "...",
                            active: false,
                            enabled: false
                        });
                    }

                    if ($scope.pageInfo.page > $scope.pageInfo.pages - 5) {
                        for (var i = $scope.pageInfo.pages - 5; i < $scope.pageInfo.pages - 1; i++) {
                            $scope.pagerPages.push({
                                id: i,
                                label: i + 1,
                                active: $scope.pageInfo.page == i,
                                enabled: true
                            });
                        }
                    }

                    $scope.pagerPages.push({
                        id: $scope.pageInfo.pages - 1,
                        label: $scope.pageInfo.pages,
                        active: $scope.pageInfo.page == $scope.pageInfo.pages - 1,
                        enabled: true
                    });
                }
                $scope.pagerActive = $scope.pageInfo.pages > 1;

                if (i18nService && $scope.pagerActive) {
                    $scope.$watch("pageInfo.start", function (newValue, oldValue) {
                        $scope.pageInfoText = i18nService.format("serverSettings.manageWem.pageInfos", $scope.pageInfo.start + 1 + "|" + $scope.pageInfo.end + "|" +
                            $scope.pageInfo.recordsTotal);
                    });
                }
            };
            init();

            $scope.goToPage = function (page) {
                DataTable.page(page).draw(false);
                init();
            };
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('filteringService', filteringService);

    filteringService.$inject = ['_'];

    function filteringService(_) {
        return {
            filterItems: filterItems
        };

        function filterItems(items, service) {
            if (service.deletedItem) {
                items = _.filter(items, function (item) {
                    return item.id != service.deletedItem
                });
            }
            if (service.newItem) {
                var found = false;
                items = _.map(items, function (item) {
                    if (item.id == service.newItem.metadata.id) {
                        found = true;
                        return service.newItem.metadata;
                    } else {
                        return item;
                    }
                });
                if (!found) {
                    items.push(service.newItem.metadata);
                }
            }
            delete service.newItem;
            delete service.deletedItem;

            return items;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('formMappingService', formMappingService);

    formMappingService.$inject = ['$http', '_'];

    function formMappingService($http, _) {
        var _formMappingContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'rules/';
        var _openMappedProp = "(";
        var _setPropertyNamePrefix = "properties" + _openMappedProp;
        var _setPropertyValuePrefix = "eventProperty::properties" + _openMappedProp;
        var _endPrefix = ")";
        var _newItem;
        var _deletedItem;

        return {
            save: save,
            getAll: getAll,
            get: get,
            serverToLocalFormMapping: serverToLocalFormMapping,
            serverToLocalFormMappingActions: serverToLocalFormMappingActions,
            localToServerFormMapping: localToServerFormMapping,
            remove: remove,
            getFormFactory: getFormFactory
        };

        function save(formMapping) {
            _newItem = formMapping;
            return $http.post(_formMappingContext, formMapping);
        }

        function getAll(offset, limit, sortby) {
            var query = {
                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.tags",
                                    "comparisonOperator": "equals",
                                    "propertyValue": "formMappingRule"
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ]
                    }
                },
                "offset": offset,
                "limit": limit,
                "sortby": sortby
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            query.condition,
                            {
                                parameterValues: {
                                    comparisonOperator: "notEquals",
                                    propertyName: "itemId",
                                    propertyValue: _deletedItem
                                },
                                type: "sessionPropertyCondition"
                            }
                        ]
                    }
                };
                _deletedItem = undefined;
            }

            return $http.post(_formMappingContext + "query", query);
        }

        function get(id) {
            return $http.get(_formMappingContext + id);
        }

        function serverToLocalFormMapping(formMapping) {
            var _formMapping = {
                name: formMapping.metadata.name,
                description: formMapping.metadata.description,
                id: formMapping.metadata.id,
                scope: formMapping.metadata.scope,
                additionalPaths: [],
                tempActions: formMapping.actions,
                actions: {},
                lists: []
            };
            _formMapping.allSite = false;

            _formMapping.formId = _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type == 'formEventCondition';
            }).parameterValues.formId;

            _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type != 'formEventCondition';
            }).parameterValues.subConditions.forEach(function (entry) {
                if (entry.parameterValues.scope) {
                    _formMapping.allSite = true;
                } else {
                    if (!_formMapping.primaryPath) {
                        _formMapping.primaryPath = entry.parameterValues.path;
                    } else if (entry.parameterValues.path != _formMapping.primaryPath) {
                        _formMapping.additionalPaths.push(entry.parameterValues.path);
                    }
                }
            });

            return _formMapping;
        }

        function serverToLocalFormMappingActions(formMapping, propertyNameChoiceListValuesIndexById) {
            formMapping.tempActions.forEach(function (action) {
                if (action.parameterValues.setPropertyName == "systemProperties.lists" && action.parameterValues.setPropertyValue) {
                    formMapping.lists.push(action.parameterValues.setPropertyValue);
                } else {
                    var propertyName = _extractNameFromPrefixedName(action.parameterValues.setPropertyName, _setPropertyNamePrefix);
                    var fieldName = action.parameterValues.setPropertyValue ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValue, _setPropertyValuePrefix) : undefined;
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueInteger ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValueInteger, _setPropertyValuePrefix) : undefined;
                    }
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueMultiple ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValueMultiple, _setPropertyValuePrefix) : undefined;
                    }
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueBoolean ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValueBoolean, _setPropertyValuePrefix) : undefined;
                    }
                    var strategy = action.parameterValues.setPropertyStrategy ? action.parameterValues.setPropertyStrategy : undefined;
                    if (!propertyNameChoiceListValuesIndexById[propertyName]) {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById["user-specified-property"],
                            newProp: propertyName,
                            strategy: strategy
                        };
                    } else {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById[propertyName],
                            strategy: strategy
                        };
                    }
                }
            });
            delete formMapping.tempActions;
        }

        function localToServerFormMapping(formMapping) {
            var _formMapping = {
                "metadata": {
                    "id": formMapping.id,
                    "name": formMapping.name,
                    "description": formMapping.description,
                    "scope": formMapping.scope,
                    "tags": ['formMappingRule']
                },

                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": []
                    }
                },

                "actions": []
            };

            //conditions
            var pathConditions = [];
            pathConditions.push({
                "parameterValues": {
                    "path": formMapping.primaryPath
                },
                "type": "sourceEventPropertyCondition"
            });
            if (formMapping.allSite) {
                pathConditions.push({
                    "parameterValues": {
                        "scope": formMapping.scope
                    },
                    "type": "sourceEventPropertyCondition"
                });
            } else {
                formMapping.additionalPaths.forEach(function (path) {
                    pathConditions.push({
                        "parameterValues": {
                            "path": path
                        },
                        "type": "sourceEventPropertyCondition"
                    })
                });
            }

            _formMapping.condition.parameterValues.subConditions.push(
                {
                    "parameterValues": {
                        "formId": formMapping.formId
                    },
                    "type": "formEventCondition"
                },
                {
                    "parameterValues": {
                        "operator": "or",
                        "subConditions": pathConditions
                    },
                    "type": "booleanCondition"
                }
            );

            //actions
            angular.forEach(formMapping.actions, function(value) {
                value.order = ManagersUtils.formMappingActionStrategies.length - ManagersUtils.formMappingActionStrategies.indexOf(value.strategy);
            });

            angular.forEach(_.sortBy(formMapping.actions, 'order'), function (value) {
                if (value.propertyType) {
                    var strategy = value.strategy;
                    var target = ((value.propertyType.metadata.id == 'user-specified-property' && value.newProp) ? value.newProp : value.propertyType.metadata.id);
                    var action = {
                        "parameterValues": {
                            "setPropertyName": _prefixPropertyName(target, _setPropertyNamePrefix),
                            "setPropertyStrategy": strategy
                        },
                        "type": "setPropertyAction"
                    };

                    if (value.propertyType.type == "integer") {
                        action.parameterValues.setPropertyValueInteger = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                    }

                    if (value.propertyType.type == "boolean") {
                        action.parameterValues.setPropertyValueBoolean = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                    }

                    else {
                        if (value.propertyType.multivalued) {
                            action.parameterValues.setPropertyValueMultiple = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                        }else{
                            action.parameterValues.setPropertyValue = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                        }
                    }
                    _formMapping.actions.push(action);
                }
            });

            //lists
            angular.forEach(formMapping.lists, function (value, key) {
                _formMapping.actions.push({
                    "parameterValues": {
                        "setPropertyName": "systemProperties.lists",
                        "setPropertyValue": value,
                        "setPropertyStrategy": "addValue"
                    },
                    "type": "setPropertyAction"
                });
            });

            return _formMapping;
        }

        function remove(id) {
            _deletedItem = id;
            return $http.delete(_formMappingContext + id);
        }

        function getFormFactory(id) {
            return $http.get(ManagersContext.cmsContext + "/modules/formfactory/live/skeleton/" + id + "/" + ManagersContext.mainResourceLocale);
        }

        // Private function under this line
        function _extractNameFromPrefixedName(prefixedName, prefix) {
            var extracted;
            if (prefixedName.indexOf(_endPrefix) > 0) {
                extracted = prefixedName.substring(prefix.length, prefixedName.length - _endPrefix.length);
            } else {
                // we have an old style property, so remove the old style prefix which wasn't using indexed properties
                // prefix contains the opening of the mapped prop ((') so remove it and add + 1 for the . that was there before
                extracted = prefixedName.substr(prefix.length - _openMappedProp.length + 1);
            }
            return extracted;
        }

        function _prefixPropertyName(name, prefix) {
            return prefix + name + _endPrefix;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('internalSearchService', internalSearchService);

    internalSearchService.$inject = ['$q', '_', 'i18nService', 'unomiQueryService'];

    function internalSearchService($q, _, i18nService, unomiQueryService) {
        var _selectedUrl;
        var _selectedSearch;
        // todo: extract scope condition to reuse it
        // todo: use ManagerUtils.booleanAnd
        var _searchEventsCondition = {
            condition: {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "source.scope",
                                comparisonOperator: "equals",
                                propertyValue: ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "eventTypeCondition",
                            parameterValues: {
                                eventTypeId: "search"
                            }
                        }
                    ],
                    operator: "and"
                }
            }
        };

        // todo: use ManagerUtils.booleanAnd
        var _viewEventsCondition = {
            condition: {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "source.scope",
                                comparisonOperator: "equals",
                                propertyValue: ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "eventTypeCondition",
                            parameterValues: {
                                eventTypeId: "view"
                            }
                        }
                    ],
                    operator: "and"
                }
            }
        };

        // todo: replace by ManagerUtils.AggregateDisplay at some point
        var Search = function (searchString, occurrences) {
            var SearchItem = function (value, occurrences, missing) {
                this.item = value;
                this.occurrences = occurrences;
                this.missing = missing;

                this.importance = function (totalOccurrences) {
                    return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
                }
            };

            this.searchString = searchString;
            this.occurrences = occurrences;

            this.origins = [];
            this.setOrigins = function (origins) {
                this.origins = this._setDetails(origins);
            };

            this.userIds = [];
            this.setUserIds = function (userIds) {
                this.userIds = this._setDetails(userIds);
            };

            this.importance = function (totalOccurrences) {
                return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
            };

            this._setDetails = function (details) {
                if (details) {
                    var detailsArray = [];
                    var totalOccurrences = 0;
                    for (var detail in details) {
                        if (ManagersUtils.acceptProperty(details, detail)) {
                            occurrences = details[detail];

                            var missing = detail == "_missing";
                            if (missing) {
                                // replace _missing by something more user friendly
                                detail = i18nService.message("serverSettings.internalSearches.missing");
                            }

                            totalOccurrences += occurrences;
                            detailsArray.push(new SearchItem(detail, occurrences, missing));
                        }
                    }
                    this.occurrences = totalOccurrences;
                    return detailsArray;
                }
            };

            this.languages = [];
            this.setLanguages = function (lang) {
                this.languages = this._setDetails(lang);
            }
        };

        return {
            setSelectedSearch: setSelectedSearch,
            setSelectedUrl: setSelectedUrl,
            getSearchOccurrencesByTerm: getSearchOccurrencesByTerm,
            getSearchOccurrencesByTermAndDate: getSearchOccurrencesByTermAndDate,
            getSearchDetails: getSearchDetails,
            getMostVisitedList: getMostVisitedList,
            getMostVisitedDetails: getMostVisitedDetails,
            getMostVisitedLanguages: getMostVisitedLanguages,
            getSearchLanguages: getSearchLanguages
        };

        function setSelectedSearch(searchTerm) {
            _selectedSearch = searchTerm;
        }

        function setSelectedUrl(urlPath) {
            _selectedUrl = urlPath;
        }

        function getSearchOccurrencesByTerm() {
            return unomiQueryService.query('event', 'properties.keyword', _searchEventsCondition);
        }

        function getSearchOccurrencesByTermAndDate(from, to, language) {
            var deferred = $q.defer();
            var searchEventsWithDateRangeCondition = {
                "condition": ManagersUtils.getDateRangeConditionBody(from, to)
            };

            if (language && language != null && language !== '_all') {
                searchEventsWithDateRangeCondition = {
                    "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                        {
                            "type": "eventPropertyCondition",
                            "parameterValues": {
                                "propertyName": "properties.language",
                                "comparisonOperator": "equals",
                                "propertyValue": language
                            }
                        }
                    )
                };
            }

            searchEventsWithDateRangeCondition.condition.parameterValues.subConditions.push({
                type: "eventTypeCondition",
                parameterValues: {
                    eventTypeId: "search"
                }
            });

            unomiQueryService.query('event', 'properties.keyword', searchEventsWithDateRangeCondition)
                .then(function (response) {
                    var data = response.data;
                    var searches = [];
                    for (var search in data) {
                        if (ManagersUtils.acceptProperty(data, search)) {
                            searches.push({
                                searchString: search,
                                nbOcc: data[search],
                                importance: ManagersUtils.getPercentage(data[search] / data._filtered)
                            });
                        }
                    }
                    var result = {data: searches};
                    deferred.resolve(result);
                },
                function () {
                    deferred.reject();
                }
            );
            return deferred.promise;
        }

        function getSearchDetails(searchTerm) {
            // if we didn't provide a search term, use the one potentially already selected
            searchTerm = searchTerm || _selectedSearch;

            var search = new Search(searchTerm, -1);

            var condition = {};
            angular.copy(_searchEventsCondition, condition);
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "properties.keyword",
                    "comparisonOperator": "equals",
                    "propertyValue": searchTerm
                }
            });

            unomiQueryService.query('event', 'properties.origin', condition).then(function (response) {
                search.setOrigins(response.data);
            });

            unomiQueryService.query('event', 'profileId', condition).then(function (response) {
                search.setUserIds(response.data);
            });

            unomiQueryService.query('event', 'properties.language', condition).then(function (response) {
                search.setLanguages(response.data);
            });

            return search;
        }

        function getMostVisitedList(limit, from, to, language) {
            var deferred = $q.defer();
            var viewEventsByDateCondition = {
                "condition":
                    ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    {
                        "type": "eventTypeCondition",
                        "parameterValues": {
                            "eventTypeId": "view"
                        }
                    })
            };

            if (language && language != null && language !== '_all') {
                viewEventsByDateCondition = {
                    "condition": ManagersUtils.booleanAndArray([
                        ManagersUtils.getDateRangeConditionBody(from, to),
                        {
                            "type": "eventTypeCondition",
                            "parameterValues": {
                                "eventTypeId": "view"
                            }
                        },
                        {
                            "type": "eventPropertyCondition",
                            "parameterValues": {
                                "propertyName": "target.properties.pageInfo.language",
                                "comparisonOperator": "equals",
                                "propertyValue": language
                            }
                        }
                    ])
                };
            }

            unomiQueryService.query("event", "target.properties.pageInfo.destinationURL", viewEventsByDateCondition)
                .then(function (response) {
                    var dataMap = _.map(response.data, function (occurrences, url) {
                        return {url: url, occurrences: occurrences}
                    });
                    dataMap.splice(0, 2);

                    var result = {data: dataMap, limit: 0};

                    if(limit > 0){
                        result.limit = limit;
                    }

                    //fnCallback(results);
                    deferred.resolve(result);

                },
                function () {
                    deferred.reject();
                });
            return deferred.promise;
        }

        function getMostVisitedDetails() {
            var urlPath = _selectedUrl;
            var search = new Search(urlPath, -1);

            var condition = {};
            angular.copy(_viewEventsCondition, condition);
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.destinationURL",
                    "comparisonOperator": "equals",
                    "propertyValue": urlPath
                }
            });


            unomiQueryService.query('event', 'profileId', condition).then(function (response) {
                search.setUserIds(response.data);
            });

            return search;
        }

        function getMostVisitedLanguages() {
            var deferred = $q.defer();
            unomiQueryService.query('event', 'target.properties.pageInfo.language', _viewEventsCondition)
                .then(function (response) {
                    var dataMap = _.reduce(response.data, function (memo, value, key) {
                        if (key !== '_filtered') {
                            var name = key;
                            if (name === '_all') {
                                name = "all";
                            }
                            memo.push({id: key, name: name});
                        }
                        return memo;
                    }, []);
                    deferred.resolve(dataMap);
                },function () {
                    deferred.reject();
                });
            return deferred.promise;
        }

        function getSearchLanguages() {
            var deferred = $q.defer();
            unomiQueryService.query('event', 'properties.language', _searchEventsCondition)
                .then(function (response) {
                    var dataMap = _.reduce(response.data, function (memo, value, key) {
                        if (key !== '_filtered') {
                            var name = key;
                            if (name === '_all') {
                                name = "all";
                            }
                            memo.push({id: key, name: name});
                        }
                        return memo;
                    }, []);
                    deferred.resolve(dataMap);
                },function () {
                    deferred.reject();
                });
            return deferred.promise;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('jcrService', jcrService);

    jcrService.$inject = ['$http'];

    function jcrService($http) {

        return {
            addMixin: addMixin,
            doGetOnPath: doGetOnPath,
            doGetOnPathSync: doGetOnPathSync
        };

        function addMixin(workspace, locale, nodeIdOrPath, mixin, mixinProperties, byPath) {
            var apiType = byPath ? "paths" : "nodes";
            nodeIdOrPath = byPath ? nodeIdOrPath : "/" + nodeIdOrPath;
            var url = _buildURL(workspace, locale, apiType, nodeIdOrPath) + "/mixins/" + mixin.replace(":", "__");
            return mixinProperties ? $http.put(url, mixinProperties) : $http.put(url);
        }

        function doGetOnPath(workspace, locale, path) {
            return $http.get(_buildURL(workspace, locale, "paths", path));
        }

        function doGetOnPathSync(workspace, locale, path) {
            var request = new XMLHttpRequest();
            request.open('GET', _buildURL(workspace, locale, "paths", path), false);
            request.send(null);

            return request.status === 200;
        }

        // Private function under this line
        function _buildURL(workspace, locale, type, endUrl) {
            return [ManagersContext.jcrRestAPIBase,
                    workspace ? workspace : "default",
                    locale ? locale : ManagersContext.currentSiteDefaultLanguage,
                    type ? type : "nodes"].join("/") + endUrl;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('loadingSpinnerService', loadingSpinnerService);

    function loadingSpinnerService() {
        var _spinner;

        return {
            register: register,
            show: show,
            hide: hide
        };

        function register(spinner) {
            _spinner = spinner;
        }

        function show() {
            if (_spinner) {
                _spinner.show();
            }
        }

        function hide() {
            if (_spinner) {
                _spinner.hide();
            }
        }
    }
})();
(function () {
    'use strict';

    angular.module('wemApp.services').service('modalService', wemAppIsNewUI ? mdModalService : uibModalService);

    mdModalService.$inject = ['$mdDialog'];

    function mdModalService($mdDialog) {
        return {
            newAlertModal: newAlertModal,
            newInfoModal: newInfoModal,
            newCustomModal: newCustomModal
        };

        function newAlertModal(modalData, okCallback, cancelCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-alert.html',
                controller: 'ModalController',
                locals: {
                    modalData: modalData
                }
            });
            var modalInstance = $mdDialog.show(modalData);

            modalInstance.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function (modalData) {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }

        function newInfoModal(modalData, okCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-info.html',
                controller: 'ModalController',
                locals: {
                    modalData: modalData
                }
            });
            var modalInstance = $mdDialog.show(modalData);

            modalInstance.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            });
        }

        function newCustomModal(modalData, event, okCallback, cancelCallback) {
            angular.extend(modalData, {
                controller: 'ModalController',
                controllerAs: 'modalCtrl',
                parent: angular.element('section[wem-main-section]'),
                targetEvent: event,
                locals: {
                    modalData: modalData
                },
                clickOutsideToClose: true
            });
            var modalInstance = $mdDialog.show(modalData);

            modalInstance.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function () {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }
    }

    uibModalService.$inject = ['$uibModal'];

    function uibModalService($uibModal) {
        return {
            newAlertModal: newAlertModal,
            newInfoModal: newInfoModal,
            newCustomModal: newCustomModal
        };

        function newAlertModal(modalData, okCallback, cancelCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-alert.legacy.html',
                controller: 'ModalController',
                size: 'sm',
                resolve: {
                    "modalData": function () {
                        return modalData;
                    }
                }
            });
            var modalInstance = $uibModal.open(modalData);

            modalInstance.result.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function (modalData) {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }

        function newInfoModal(modalData, okCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-info.legacy.html',
                controller: 'ModalController',
                resolve: {
                    "modalData": function () {
                        return modalData;
                    }
                }
            });
            var modalInstance = $uibModal.open(modalData);

            modalInstance.result.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            });
        }

        function newCustomModal(modalData, event, okCallback, cancelCallback) {
            angular.extend(modalData, {
                controller: 'ModalController',
                controllerAs: 'modalCtrl',
                resolve: {
                    "modalData": function () {
                        return modalData;
                    }
                }
            });
            var modalInstance = $uibModal.open(modalData);

            modalInstance.result.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function (modalData) {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }
    }

    angular.module('wemApp.services').controller('ModalController', wemAppIsNewUI ? mdModalController : uibModalController);

    mdModalController.$inject = ['$scope', '$mdDialog', 'i18nService', 'modalData'];

    function mdModalController($scope, $mdDialog, i18nService, modalData) {
        $scope.modal = modalData;
        if ($scope.modal.init) {
            $scope.modal.init($scope, i18nService, modalData);
        }
        if (!$scope.modal.okMsg) {
            $scope.modal.okMsg = i18nService.message("wem.label.ok");
        }
        if (!$scope.modal.cancelMsg) {
            $scope.modal.cancelMsg = i18nService.message("wem.label.cancel");
        }

        $scope.ok = function () {
            $mdDialog.hide($scope.modal);
        };

        $scope.cancel = function () {
            $mdDialog.cancel($scope.modal);
        }
    }

    uibModalController.$inject = ['$scope', '$uibModalInstance', 'i18nService', 'modalData'];

    function uibModalController($scope, $uibModalInstance, i18nService, modalData) {
        $scope.modal = modalData;
        if ($scope.modal.init) {
            $scope.modal.init($scope, i18nService, modalData);
        }
        if (!$scope.modal.okMsg) {
            $scope.modal.okMsg = i18nService.message("wem.label.ok");
        }
        if (!$scope.modal.cancelMsg) {
            $scope.modal.cancelMsg = i18nService.message("wem.label.cancel");
        }

        $scope.ok = function () {
            $uibModalInstance.close($scope.modal);
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
        }
    }

})();
(function() {
    'use strict';

    if (wemAppIsNewUI) {
        angular.module('wemApp.services').service('notificationService', notificationNewUiService);

        notificationNewUiService.$inject = ['$mdToast'];
    } else {
        angular.module('wemApp.services').service('notificationService', notificationOldUiService);

        notificationOldUiService.$inject = ['Notification'];
    }

    function notificationOldUiService(Notification) {
        return {
            successToast: successToast,
            errorToast: errorToast
        };

        function successToast(message) {
            Notification.success(message);
        }

        function errorToast(message) {
            Notification.error(message);
        }
    }

    function notificationNewUiService($mdToast) {
        return {
            successToast: successToast,
            errorToast: errorToast
        };

        function successToast(message) {
            $mdToast.show(
                $mdToast.simple()
                    .textContent(message)
                    .position('bottom left')
                    .hideDelay(5000)
                    .parent(angular.element("section[wem-main-section]"))
            );
        }

        function errorToast(message) {
            $mdToast.show(
                $mdToast.simple()
                    .textContent(message)
                    .theme('error-toast')
                    .position('bottom left')
                    .hideDelay(5000)
                    .parent(angular.element("section[wem-main-section]"))
            );
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('referrerService', referrerService);

    referrerService.$inject = ['$q', 'i18nService', 'unomiQueryService', 'unomiProfileService'];

    function referrerService($q, i18nService, unomiQueryService, unomiProfileService) {
        var _selectedQuery;
        var _selectedReferrer;
        var _scopeCondition = {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": "scope",
                "comparisonOperator": "equals",
                "propertyValue": ManagersContext.dmfScope
            }
        };

        var _viewFromReferrerEventCondition = {
            "type": "eventTypeCondition",
            "parameterValues": {
                "eventTypeId": "viewFromReferrer"
            }
        };

        return {
            getReferrers: getReferrers,
            getReferrerDetails: getReferrerDetails,
            getReferrerQueries: getReferrerQueries,
            getQueries: getQueries,
            getQueryDetails: getQueryDetails,
            getQueryReferrers: getQueryReferrers,
            setSelectedReferrer: setSelectedReferrer,
            setSelectedQuery: setSelectedQuery,
            decodeQuery: decodeQuery
        };

        function getReferrers(from, to) {
            var deferred = $q.defer();
            var referrerEventsByDateCondition = {
                "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    _viewFromReferrerEventCondition)
            };

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerHost', referrerEventsByDateCondition)
                .then(function (response) {
                    var data = response.data;
                    var referrers = [];
                    for (var referrer in data) {
                        if (ManagersUtils.acceptProperty(data, referrer)) {
                            var occurrences = data[referrer];
                            referrers.push({
                                host: referrer,
                                occurrences: occurrences,
                                importance: ManagersUtils.getPercentage(occurrences / data._filtered)
                            });
                        }
                    }

                    var result = {data: referrers};
                    deferred.resolve(result);
                },function () {
                    deferred.reject();
                });

            return deferred.promise;
        }

        function getReferrerDetails(host, offset, length, order) {
            // if we didn't provide a host, use the one potentially already selected
            host = host || _selectedReferrer;

            var condition = {
                "condition": ManagersUtils.booleanAnd(_scopeCondition, {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "properties.referrerHost",
                        "comparisonOperator": "equals",
                        "propertyValue": host
                    }
                }),
                "limit": length,
                "offset": offset,
                "sortby": order,
                "text": null
            };

            return unomiProfileService.searchSessions(condition, 'profiles');
        }

        function getReferrerQueries(host) {
            // if we didn't provide a host, use the one potentially already selected
            host = host || _selectedReferrer;

            var referrerQueries = new ManagersUtils.AggregateDisplay(host, -1, i18nService);

            var condition = {"condition": ManagersUtils.booleanAnd(_scopeCondition, _viewFromReferrerEventCondition)};
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.referrerHost",
                    "comparisonOperator": "equals",
                    "propertyValue": host
                }
            });

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerQuery', condition).then(function (response) {
                referrerQueries.setDetails('queries', response.data);
            });

            return referrerQueries;
        }

        function getQueries(from, to) {
            var deferred = $q.defer();
            var referrerEventsByDateCondition = {
                "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    _viewFromReferrerEventCondition)
            };

            var eventTypeCondition = angular.copy(referrerEventsByDateCondition.condition.parameterValues.subConditions[1]);
            referrerEventsByDateCondition.condition.parameterValues.subConditions[1] = ManagersUtils.booleanAnd(eventTypeCondition, {
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.referrerQuery",
                    "comparisonOperator": "exists"
                }
            });

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerQuery', referrerEventsByDateCondition)
                .then(function (response) {
                    var data = response.data;
                    var queryList = [];
                    for (var query in data) {
                        if (ManagersUtils.acceptProperty(data, query)) {
                            var occurrences = data[query];
                            queryList.push({
                                referrerQuery: query,
                                occurrences: occurrences,
                                importance: ManagersUtils.getPercentage(occurrences / data._filtered)
                            });
                        }
                    }

                    var result = {data: queryList};
                    deferred.resolve(result);
                },function () {
                    deferred.reject();
                });

            return deferred.promise;
        }

        function getQueryDetails(query, offset, length, order) {
            query = decodeQuery(query) || _selectedQuery;

            var condition;
            if ('_missing' === query) {
                condition = {
                    "condition": ManagersUtils.booleanAnd(_scopeCondition, {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "properties.referrerQuery",
                            "comparisonOperator": "missing"
                        }
                    }),
                    "limit": length,
                    "offset": offset,
                    "sortby": order,
                    "text": null
                };
            } else {
                condition = {
                    "condition": ManagersUtils.booleanAnd(_scopeCondition, {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "properties.referrerQuery",
                            "comparisonOperator": "equals",
                            "propertyValue": query
                        }
                    }),
                    "limit": length,
                    "offset": offset,
                    "sortby": order,
                    "text": null
                };
            }

            return unomiProfileService.searchSessions(condition, 'profiles');
        }

        function getQueryReferrers(query) {
            // if we didn't provide a search term, use the one potentially already selected
            query = decodeQuery(query) || _selectedQuery;

            var incomingSearch = new ManagersUtils.AggregateDisplay(query, -1, i18nService);

            var condition = {"condition": ManagersUtils.booleanAnd(_scopeCondition, _viewFromReferrerEventCondition)};
            if ('_missing' === query) {
                condition.condition.parameterValues.subConditions.push({
                    "type": "eventPropertyCondition",
                    "parameterValues": {
                        "propertyName": "target.properties.pageInfo.referrerQuery",
                        "comparisonOperator": "missing"
                    }
                });
            } else {
                condition.condition.parameterValues.subConditions.push({
                    "type": "eventPropertyCondition",
                    "parameterValues": {
                        "propertyName": "target.properties.pageInfo.referrerQuery",
                        "comparisonOperator": "equals",
                        "propertyValue": query
                    }
                });
            }

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerHost', condition).then(function (response) {
                incomingSearch.setDetails('hosts', response.data);
            });

            incomingSearch.incomingSearchString = ManagersUtils.humanReadableMissingIfNeeded(query, i18nService);

            return incomingSearch;
        }

        function setSelectedReferrer(referrer) {
            _selectedReferrer = referrer;
        }

        function setSelectedQuery(query) {
            _selectedQuery = query;
            return _encodeQuery(query);
        }

        function decodeQuery(query) {
            return query.replace(/\+/g, ' ')
        }

        // Private functions under this line
        function _encodeQuery(query) {
            return query.replace(/ /g, '+')
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiCampaignService', unomiCampaignService);

    unomiCampaignService.$inject = ['$http', 'i18nService'];

    function unomiCampaignService($http, i18nService) {
        var _campaignsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'campaigns/';
        var _deletedCampaign;
        var _deletedEvent;
        var _newEvent;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            queryDetails: queryDetails,
            setCampaign: setCampaign,
            setCampaignEvent: setCampaignEvent,
            getCampaign: getCampaign,
            getCampaignDetail: getCampaignDetail,
            getLocalizedCampaignType: getLocalizedCampaignType,
            getLocalizedCampaignParam: getLocalizedCampaignParam,
            getTypeSpecificCampaignParameter: getTypeSpecificCampaignParameter,
            getCampaignEvents: getCampaignEvents,
            deleteCampaign: deleteCampaign,
            deleteCampaignEvent: deleteCampaignEvent,
            getCampaignType: getCampaignType,
            createCampaignEntryCondition: createCampaignEntryCondition
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (_deletedCampaign) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedCampaign
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedCampaign = undefined
            }

            return query;
        }

        function getAll() {
            return $http.post(_campaignsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function queryDetails(query) {
            return $http.post(_campaignsContext + "query/detailed", query);
        }

        function setCampaign(campaign) {
            return $http.post(_campaignsContext, campaign);
        }

        function setCampaignEvent(campaignEvent) {
            _newEvent = campaignEvent;
            return $http.post(_campaignsContext + "event", campaignEvent);
        }

        function getCampaign(id) {
            return $http.get(_campaignsContext + id);
        }

        function getCampaignDetail(id) {
            return $http.get(_campaignsContext + id + "/detailed");
        }

        function getLocalizedCampaignType(campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType);
        }

        function getLocalizedCampaignParam(campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType + '.paramName');
        }

        function getTypeSpecificCampaignParameter(campaign) {
            var campaignType = getCampaignType(campaign);
            switch (campaignType) {
                case 'landing':
                    return campaign.entryCondition.parameterValues.subConditions[1].parameterValues.pagePath;
                case 'referrer':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
                case 'parameter':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
            }
        }

        function getCampaignEvents(campaignId, length, start, order, additionalCondition) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaignId
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (additionalCondition) {
                query.condition.parameterValues.subConditions.push(additionalCondition);
            }

            if (_newEvent) {
                query.forceRefresh = true;
                _newEvent = undefined;
            }

            if (_deletedEvent) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedEvent
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedEvent = undefined;
            }
            return $http.post(_campaignsContext + "events/query", query);
        }

        function deleteCampaign(id) {
            _deletedCampaign = id;
            return $http.delete(_campaignsContext + id);
        }

        function deleteCampaignEvent(eventId) {
            _deletedEvent = eventId;
            return $http.delete(_campaignsContext + "event/" + eventId);
        }

        function getCampaignType(campaign) {
            return campaign.metadata.tags[0];
        }

        function createCampaignEntryCondition(type) {
            var entryCondition = {
                "parameterValues": {
                    "subConditions": [],
                    "operator": "and"
                },
                "type": "booleanCondition"
            };
            switch (type.value) {
                case 'landing':
                    entryCondition.parameterValues.subConditions.push({
                        "parameterValues": {
                            "propertyName": "duration",
                            "propertyValueInteger": 0,
                            "comparisonOperator": "equals"
                        },
                        "type": "sessionPropertyCondition"
                    });
                    entryCondition.parameterValues.subConditions.push({
                            "parameterValues": {
                                "pagePath": type.param
                            },
                            "type": "pageViewEventCondition"
                        }
                    );
                    break;
                case 'referrer':
                    entryCondition.parameterValues.subConditions.push({
                        "parameterValues": {
                            "propertyName": "duration",
                            "propertyValueInteger": 0,
                            "comparisonOperator": "equals"
                        },
                        "type": "sessionPropertyCondition"
                    });
                    entryCondition.parameterValues.subConditions.push({
                            "type": "pageViewEventCondition"
                        }
                    );
                    entryCondition.parameterValues.subConditions.push({
                            "parameterValues": {
                                "propertyName": "target.properties.pageInfo.referringURL",
                                "propertyValue": _escapeRegex(type.param),
                                "comparisonOperator": "matchesRegex"
                            },
                            "type": "eventPropertyCondition"

                        }
                    );
                    break;
                case 'parameter':
                    entryCondition.parameterValues.subConditions.push({
                        "parameterValues": {
                            "propertyName": "duration",
                            "propertyValueInteger": 0,
                            "comparisonOperator": "equals"
                        },
                        "type": "sessionPropertyCondition"
                    });
                    entryCondition.parameterValues.subConditions.push({
                            "type": "pageViewEventCondition"
                        }
                    );
                    entryCondition.parameterValues.subConditions.push({
                            "parameterValues": {
                                "propertyName": "target.properties.pageInfo.destinationURL",
                                "propertyValue": _escapeRegex(type.param),
                                "comparisonOperator": "matchesRegex"
                            },
                            "type": "eventPropertyCondition"

                        }
                    );
                    break;
            }
            return entryCondition;
        }

        // Private function under this line
        function _escapeRegex(regex) {
            var result = regex.replace(/[\\\.\[\]\{\}\(\)\+\-\?\^\$\|]/g, "\\$&").replace(/\*/g, ".*");
            if (result.substr(0, 2) != ".*") {
                result = ".*" + result;
            }
            if (result.substr(result.length - 2) != ".*") {
                result += ".*";
            }
            return result;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiDefinitionService', unomiDefinitionService);

    unomiDefinitionService.$inject = ['$http'];

    function unomiDefinitionService($http) {
        var _definitionsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'definitions/';

        return {
            getTag: getTag,
            getConditionType: getConditionType,
            getConditionTypes: getConditionTypes,
            getConditionTypesByTag: getConditionTypesByTag,
            fillTags: fillTags,
            resolveProfileTags: resolveProfileTags
        };

        function getTag(tag) {
            return $http.get(_definitionsContext + 'tags/' + tag)
        }

        function getConditionType(conditionType) {
            return $http.get(_definitionsContext + 'conditions/' + conditionType)
        }
        
        function getConditionTypes() {
            return $http.get(_definitionsContext + 'conditions')
        }

        function getConditionTypesByTag(tag) {
            return $http.get(_definitionsContext + 'conditions/tags/' + tag)
        }

        function fillTags(arrayName, $scope) {
            $scope.$watch(arrayName + ".length", function (newVal, oldVal, scope) {
                angular.forEach($scope[arrayName], function (item) {
                    if (ManagersContext.conditionsTags && ManagersContext.conditionsTags[item.id]) {
                        item.profileTag = ManagersContext.conditionsTags[item.id];
                    } else {
                        angular.forEach(item.tags, function (tag) {
                            if ($scope.tags[tag]) {
                                item.profileTag = tag;
                            }
                        });
                    }
                });
            });
        }

        function resolveProfileTags(arrayToFill, tags) {
            angular.forEach(arrayToFill, function (item) {
                if (ManagersContext.conditionsTags && ManagersContext.conditionsTags[item.id]) {
                    item.profileTag = ManagersContext.conditionsTags[item.id];
                } else {
                    angular.forEach(item.tags, function (tag) {
                        if (tags[tag]) {
                            item.profileTag = tag;
                        }
                    });
                }
            });
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiExportConfigService', unomiExportConfigService);

    unomiExportConfigService.$inject = ['$http', '$q'];

    function unomiExportConfigService($http, $q) {
        var _exportConfigContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'exportConfiguration/';
        var _exportConfigOneshotContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'exportConfiguration/oneshot/';
        var _selectedTab;

        return {
            getAll: getAll,
            setExportConfig: setExportConfig,
            setOneshotExportConfig: setOneshotExportConfig,
            getExportConfig: getExportConfig,
            deleteExportConfig: deleteExportConfig
        };

        function getAll(limit, offset, sortby) {
            var actualLimit = limit || 100;
            var actualOffset = offset || 0;
            var actualSortBy = sortby || null;
            return $http.get(_exportConfigContext);
        }

        function setExportConfig(exportConfig) {
            return $http.post(_exportConfigContext, exportConfig);
        }

        function setOneshotExportConfig(exportConfig) {
            var deferred = $q.defer();
            setExportConfig(exportConfig).then(function(response) {
                $http({
                    method: 'POST',
                    url: _exportConfigOneshotContext,
                    data: response.data,
                    responseType: 'arraybuffer'
                }).then(function (response) {
                    var headers = response.headers();
                    var fileNameAttr = headers["content-disposition"].split(';')[1];
                    var filename = fileNameAttr.substring(fileNameAttr.indexOf('=') + 1, fileNameAttr.length);
                    var contentType = headers['content-type'];
                    var linkElement = document.createElement('a');
                    try {
                        var blob = new Blob([response.data], {type: contentType});
                        var url = window.URL.createObjectURL(blob);

                        linkElement.setAttribute('href', url);
                        linkElement.setAttribute("download", filename);

                        var clickEvent = new MouseEvent("click", {
                            "view": window,
                            "bubbles": true,
                            "cancelable": false
                        });
                        linkElement.dispatchEvent(clickEvent);
                        deferred.resolve({msg: "success"});
                    } catch (ex) {
                        console.log(ex);
                        deferred.resolve({msg: ex});
                    }
                }, function (data) {
                    console.log(data);
                    deferred.resolve({msg: data});
                });
            });
            return deferred.promise;
        }

        function getExportConfig(id) {
            return $http.get(_exportConfigContext + id);
        }

        function deleteExportConfig(id) {
            return $http.delete(_exportConfigContext + id);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiGeonamesService', unomiGeonamesService);

    unomiGeonamesService.$inject = ['$http'];

    function unomiGeonamesService($http) {
        return {
            getEntries: getEntries,
            getHierarchy: getHierarchy,
            getCities: getCities,
            getCapital: getCapital,
            reverseGeoCode: reverseGeoCode
        };

        function getEntries() {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/entries';
            var length = url.length;
            if (arguments.length > 0) {
                for (var i = 0; i < arguments.length; i++) {
                    url = url + ("/" + arguments[i]);
                }
            }
            if (length == url.length) {
                url += "/";
            }

            return $http.get(url);
        }

        function getHierarchy(administrativeSbudivision) {
            return $http.get(ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/hierarchy/' + administrativeSbudivision);
        }

        function getCities() {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/cities';
            var length = url.length;
            if (arguments.length > 0) {
                for (var i = 0; i < arguments.length; i++) {
                    if (arguments[i]) {
                        url = url + ("/" + arguments[i]);
                    }
                }
            }
            if (length == url.length) {
                url += "/";
            }
            return $http.get(url);
        }

        function getCapital(itemId) {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/capitals/' + itemId;
            return $http.get(url);
        }

        function reverseGeoCode(lat, lon) {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/reverseGeoCode/' + lat + ',' + lon;
            return $http.get(url);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiGoalService', unomiGoalService);

    unomiGoalService.$inject = ['$http'];

    function unomiGoalService($http) {
        var ADD_TO_LIST_RULE = "addToListRULE_";
        var _goalsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'goals/';

        return {
            localToServerGoalRule: localToServerGoalRule,
            getAll: getAll,
            getAllForSite: getAllForSite,
            getAllForCampaign: getAllForCampaign,
            getGoalReport: getGoalReport,
            getGoalReportAggregated: getGoalReportAggregated,
            getAddToListRulePrefix: getAddToListRulePrefix,
            setGoal: setGoal,
            getGoal: getGoal,
            deleteGoal: deleteGoal
        };

        function localToServerGoalRule(goal, lists) {
            if (goal) {
                var _goalRule = {
                    "metadata": {
                        "id": ADD_TO_LIST_RULE + goal.metadata.id,
                        "name": goal.metadata.name,
                        "description": goal.metadata.description,
                        "scope": goal.metadata.scope
                    },
                    "condition": {
                        type: "booleanCondition",
                        parameterValues: {
                            subConditions: [
                                {
                                    type: "eventPropertyCondition",
                                    parameterValues: {
                                        propertyName: "target.itemId",
                                        comparisonOperator: "equals",
                                        propertyValue: goal.metadata.id+'TargetEvent'
                                    }
                                },
                                {
                                    type: "eventTypeCondition",
                                    parameterValues: {
                                        eventTypeId: "ruleFired"
                                    }
                                }
                            ],
                            operator: "and"
                        }
                    },

                    "actions": []
                };

                //lists
                angular.forEach(lists, function (value, key) {
                    _goalRule.actions.push({
                        "parameterValues": {
                            "setPropertyName": "systemProperties.lists",
                            "setPropertyValue": value,
                            "setPropertyStrategy": "addValue"
                        },
                        "type": "setPropertyAction"
                    });
                });

                return _goalRule;
            }

            return null;
        }

        function getAll() {
            return $http.post(_goalsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function getAllForSite() {
            return $http.post(_goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "missing"
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function getAllForCampaign(campaign) {
            return $http.post(_goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaign
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function getGoalReport(id) {
            var uri = _goalsContext + id + "/report";
            return $http.get(uri);
        }

        function getGoalReportAggregated(id, query) {
            var uri = _goalsContext + id + "/report";
            return $http.post(uri, query);
        }

        function getAddToListRulePrefix(){
            return ADD_TO_LIST_RULE;
        }

        function setGoal(goal) {
            return $http.post(_goalsContext, goal);
        }

        function getGoal(id) {
            return $http.get(_goalsContext + id);
        }

        function deleteGoal(id) {
            return $http.delete(_goalsContext + id);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiImportConfigService', unomiImportConfigService);

    unomiImportConfigService.$inject = ['$http', '$q'];

    function unomiImportConfigService($http, $q) {
        var _importConfigContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'importConfiguration/';
        var _importConfigOneshotContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'importConfiguration/oneshot/';
        var _selectedTab;

        return {
            getAll: getAll,
            setImportConfig: setImportConfig,
            setOneshotImportConfig: setOneshotImportConfig,
            getImportConfig: getImportConfig,
            deleteImportConfig: deleteImportConfig
        };

        function getAll(limit, offset, sortby) {
            var actualLimit = limit || 100;
            var actualOffset = offset || 0;
            var actualSortBy = sortby || null;
            return $http.get(_importConfigContext);
        }

        function setImportConfig(importConfig) {
            return $http.post(_importConfigContext, importConfig);
        }

        function setOneshotImportConfig(importConfig, files) {
            var deferred = $q.defer();
            setImportConfig(importConfig).then(function(savedImportConfig) {
                var formData = new FormData();
                formData.append('importConfigId', savedImportConfig.data.itemId);
                if(files[0] && !files[0].isRemote){
                    formData.append('file', files[0].lfFile);
                }
                $http.post(_importConfigOneshotContext, formData, {
                    transformRequest: angular.identity,
                    headers: {'Content-Type': undefined}
                }).then(function (response) {
                    deferred.resolve({msg:response});
                });
            });

            return deferred.promise;

        }

        function getImportConfig(id) {
            return $http.get(_importConfigContext + id);
        }

        function deleteImportConfig(id) {
            return $http.delete(_importConfigContext + id);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiListService', unomiListService);

    unomiListService.$inject = ['$http'];

    function unomiListService($http) {
        var _listsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'lists/';
        var _newItem;
        var _deletedItem;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            query: query,
            setList: setList,
            getList: getList,
            deleteList: deleteList
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedItem = undefined;
            }

            return query;
        }

        function getAll() {
            return $http.get(_listsContext);
        }

        function query(query) {
            return $http.post(_listsContext + "query", ManagersUtils.getQueryOrDefault(query));
        }

        function setList(list, shared) {
            if (shared) {
                list.metadata.scope = 'systemscope';
            }
            _newItem = list;
            return $http.post(_listsContext, list);
        }

        function getList(id) {
            return $http.get(_listsContext + id);
        }

        function deleteList(id) {
            _deletedItem = id;
            return $http.delete(_listsContext + id);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiProfileService', unomiProfileService);

    unomiProfileService.$inject = ['$http', 'unomiQueryService'];

    function unomiProfileService($http, unomiQueryService) {
        var _profilesContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'profiles/';
        var _personasContext = _profilesContext + 'personas/';
        var _personasWithSessionsContext = _profilesContext + 'personasWithSessions/';
        var _selectedTab;

        return {
            search: search,
            searchSessions: searchSessions,
            exportUsingForm: exportUsingForm,
            getProfile: getProfile,
            saveProfile: saveProfile,
            newProfile: newProfile,
            deleteProfile: deleteProfile,
            batchProfilesUpdate: batchProfilesUpdate,
            getSessions: getSessions,
            savePersonaWithSessions: savePersonaWithSessions,
            getPropertyTypes: getPropertyTypes,
            createPropertyType: createPropertyType,
            deletePropertyType: deletePropertyType,
            getPropertyTypesByTarget: getPropertyTypesByTarget,
            getPropertyTypesByTag: getPropertyTypesByTag,
            getCount: getCount,
            constructCondition: constructCondition,
            constructQuery: constructQuery,
            constructQueryOnCurrentSelectedTab: constructQueryOnCurrentSelectedTab,
            setSelectedTab: setSelectedTab,
            setSelectedTabType: setSelectedTabType,
            setSelectedTabDeletedProfileId: setSelectedTabDeletedProfileId,
            setSelectedTabConditionTypes: setSelectedTabConditionTypes,
            setSelectedTabEventConditionTypes: setSelectedTabEventConditionTypes,
            setSelectedTabSearchCondition: setSelectedTabSearchCondition,
            setSelectedTabSearchConditionType: setSelectedTabSearchConditionType,
            setSelectedTabSearchConditionParameterValues: setSelectedTabSearchConditionParameterValues,
            getSelectedTab: getSelectedTab,
            deleteSelectedTabBaseCondition: deleteSelectedTabBaseCondition,
            setSelectedTabBaseCondition: setSelectedTabBaseCondition
        };

        function search(query, profileType) {
            return $http.post(_getContextFor(profileType) + "search", query);
        }

        function searchSessions(query, profileType) {
            return $http.post(_getContextFor(profileType) + 'search/sessions', query);
        }

        function exportUsingForm(query, profileType, method) {
            ManagersUtils.downloadForm(_getContextFor(profileType) + "export", query, method);
        }

        function getProfile(id, profileType) {
            return $http.get(_getContextFor(profileType) + id);
        }

        function saveProfile(profile, profileType) {
            // remove empty props
            var cleanedProp = {};
            angular.forEach(profile.properties, function (value, key) {
                if (value) {
                    cleanedProp[key] = value;
                } else {
                    if(typeof value != 'boolean') {
                        cleanedProp[key] = null;
                    } else {
                        cleanedProp[key] = value;
                    }
                }
            });
            profile.properties = cleanedProp;
            return $http.post(_getContextFor(profileType), profile);
        }

        function newProfile(id, profileType) {
            return $http.put(_getContextFor(profileType) + id);
        }

        function deleteProfile(id, profileType) {
            return $http.delete(_getContextFor(profileType) + id);
        }

        function batchProfilesUpdate(batchUpdate) {
            return $http.post(_profilesContext + 'batchProfilesUpdate', batchUpdate)
        }

        function getSessions(id, profileType, offset, numberOfSessions, sort) {
            return $http.get(_getContextFor(profileType) + id + '/sessions?offset=' + offset + '&size=' + numberOfSessions + '&sort=' + sort);
        }

        function savePersonaWithSessions(personaObject) {
            return $http.post(_personasWithSessionsContext, personaObject);
        }

        function getPropertyTypes() {
            return $http.get(_profilesContext + 'properties');
        }

        function createPropertyType(propertyType) {
            return $http.post(_profilesContext + 'properties', propertyType);
        }

        function deletePropertyType(propertyId) {
            return $http.delete(_profilesContext + 'properties/' + propertyId);
        }

        function getPropertyTypesByTarget(target) {
            return $http.get(_profilesContext + 'properties/' + target)
        }

        function getPropertyTypesByTag(tag, recursive) {
            if (recursive) {
                return $http.get(_profilesContext + 'properties/tags/' + tag + '?recursive=true')
            } else {
                return $http.get(_profilesContext + 'properties/tags/' + tag)
            }
        }

        function getCount() {
            return unomiQueryService.getMatchingItemsCountForCondition('profile', constructCondition(null,null,null, true,'profile'), false);
        }

        function constructCondition(baseCondition, searchCondition, deletedProfileId, excludeMergedAndAnonymous, profileType) {
            var subConditions = [];
            if (baseCondition) {
                subConditions.push(baseCondition);
            }
            if (excludeMergedAndAnonymous && profileType != "personas") {
                // we do not want the merged profiles:
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "missing",
                        propertyName: "mergedWith"
                    },
                    type: "profilePropertyCondition"
                });
                subConditions.push({
                    parameterValues: {
                        propertyName: "systemProperties.isAnonymousProfile",
                        comparisonOperator: "missing"
                    },
                    type: "profilePropertyCondition"
                });
            }
            if (searchCondition && searchCondition.type) {
                if (searchCondition.type == "booleanCondition"
                    && searchCondition.parameterValues.operator
                    && searchCondition.parameterValues.operator == "and"
                    && searchCondition.parameterValues.subConditions) {
                    subConditions = subConditions.concat(searchCondition.parameterValues.subConditions);
                } else {
                    subConditions.push(searchCondition);
                }
            }
            if (deletedProfileId) {
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: deletedProfileId
                    },
                    type: "profilePropertyCondition"
                });
            }

            if (subConditions.length == 0) {
                return null;
            } else if (subConditions.length == 1) {
                return subConditions[0];
            } else {
                return {
                    parameterValues: {
                        operator: "and",
                        subConditions: subConditions
                    },
                    type: "booleanCondition"
                };
            }
        }

        function constructQuery(baseCondition, searchCondition, deletedProfileId, text, offset, limit, orderBy, profileType) {
            return {
                "condition": constructCondition(baseCondition, searchCondition, deletedProfileId, true, profileType),
                "limit": limit ? limit : 50,
                "offset": offset ? offset : 0,
                "sortby": orderBy != undefined ? orderBy : null,
                "text": text != undefined ? text : null
            };
        }

        function constructQueryOnCurrentSelectedTab(text, offset, limit, orderBy) {
            if (_selectedTab) {
                return {
                    "condition": constructCondition(_selectedTab.baseCondition, _selectedTab.searchCondition, _selectedTab.deletedProfileId, true, _selectedTab.type),
                    "limit": limit ? limit : -1,
                    "offset": offset ? offset : 0,
                    "sortby": orderBy != undefined ? orderBy : null,
                    "text": text != undefined ? text : null
                };
            }
        }

        // TODO: selectedTab is used outside of the service so I put some getter and setter here but I believe we should rewrite most of this
        function getSelectedTab() {
            return _selectedTab;
        }

        function setSelectedTab(selectedTab) {
            _selectedTab = selectedTab;
        }

        function setSelectedTabSearchCondition(searchCondition) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.searchCondition = searchCondition;
        }

        function setSelectedTabSearchConditionType(searchConditionType) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            if (!_selectedTab.searchCondition) {
                _selectedTab.searchCondition = {};
            }
            _selectedTab.searchCondition.type = searchConditionType;
        }

        function setSelectedTabSearchConditionParameterValues(searchConditionParameterValues) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            if (!_selectedTab.searchCondition) {
                _selectedTab.searchCondition = {};
            }
            _selectedTab.searchCondition.parameterValues = searchConditionParameterValues;
        }

        function setSelectedTabType(selectedTabType) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.type = selectedTabType;
        }

        function setSelectedTabDeletedProfileId(deletedProfileId) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.deletedProfileId = deletedProfileId;
        }

        function setSelectedTabConditionTypes(conditionTypes) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.conditionTypes = conditionTypes;
        }

        function setSelectedTabEventConditionTypes(eventConditionTypes) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.eventConditionTypes = eventConditionTypes;
        }

        function deleteSelectedTabBaseCondition() {
            delete _selectedTab.baseCondition;
        }

        function setSelectedTabBaseCondition(baseCondition) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.baseCondition = baseCondition;
        }
        
        // Private functions under this line
        function _getContextFor(profileType) {
            return profileType == 'personas' ? _personasContext : _profilesContext;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiQueryService', unomiQueryService);

    unomiQueryService.$inject = ['$http', 'i18nService', 'notificationService'];

    function unomiQueryService($http, i18nService, notificationService) {
        var _queryContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'query/';

        return {
            query: query,
            getMatchingItemsCountForCondition: getMatchingItemsCountForCondition,
            createWithCallbackIfDoesNotExist: createWithCallbackIfDoesNotExist,
            nbOfVisits: nbOfVisits
        };

        function query(type, property, aggregate) {
            if (!aggregate) {
                return $http.get(_queryContext + type + '/' + property);
            }
            return $http.post(_queryContext + type + '/' + property, aggregate);
        }

        function getMatchingItemsCountForCondition(type, condition, validate) {
            return $http.post(_queryContext + type + '/count' + (validate != undefined ? ("?validate="+validate) : ""), condition);
        }

        function createWithCallbackIfDoesNotExist(itemType, scope, name, typeId, errorMessageKey, errorMessageParams, callback, failCallback) {
            var condition = {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.scope",
                                comparisonOperator: "equals",
                                propertyValue: scope ? scope : ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.name",
                                comparisonOperator: "equals",
                                propertyValue: name
                            }
                        }
                    ],
                    operator: "and"
                }
            };

            if (typeId) {
                condition.parameterValues.subConditions.push({
                    type: "sessionPropertyCondition",
                    parameterValues: {
                        propertyName: "metadata.tags",
                        comparisonOperator: "contains",
                        propertyValue: typeId
                    }
                });
            }

            getMatchingItemsCountForCondition(itemType, condition)
                .then(function (response) {
                // if we have matching elements, display an error
                if (response.data > 0) {
                    if (failCallback) {
                        failCallback();
                    }
                    notificationService.errorToast(errorMessageParams ? i18nService.format(errorMessageKey, errorMessageParams) : i18nService.message(errorMessageKey));
                } else {
                    // otherwise perform the provide callback
                    callback();
                }
            });
        }

        // todo: move to unomiCampaignService?
        function nbOfVisits(campaignId, startDate, endDate, callback) {
            getMetricsForCondition("session", "profile.properties.nbOfVisits", ["sum", "avg"], {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "scope",
                                "comparisonOperator": "equals",
                                "propertyValue": ManagersContext.dmfScope
                            }
                        },
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "systemProperties.campaigns." + campaignId + "Engaged",
                                "comparisonOperator": "between",
                                "propertyValuesDate": [startDate, endDate]
                            }
                        }
                    ]
                }
            }).then(callback);
        }

        // Private functions under this line
        function getMetricsForCondition(type, property, metrics, condition) {
            return $http.post(_queryContext + type + '/' + property + '/' + ($.isArray(metrics) ? metrics.join('/') : metrics), condition);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiRulesService', unomiRulesService);

    unomiRulesService.$inject = ['$http'];

    function unomiRulesService($http) {
        var _rulesContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'rules/';
        var _rulesStatisticsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'rules/statistics';

        return {
            getAll: getAll,
            setRule: setRule,
            getRule: getRule,
            deleteRule: deleteRule,
            getStatistics : getStatistics
        };

        function getAll(limit, offset, sortby) {
            var actualLimit = limit || 100;
            var actualOffset = offset || 0;
            var actualSortBy = sortby || null;
            return $http.post(_rulesContext + "query", {
                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "or",
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": "systemscope"
                                }
                            }
                        ]
                    }
                },
                "limit": actualLimit,
                "offset": actualOffset,
                "sortby": actualSortBy
            });
        }

        function setRule(rule) {
            return $http.post(_rulesContext, rule);
        }

        function getRule(id) {
            return $http.get(_rulesContext + id);
        }

        function deleteRule(id) {
            return $http.delete(_rulesContext + id);
        }

        function getStatistics() {
            return $http.get(_rulesStatisticsContext);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiScoringService', unomiScoringService);

    unomiScoringService.$inject = ['$http', 'jcrService'];

    function unomiScoringService($http, jcrService) {
        var _scoringContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'scoring/';
        var _newItem;
        var _deletedItem;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            getImpacted: getImpacted,
            query: query,
            setScoring: setScoring,
            getScoring: getScoring,
            deleteScoring: deleteScoring,
            buildUpdateBatchRemoveScoring: buildUpdateBatchRemoveScoring
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedItem = undefined;
            }

            return query;
        }

        function getAll() {
            return $http.get(_scoringContext);
        }

        function getImpacted(segmentId) {
            return $http.get(_scoringContext + segmentId + "/impacted");
        }

        function query(query) {
            return $http.post(_scoringContext + "query", query);
        }

        function setScoring(scoring, shared) {
            scoring.elements.forEach(function (element) {
                _addDownloadMixin(element.condition);
            });
            if (shared) {
                scoring.metadata.scope = 'systemscope';
            }
            _newItem = scoring;
            return $http.post(_scoringContext, scoring);
        }

        function getScoring(id) {
            return $http.get(_scoringContext + id);
        }

        function deleteScoring(id, validate) {
            var uri = _scoringContext + id;
            if (validate) {
                uri += '?validate=true';
            }
            return $http.delete(uri);
        }

        function buildUpdateBatchRemoveScoring(scoreId) {
            var baseCondition = {
                parameterValues: {
                    comparisonOperator: "exists",
                    propertyName: "scores."+scoreId
                },
                type: "profilePropertyCondition"
            };
            return {
                condition: baseCondition,
                propertyName: "systemProperties.notificationAck.scoring."+scoreId,
                propertyValue: null,
                strategy: "remove"
            };
        }

        // Private function
        function _addDownloadMixin(condition) {
            var pastEventConditions = _getSubConditionsOfType(condition, [], "pastEventCondition");
            pastEventConditions.forEach(function (pastEventCondition) {
                var downloadConditions = _getSubConditionsOfType(pastEventCondition.parameterValues.eventCondition, [], "downloadEventCondition");
                downloadConditions.forEach(function (downloadCondition) {
                    jcrService.addMixin("live", null, downloadCondition.parameterValues.filePath, "wemmix:trackDownloadedFile", {}, true).then(function (response) {
                        // mixin added
                    });
                });
            });
        }

        function _getSubConditionsOfType(rootCondition, list, conditionType) {
            if (rootCondition.type == "booleanCondition" && rootCondition.parameterValues && rootCondition.parameterValues.subConditions) {
                rootCondition.parameterValues.subConditions.forEach(function(subCondition) {
                    _getSubConditionsOfType(subCondition, list, conditionType);
                })
            } else if (rootCondition.type == conditionType) {
                list.push(rootCondition);
            }

            return list;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiSegmentService', unomiSegmentService);

    unomiSegmentService.$inject = ['$http', 'jcrService'];

    function unomiSegmentService($http, jcrService) {
        var _segmentsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'segments/';
        var _newItem;
        var _deletedItem;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            getImpacted: getImpacted,
            query: query,
            addDownloadMixin: addDownloadMixin,
            setSegment: setSegment,
            getSegment: getSegment,
            getSegmentCount: getSegmentCount,
            deleteSegment: deleteSegment
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedItem = undefined;
            }

            return query;
        }

        function getAll() {
            return $http.get(_segmentsContext);
        }

        function getImpacted(segmentId) {
            return $http.get(_segmentsContext + segmentId + "/impacted");
        }

        function query(query) {
            return $http.post(_segmentsContext + "query", query);
        }

        function addDownloadMixin(condition) {
            var pastEventConditions = _getSubConditionsOfType(condition, [], "pastEventCondition");
            pastEventConditions.forEach(function (pastEventCondition) {
                var downloadConditions = _getSubConditionsOfType(pastEventCondition.parameterValues.eventCondition, [], "downloadEventCondition");
                downloadConditions.forEach(function (downloadCondition) {
                    jcrService.addMixin("live", null, downloadCondition.parameterValues.filePath, "wemmix:trackDownloadedFile", {}, true).then(function (response) {
                        // mixin added
                    });
                })
            });
        }

        function setSegment(segment, shared) {
            addDownloadMixin(segment.condition);
            if (shared) {
                segment.metadata.scope = 'systemscope';
            }
            _newItem = segment;
            return $http.post(_segmentsContext, segment);
        }

        function getSegment(id) {
            return $http.get(_segmentsContext + id);
        }

        function getSegmentCount(id) {
            return $http.get(_segmentsContext + id + "/count");
        }

        function deleteSegment(id, validate) {
            var uri = _segmentsContext + id;
            if (validate) {
                uri += '?validate=true';
            }
            return $http.delete(uri);
        }

        // Private function under this line
        function _getSubConditionsOfType(rootCondition, list, conditionType) {
            if (rootCondition.type == "booleanCondition" && rootCondition.parameterValues && rootCondition.parameterValues.subConditions) {
                rootCondition.parameterValues.subConditions.forEach(function(subCondition) {
                    _getSubConditionsOfType(subCondition, list, conditionType);
                })
            } else if (rootCondition.type == conditionType) {
                list.push(rootCondition);
            }

            return list;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiUserListService', unomiUserListService);

    unomiUserListService.$inject = ['$http'];

    function unomiUserListService($http) {
        var _userListsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'userList/';

        return {
            getAll: getAll
        };

        function getAll() {
            return $http.get(_userListsContext);
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.services').service('validationUtilsService', validationUtilsService);

    validationUtilsService.$inject = ['loadingSpinnerService'];

    function validationUtilsService(loadingSpinnerService) {
        return {
            getFormInPage: getFormInPage
        };

        function getFormInPage($scope, formId, path, callback) {
            var id = "formFrame" + Math.random().toString(16).substring(2);
            var tries = 5;
            console.log("creating frame " + id);
            var $iframe = $("<iframe id='" + id + "'></iframe>");
            $iframe.css("display", "none");
            $("body").append($iframe);
            loadingSpinnerService.show();
            $iframe.ready(function () {
                console.log("iframe " + id + " loaded");
                var _timeOut = function () {
                    $scope.$apply(function () {
                        var _form = $iframe.contents().find("form[name='" + formId + "']");
                        if (!_form || _form.length == 0) {
                            _form = $iframe.contents().find("#" + formId);
                        }
                        if ((_form && _form.length > 0 && _form.find(":input").length > 0) || tries == 0) {
                            console.log("found form : " + _form.length);
                            $("#" + id).remove();
                            loadingSpinnerService.hide();
                            return callback(_form);
                        } else {
                            tries--;
                            console.log("not found, retrying : " + tries);
                            window.setTimeout(_timeOut, 1000);
                        }
                    });
                };
                window.setTimeout(_timeOut, 1000);
            });
            $iframe.attr("src", ManagersContext.baseLive + path + ".html?skipWem=true");
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemFoldableInput', wemFoldableInput);

    wemFoldableInput.$inject = ['i18nService'];

    function wemFoldableInput(i18nService) {
        return {
            restrict: 'E',
            scope: {
                model: '=',
                name: '@',
                multivalued: '=',
                required: '=',
                labelKey: '@',
                label: '@',
                altClass: '@',
                type: '@',
                disabled: '=',
                pattern: '@',
                hasError: '='
            },
            transclude:true,
            controller: WemFoldableInputController,
            link: linkFunc,
            templateUrl:  ManagersContext.wemAppPath + "/components/foldable-input/base.directive.html"
        };

        function linkFunc(scope, element, attrs) {
            if (scope.labelKey && i18nService.contains(scope.labelKey)) {
                scope.label = i18nService.message(scope.labelKey);
            }
            if (scope.required) {
                scope.label = scope.label + " (" + i18nService.message('label.required') + ")";
            }
            if (attrs.type !== "boolean") {
                element.find("label").text(scope.label).click(function () {
                    // need for IE that doesn't support  pointer-events: none; in CSS
                    if (attrs.type == "textarea") {
                        element.find("textarea").focus();
                    } else {
                        element.find("input").focus();
                    }
                });
            }

            if (attrs.type == "textarea") {
                scope.templateUrl = ManagersContext.wemAppPath + "/components/foldable-input/textarea.directive.html";
            } else if(attrs.type == "date") {
                scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/date.directive.html";
            } else if(attrs.type == "boolean") {
                scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/boolean.directive.html";
            } else {
                if(scope.multivalued){
                    scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/multipleInput.directive.html";
                }else{
                    scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/generic.directive.html";
                }
            }
        }
    }

    WemFoldableInputController.$inject = ['$scope', 'uibDatepickerConfig', 'i18nService'];

    function WemFoldableInputController($scope, uibDatepickerConfig, i18nService) {
        if ($scope.type == 'date' && $scope.model != null && typeof $scope.model == "string") {
            $scope.model = moment($scope.model).toDate();
        }

        $scope.inner = {
            model: $scope.model
        };

        $scope.tempValue = null;

        var foldedClass;
        if ($scope.altClass) {
            foldedClass = "foldedLabel " + $scope.altClass;
        } else {
            foldedClass = "foldedLabel";
        }

        function updateLabel(newValue) {
            if (((newValue != null && (typeof newValue == "number" || newValue != '')) && !$scope.multivalued) || $scope.type == "boolean") {
                $scope.labelClass = foldedClass;
            } else {
                $scope.labelClass = "";
            }
        }

        $scope.focusInput = function (event) {
            $scope.labelClass = foldedClass;
            $scope.focused = true;
            if ($scope.type == "date") {
                $scope.datepicker.datePickerOpen = true;
                event.target.blur()
            }
        };

        $scope.blurInput = function (event) {
            //Callback for blur event defined on the parent Scope
            if ($scope.type != "date" && $scope.$parent["blurCallback"]) {
                $scope.$parent["blurCallback"](event);
            }
            updateLabel(event.target.value);
            $scope.focused = false;
        };

        $scope.$watch("inner.model", function(newVal) {
            $scope.model = newVal;
        });

        $scope.$watch("model", function (newValue, oldValue) {
            $scope.inner.model = newValue;
            if (!$scope.focused) {
                updateLabel(newValue);
            }
        });

        updateLabel($scope.model);
        $scope.focused = false;

        if ($scope.type == 'date') {
            uibDatepickerConfig.startingDay = 1;
            uibDatepickerConfig.minMode = "day";
            uibDatepickerConfig.datepickerMode = "day";
            uibDatepickerConfig.showWeeks = false;

            $scope.openDatePopup = function ($event, selector) {
                $event.preventDefault();
                $event.stopPropagation();

                $scope.datepicker.datePickerOpen = true;
            };
            $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
            $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
            $scope.clearText = i18nService.message('wem.label.clear');
            $scope.datepicker = {
                datePickerOpen: false
            };
        }

        $scope.onDeleteMultiValue = function (property, value) {
            return _.reject(property, function(val){ return val === value; });
        };

        $scope.onAddMultiValue = function (property, newValue) {
            if (!property) {
                property = [];
            }
            if (!Array.isArray(property)) {
                property = Array.of(property);
            }
            if (newValue && newValue != null && newValue != '') {
                if (!_.contains(property, newValue)) {
                    property.push(angular.copy(newValue));
                }
            }

            return property;
        };
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemHtmlTagPicker', wemHtmlTagPicker);

    function wemHtmlTagPicker() {
        return {
            restrict: "E",
            scope: {
                selectedTagId: '=ngModel',
                origin: '@',
                tag: '@',
                attrs: '@',
                displayAttr: '@',
                disabled: '@',
                label: '@',
                name: '@',
                required: '='
            },
            templateUrl: ManagersContext.wemAppPath + "/components/pickers/wem-html-tag-picker.directive.html",
            controller: WemHtmlTagPickerController,
            link: linkFunc
        };

        function linkFunc(scope, element, attributes) {
            var tag = attributes.tag ? attributes.tag : "form";
            var attrs = attributes.attrs ? attributes.attrs.split('|') : ["name", "id"];
            var displayAttr = attributes.displayAttr;

            scope.toBoolean = function (value) {
                if (typeof value === 'function') {
                    value = true;
                } else if (value && value.length !== 0) {
                    var v = angular.lowercase("" + value);
                    value = !(v == 'f' || v == '0' || v == 'false' || v == 'no' || v == 'n' || v == '[]');
                } else {
                    value = false;
                }
                return value;
            };

            var displayIframe = function (src) {
                var $iframe = $("<iframe></iframe>");
                $iframe.css("display", "none");
                $iframe.on("load", function () {
                    var tags = [];
                    $iframe.contents().find(tag).each(function () {
                        var $element = $(this);
                        for (var i = 0; i < attrs.length; i++) {
                            var attr = attrs[i];
                            if ($element.is("[" + attr + "]")) {
                                var id = $element.attr(attr);
                                tags.push({
                                    "id": id,
                                    "display": (displayAttr && $element.is("[" + displayAttr + "]")) ? $element.attr(displayAttr) : id
                                });
                                break;
                            }
                        }
                    });
                    scope.$apply(function () {
                        scope.tags = tags;
                        scope.loaded = true;
                        scope.loading = false;
                    });
                });
                $iframe.attr("src", src);
                element.append($iframe);
            };

            attributes.$observe('origin', function (value) {
                if (value) {
                    scope.loaded = false;
                    scope.loading = true;
                    element.find("iframe").remove();
                    displayIframe(ManagersContext.baseLive + value + ".html?skipWem=true");
                }
            });
        }
    }

    WemHtmlTagPickerController.$inject = ['$scope'];

    function WemHtmlTagPickerController($scope) {
        $scope.tags = [];
        $scope.loaded = false;
        $scope.loading = false;
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemHtmlTagVisualPicker', wemHtmlTagVisualPicker);

    function wemHtmlTagVisualPicker() {
        return {
            restrict: "E",
            scope: {
                selectedTagId: '=ngModel',
                origin: '@',
                tag: '@',
                attrs: '@',
                displayAttr: '@',
                disabled: '@',
                callback: '@'
            },
            templateUrl: ManagersContext.wemAppPath + "/components/pickers/wem-html-tag-visual-picker.directive.html",
            controller: WemHtmlTagVisualPickerController
        };
    }

    WemHtmlTagVisualPickerController.$inject = ['$scope', '$uibModal'];

    function WemHtmlTagVisualPickerController($scope, $uibModal) {
        $scope.showVisualPicker = function () {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/components/pickers/wem-html-tag-visual-picker-modal.directive.html',
                controller: 'WemHtmlTagVisualPickerModalController',
                size: 'full',
                windowClass: 'tagPickerModal',
                backdrop: false,
                resolve: {
                    options: function () {
                        return {
                            selectedTagId: $scope.selectedTagId,
                            src: ManagersContext.baseLive + $scope.origin + ".html?skipWem=true",
                            tag: $scope.tag,
                            attrs: $scope.attrs ? $scope.attrs.split('|') : ["name", "id"],
                            displayAttr: $scope.displayAttr,
                            disabled: $scope.disabled,
                            callback: $scope.callback
                        };
                    }
                }
            });

            modalInstance.result.then(function (selectedTagId) {
                $scope.selectedTagId = selectedTagId;
            });
        }
    }

    angular.module('wemApp.components').controller('WemHtmlTagVisualPickerModalController', WemHtmlTagVisualPickerModalController);

    WemHtmlTagVisualPickerModalController.$inject = ['$rootScope', '$scope', '$uibModalInstance', 'options'];

    function WemHtmlTagVisualPickerModalController($rootScope, $scope, $uibModalInstance, options) {
        $scope.options = options;

        $scope.ok = function () {
            $uibModalInstance.close($scope.options.selectedTagId);
            $('.modal-content').css('height', $('#newGoalForm').height());
        };

        $scope.iframeLoadedCallBack = function (element) {
            ["keypress","submit","focus","blur","tab"].forEach(function(eventName){
                frames['visualPickerFrame'].addEventListener(eventName, function(e){
                    e.preventDefault();
                    e.stopPropagation();

                    // if an input is somehow focused, unfocus it
                    var target = e.target;
                    setTimeout(function(){
                        if (target.blur) target.blur();
                    }, 50);
                }, true);
            });

            $('body', frames['visualPickerFrame'].document).bind("click", function (event) {
                var $targetElement = $(event.target);
                if ($scope.options.callback) {
                    if ($rootScope[$scope.options.callback]) {
                        $rootScope[$scope.options.callback](event, $scope);
                    }
                } else {
                    var found = $targetElement.closest($scope.options.tag);
                    if (found && found.length > 0) {
                        for (var i = 0; i < $scope.options.attrs.length; i++) {
                            var attr = $scope.options.attrs[i];
                            if (found.is("[" + attr + "]")) {
                                $scope.$apply(function () {
                                    $scope.options.selectedTagId = found.attr(attr);
                                });
                                break;
                            }
                        }
                    }
                }

                // disable links that trigger navigation on other pages, keep ajax links start with # and empty href links that can be used to display hided contents
                if(($targetElement.is("a") && $targetElement.attr("href") && $targetElement.attr("href").length > 0 && $targetElement.attr("href")[0] != "#")){
                    event.preventDefault();
                    event.stopPropagation();
                    return false;
                }
            });
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
            $('.modal-content').css('height', $('#newGoalForm').height());
        };
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemNodePicker', wemNodePicker);

    function wemNodePicker() {
        return {
            restrict: "E",
            scope: {
                selectedPath: '=ngModel',
                selectedId: '=',
                nodeTypes: '@',
                selectableNodeTypes: '@',
                displayableNodesOnly: '@',
                basePath: '@',
                baseUrl: '@',
                label: '@'
            },
            link: linkFunc,
            templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? "/components/pickers/wem-node-picker-modal.directive.html" : "/components/pickers/wem-node-picker-modal.directive.legacy.html"),
            controller: WemNodePickerController
        };

        function linkFunc(scope, element, attrs) {
            if (!attrs.selectedId) {
                scope.selectedIdSpecified = false;
            } else {
                scope.selectedIdSpecified = true;
            }
        }
    }

    WemNodePickerController.$inject = ['$scope', 'modalService'];

    function WemNodePickerController($scope, modalService) {
        $scope.openPicker = function () {
            var modalData = {
                directiveScope : $scope,
                init : function ($scope, i18nService, modalData) {
                    $scope.root = {};
                    $scope.base = (modalData.directiveScope.baseUrl && ManagersContext[modalData.directiveScope.baseUrl]) ? ManagersContext[modalData.directiveScope.baseUrl] : ManagersContext.baseLive;

                    function selectInTree() {
                        if ($scope.selectedPath) {
                            $scope.root.children.forEach(function (child) {
                                if ($scope.selectedPath != child.path && $scope.selectedPath.indexOf(child.path) == 0) {
                                    $scope.toggle(child, selectItemInTree)
                                }
                            });
                        }
                    }

                    function selectItemInTree(item) {
                        if ($scope.selectedPath && item && item.hasChildren && item.children.length > 0) {
                            item.displayed = true;
                            item.children.forEach(function (child) {
                                if ($scope.selectedPath != child.path && $scope.selectedPath.indexOf(child.path) == 0) {
                                    $scope.toggle(child, selectItemInTree)
                                }
                            });
                        }
                    }

                    if (!modalData.directiveScope.basePath) {
                        modalData.directiveScope.basePath = "";
                    }

                    // init get root node
                    $.ajax({
                        type: "GET",
                        dataType: "json",
                        url: $scope.base + ManagersContext.currentSitePath + modalData.directiveScope.basePath + ".treeRootItem.json?nodeTypes=" + modalData.directiveScope.nodeTypes
                        + (modalData.directiveScope.selectableNodeTypes ? "&selectableNodeTypes=" + modalData.directiveScope.selectableNodeTypes : "")
                        + (modalData.directiveScope.displayableNodesOnly ? "&displayablenodeonly=true" : "")
                    }).done(function (data) {
                        $scope.$apply(function () {
                            if (data && data.length > 0) {
                                $scope.root = data[0];
                                $scope.root.displayed = true;
                                $scope.$watch('selectedPath', selectInTree);
                            }
                        });
                    });

                    $scope.getIcon = function (isRoot, item) {
                        if (isRoot) {
                            return "web";
                        }

                        if (item) {
                            if ($scope.isSelectable(item)) {
                                return "insert_drive_file";
                            } else {
                                return "folder_open"
                            }
                        }

                        return "insert_drive_file";
                    };

                    $scope.getArrow = function (item) {
                        if (item.hasChildren) {
                            if (item.displayed) {
                                return "keyboard_arrow_down";
                            } else {
                                return "keyboard_arrow_right";
                            }
                        }
                        return "";
                    };

                    $scope.getItemClass = function (item) {
                        if ($scope.isSelected(item)) {
                            return "selected";
                        }

                        if ($scope.isSelectable(item)) {
                            return "selectable";
                        }
                    };

                    $scope.isSelectable = function (item) {
                        return "selectable" == item.classes;
                    };

                    $scope.isSelected = function (item) {
                        return item.path == $scope.selectedPath;
                    };

                    $scope.select = function (item) {
                        if ("selectable" == item.classes) {
                            modalData.item = item;
                            $scope.ok();
                        }
                    };

                    $scope.toggle = function (item, cb) {
                        if (item.hasChildren) {
                            if (item.children && item.children.length > 0) {
                                item.displayed = !item.displayed;
                                if (cb) {
                                    cb(item)
                                }
                            } else {
                                $.ajax({
                                    type: "GET",
                                    dataType: "json",
                                    url: $scope.base + item.path + ".treeRootItem.json?nodeTypes=" + modalData.directiveScope.nodeTypes
                                    + (modalData.directiveScope.selectableNodeTypes ? "&selectableNodeTypes=" + modalData.directiveScope.selectableNodeTypes : "")
                                    + (modalData.directiveScope.displayableNodesOnly ? "&displayablenodeonly=true" : "")
                                }).done(function (data) {
                                    var children = data && data.length > 0 ? data[0].children : [];
                                    $scope.$apply(function () {
                                        item.children = children;
                                        item.displayed = true;
                                        if (cb) {
                                            cb(item)
                                        }
                                    });
                                });
                            }
                        }
                    };
                },
                templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? "/components/pickers/wem-node-picker.directive.html" : "/components/pickers/wem-node-picker.directive.legacy.html")
            };

            var okCallback = function (modalData) {
                $scope.selectedPath = modalData.item.path;
                if ($scope.selectedIdSpecified) {
                    $scope.selectedId = modalData.item.id;
                }
            };

            var modalInstance = modalService.newCustomModal(
                modalData,
                null,
                okCallback,
                null
            );

        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemSiteLanguagePicker', wemSiteLanguagePicker);

    function wemSiteLanguagePicker() {
        return {
            restrict: "E",
            scope: {
                selectedLanguage: '=ngModel'
            },
            templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? "/components/pickers/wem-site-language-picker.directive.html" : "/components/pickers/wem-site-language-picker.directive.legacy.html"),
            controller: WemSiteLanguagePickerController
        };
    }

    WemSiteLanguagePickerController.$inject = ['$scope', 'i18nService'];

    function WemSiteLanguagePickerController($scope, i18nService) {
        $scope.root = {};
        $scope.loaded = false;

        $.ajax({
            type: "GET",
            dataType: "json",
            url: ManagersContext.baseLive + ManagersContext.currentSitePath + ".languages.json"
        }).done(function (data) {
            $scope.$apply(function () {
                if (data && data.length > 0) {
                    data.push({
                        value: i18nService.message('serverSettings.languagePicker.anyLanguages')
                    });
                    $scope.siteLanguages = data;
                    $scope.loaded = true;
                }
            });
        });

    }
})();
(function() {
    'use strict';

    // Todo: I don't know why we have this directive which is using 'ng' as prefix

    angular.module('wemApp.components').directive('ngEnter', ngEnter);

    function ngEnter() {
        return function (scope, element, attrs) {
            element.bind("keydown keypress", function (event) {
                if(event.which === 13) {
                    scope.$apply(function (){
                        scope.$eval(attrs.ngEnter);
                    });

                    event.preventDefault();
                }
            });
        };
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemAction', wemActionDirective);

    function wemActionDirective() {
        return {
            restrict: "E",
            scope: {
                ref: '=',
                parentList: '=',
                parentIndex: '=',
                types: '='
            },
            templateUrl: ManagersContext.wemAppPath + "/components/wem-action.directive.html",
            controller: WemActionDirectiveController
        };
    }

    WemActionDirectiveController.$inject = ['$scope'];

    function WemActionDirectiveController($scope) {

        $scope.updateTemplate = function() {
            if( Object.prototype.toString.call( $scope.ref ) === '[object Array]' ) {
                $scope.actionTemplateUrl = ManagersContext.wemAppPath + "/components/wem-actions-list.directive.html";
            } else if ($scope.ref && $scope.ref.type != "empty") {
                $scope.actionType = $scope.getActionType($scope.ref);
                if ($scope.actionType != null) {
                    $scope.actionTemplateUrl = ManagersContext.wemAppPath + "/actions/templates/" + $scope.actionType.id + ".html";
                }
            }
        };

        $scope.getActionType = function (action) {
            for (var i = 0; i < $scope.types.length; i++) {
                var actionType = $scope.types[i];
                if (actionType.id == action.type) {
                    return actionType;
                }
            }
            return null;
        };

        $scope.addAction = function() {
            var actionType = $scope.added.addedActionType;
            $scope.added.addedActionType = null;
            var newAction = {
                "type" : actionType.id,
                "parameterValues" : {}
            };
            for (var i=0; i < actionType.parameters.length; i++) {
                var parameter = actionType.parameters[i];
                if (parameter.multivalued) {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null && parameter.defaultValue.length > 0) {
                        newAction.parameterValues[parameter.id] = parameter.defaultValue.split(",");
                    } else {
                        newAction.parameterValues[parameter.id] = [];
                    }
                } else {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null && parameter.defaultValue.length > 0) {
                        newAction.parameterValues[parameter.id] = parameter.defaultValue;
                    } else {
                        if (parameter.type == "string") {
                            newAction.parameterValues[parameter.id] = "";
                        } else {
                            newAction.parameterValues[parameter.id] = null;
                        }
                    }
                }
            }
            $scope.ref.push(newAction);
        };

        $scope.onRemove = function (index) {
            console.log("wem-action.directive.js : onRemove("+ index +") called");
            $scope.parentList.splice(index, 1);
        };

        if (!$scope.ref) {
            console.log("wem-action.directive.js: no action passed, creating empty one.");
            $scope.ref = {};
            $scope.ref.type = "empty";
        }

        console.log("wem-action.directive.js: Initializing action controller for action " + $scope.ref + " of type="+$scope.ref.type + " parentIndex=" + $scope.parentIndex);

        $scope.added = {};
        $scope.updateTemplate();

        $scope.$watch("ref", function (newValue, oldValue) {
            console.log("wem-action.directive.js : action changed, refreshing...");
            $scope.updateTemplate();
        });

        $scope.$watch("types.length", function (newValue, oldValue) {
            console.log("wem-action.directive.js : action type list changed, refreshing...");
            $scope.updateTemplate();
        });
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemAutofocus', wemAutofocus);

    wemAutofocus.$inject = ['$timeout'];

    function wemAutofocus($timeout) {
        return {
            restrict: 'A',
            link: function (scope, element) {
                $timeout(function () {
                    element[0].focus();
                });
            }
        };
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemChecklistModel', wemChecklistModel);

    wemChecklistModel.$inject = ['$parse', '$compile'];

    function wemChecklistModel($parse, $compile) {
        return {
            restrict: 'A',
            priority: 1000,
            terminal: true,
            scope: true,
            compile: function (tElement, tAttrs) {
                if (tElement[0].tagName !== 'INPUT' || tAttrs.type !== 'checkbox') {
                    throw 'wem-checklist-model should be applied to `input[type="checkbox"]`.';
                }

                if (!tAttrs.wemChecklistValue) {
                    throw 'You should provide `wem-checklist-value`.';
                }

                // exclude recursion
                tElement.removeAttr('wem-checklist-model');

                // local scope var storing individual checkbox model
                tElement.attr('ng-model', 'checked');

                return postLinkFn;
            }
        };

        // http://stackoverflow.com/a/19228302/1458162
        function postLinkFn(scope, elem, attrs) {
            // compile with `ng-model` pointing to `checked`
            $compile(elem)(scope);

            // getter / setter for original model
            var getter = $parse(attrs.wemChecklistModel);
            var setter = getter.assign;
            var checklistChange = $parse(attrs.checklistChange);

            // value added to list
            var value = $parse(attrs.wemChecklistValue)(scope.$parent);


            var comparator = angular.equals;

            if (attrs.hasOwnProperty('wemChecklistComparator')) {
                comparator = $parse(attrs.wemChecklistComparator)(scope.$parent);
            }

            // watch UI checked change
            scope.$watch('checked', function (newValue, oldValue) {
                if (newValue === oldValue) {
                    return;
                }
                var current = getter(scope.$parent);
                if (newValue === true) {
                    setter(scope.$parent, add(current, value, comparator));
                } else {
                    setter(scope.$parent, remove(current, value, comparator));
                }

                if (checklistChange) {
                    checklistChange(scope);
                }
            });

            // declare one function to be used for both $watch functions
            function setChecked(newArr, oldArr) {
                scope.checked = contains(newArr, value, comparator);
            }

            // watch original model change
            // use the faster $watchCollection method if it's available
            if (angular.isFunction(scope.$parent.$watchCollection)) {
                scope.$parent.$watchCollection(attrs.wemChecklistModel, setChecked);
            } else {
                scope.$parent.$watch(attrs.wemChecklistModel, setChecked, true);
            }
        }

        // contains
        function contains(arr, item, comparator) {
            if (angular.isArray(arr)) {
                for (var i = arr.length; i--;) {
                    if (comparator(arr[i], item)) {
                        return true;
                    }
                }
            }
            return false;
        }

        // add
        function add(arr, item, comparator) {
            arr = angular.isArray(arr) ? arr : [];
            if (!contains(arr, item, comparator)) {
                arr.push(item);
            }
            return arr;
        }

        // remove
        function remove(arr, item, comparator) {
            if (angular.isArray(arr)) {
                for (var i = arr.length; i--;) {
                    if (comparator(arr[i], item)) {
                        arr.splice(i, 1);
                        break;
                    }
                }
            }
            return arr;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive("wemCondition", wemConditionDirective);

    wemConditionDirective.$inject = ['RecursionHelper'];

    function wemConditionDirective(RecursionHelper) {
        return {
            restrict: "E",
            scope: {
                countlabel:'@',
                countlabeli18n:'@',
                disableEdition: '=',
                disableBoolean: '=',
                displayCount: '=',
                form: '=',
                itemType: '@',
                labelNew: '=',
                level: '@',
                parentIndex: '=',
                parentName: '=',
                parentRef: '=',
                ref: '=',
                showCount: '=',
                types: '='
            },
            templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? "/components/wem-condition-builder.directive.html" : "/components/wem-condition-builder.directive.legacy.html"),
            compile: function (element) {
                // Use the compile function from the RecursionHelper,
                // And return the linking function(s) which it returns
                return RecursionHelper.compile(element);
            },
            controller: WemConditionController
        };
    }

    WemConditionController.$inject = ['$scope', '$rootScope', '$injector', 'unomiQueryService', 'i18nService', 'modalService', '_', 'autocompleteService'];

    function WemConditionController($scope, $rootScope, $injector, unomiQueryService, i18nService, modalService, _, autocompleteService) {
        $scope.autoCompletedProperties = [];
        autocompleteService.getAutocompletedProperties().then(function(response){
            $scope.autoCompletedProperties = response.data;
        });

        function isMultivalued (propertyName){
            var propertyIndex = _.findIndex($scope.autoCompletedProperties, function(value) {
                return value.itemId == propertyName.replace("properties.", "") && value.multivalued == true;
            });
            return propertyIndex > -1;
        }
        //Set autocomplete options
        $scope.autocompleteValue = "on";
        $scope.autoCompleteOptions = {
            minimumChars: 1,
            data: function (term) {
                var selectedProperty = $scope.ref.parameterValues.propertyName;
                var propertyType = "";
                if(!$scope.currentItemType){
                    if($scope.ref.type == "profilePropertyCondition"){
                        propertyType = "profile"
                    }
                    if($scope.ref.type == "sessionPropertyCondition"){
                        propertyType = "session"
                    }
                } else {
                    propertyType = $scope.currentItemType;
                }
                if(autocompleteService.isAutocomplete(selectedProperty,$scope.autoCompletedProperties)){
                    $scope.autocompleteValue = "off";
                    var multivalued = isMultivalued(selectedProperty);
                    term = term.toUpperCase();
                    //Candidate values are calculated by the autocomplete service
                    return autocompleteService.completePropertyValue(selectedProperty,term,propertyType, multivalued);
                }
            },
            selectedTextAttr : 'value',
            itemTemplate : "{{item.value}} - {{item.count}}"
        };

        var $uibModal = null;
        if ($injector.has('$uibModal')) {
            $uibModal = $injector.get('$uibModal');
        }
        var uibDatepickerConfig = null;
        if ($injector.has('uibDatepickerConfig')) {
            uibDatepickerConfig = $injector.get('uibDatepickerConfig');
        }

        if(angular.isUndefined($scope.disableEdition)) {
            $scope.disableEdition = false;
        }
        if (angular.isUndefined($scope.disableBoolean)) {
            $scope.disableBoolean = false;
        }

        $scope.getCountLabel =  function(){
            if(!$scope.countlabeli18n){
                if(!$scope.countlabel) {
                    return "";
                } else {
                    return $scope.countlabel;
                }
            } else {
                return i18nService.message($scope.countlabeli18n);
            }
        };

        if(!$scope.level) {
            $scope.level = 0;
        }

        if (angular.isUndefined($scope.displayCount)) {
            $scope.displayCount = true;
        }

        if (angular.isUndefined($scope.labelNew)) {
            $scope.labelNew = "condition.new";
        }
        $scope.newConditionText = i18nService.message($scope.labelNew);
        if($scope.form && $scope.form.scoringId){
            $scope.scoringId = $scope.form.scoringId.$modelValue;
        }

        if (uibDatepickerConfig) {
            // date config
            uibDatepickerConfig.startingDay = 1;
            uibDatepickerConfig.minMode = "day";
            uibDatepickerConfig.datepickerMode = "day";
            uibDatepickerConfig.showWeeks = false;
        }
        $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
        $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
        $scope.clearText = i18nService.message('wem.label.clear');
        $scope.datePickers = {};

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();
            angular.forEach($scope.datePickers, function(value, key) {
                $scope.datePickers[key] = false;
            });
            $scope.datePickers[selector] = true;
        };

        $scope.hasError = function(form, propertyName) {
            return ManagersUtils.hasErrors($scope.form, form, propertyName);
        };

        $scope.getHumanizeTime = function (value, unit) {
            return moment.duration(value, unit).humanize();
        };

        $scope.updateTemplate = function () {
            if ($scope.ref && $scope.ref.type) {
                $scope.conditionType = $scope.getConditionType($scope.ref);
                if ($scope.conditionType !== null) {
                    var conditionTypeId = $scope.conditionType.id;
                    var conditionTemplatesDir = (wemAppIsNewUI) ? ManagersContext.wemAppPath + '/conditions/templates/' : ManagersContext.wemAppPath + '/conditions/templates/legacy/';
                    var propertyTemplatesDir = conditionTemplatesDir + 'property/';

                    // special handling of property templates
                    if (conditionTypeId.indexOf('Property') !== -1) {
                        $scope.conditionLabelKey = 'condition.propertyCondition.' + conditionTypeId;
                        $scope.conditionTemplateUrl = propertyTemplatesDir + 'propertyCondition.html';
                        $scope.conditionSelectTemplate = conditionTypeId == 'eventPropertyCondition' ? propertyTemplatesDir + conditionTypeId + 'Select.html'
                            : propertyTemplatesDir + 'defaultPropertyConditionSelect.html';
                    } else {
                        $scope.conditionTemplateUrl = conditionTemplatesDir + conditionTypeId + '.html';
                    }
                }
            }
        };

        $scope.getConditionType = function (condition) {
            function resolveConditionTypeIn(list, condition) {
                if (list) {
                    for (var i = 0; i < list.length; i++) {
                        var conditionType = list[i];
                        if (conditionType.id == condition.type) {
                            return conditionType;
                        }
                    }
                }
            }

            if (condition.conditionType) {
                return condition.conditionType;
            }
            var lists = [$scope.types, $scope.conditionTypes, $scope.eventConditionTypes];
            for (var i = 0; i < lists.length; i++) {
                var resolved = resolveConditionTypeIn(lists[i], condition);
                if (resolved) {
                    return resolved;
                }
            }
            return null;
        };

        $scope.addCondition = function (conditionType, targetName, wrapIn) {
            if (wrapIn) {
                var copyCurrentRef = angular.copy($scope.ref);
                targetName = "subConditions";
                $scope.ref = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": wrapIn,
                        "subConditions": [copyCurrentRef]
                    }
                };
                if ($scope.parentRef) {
                    $scope.parentRef.parameterValues[$scope.parentName][$scope.parentIndex] = $scope.ref;
                }
            }


            var newCondition = {
                "type": conditionType.id,
                "parameterValues": {}
            };
            for (var i = 0; i < conditionType.parameters.length; i++) {
                var parameter = conditionType.parameters[i];
                if (parameter.multivalued) {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null && parameter.defaultValue.length > 0) {
                        newCondition.parameterValues[parameter.id] = parameter.defaultValue.split(",");
                    } else {
                        newCondition.parameterValues[parameter.id] = [];
                    }
                } else {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null) {
                        if (parameter.type == "integer") {
                            newCondition.parameterValues[parameter.id] = parseInt(parameter.defaultValue);
                        } else {
                            newCondition.parameterValues[parameter.id] = parameter.defaultValue;
                        }
                    }
                }
            }

            if (targetName) {
                if (Object.prototype.toString.call($scope.ref.parameterValues[targetName]) === '[object Array]') {
                    $scope.ref.parameterValues[targetName].push(newCondition);
                } else {
                    $scope.ref.parameterValues[targetName] = newCondition;
                }
            } else {
                $scope.ref = newCondition;
            }
        };

        $scope.onRemove = function (parentName, index) {
            console.log("wem-condition.directive.js : onRemove(" + parentName + "," + index + ") called");
            var parent = $scope.ref.parameterValues[parentName];
            if (parent instanceof Array) {
                parent.splice(index, 1);
                $scope.ref.parameterValues[parentName] = parent;
                if ($scope.ref.type == 'booleanCondition') {
                    if (parent.length == 1) {
                        $scope.ref = $scope.ref.parameterValues.subConditions[0]; // unwrap boolean condition
                        if ($scope.parentRef) {
                            $scope.parentRef.parameterValues[$scope.parentName][$scope.parentIndex] = $scope.ref;
                        }
                    } else if (parent.length == 0) {
                        // delete empty boolean condition
                        if ($scope.parentRef) {
                            parent = $scope.parentRef.parameterValues[$scope.parentName];
                            if (parent instanceof Array) {
                                parent.splice($scope.parentIndex, 1);
                                $scope.parentRef.parameterValues[$scope.parentName] = parent;
                            } else {
                                $scope.parentRef.parameterValues[$scope.parentName] = null;
                            }
                        } else {
                            $scope.ref = {};
                        }
                    }
                }
            } else {
                $scope.ref.parameterValues[parentName] = null;
            }
        };

        $scope.onRemoveRoot = function () {
            console.log("wem-condition.directive.js : onRemoveRoot() called");
            $scope.ref = {};
        };

        $scope.onAddValue = function (values, newVal, $event) {
            // if we're dealing with dates, convert the given value to a date first
            if (values == $scope.ref.parameterValues.propertyValuesDate && typeof newVal === 'string') {
                console.log("Condition builder : Dealing with date !");
                newVal = new Date(newVal);
            }
            values.push(newVal);

            $event.preventDefault();
        };

        $scope.onRemoveValue = function (values, pos, $event) {
            values.splice(pos, 1);

            $event.preventDefault();
        };

        $scope.canWrapBoolCondition = function (operator) {
            return (!$scope.disableBoolean && $scope.ref.type != 'booleanCondition'
            && ($scope.parentRef == null || $scope.parentRef.type != 'booleanCondition' || $scope.parentRef.parameterValues.operator != operator));
        };

        $scope.openAddConditionModal = function (targetName, wrapIn) {
            if($scope.form && $scope.form.scoringType){
                $scope.scoringType = $scope.form.scoringType.$viewValue;
            }
            var filteredTypes = $scope.types;
            if ($scope.scoringType) {
                if ($scope.scoringType !== 'st:all') {
                    filteredTypes = _.filter(filteredTypes, function (conditionType) {
                        return $scope.scoringType.split(':')[1] == conditionType.profileTag;
                    });
                } else {
                    filteredTypes = _.filter(filteredTypes, function (conditionType) {
                        return _.contains($rootScope.allowedTags, conditionType.profileTag);
                    });
                }
            }
            var conditionTypes = _.filter(filteredTypes, function (conditionType) {
                return (conditionType.id != "booleanCondition" && conditionType.id != "notCondition" &&
                conditionType.id != "matchAllCondition");
            });
            var conditionTypeGroups = [];
            if (conditionTypes) {
                for (var i = 0; i < conditionTypes.length; i++) {
                    var profileTag = conditionTypes[i].profileTag;
                    if (!_.contains(conditionTypeGroups, profileTag)) {
                        conditionTypeGroups.push(profileTag);
                    }
                }
            }

            var modalData = {
                conditionTypes : _.filter(conditionTypes, function (value) {
                    return i18nService.message('condition.' + value.id + '.name').indexOf("???") == -1;
                }),
                conditionTypeGroups : conditionTypeGroups,
                title : $scope.newConditionText,
                templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? '/components/wem-condition-add-modal.directive.html' : '/components/wem-condition-add-modal.directive.legacy.html')
            };
            var okCallback = function (modalData) {
                if (modalData.conditionType) {
                    $scope.addCondition(modalData.conditionType, targetName, wrapIn);
                }
            };
            var cancelCallback = function (modalData) {};
            modalService.newCustomModal(
                modalData,
                null,
                okCallback,
                cancelCallback);
        };

        // init phase
        if (!$scope.ref) {
            console.log("wem-condition.directive.js: no condition passed, creating empty one.");
            $scope.ref = {};
        }

        console.log("wem-condition.directive.js: Initializing condition controller for condition " + $scope.ref + " of type=" + $scope.ref.type + " parentIndex=" + $scope.parentIndex);

        $scope.updateTemplate();
        $scope.$watch("ref", function (newValue, oldValue) {
            if (newValue != oldValue) {
                console.log("wem-condition.directive.js : condition changed, refreshing...");
                $scope.updateTemplate();
            }
        });

        if ($scope.itemType) {
            $scope.currentItemType = $scope.itemType;
        } else {
            $scope.currentItemType = $scope.$parent.currentItemType;
        }

        if (!$scope.showCount) {
            $scope.showCount = $scope.$parent.showCount;
        }

        $scope.validateCondition = function(condition) {
            if (!condition || !condition.type || !condition.parameterValues) {
                return false;
            }

            for (var parameter in condition.parameterValues) {
                if (condition.parameterValues.hasOwnProperty(parameter)) {
                    if (condition.parameterValues[parameter] == undefined) {
                        return false;
                    } else if (condition.parameterValues[parameter].constructor == Array
                        && condition.parameterValues[parameter][0] == undefined) { // multiple parameters are initialized
                        return false;
                    }
                }
            }

            if (condition.type == 'booleanCondition') {
                if (condition.parameterValues.subConditions.length == 0) {
                    return false;
                }
                for (var c = 0; c < condition.parameterValues.subConditions.length; c++) {
                    var subCondition = condition.parameterValues.subConditions[c];
                    var result = $scope.validateCondition(subCondition);
                    if (!result) {
                        return false;
                    }
                }
            }

            if (condition.type == 'pastEventCondition') {
                var result = $scope.validateCondition(condition.parameterValues.eventCondition);
                if (!result) {
                    return false;
                }
            }

            if (condition.type == 'campaignEngagedCondition') {
                if (!condition.parameterValues.campaignId) {
                    return false
                }
            }

            if (condition.parameterValues.comparisonOperator == 'between') {
                if (condition.parameterValues.propertyValuesDate) {
                    if (!condition.parameterValues.propertyValuesDate[1]) {
                        return false;
                    } else {
                        condition.parameterValues.propertyValuesDate[1] = moment(condition.parameterValues.propertyValuesDate[1]).endOf('day').toDate();
                    }
                }
            }

            return true;
        };

        if ($scope.displayCount && $scope.currentItemType && $scope.showCount!="false") {
            $scope.$watch("ref.parameterValues", function (newValue, oldValue) {
                // Wait for other watchers to have been executed before running query on server
                window.setTimeout(function() {
                    if ($scope.validateCondition($scope.ref)) {

                        var condition = $scope.ref;
                        if($scope.ref.parameterValues.propertyValues){
                            $scope.ref.parameterValues.propertyValues = _.map($scope.ref.parameterValues.propertyValues, function (value) {
                                if(value == undefined || value == null){
                                    return "";
                                }else {
                                    return value;
                                }
                            });
                        }
                        if ($scope.currentItemType == 'profile') {
                            var subCondition = $scope.ref;
                            if ($scope.ref.parameterValues.comparisonOperator === 'notContains') {
                                subCondition = {
                                    "parameterValues": {
                                        "subCondition":
                                            {
                                                type: "profilePropertyCondition",
                                                parameterValues: {
                                                    propertyName: $scope.ref.parameterValues.propertyName,
                                                    propertyValue: $scope.ref.parameterValues.propertyValue,
                                                    comparisonOperator: "contains"
                                                }
                                            }
                                    },
                                    "type": "notCondition"
                                };
                            }

                            condition = {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            type: "profilePropertyCondition",
                                            parameterValues: {
                                                propertyName: "mergedWith",
                                                comparisonOperator: "missing"
                                            }
                                        },
                                        {
                                            type: "profilePropertyCondition",
                                            parameterValues: {
                                                propertyName: "systemProperties.isAnonymousProfile",
                                                comparisonOperator: "missing"
                                            }
                                        },
                                        subCondition
                                    ],
                                    "operator": "and"
                                },
                                "type": "booleanCondition"
                            };

                        }

                        unomiQueryService.getMatchingItemsCountForCondition($scope.currentItemType, condition).then(function (response) {
                            $scope.matching = response.data;
                            if($scope.countlabel || $scope.countlabeli18n){
                                $scope.matching = $scope.getCountLabel() + " " + $scope.matching;
                            } else {
                                $scope.countClass="subcount";
                            }
                        });
                    } else {
                        $scope.matching = 0;
                        if($scope.countlabel || $scope.countlabeli18n){
                            $scope.matching = $scope.getCountLabel() + " " + $scope.matching;
                        } else {
                            $scope.countClass="subcount";
                        }
                    }
                });
            }, true);
        }

        //Handle date conditions
        if ($scope.ref.parameterValues) {
            if ($scope.ref.parameterValues.propertyValuesDate) {
                $scope.ref.parameterValues.propertyValuesDate[0] = moment($scope.ref.parameterValues.propertyValuesDate[0]).toDate();
                $scope.ref.parameterValues.propertyValuesDate[1] = moment($scope.ref.parameterValues.propertyValuesDate[1]).endOf('day').toDate();
            } else if ($scope.ref.parameterValues.propertyValueDate) {
                $scope.ref.parameterValues.propertyValueDate = moment($scope.ref.parameterValues.propertyValueDate).toDate();
            }
        }

        $scope.$watch("types.length", function (newValue, oldValue) {
            console.log("wem-condition.directive.js : condition type list changed, refreshing...");
            $scope.updateTemplate();
        });
        $scope.alternate = !$scope.$parent.alternate;
        $scope.conditionTypes = $scope.$parent.conditionTypes;
        $scope.eventConditionTypes = $scope.$parent.eventConditionTypes;
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemContextServerIssue', wemContextServerIssue);

    function wemContextServerIssue() {
        return {
            restrict: 'E',
            scope: {},
            templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? '/components/wem-context-server-issue.directive.html' : '/components/wem-context-server-issue.directive.legacy.html'),
            controller: WemContextServerIssueController,
            controllerAs: 'wemContextServerIssueCtrl'
        };
    }

    function WemContextServerIssueController() {
        var vm = this;
        vm.isAllowedByLicense = ManagersContext.isAllowedByLicense;
        vm.contextServerIsNotSetup = !ManagersContext.contextServerSetup && ManagersContext.isAllowedByLicense;
        // set to false because we don't want to display the associate message if not setup
        vm.contextServerIsNotOnline = (ManagersContext.contextServerSetup)?!ManagersContext.contextServerOnline && ManagersContext.isAllowedByLicense:false;
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemDtPager', wemDtPager);

    function wemDtPager() {
        return {
            restrict: 'E',
            templateUrl: ManagersContext.wemAppPath + '/components/wem-dt-pager.directive.html',
            controller: WemDtPagerController
        }
    }

    WemDtPagerController.$inject = ['$scope', 'dtPagerService'];

    function WemDtPagerController($scope, dtPagerService) {
        $(document).on('length.dt', function (event) {
            dtPagerService.refreshPager($scope);
        });

        $scope.$watch('dtInstance', function (val) {
            if (val) {
                dtPagerService.refreshPager($scope);
            }
        });
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemEditframeMenu', wemEditframeMenu);

    wemEditframeMenu.$inject = ['$window'];

    function wemEditframeMenu($window) {
        return {
            restrict: 'A',
            templateUrl: ManagersContext.wemAppPath + '/components/wem-editframe-menu.directive.html',
            controller: WemEditframeMenuController,
            controllerAs: 'wemEditframeMenuCtrl',
            link: linkFunc
        };

        function linkFunc(scope, el) {
            scope.displayMenu = false;
            if ($window.parent.location.pathname.indexOf('/editframe/') != -1) {
                el.addClass('wem-editframe-menu');
                scope.displayMenu = true;
            }
        }
    }

    WemEditframeMenuController.$inject = ['$window', '$filter'];

    function WemEditframeMenuController($window, $filter) {
        var wemEditframeMenuCtrl = this;
        wemEditframeMenuCtrl.getLink = getLink;
        wemEditframeMenuCtrl.goTo = goTo;
        wemEditframeMenuCtrl.toggleMenu = toggleMenu;
        wemEditframeMenuCtrl.getMenuEntryClass = getMenuEntryClass;
        wemEditframeMenuCtrl.menuEntries = $filter('orderBy')(ManagersContext.editframeMenu, 'name');

        function getLink(entryName) {
            return ManagersContext.baseEdit.replace('/edit/', '/editframe/') + ManagersContext.currentSitePath + '.' + entryName + '.html';
        }

        function goTo(entryName) {
            $window.parent.location = ManagersContext.baseEdit.replace('/edit/', '/editframe/') + ManagersContext.currentSitePath + '.' + entryName + '.html';
        }

        function toggleMenu(index) {
            wemEditframeMenuCtrl.menuEntries[index].toggle = !wemEditframeMenuCtrl.menuEntries[index].toggle;
        }

        function getMenuEntryClass(entryPathKey, parent) {
            var isOpen = ($window.parent.location.pathname.indexOf(entryPathKey) != -1);
            if (parent && isOpen) {
                parent.toggle = true;
            }
            return isOpen ? 'active' : '';
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemIframeOnload', wemIframeOnload);

    function wemIframeOnload() {
        return {
            scope: {
                callBack: '@wemIframeOnload'
            },
            link: linkFunc
        };

        function linkFunc(scope, element, attrs) {
            element.on('load', function () {
                if (scope.$parent[scope.callBack]) {
                    scope.$parent[scope.callBack](element);
                }
            })
        }
    }
})();
(function() {
    'use strict';

    // doc and example https://github.com/ifwe/infinite-scroll
    angular.module('wemApp.components').directive('wemInfiniteScroll', wemInfiniteScroll);

    wemInfiniteScroll.$inject = ['$window', '$timeout'];

    function wemInfiniteScroll($window, $timeout) {
        return {
            scope: {
                callback: '&wemInfiniteScroll',
                distance: '=wemInfiniteScrollDistance',
                disabled: '=wemInfiniteScrollDisabled'
            },
            link: linkFunc
        };

        function linkFunc(scope, elem, attrs) {
            var win = angular.element($window);

            var onScroll = function(oldValue, newValue) {
                // Do nothing if infinite scroll has been disabled
                if (scope.disabled) {
                    return;
                }
                var windowHeight = win[0].innerHeight;
                var elementBottom = elem[0].offsetTop + elem[0].offsetHeight;
                var windowBottom = windowHeight + (win[0].scrollY || win[0].pageYOffset);
                var remaining = elementBottom - windowBottom;
                var shouldGetMore = (remaining - parseInt(scope.distance || 0, 10) <= 0);

                if (shouldGetMore) {
                    $timeout(scope.callback);
                }
            };

            // Check immediately if we need more items upon reenabling.
            scope.$watch('disabled', function(isDisabled){
                if (false === isDisabled) onScroll();
            });

            win.bind('scroll', onScroll);

            // Remove window scroll handler when this element is removed.
            scope.$on('$destroy', function() {
                win.unbind('scroll', onScroll);
            });

            // Check on next event loop to give the browser a moment to paint.
            // Otherwise, the calculations may be off.
            $timeout(onScroll);
        }
    }
})();
(function() {
    'use strict';
    // Todo: replace by angular material messages
    angular.module('wemApp.components').directive('wemLettersOrNumbersOnly', wemLettersOrNumbersOnly);

    function wemLettersOrNumbersOnly() {
        return {
            require: 'ngModel',
            link: linkFunc
        };

        function linkFunc(scope, element, attrs, ctrl) {
            var validate = function (viewValue) {
                ctrl.$setValidity('lettersOrNumbersOnly', new RegExp("^[A-Za-z0-9]+$").test(viewValue));
                return viewValue;
            };
            ctrl.$parsers.unshift(validate);
            ctrl.$formatters.push(validate);

            attrs.$observe('lettersOrNumbersOnly', function (comparisonModel) {
                // Whenever the comparison model changes we'll re-validate
                return validate(ctrl.$viewValue);
            });

        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemLoadingSpinner', wemLoadingSpinner);

    function wemLoadingSpinner() {
        return {
            restrict: 'E',
            scope: {
                show: '@'
            },
            templateUrl: ManagersContext.wemAppPath + '/components/wem-loading-spinner.directive.html',
            controller: WemLoadingSpinnerController,
            controllerAs: 'loadingSpinnerCtrl',
            bindToController: true
        };
    }

    WemLoadingSpinnerController.$inject = ['loadingSpinnerService'];

    function WemLoadingSpinnerController(loadingSpinnerService) {
        var vm = this;

        loadingSpinnerService.register({
            'show': function() {
                vm.show = true;
            },
            'hide': function() {
                vm.show = false;
            }
        });
    }
})();
(function() {
    'use strict';
    // Todo: replace by angular material messages
    angular.module('wemApp.components').directive('wemLowerThanNumber', wemLowerThanNumber);

    function wemLowerThanNumber() {
        return {
            require: 'ngModel',
            link: linkFunc
        };

        function linkFunc(scope, element, attrs, ctrl) {
            var validate = function (viewValue) {
                var comparisonModel = attrs.lowerThanNumber;
                var view = parseInt(viewValue, 10);
                var comparison = parseInt(comparisonModel, 10);

                if (view && comparison) {
                    // It's valid because we have nothing to compare against
                    ctrl.$setValidity('lowerThan', view < comparison);
                } else {
                    ctrl.$setValidity('lowerThan', true);
                }

                return viewValue;
            };

            ctrl.$parsers.unshift(validate);
            ctrl.$formatters.push(validate);

            attrs.$observe('lowerThanNumber', function (comparisonModel) {
                // Whenever the comparison model changes we'll re-validate
                return validate(ctrl.$viewValue);
            });
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemMainSection', wemSection);

    wemSection.$inject = ['$window'];

    function wemSection($window) {
        return {
            restrict: 'A',
            scope: {},
            link: linkFunc
        };

        function linkFunc(scope, el) {
            if ($window.parent.location.pathname.indexOf('/editframe/') != -1) {
                el.addClass('wem-main-section-with-editframe-menu');
            } else {
                el.addClass('wem-main-section');
            }
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wem-page-head', wemPageHead);

    wemPageHead.$inject = ['$window'];

    function wemPageHead($window) {
        return {
            restrict: 'C',
            scope: {
                headerShadow: '@'
            },
            link: linkFunc
        };

        function linkFunc(scope, element, attrs) {
            angular.element($window).bind("scroll", function () {
                if (this.pageYOffset > (scope.headerShadow ? scope.headerShadow : 0)) {
                    element.addClass("wem-page-head-shadow");
                } else {
                    element.removeClass("wem-page-head-shadow");
                }
            });
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemPrefixModelByProperties', wemPrefixModelByProperties);

    function wemPrefixModelByProperties() {
        return {
            require: 'ngModel',
            restrict: 'A',
            scope: {ngModel: '='},
            link: linkFunc
        };

        function linkFunc(scope, element, attr, controller) {
            controller.$formatters.unshift(function (valueFromModel) {
                // what you return here will be passed to the text field
                if(valueFromModel && valueFromModel.lastIndexOf("properties.", 0) === 0) {
                    return valueFromModel.substr("properties.".length);
                }
                return valueFromModel;
            });

            controller.$parsers.push(function (valueFromInput) {
                // put the inverse logic, to transform formatted data into model data
                // what you return here, will be stored in the $scope
                return "properties." + valueFromInput;
            });
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemProperty', wemPropertyDirective);

    function wemPropertyDirective() {
        return {
            restrict: 'E',
            scope: {
                ref: '=',
                types: '=',
                currentProfile: '='
            },
            template: '<div ng-include="templateUrl"></div>',
            controller: WemPropertyDirectiveController
        };
    }

    WemPropertyDirectiveController.$inject = ['$scope'];

    function WemPropertyDirectiveController($scope) {

        function updateTemplate() {
            if ($scope.ref && $scope.ref.type != "empty") {
                $scope.valueType = $scope.getValueType($scope.ref);
                if ($scope.valueType != null) {
                    $scope.templateUrl = ManagersContext.wemAppPath + "/properties/templates/" + $scope.valueType.id + ".html";
                }
            }
        }

        $scope.getValueType = function (property) {
            for (var i = 0; i < $scope.types.length; i++) {
                var valueType = $scope.types[i];
                if (valueType.id == property.type) {
                    return valueType;
                }
            }
            return null;
        };

        if (!$scope.ref) {
            console.log("wem-property.directive.js: no property passed, creating empty one.");
            $scope.ref = {};
            $scope.ref.type = "empty";
        }

        console.log("wem-property.directive.js: Initializing property controller for property " + $scope.ref + " of type=" + $scope.ref.type);

        updateTemplate();

        $scope.$watch("ref", function (newValue, oldValue) {
            console.log("wem-property.directive.js : property changed, refreshing...");
            updateTemplate();
        });

        $scope.$watch("types.length", function (newValue, oldValue) {
            console.log("wem-property.directive.js : property type list changed, refreshing...");
            updateTemplate();
        });

    }
})();

(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemTimeline', wemTimeline);

    function wemTimeline() {
        return {
            restrict: 'E',
            templateUrl: ManagersContext.wemAppPath + '/components/wem-timeline.directive.html',
            controller: WemTimelineController,
            controllerAs: 'wemTimeLineCtrl',
            scope: {},
            bindToController: {
                data: '=ngModel',
                colours: '=',
                report: '=',
                preset: '@'
            },
            link: linkFunc
        };

        function linkFunc(scope, el, attrs, ctrl) {
            ctrl._init();
        }
    }

    WemTimelineController.$inject = ['i18nService'];

    function WemTimelineController(i18nService) {
        var vm = this;
        vm.currentText = i18nService.message('serverSettings.manageWem.datepicker.today');
        vm.closeText = i18nService.message('serverSettings.manageWem.datepicker.close');
        vm.clearText = i18nService.message('wem.label.clear');

        vm.toggleDatePicker = toggleDatePicker;
        vm.openDatePopup = openDatePopup;
        vm.updateFromDate = updateFromDate;
        vm.updateToDate = updateToDate;
        vm.updateInterval = updateInterval;
        vm.updatePresetDate = updatePresetDate;
        vm._init = _init;

        function openDatePopup($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();

            if (selector == 'start') {
                vm.startOpened = true;
                vm.endOpened = false;
            } else {
                vm.endOpened = true;
                vm.startOpened = false;
            }
        }

        function updateFromDate() {
            if (vm.data.interval.key == '1M') {
                var fromDateMoment = moment(vm.data.fromDate);
                vm.data.fromDate = fromDateMoment.startOf('month').toDate();
            }
            _validateDate('fromDate');
            _updateDatePickerOptions();
            _queryData();
        }

        function updateToDate() {
            if (vm.data.interval.key == '1M') {
                var toDateMoment = moment(vm.data.toDate);
                vm.data.toDate = toDateMoment.endOf('month').toDate();
            }
            _validateDate('toDate');
            _updateDatePickerOptions();
            _queryData();
        }

        // convert dates when interval change
        function updateInterval() {
            var fromDateMoment = moment(vm.data.fromDate);
            var toDateMoment = moment(vm.data.toDate);
            if (vm.data.interval.key == '1M') {
                fromDateMoment.startOf('month');
                toDateMoment.endOf('month');
            }
            vm.data.fromDate = fromDateMoment.toDate();
            vm.data.toDate = toDateMoment.toDate();

            _validateDate('toDate');
            _updateDatePickerOptions();
            _queryData();
        }

        function updatePresetDate(selection) {
            var toDateMoment = moment();
            var fromDateMoment;
            var intervalUnit;
            vm.selectedPreset = selection;
            switch (selection) {
                case '1week':
                    intervalUnit = 'day';
                    fromDateMoment = moment().subtract(7, 'days');
                    break;
                case '1month':
                    intervalUnit = 'day';
                    fromDateMoment = moment().subtract(1, 'months');
                    break;
                case '3month':
                    intervalUnit = 'day';
                    fromDateMoment = moment().subtract(3, 'months');
                    break;
                case '6month':
                    intervalUnit = 'month';
                    toDateMoment.endOf('month');
                    fromDateMoment = moment().subtract(6, 'months').startOf('month');
                    break;
                case '1year':
                    intervalUnit = 'month';
                    toDateMoment.endOf('month');
                    fromDateMoment = moment().subtract(1, 'years').startOf('month');
                    break;
            }

            if (vm.data.interval.unit != intervalUnit) {
                for (var interval in vm.intervals) {
                    if (vm.intervals[interval].unit == intervalUnit) {
                        vm.data.interval = vm.intervals[interval];
                    }
                }
            }
            vm.data.fromDate = fromDateMoment.toDate();
            vm.data.toDate = toDateMoment.toDate();

            _updateDatePickerOptions();
            _queryData();
        }

        function toggleDatePicker() {
            vm.showDatePicker = !vm.showDatePicker;
            if (vm.showDatePicker) {
                vm.selectedPreset = 'custom';
            }
        }

        // Private functions under this line
        function _init() {
            // compute interval
            vm.data.interval = vm.data.interval || ManagersUtils.supportedTimelineIntervales['1M'];
            vm.intervals = ManagersUtils.supportedTimelineIntervales;

            if (vm.preset) {
                vm.showDatePicker = false;
                updatePresetDate(vm.preset);
            } else {
                vm.showDatePicker = true;
                vm.selectedPreset = 'custom';

                _updateDatePickerOptions();
                _queryData();
            }
        }

        function _updateDatePickerOptions() {
            vm.firstPickerOptions = {
                datepickerMode: vm.data.interval.unit,
                // maxDate: toDateMoment.toDate(),
                maxMode: vm.data.interval.unit,
                minMode: vm.data.interval.unit
            };

            vm.secondPickerOptions = {
                datepickerMode: vm.data.interval.unit,
                maxMode: vm.data.interval.unit,
                // minDate: fromDateMoment.toDate(),
                minMode: vm.data.interval.unit
            };
        }

        function _queryData() {
            vm.data.loadData(function(datasets) {
                vm.datasets = datasets;
                _redrawGraph();
            });
        }

        function _validateDate(selected) {
            var fromDateMoment = moment(vm.data.fromDate);
            var toDateMoment = moment(vm.data.toDate);
            if (selected == 'fromDate') {
                var minToDate = moment(fromDateMoment);
                minToDate.add(vm.data.interval.minpoints, vm.data.interval.units);
                if (toDateMoment.isBefore(minToDate)) {
                    if (vm.data.interval.key == '1M') {
                        minToDate.endOf('month');
                    }
                    vm.data.toDate = minToDate.toDate();
                }
            }
            if (selected == 'toDate') {
                var minFromDate = moment(toDateMoment);
                minFromDate.subtract(vm.data.interval.minpoints, vm.data.interval.units);
                if (fromDateMoment.isAfter(minFromDate)) {
                    if (vm.data.interval.key == '1M') {
                        minFromDate.startOf('month');
                    }
                    vm.data.fromDate = minFromDate.toDate();
                }
            }
        }

        function _redrawGraph() {
            var filtered = {};
            var fromDateMoment = moment(vm.data.fromDate);
            var toDateMoment = moment(vm.data.toDate);
            var dates = [];

            // Fill empty points with 0
            if (fromDateMoment.isBefore(toDateMoment)) {
                var count = 0;
                var dif = toDateMoment.diff(fromDateMoment,  vm.data.interval.units);
                dates.push(fromDateMoment.format('YYYY-MM-DD'));
                while (count < dif) {
                    dates.push(fromDateMoment.add(vm.data.interval.unitcount,  vm.data.interval.units).format('YYYY-MM-DD'));
                    count++;
                }
            }

            filtered = [];
            angular.forEach(vm.datasets, function(d,i) {
                filtered[i] = [];
            });

            angular.forEach(dates, function (v, k) {
                angular.forEach(vm.datasets, function(d,i) {
                    filtered[i].push(vm.datasets[i][v] ? vm.datasets[i][v] : 0);
                })
            });
            var labels = _.map(dates, function(f) { return moment(f).format( vm.data.interval.scaleFormat) });
            vm.lineChartData = {
                labels: labels,
                datasets: filtered,
                series: vm.data.series,
                colours: vm.colours ? vm.colours : [
                        {
                            backgroundColor: 'rgba(220,220,220,0.2)',
                            borderColor: 'rgba(220,220,220,1)',
                            pointBackgroundColor: 'rgba(220,220,220,1)',
                            pointBorderColor: '#fff',
                            pointHoverBackgroundColor: '#fff',
                            pointHoverBorderColor: 'rgba(220,220,220,1)'
                        },
                        {
                            backgroundColor: 'rgba(0, 188, 212, 0.2)',
                            borderColor: '#00A0E3',
                            pointBackgroundColor: '#00A0E3',
                            pointBorderColor: '#fff',
                            pointHoverBackgroundColor: '#fff',
                            pointHoverBorderColor: '#00A0E3'
                        }
                    ],
                options: angular.extend({
                    scales: {
                        yAxes: [{
                            ticks: {
                                beginAtZero:true
                            }
                        }]
                    },
                    legend: {
                        display: true,
                        position: 'bottom',
                        labels: {
                            usePointStyle: true
                        }
                    },
                    maintainAspectRatio: false
                }, vm.data.chartOptions)
            };
        }
    }
})();
