(function() {
    'use strict';

    angular.module('wemExportConfigsApp', ['wemApp']);
})();
(function() {
    'use strict';

    angular.module('wemExportConfigsApp').run(wemExportConfigsApp);

    wemExportConfigsApp.$inject = ['$rootScope'];

    function wemExportConfigsApp($rootScope) {
        $rootScope.getCurrentModuleResource = function(resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };
    }
})();
(function () {
    'use strict';

    angular.module('wemExportConfigsApp').controller('ExportConfigsEditController', ExportConfigsEditController);

    ExportConfigsEditController.$inject = ['$route', '$routeParams', '$timeout', 'loadingSpinnerService', 'notificationService','modalService', 'i18nService', 'unomiProfileService', 'unomiExportConfigService', 'unomiSegmentService','currentExportConfig', '$location'];

    function ExportConfigsEditController($route, $routeParams, $timeout, loadingSpinnerService, notificationService, modalService, i18nService, unomiProfileService, unomiExportConfigService, unomiSegmentService,currentExportConfig, $location) {
        var vm = this;

        vm.saveExportConfig = saveExportConfig;
        vm.showConfirm = showConfirm;
        vm.addMapping = addMapping;
        vm.deleteMapping = deleteMapping;
        vm.reload = reload;

        _init();

        function saveExportConfig(execute) {
            if (!vm.currentExportConfig.properties) {
                vm.currentExportConfig.properties = {};
            }
            vm.currentExportConfig.properties.mapping = {};
            R.forEach(function (item) {
                vm.currentExportConfig.properties.mapping[item.index] = item.property;
            }, vm.currentExportConfig.mappingArray);
            vm.currentExportConfig = R.omit("mappingArray", vm.currentExportConfig);
            if(execute || vm.currentExportConfig.configType == 'recurrent') {
                vm.currentExportConfig.status = "RUNNING";
            }

            if(vm.currentExportConfig.configType == 'oneshot') {
                vm.currentExportConfig.active = false;
                if(vm.currentExportConfig.properties.destination) {
                    vm.currentExportConfig.properties = R.omit(['destination'], vm.currentExportConfig.properties);
                }
                if(vm.currentExportConfig.properties.period) {
                    vm.currentExportConfig.properties = R.omit(['period'], vm.currentExportConfig.properties);
                }
                if(execute) {
                    unomiExportConfigService.setOneshotExportConfig(vm.currentExportConfig)
                        .then(function () {
                                $location.path("/export-configs");
                            },
                            function () {
                                _convertMappingProperties();
                            });
                    return;
                } else {
                    if(!vm.currentExportConfig.name) {
                        notificationService.errorToast(i18nService.format("serverSettings.manageImportExportConfigs.error.nameRequired", ''));
                        return;
                    }
                }
            }

            unomiExportConfigService.setExportConfig(vm.currentExportConfig)
                .then(function (response) {
                        if(response.status == 206) {
                            notificationService.errorToast(i18nService.format("serverSettings.manageImportExportConfigs.error."+response.data, ''))
                        } else {
                            $location.path("/export-configs");
                        }
                    },
                    function () {
                        _convertMappingProperties();
                    });

        }

        function showConfirm($event) {
            modalService.newAlertModal({
                title: i18nService.message('serverSettings.manageExportConfigs.modal.delete.title'),
                bodyMessage: i18nService.message('serverSettings.manageExportConfigs.modal.delete.text')
            }, function (modalData) {
                loadingSpinnerService.show();
                console.info("Delete confirmed, deleting export config", vm.currentExportConfig.itemId);
                unomiExportConfigService.deleteExportConfig(vm.currentExportConfig.itemId)
                    .then(function () {
                        $timeout(function () {
                            loadingSpinnerService.hide();
                            $location.path("/export-configs", true);
                        }, 1000);
                    });
            }, function (modalData) {
                console.info("Delete cancelled");
            });
        }

        function addMapping() {
            if (!vm.currentExportConfig.mappingArray || vm.currentExportConfig.mappingArray.length == 0) {
                vm.currentExportConfig.mappingArray = [];
            }
            var maxIndex = 0;
            R.forEach(function (mapping) {
                if(mapping.index > maxIndex) {
                    maxIndex = mapping.index;
                }
            }, vm.currentExportConfig.mappingArray);
            if(vm.currentExportConfig.mappingArray.length > 0) {
                maxIndex++;
            }
            vm.currentExportConfig.mappingArray.push({property: "", index: maxIndex});
            _getMappingPropertiesList();
        }

        function deleteMapping(property) {
            vm.currentExportConfig.mappingArray = R.reject(function (mapping) {
                return mapping.property == property
            }, vm.currentExportConfig.mappingArray);
            _getMappingPropertiesList();
        }

        function reload() {
            $route.reload();
        }

        // Private functions
        function _init() {
            vm.getMappingPropertiesList = [];
            vm.propertiesList = [];
            vm.isNewExport = $routeParams.configId == 'new';
            unomiProfileService.getPropertyTypesByTarget('profiles').then(function (response) {
                vm.propertiesList = response.data;
                _getMappingPropertiesList();
            });

            unomiSegmentService.getAll().then(function (response) {
                vm.segments = response.data;
            });

            vm.currentExportConfig = currentExportConfig;

            if(vm.isNewExport) {
                vm.currentExportConfig.columnSeparator = ',';
                vm.currentExportConfig.lineSeparator = '\\n';
            }

            _convertMappingProperties();

            if (!vm.currentExportConfig.configType) {
                vm.currentExportConfig.configType = 'oneshot';
            }

        }

        function _convertMappingProperties() {
            vm.currentExportConfig.mappingArray = [];
            if (vm.currentExportConfig.properties) {
                Object.keys(vm.currentExportConfig.properties.mapping).forEach(function (key) {
                    vm.currentExportConfig.mappingArray.push({
                        index: Number(key),
                        property: vm.currentExportConfig.properties.mapping[key]
                    })
                });
            }
        }

        function _getMappingPropertiesList() {
            if (vm.currentExportConfig.properties) {
                vm.mappingPropertiesList = R.reject(function (prop) {
                    return R.find(R.propEq('property', prop.itemId))(vm.currentExportConfig.mappingArray);
                }, vm.propertiesList);
            } else {
                vm.mappingPropertiesList = vm.propertiesList;
            }
        }
    }
})();
(function () {
    'use strict';

    angular.module('wemExportConfigsApp').controller('ExportConfigsListController', ExportConfigsListController);

    ExportConfigsListController.$injecet = ['$route', '$location', '$filter', 'unomiExportConfigService', 'unomiSegmentService'];

    function ExportConfigsListController($route, $location, $filter, unomiExportConfigService, unomiSegmentService) {
        var vm = this;

        vm.onChangeFilterByType = onChangeFilterByType;
        vm.newExportConfig = newExportConfig;
        vm.editExportConfig = editExportConfig;
        vm.getLastExecution = getLastExecution;
        vm.reload = reload;

        _init();

        function onChangeFilterByType() {
            if (vm.filterByType == 'all') {
                vm.exportConfigs = angular.copy(vm.exportConfigsAll);
            } else {
                vm.exportConfigs = R.filter(function (elt) {
                    return elt.configType == vm.filterByType;
                }, vm.exportConfigsAll);
            }
        }

        function newExportConfig() {
            $location.path('/export-configs/new');
        }

        function editExportConfig(exportConfigId) {
            $location.path('/export-configs/' + exportConfigId);
        }

        function getLastExecution(executions) {
            if (executions && executions.length) {
                return $filter('orderBy')(executions, 'date', true)[0];
            } else {
                return null;
            }

        }

        function reload() {
            $route.reload();
        }

        vm.segments = [];

        // Private functions
        function _init() {
            vm.filterByType = 'all';
            unomiSegmentService.getAll().then(
                function (segments) {
                    vm.segments = segments.data;
                    _getExportConfigs();
                },
                function () {
                    console.error("Error while retrieving segments list!");
                }
            );
        }

        function _getExportConfigs() {
            unomiExportConfigService.getAll().then(function (response) {
                vm.exportConfigsAll = response.data;
                R.forEach(function (exportConfig) {
                    var segment = R.find(R.propEq('id', exportConfig.properties.segment))(vm.segments);
                    exportConfig.segmentName = segment ? segment.name : '';
                }, vm.exportConfigsAll);
                vm.exportConfigs = angular.copy(vm.exportConfigsAll);
            });
        }
    }
})();
