(function() {
    'use strict';

    angular.module('wemImportConfigsApp', ['wemApp']);
})();
(function() {
    'use strict';

    angular.module('wemImportConfigsApp').run(wemImportConfigsApp);

    wemImportConfigsApp.$inject = ['$rootScope'];

    function wemImportConfigsApp($rootScope) {
        $rootScope.getCurrentModuleResource = function(resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };
    }
})();
(function () {
    'use strict';

    angular.module('wemImportConfigsApp').controller('ImportConfigsEditController', ImportConfigsEditController);

    ImportConfigsEditController.$inject = ['$route', '$routeParams', '$timeout', 'loadingSpinnerService', 'notificationService', 'modalService', 'i18nService', 'unomiProfileService', 'unomiImportConfigService', 'currentImportConfig', '$location'];

    function ImportConfigsEditController($route, $routeParams, $timeout, loadingSpinnerService, notificationService, modalService, i18nService, unomiProfileService, unomiImportConfigService, currentImportConfig, $location) {
        var vm = this;

        vm.saveImportConfig = saveImportConfig;
        vm.showConfirm = showConfirm;
        vm.updatePropertiesOverwrite = updatePropertiesOverwrite;
        vm.addMapping = addMapping;
        vm.deleteMapping = deleteMapping;
        vm.reload = reload;

        _init();

        function saveImportConfig(execute) {
            if (!vm.currentImportConfig.properties) {
                vm.currentImportConfig.properties = {};
            }
            vm.currentImportConfig.properties.mapping = {};
            R.forEach(function (item) {
                vm.currentImportConfig.properties.mapping[item.property] = item.index;
            }, vm.currentImportConfig.mappingArray);
            vm.currentImportConfig = R.omit("mappingArray", vm.currentImportConfig);
            if(execute || vm.currentImportConfig.configType == 'recurrent') {
                vm.currentImportConfig.status = "RUNNING";
            }

            if(!vm.currentImportConfig.lineSeparator) {
                vm.currentImportConfig.lineSeparator = '\n';
            }
            if(!vm.currentImportConfig.columnSeparator) {
                vm.currentImportConfig.columnSeparator = ',';
            }

            vm.currentImportConfig = R.omit(['overwriteAllProperties'], vm.currentImportConfig);

            if (vm.currentImportConfig.configType == 'oneshot') {
                vm.currentImportConfig.active = false;
                if(vm.currentImportConfig.properties.source) {
                    vm.currentImportConfig.properties = R.omit(['source'], vm.currentImportConfig.properties);
                }
                if(execute) {
                    unomiImportConfigService.setOneshotImportConfig(vm.currentImportConfig, vm.files)
                        .then(function () {
                                $location.path("/import-configs");
                            },
                            function () {
                                _convertMappingProperties();
                            });
                    return;
                } else {
                    if(!vm.currentImportConfig.name) {
                        notificationService.errorToast(i18nService.format("serverSettings.manageImportExportConfigs.error.nameRequired", ''));
                        return;
                    }
                }
            }

            unomiImportConfigService.setImportConfig(vm.currentImportConfig)
                .then(function (response) {
                        if(response.status == 206) {
                            notificationService.errorToast(i18nService.format("serverSettings.manageImportExportConfigs.error."+response.data, ''))
                        } else {
                            $location.path("/import-configs");
                        }
                    },
                    function () {
                        _convertMappingProperties();
                    });
        }

        function showConfirm($event) {
            modalService.newAlertModal({
                title: i18nService.message('serverSettings.manageImportConfigs.modal.delete.title'),
                bodyMessage: i18nService.message('serverSettings.manageImportConfigs.modal.delete.text')
            }, function (modalData) {
                loadingSpinnerService.show();
                console.info("Delete confirmed, deleting import config", vm.currentImportConfig.itemId);
                unomiImportConfigService.deleteImportConfig(vm.currentImportConfig.itemId)
                    .then(function () {
                        $timeout(function () {
                            loadingSpinnerService.hide();
                            $location.path("/import-configs", true);
                        }, 1000);
                    });
            }, function (modalData) {
                console.info("Delete cancelled");
            });
        }

        function updatePropertiesOverwrite() {
            if(vm.currentImportConfig.overwriteAllProperties) {
                vm.currentImportConfig.propertiesToOverwrite = null;
            }
        }

        function addMapping(newKey, newValue) {
            if (!vm.currentImportConfig.mappingArray || vm.currentImportConfig.mappingArray.length == 0) {
                vm.currentImportConfig.mappingArray = [];
            }
            var maxIndex = 0;
            R.forEach(function (mapping) {
                if(mapping.index > maxIndex) {
                    maxIndex = mapping.index;
                }
            }, vm.currentImportConfig.mappingArray);
            if(vm.currentImportConfig.mappingArray.length > 0) {
                maxIndex++;
            }
            vm.currentImportConfig.mappingArray.push({property: "", index: maxIndex});
            _getMappingPropertiesList();
        }

        function deleteMapping(property) {
            vm.currentImportConfig.mappingArray = R.reject(function (mapping) {
                return mapping.property == property
            }, vm.currentImportConfig.mappingArray);
            _getMappingPropertiesList();
        }

        function reload() {
            $route.reload();
        }

        // Private functions
        function _init() {
            vm.getMappingPropertiesList = [];
            vm.propertiesList = [];
            vm.isNewImport = $routeParams.configId == 'new';
            unomiProfileService.getPropertyTypesByTarget('profiles').then(function (response) {
                vm.propertiesList = response.data;
                _getMappingPropertiesList();
            });
            vm.currentImportConfig = currentImportConfig;
            if(vm.isNewImport) {
                vm.currentImportConfig.columnSeparator = ',';
                vm.currentImportConfig.overwriteAllProperties = true;
            } else {
                vm.currentImportConfig.overwriteAllProperties = !vm.currentImportConfig.propertiesToOverwrite
            }

            _convertMappingProperties();

            if (!vm.currentImportConfig.configType) {
                vm.currentImportConfig.configType = 'oneshot';
            }

            vm.option = {
                "browseIconCls": "fileUpload-browseIco",
                "removeIconCls": "fileUpload-removeIco",
                "captionIconCls": "fileUpload-captionIco"
            };
        }

        function _convertMappingProperties() {
            vm.currentImportConfig.mappingArray = [];
            if (vm.currentImportConfig.properties) {
                Object.keys(vm.currentImportConfig.properties.mapping).forEach(function (key) {
                    vm.currentImportConfig.mappingArray.push({
                        property: key,
                        index: vm.currentImportConfig.properties.mapping[key]
                    })
                });
            }
        }

        function _getMappingPropertiesList() {
            if (vm.currentImportConfig.properties) {
                vm.mappingPropertiesList = R.reject(function (prop) {
                    return R.find(R.propEq('property', prop.itemId))(vm.currentImportConfig.mappingArray);
                }, vm.propertiesList);
            } else {
                vm.mappingPropertiesList = vm.propertiesList;
            }
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemImportConfigsApp').controller('ImportConfigsListController', ImportConfigsListController);

    ImportConfigsListController.$injecet = ['$route', '$location', 'unomiImportConfigService'];

    function ImportConfigsListController($route, $location, unomiImportConfigService) {
        var vm = this;

        vm.onChangeFilterByType = onChangeFilterByType;
        vm.newImportConfig = newImportConfig;
        vm.editImportConfig = editImportConfig;
        vm.reload = reload;

        _init();

        function onChangeFilterByType() {
            if(vm.filterByType == 'all') {
                vm.importConfigs = angular.copy(vm.importConfigsAll);
            } else {
                vm.importConfigs = R.filter(function(elt) {
                    return elt.configType == vm.filterByType;
                }, vm.importConfigsAll);
            }
        }

        function newImportConfig(importConfigId) {
            $location.path('/import-configs/new');
        }

        function editImportConfig(importConfigId) {
            $location.path('/import-configs/' + importConfigId);
        }

        function reload() {
            $route.reload();
        }

        // Private functions
        function _init() {
            vm.filterByType = 'all';

            _getImportConfigs();
        }

        function _getImportConfigs() {
            unomiImportConfigService.getAll().then(function(response) {
                vm.importConfigsAll = response.data;
                vm.importConfigs = angular.copy(vm.importConfigsAll);
            });
        }
    }
})();
