(function() {
    'use strict';

    angular.module('wemOptimizationDashboardApp', ['wemApp']);
})();
(function() {
    'use strict';

    angular.module('wemOptimizationDashboardApp').controller('OptimizationDashboardController', OptimizationDashboardController);

    OptimizationDashboardController.$inject = ['$scope', '$http', '$timeout', '$uibModal', 'i18nService', 'unomiGoalService', 'jcrService', 'validationUtilsService'];

    function OptimizationDashboardController($scope, $http, $timeout, $uibModal, i18nService, unomiGoalService, jcrService, validationUtilsService) {
        $scope.optimizedContents = [];
        $scope.optimizedContentsPath = {};
        $scope.limit = 50;
        $scope.currentOffset = 0;
        $scope.currentOrderBy = 'optimizationName';
        $scope.currentOrderDirection = 'ASC';
        $scope.limitPage = 50;
        $scope.currentOffsetPage = 0;
        $scope.currentOrderByPage = 'optimizationName';
        $scope.currentOrderDirectionPage = 'ASC';
        $scope.disabled = true;
        $scope.fetching = false;
        $scope.searchValue = '';

        var getData = function(orderBy, direction, offset, limit, increaseCurrentOffset) {
            $scope.fetching = true;
            $http.get(ManagersContext.baseEdit + ManagersContext.currentSitePath + ".optimizationDashboardAction.do?orderBy=" + orderBy + "&orderDirection=" + direction + "&offset=" + offset + "&limit=" + limit + "&orderByPages=" + orderBy + "&orderDirectionPages=" + direction + "&offsetPages=" + offset + "&limitPages=" + limit).then(function(response) {
                var data = response.data;
                $scope.fetching = false;
                var length = data.optimizedContents.length;
                if (length < $scope.limit) {
                    $scope.disabled = true;
                }

                if (increaseCurrentOffset) {
                    $scope.currentOffset += length;
                }

                $scope.optimizedPages = data.optimizedPages;
                angular.forEach(data.optimizedContents, function(optimizedContent) {
                    $scope.optimizedContents.push(optimizedContent);
                });

                angular.forEach(data.optimizedContentsPath, function (value, key) {
                    $scope.optimizedContentsPath[key] = value;
                });
            });
        };

        $scope.updateOrderBy = function(orderBy) {
            $scope.disabled = false;
            $scope.currentOrderBy = orderBy;
            $scope.currentOffset = 0;
            $scope.optimizedContents = [];
            $scope.optimizedContentsPath = {};
            getData($scope.currentOrderBy, $scope.currentOrderDirection, $scope.currentOffset, $scope.limit, true);
        };

        $scope.updateOrderDirection = function() {
            $scope.currentOrderDirection = $scope.currentOrderDirection=='ASC'?'DESC':'ASC';
            $scope.optimizedContents = [];
            $scope.optimizedContentsPath = {};
            // To not loose current data with use the current offset as a limit and we call from offset 0
            getData($scope.currentOrderBy, $scope.currentOrderDirection, 0, $scope.currentOffset, false);
        };

        $scope.getMoreData = function() {
            getData($scope.currentOrderBy, $scope.currentOrderDirection, $scope.currentOffset, $scope.limit, true);
        };

        $scope.getMoreDataForPages = function() {
            getData($scope.currentOrderByPage, $scope.currentOrderDirectionPage, $scope.currentOffsetPage, $scope.limitPage, true);
        };

        var getNodeInfo = function(optimizedContent, path) {
            jcrService.doGetOnPath('live', null, path).then(function (response) {
                optimizedContent.goalInfo.name = response.data.name;
            });
        };

        $scope.toggleCardVariants = function(optimizedContent) {
            optimizedContent.collapsed = !optimizedContent.collapsed;
            if (optimizedContent.goalId && !optimizedContent.goalInfo && $scope.optimizedContentsPath.length) {
                unomiGoalService.getGoal(optimizedContent.goalId).then(function(response) {
                    var data = response.data;
                    var tag = data.metadata.tags[0];

                    optimizedContent.goalInfo = {};
                    // Path are stored in another object so angular filter is not filtering on them
                    $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo = {};
                    optimizedContent.goalInfo.type = i18nService.message("serverSettings.manageGoals.tag." + tag + ".description");

                    // May need to add additional goal in the future
                    switch (tag) {
                        case "downloadGoal":
                            $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.filePath = data.targetEvent.parameterValues.filePath;
                            getNodeInfo(optimizedContent, $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.filePath);
                            break;
                        case "videoGoal":
                            angular.forEach(data.targetEvent.parameterValues.subConditions, function(subCondition) {
                                if (subCondition.type == 'videoViewEventCondition') {
                                    optimizedContent.goalInfo.videoId = subCondition.parameterValues.videoId;
                                } else {
                                    $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.pagePath = subCondition.parameterValues.path;
                                }
                            });
                            getNodeInfo(optimizedContent, $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.pagePath);
                            break;
                        case "formGoal":
                            angular.forEach(data.targetEvent.parameterValues.subConditions, function(subCondition) {
                                if (subCondition.type == 'formEventCondition') {
                                    optimizedContent.goalInfo.formId = subCondition.parameterValues.formId;
                                } else {
                                    $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.pagePath = subCondition.parameterValues.path;
                                }
                            });
                            getNodeInfo(optimizedContent, $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.pagePath);
                            $timeout(function () {
                                validationUtilsService.getFormInPage($scope, optimizedContent.goalInfo.formId, $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.pagePath, function ($form) {
                                    if ($form.length > 0) {
                                        var formfactoryId = $form.attr("data-form-id");
                                        if (formfactoryId) {
                                            $http.get(ManagersContext.cmsContext + "/modules/formfactory/live/" + formfactoryId + "/" + ManagersContext.mainResourceLocale).then(function (response) {
                                                optimizedContent.goalInfo.displayableName = response.data.displayableName;
                                            });
                                        }
                                    }
                                });
                            }, 300);
                            break;
                        default: // use for pageVisitGoal
                            $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.pagePath = data.targetEvent.parameterValues.pagePath;
                            getNodeInfo(optimizedContent, $scope.optimizedContentsPath[optimizedContent.nodeIdentifier].goalInfo.pagePath);
                            break;
                    }
                });
            }
        };

        $scope.getEditPageLink = function(node) {
            var editPageUrl =  ManagersContext.baseEdit + $scope.optimizedContentsPath[node.nodeIdentifier].pagePath + '.html';
            if(node.pageVariantNodeName) {
                editPageUrl += '?wemActiveVariant=' + node.pageVariantNodeName;
            }
            return editPageUrl;
        };

        $scope.buildPageEditUrl = function(nodePath) {
            return ManagersContext.baseEdit + nodePath + '.html';
        };

        $scope.getEditPageLinkInGoal = function(nodeIdentifier) {
            return ManagersContext.baseEdit + $scope.optimizedContentsPath[nodeIdentifier].goalInfo.pagePath + '.html';
        };

        $scope.getFileUrlInGoal = function(nodeIdentifier) {
            return ManagersContext.baseFile + $scope.optimizedContentsPath[nodeIdentifier].goalInfo.filePath;
        };

        $scope.getLocalizedDateTime = function(dateTime) {
            return moment(dateTime).locale(ManagersContext.jahiaUILocale).format('llll');
        };

        $scope.trimVariantName = function (variantName) {
            if (variantName.length > 25)
                return variantName.substring(0, 25) + '...';
            else
                return variantName;
        };

        $scope.previewVariant = function (optiIdentifier, variantIdentifier) {
            $uibModal.open({
                template: '<div class="modal-body"><iframe style="height: 100%; width: 100%; display: block; border: none;" wem-iframe-onload="iframeLoadedCallBack" src="' + ManagersContext.basePreview + $scope.optimizedContentsPath[optiIdentifier][variantIdentifier] + '.html.ajax"></iframe></div><div class="modal-footer"><button class="btn btn-default pull-right" type="button" message-key="wemnt_serverSettingsOptimizationDashboard.label.close" ng-click="closeModal()"></button></div>',
                size: 'lg',
                windowClass: 'opti-modal',
                controller: function ($scope, $uibModalInstance) {
                    $scope.closeModal = function() {
                        $uibModalInstance.close();
                    }
                }
            })
        };

        $scope.previewPageVariant = function (pageVariantPath, variantNodeName) {
            $uibModal.open({
                template: '<div class="modal-body"><iframe style="height:100%; width: 100%; display: block; border: none;" wem-iframe-onload="iframeLoadedCallBack" src="' + ManagersContext.basePreview + pageVariantPath + '.html?wemActiveVariant='+ variantNodeName +'&hidePersonaPanel=true"></iframe></div><div class="modal-footer"><button class="btn btn-default pull-right" type="button" message-key="wemnt_serverSettingsPersonalizationDashboard.label.close" ng-click="closeModal()"></button></div>',
                size: 'lg',
                windowClass: 'opti-modal',
                controller: function ($scope, $uibModalInstance) {
                    $scope.closeModal = function() {
                        $uibModalInstance.close();
                    }
                }
            })
        };

        $scope.getMessage = function (rbKey) {
            return i18nService.message(rbKey);
        };

        $scope.getMessageWithParams = function (rbKey, params) {
            return i18nService.format(rbKey, params);
        };

        var init = function() {
            getData($scope.currentOrderBy, $scope.currentOrderDirection, $scope.currentOffset, $scope.limit, true);
            $scope.disabled = false;
        };

        init()
    }
})();
