(function() {
    'use strict';

    angular.module('wemPersonaPanelApp', ['wemApp']);
})();

(function() {
    'use strict';

    angular.module('wemPersonaPanelApp').controller('PersonaPanelController', PersonaPanelController);

    PersonaPanelController.$inject = ['$scope', '$timeout',
                                        '_', 'i18nService',
                                        'unomiProfileService', 'unomiGeonamesService', 'unomiSegmentService', 'unomiScoringService'];

    function PersonaPanelController($scope, $timeout,
                                    _, i18nService,
                                    unomiProfileService, unomiGeonamesService, unomiSegmentService, unomiScoringService) {
        var vm = this;
        vm.wemScope = ManagersContext.currentSiteKey;
        vm.filterTargets = [];

        //Google map object
        vm.map = ManagersUtils.personaMap;

        vm.marker = {
            id: 0,
            coords: {
                latitude: 45,
                longitude: -73
            },
            options: {draggable: true},
            events: {
                dragend: function (marker, eventName, model, args) {
                    //Set new location
                    _reverseGeoCode(args[0].latLng.lat(), args[0].latLng.lng());
                }
            }
        };

        //togglable categories
        vm.togglable = {
            optimizationContent: true,
            optimizationPage: true,
            profile: true,
            interests: true,
            pageViews: true,
            session: true,
            segments: true,
            location: true,
            scoringPlan: true
        };

        vm.locationHandler = {
            cityLoaded: false,
            countryLoaded: false,
            adm1Loaded: false,
            adm2Loaded: false
        };

        vm.propertiesHandler = {
            segmentsToAdd: [],
            availableProfileProperties: [],
            availableSessionProperties: [],
            availableScoringPlans: [],
            profilePropertyToAdd: null,
            sessionPropertyToAdd: null,
            scoringPlanToAdd: null,
            interestToAdd: null,
            interestToAddValue: 1,
            profileListNotEmpty: false,
            sessionListNotEmpty: false,
            addProfileProperties: false,
            addSessionProperties: false,
            addInterest: false,
            addSegment: false,
            addScoringPlans: false
        };

        vm.choiceListValues = {
            session : {
                deviceCategory : [
                    "Game console",
                    "Other",
                    "PDA",
                    "Personal computer",
                    "Smart TV",
                    "Smartphone",
                    "Tablet",
                    "Wearable computer"
                ]
            }
        };

        //Display size limit
        vm.propertyNameLimit = 20;
        vm.propertyValueLimit = 20;

        vm.reloadWithSelectedVariant = reloadWithSelectedVariant;
        vm.reloadWithSelectedContentVariant = reloadWithSelectedContentVariant;
        vm.filterAvailableSegments = filterAvailableSegments;
        vm.filterAvailableProps = filterAvailableProps;
        vm.filterAvailableScoringPlans = filterAvailableScoringPlans;
        vm.filteredEditableProp = filteredEditableProp;
        vm.filteredEditableScoring = filteredEditableScoring;
        vm.isFilterOnTarget = isFilterOnTarget;
        vm.removeProp = removeProp;
        vm.removeScoringPlan = removeScoringPlan;
        vm.addProfileProp = addProfileProp;
        vm.addInterest = addInterest;
        vm.addSessionProp = addSessionProp;
        vm.addScoringPlan = addScoringPlan;
        vm.addSegments = addSegments;
        vm.removeSegment = removeSegment;
        vm.getSegmentName = getSegmentName;
        vm.togglableClick = togglableClick;
        vm.getPropertyInputType = getPropertyInputType;
        vm.getPropertyName = getPropertyName;
        vm.locationEdited = locationEdited;
        vm.hasOverridePersona = hasOverridePersona;
        vm.personaSelected = personaSelected;
        vm.updatePersonaOverride = updatePersonaOverride;
        vm.getPropertyResourceName = getPropertyResourceName;
        vm.resetPersona = resetPersona;
        vm.isMultivalue = isMultivalue;

        _init();

        // View functions
        function reloadWithSelectedVariant(variantToApply) {
            vm.variantToApply = variantToApply;
            var location = window.parent.location;
            var str = location.search;
            if (new RegExp("[&?]wemActiveVariant([=&].*)?$").test(str)) {
                str = str.replace(new RegExp("(?:[&?])wemActiveVariant[^&]*", "g"), "")
            }
            str += "&";
            str += "wemActiveVariant=" + vm.variantToApply;
            str = "?" + str.slice(1);
            location.assign(location.origin + location.pathname + str + location.hash)
        }

        function reloadWithSelectedContentVariant(areaId) {
            _.each(vm.areas[areaId].variants, function(variant) {
                parent.document.getElementById(variant.content).style.display = 'none';
                parent.document.getElementById(variant.content).classList.remove('elementUpdated');
                if(variant.content === vm.contentVariantToApply[areaId]){
                    parent.document.getElementById(variant.content).style.display = '';
                    parent.document.getElementById(variant.content).classList.add('elementUpdated');
                }
            });
        }

        function filterAvailableSegments(segment) {
            if(vm.profileSegments && vm.profileSegments.length > 0){
                return !_.contains(vm.profileSegments, segment.id);
            }
            return true;
        }

        function filterAvailableProps(property) {
            if (property.target == "profiles") {
                return vm.profileProperties && !vm.profileProperties.hasOwnProperty(property.itemId);
            } else {
                return vm.sessionProperties && !vm.sessionProperties.hasOwnProperty(property.itemId) &&
                    ['sessionCountryCode','sessionCity','sessionCityId','sessionCountryName','sessionAdminSubDiv1','sessionAdminSubDiv2','location'].indexOf(property.itemId) == -1
            }
        }

        /**
         * Filter the list of addable scoring plan in persona panel.
         * Only the socring plans that are not already in the persona are pickable
         * @param scoring : The scoring plan to evaluate
         * @returns {boolean}
         */
        function filterAvailableScoringPlans(scoring) {
            return vm.scoringPlans && !vm.scoringPlans.hasOwnProperty(scoring.id);
        }

        /**
         * This function filter the editable properties (when filling the persona panel foldable inputs)
         * type is either 'profile' or 'session', it is use to display only the editable properties that are also
         * in available lists
         * @param type
         * @returns {*|{filterid, filters}|Array.<T>}
         */
        function filteredEditableProp(type) {
            if(type == "profile") {
                return _.filter(_.map(vm.profileProperties, function(value, name){ return name }), function(name) {
                    return vm.availableProfilePropertiesIndexed.hasOwnProperty(name);
                });
            } else {
                return  _.filter(_.map(vm.sessionProperties, function(value, name){ return name }), function(name) {
                    return vm.availableSessionPropertiesIndexed.hasOwnProperty(name) &&
                        ['sessionCountryCode','sessionCity','sessionCityId','sessionCountryName','sessionAdminSubDiv1','sessionAdminSubDiv2','location'].indexOf(name) == -1
                });
            }
        }

        /**
         * Filter Scoring plan foldable input in persona panel, display only the scoring that are also in the
         * Available scoring plan list
         * @returns {*|{filterid, filters}|Array.<T>}
         */
        function filteredEditableScoring() {
            return _.filter(_.map(vm.scoringPlans, function(value, scoreId){ return scoreId }), function(scoreId) {
                return vm.availableScoringPlansIndexed.hasOwnProperty(scoreId);
            });
        }

        function isFilterOnTarget(target, value) {
            for (var i = 0; i < vm.filterTargets.length; i++) {
                if (vm.filterTargets[i].target == target) {
                    if (value) {
                        return vm.filterTargets[i].values.indexOf(value) > -1;
                    } else {
                        return true;
                    }
                }
            }
            return false;
        }

        function removeProp(name, type) {
            if(type == "profile") {
                if(vm.profileProperties.hasOwnProperty(name)) {
                    delete vm.profileProperties[name];
                }
            } else if(type == "interest") {
                if(vm.profileProperties.interests && vm.profileProperties.interests.hasOwnProperty(name)) {
                    delete vm.profileProperties.interests[name];
                }
            } else {
                if(vm.sessionProperties.hasOwnProperty(name)) {
                    delete vm.sessionProperties[name];
                }
            }
            vm.updatePersonaOverride();
        }

        function removeScoringPlan(id) {
            if(vm.scoringPlans.hasOwnProperty(id)) {
                delete vm.scoringPlans[id];
            }
            vm.updatePersonaOverride();
        }

        function addProfileProp() {
            if(vm.propertiesHandler.profilePropertyToAdd) {
                if(vm.propertiesHandler.profilePropertyToAdd.type == "integer") {
                    vm.profileProperties[vm.propertiesHandler.profilePropertyToAdd.itemId] = 0;
                } else if(vm.propertiesHandler.profilePropertyToAdd.type == "date") {
                    vm.profileProperties[vm.propertiesHandler.profilePropertyToAdd.itemId] = new Date();
                } else {
                    vm.profileProperties[vm.propertiesHandler.profilePropertyToAdd.itemId] = "";
                }

                vm.updatePersonaOverride();
                vm.propertiesHandler.profilePropertyToAdd = null;
            }
        }

        function addInterest() {
            if (vm.propertiesHandler.interestToAdd) {
                if (!vm.profileProperties.interests) {
                    vm.profileProperties.interests = {};
                }
                vm.profileProperties.interests[vm.propertiesHandler.interestToAdd.toLowerCase()] = (vm.propertiesHandler.interestToAddValue != undefined && vm.propertiesHandler.interestToAddValue > 0) ? vm.propertiesHandler.interestToAddValue : 1;

                vm.updatePersonaOverride();
                vm.propertiesHandler.interestToAdd = null;
                vm.propertiesHandler.interestToAddValue = 1;
                vm.propertiesHandler.addInterest = false;
            }
        }

        function addSessionProp() {
            if(vm.propertiesHandler.sessionPropertyToAdd) {
                if(vm.propertiesHandler.sessionPropertyToAdd.type == "integer") {
                    vm.sessionProperties[vm.propertiesHandler.sessionPropertyToAdd.itemId] = 0;
                } else if(vm.propertiesHandler.sessionPropertyToAdd.type == "date") {
                    vm.sessionProperties[vm.propertiesHandler.sessionPropertyToAdd.itemId] = new Date();
                } else {
                    vm.sessionProperties[vm.propertiesHandler.sessionPropertyToAdd.itemId] = "";
                }

                vm.updatePersonaOverride();
                vm.propertiesHandler.sessionPropertyToAdd = null;
            }
        }

        function addScoringPlan() {
            if(vm.propertiesHandler.scoringPlanToAdd) {
                vm.scoringPlans[vm.propertiesHandler.scoringPlanToAdd.id] = 0;
                vm.updatePersonaOverride();
                vm.propertiesHandler.scoringPlansToAdd = null;
            }
        }

        function addSegments() {
            angular.forEach(vm.propertiesHandler.segmentsToAdd, function (segment) {
                vm.profileSegments.push(segment.id);
            });
            vm.updatePersonaOverride();
            vm.propertiesHandler.addSegment = false;
            vm.propertiesHandler.segmentsToAdd = [];
        }

        function removeSegment(segmentName) {
            vm.profileSegments = _.filter(vm.profileSegments, function (segName) {
                return segName != segmentName;
            });
            vm.updatePersonaOverride();
        }

        function getSegmentName(segmentId) {
            var selectedSegment = _.find(vm.allSegments, function (segment) {
                return segment.id == segmentId;
            });
            if (selectedSegment) {
                return selectedSegment.name;
            }
            else {
                return segmentId;
            }
        }

        function togglableClick(toogleName) {
            vm.togglable[toogleName] = !vm.togglable[toogleName];
        }

        function getPropertyInputType(propertyName, type) {
            var property = type == "profile" ? vm.availableProfilePropertiesIndexed[propertyName] : vm.availableSessionPropertiesIndexed[propertyName];
            if(!property) {
                return "text";
            }
            if(property.type == "integer") {
                return "number";
            } else if (property.type == "email") {
                return "email";
            } else if (property.type == "date") {
                return "date";
            }
            return "text";
        }

        function getPropertyName(type, name) {
            if(type == "profile") {
                return (vm.availableProfilePropertiesIndexed && vm.availableProfilePropertiesIndexed[name]) ? vm.availableProfilePropertiesIndexed[name].metadata.name : name;
            } else {
                return (vm.availableSessionPropertiesIndexed && vm.availableSessionPropertiesIndexed[name]) ? vm.availableSessionPropertiesIndexed[name].metadata.name : name;
            }
        }

        function isMultivalue(pType, pName) {
            if(pType == "profile") {
                return (vm.availableProfilePropertiesIndexed && vm.availableProfilePropertiesIndexed[pName]) ? vm.availableProfilePropertiesIndexed[pName].multivalued : false;
            } else {
                return (vm.availableSessionPropertiesIndexed && vm.availableSessionPropertiesIndexed[pName]) ? vm.availableSessionPropertiesIndexed[pName].multivalued : false;
            }
        }

        /**
         * Update all address and location field when user changes country
         * Then save them to Browser session and refresh google Map
         * @param propName
         */
        function locationEdited(propName) {
            var locationValue = vm.sessionProperties[propName];
            switch (propName) {
                case "sessionCountryName" :
                    //Get country from country name
                    var locationObject = _.find(vm.countries, function(scopeCountry) {
                        return scopeCountry.name == locationValue;
                    });
                    break;
                case "sessionAdminSubDiv1" :
                    var locationObject = _.find(vm.adm1List, function(oAdm) {
                        return oAdm.itemId == locationValue;
                    });
                    break;
                case "sessionAdminSubDiv2" :
                    var locationObject = _.find(vm.adm2List, function(oAdm) {
                        return oAdm.itemId == locationValue;
                    });
                    break;
                case "sessionCityId" :
                    var locationObject = _.find(vm.cities, function(oAdm) {
                        return oAdm.itemId == locationValue;
                    });
                    break;
            }
            //Get capital city and update adm1 and adm2 if needed
            _setLocation(locationObject, propName);
        }

        //Get modified persona
        function hasOverridePersona() {
            return vm.personaOverrides[vm.selectedPersona];
        }

        //Save selected persona
        function personaSelected() {
            var personaOverride = _getPersonaOverride();
            vm.propertiesHandler.addProfileProperties = false;
            vm.propertiesHandler.addSessionProperties = false;
            if (vm.selectedPersona != 'none') {
                console.log('IF');
                console.log(vm.selectedPersona);
                window.parent.wem.loadPersonaContext(vm.selectedPersona, personaOverride);
            } else {
                console.log('ELSE');
                window.parent.wem.loadContext(true, true);
            }
            if (_storageAvailable('sessionStorage')) {
                sessionStorage.personaId = vm.selectedPersona;
            }
        }

        function updatePersonaOverride() {
            var personaOverride = _getPersonaOverride();

            personaOverride.profileProperties = angular.copy(vm.profileProperties);
            personaOverride.sessionProperties = angular.copy(vm.sessionProperties);
            personaOverride.segments = angular.copy(vm.profileSegments);
            personaOverride.scoringPlans = angular.copy(vm.scoringPlans);

            vm.personaOverrides[vm.selectedPersona] = personaOverride;
            if (_storageAvailable('sessionStorage')) {
                sessionStorage.personaOverrides = angular.toJson(vm.personaOverrides);
            }

            vm.propertiesHandler.addProfileProperties = false;
            vm.propertiesHandler.addSessionProperties = false;
            vm.propertiesHandler.addScoringPlans = false;
            window.parent.wem.loadPersonaContext(vm.selectedPersona,personaOverride);
            if (_storageAvailable('sessionStorage')) {
                sessionStorage.personaId = vm.selectedPersona;
            }
        }

        /**
         * This function gets the properties labels starting from resource bundle or returning label if nothing is found there
         * @param category : (profile or session)
         * @param propName : The property name
         * @returns An object containing the labels available in resource bundle for available properties
         */
        function getPropertyResourceName(category, propName) {
            var label = i18nService.message(category + "Properties." + propName);
            if (label.indexOf("???") == -1) {
                return label;
            }
            return vm.getPropertyName(category, propName);
        }

        /**
         *This function is called to reset the persona properties and segment to the values stored in Apache Unomi
         */
        function resetPersona() {
            if (_storageAvailable('sessionStorage')) {
                delete sessionStorage.personaOverrides;
            }
            vm.personaOverrides = {};
            window.parent.wem.loadPersonaContext(vm.selectedPersona, vm.personaOverrides);
            vm.map.control.refresh(vm.sessionProperties.location.lat, vm.sessionProperties.location.lon);
        }

        // Private functions under this line
        function _init() {
            var pagePersonalization = window.parent.pagePersonalization;
            if(window.wem && pagePersonalization) {
                window.parent.wem.registerPersonalization(pagePersonalization.filters, pagePersonalization.strategy, pagePersonalization.identifier, pagePersonalization.fallback, false, function(successfulFilters, selectedFilter) {
                    if (selectedFilter.content != pagePersonalization.wemActiveVariant) {
                        console.log("selected page change: " +selectedFilter.content);
                        var location = window.parent.location;
                        var str = location.search;
                        if (new RegExp("[&?]wemActiveVariant([=&].*)?$").test(str)) {
                            str = str.replace(new RegExp("(?:[&?])wemActiveVariant[^&]*", "g"), "")
                        }
                        str += "&";
                        str += "wemActiveVariant=" + selectedFilter.content;
                        str = "?" + str.slice(1);
                        location.assign(location.origin + location.pathname + str + location.hash)
                    }
                });
            }

            vm.contentVariantToApply = {};
            if (window.wem && window.parent.optimizedContentAreas) {
                vm.areas = window.parent.optimizedContentAreas;
                _.each(_.keys(vm.areas), function(key) {
                    _.each(vm.areas[key].variants, function(variant){
                        if(parent.document.getElementById(variant.content).style.display != 'none'){
                            vm.contentVariantToApply[key] = variant.content;
                        }
                    });
                });
            }

            if (window.wem && getPageVariants) {
                vm.pageVariants = getPageVariants();
                var match = parent.location.href.match('[?&]wemActiveVariant=([^&]+)');
                if(match && match.length > 0) {
                    vm.variantToApply = match[1].replace('#', '');
                } else {
                    vm.variantToApply = '';
                }
            }

            if (_storageAvailable('sessionStorage') && sessionStorage.personaId) {
                vm.selectedPersona = sessionStorage.personaId;
            } else {
                vm.selectedPersona = "none";
            }

            unomiSegmentService.getAll().then(function(response) {
                vm.allSegments = _.sortBy(response.data, 'name');
            });
            unomiProfileService.getPropertyTypesByTag("profileProperties", true).then(function(response) {
                var propertyTypes = response.data;
                vm.availableProfileProperties= propertyTypes;
                vm.availableProfilePropertiesIndexed= _.indexBy(propertyTypes, 'itemId');
            });
            unomiProfileService.getPropertyTypesByTag("sessionProperties", true).then(function(response) {
                var propertyTypes = response.data;
                vm.availableSessionProperties= propertyTypes;
                vm.availableSessionPropertiesIndexed= _.indexBy(propertyTypes, 'itemId');
            });

            //List all available scoring plans
            _getScoringPlans();

            if (_storageAvailable('sessionStorage')) {
                vm.personaOverrides = angular.fromJson(sessionStorage.personaOverrides) || {};
            } else {
                vm.personaOverrides = {};
            }

            window.parent.wem.registerCallbacks(_contextLoaded);

            //Get personas list from Apache Unomi using Profile Service
            unomiProfileService.search({
                condition: null,
                limit: 10,
                offset: 0,
                sortby: "itemId:asc",
                text: ""
            }, "personas").then(function(response) {
                vm.personas = response.data;
                angular.forEach(vm.personas.list, function(value, key, obj) {
                    if (value.properties.firstName) {
                        value.displayName = value.properties.firstName;
                    }
                    if (value.properties.lastName) {
                        if(value.displayName ) {
                            value.displayName  += (" " + value.properties.lastName);
                        } else {
                            value.displayName  = value.properties.lastName;
                        }
                    }
                    if(!value.displayName ) {
                        // TODO i18n this
                        value.displayName  = "Visitor";
                    }
                });
                //Pushing currentProfile to the personas list
                vm.personas.list.push({
                    "itemId": "none",
                    "displayName" : "none"
                });
            });

            var personaOverride = _getPersonaOverride();
            if (vm.selectedPersona != 'none') {
                window.parent.wem.loadPersonaContext(vm.selectedPersona, personaOverride);
            } else {
                window.parent.wem.loadContext(true, true);
            }
        }

        function _getScoringPlans(){
            unomiScoringService.getAll().then(function (response) {
                    var data = response.data;
                    vm.availableScoringPlans = data;
                    vm.availableScoringPlansIndexed = _.indexBy(data, 'id');
                },
                function () {
                    fnCallback(getErrorResult());
                    return [];
                });
        }

        function _storageAvailable(type) {
            try {
                var storage = window[type],
                    x = '__storage_test__';
                storage.setItem(x, x);
                storage.removeItem(x);
                return true;
            }
            catch (e) {
                return false;
            }
        }

        function _watchDate(name) {
            $scope.$watch(angular.bind(vm, function () {
                return vm[name];
            }), function(newVal, oldVal) {
                if(newVal && oldVal && Object.prototype.toString.call(newVal) === '[object Date]' && Object.prototype.toString.call(oldVal) === '[object Date]' && newVal.getTime() != oldVal.getTime()) {
                    vm.updatePersonaOverride();
                }
            });
        }

        /**
         * This function gets the full country list from Apache Unomi
         */
        function _getCountries() {
            unomiGeonamesService.getEntries().then(function (response) {
                var data = response.data;
                //Order list by name before returning it
                vm.countries = _.sortBy(data.list, 'name');
                vm.locationHandler.countryLoaded = true;
            });
        }

        /**
         * This function gets the full country list from Apache Unomi
         */
        function _getAdm1(countryCode, callback) {
            unomiGeonamesService.getEntries(countryCode).then(function (response) {
                var data = response.data;
                //Order list by name before returning it
                if (_.size(data.list) > 0) {
                    if (data.list[0].featureClass == "P") {
                        vm.adm1List = null;
                    }
                    else {
                        vm.adm1List = _.sortBy(data.list, 'name');
                    }
                }
                else {
                    vm.adm1List = null;
                }
                vm.locationHandler.adm1Loaded = true;
                callback();
            });
        }

        /**
         * This function gets the full country list from Apache Unomi
         */
        function _getAdm2(countryCode, adm1Id, callback) {
            var adm1Code = _.find(vm.adm1List, function(adm) {
                return adm.itemId == adm1Id;
            }).admin1Code;
            unomiGeonamesService.getEntries(countryCode, adm1Code).then(function (response) {
                var data = response.data;
                //Order list by name before returning it
                if (_.size(data.list) > 0) {
                    if (data.list[0].featureClass == "P") {
                        vm.adm2List = null;
                    }
                    else {
                        vm.adm2List = _.sortBy(data.list, 'name');
                    }
                }
                else {
                    vm.adm2List = null;
                }
                vm.locationHandler.adm2Loaded = true;
                if (callback != undefined) {
                    callback();
                }
            });
        }
        
        /**
         * This function gets the full country list from Apache Unomi
         */
        function _getCities(countryCode, admin1Id, admin2Id) {
            var oAdm1 = _.find(vm.adm1List, function(adm1) {
                return adm1.itemId == admin1Id;
            });
            var oAdm2 = _.find(vm.adm2List, function(adm2) {
                return adm2.itemId == admin2Id;
            });
            if (oAdm1) {
                var adm1Code = oAdm1.admin1Code;
            }
            if (oAdm2) {
                var adm2Code = oAdm2.admin2Code;
            }
            unomiGeonamesService.getCities(countryCode, adm1Code, adm2Code).then(function (response) {
                //Order list by name before returning it
                vm.cities = _.sortBy(response.data.list, 'name');
                vm.locationHandler.cityLoaded = true;
            });
        }

        function _updateLocation(location, cityName, cityId, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2) {
            //Update scope properties
            if(!vm.sessionProperties.location) {
                vm.sessionProperties.location = {};
            }
            vm.sessionProperties.location.lat = location.lat;
            vm.sessionProperties.location.lon = location.lon;
            vm.sessionProperties.sessionCity = cityName;
            vm.sessionProperties.sessionCityId = cityId;
            vm.sessionProperties.sessionCountryCode = countryCode;
            vm.sessionProperties.sessionCountryName = countryName;
            vm.sessionProperties.sessionAdminSubDiv1 = sessionAdminSubDiv1;
            vm.sessionProperties.sessionAdminSubDiv2 = sessionAdminSubDiv2;
            vm.updatePersonaOverride();

            //Updating the google map marker
            vm.marker.coords = {
                latitude: location.lat,
                longitude: location.lon
            };

            //Refresh google map
            vm.map.control.refresh({latitude: location.lat, longitude: location.lon});
        }

        /**
         * This function gets the address of a point defined with latitude and longitude
         * @param lat : latitude
         * @param lon : longitude
         */
        function _reverseGeoCode(lat, lon) {
            //Reverse geocode resource url
            unomiGeonamesService.reverseGeoCode(lat, lon)
                .then(function(response) {
                    var data = response.data;
                    /* Found the address of the location*/
                    //Getting the proper city object from data
                    var city = _.find(data, function(subdiv) {
                        return subdiv.featureCode.indexOf("PPL") > -1;
                    });
                    //Getting the proper country object from data
                    var country = _.find(data, function(subdiv) {
                        return subdiv.featureCode.indexOf("PCL") > -1;
                    });
                    var admSub1 = _.find(data, function(subdiv) {
                        return subdiv.featureCode == "ADM1";
                    });

                    var admSub2 = _.find(data, function(subdiv) {
                        return subdiv.featureCode == "ADM2";
                    });
                    if (admSub1) {
                        var sessionAdminSubDiv1 = admSub1.itemId;
                    }
                    if (admSub2) {
                        var sessionAdminSubDiv2 = admSub2.itemId;
                    }
                    //We need at least to change the country to update the properties
                    if (country) {
                        _refreshLocation(city, country.countryCode, country.name, sessionAdminSubDiv1, sessionAdminSubDiv2);
                        //_updateLocation({lat:lat,lon:lon},city.name,city.itemId,country.countryCode,country.name,sessionAdminSubDiv1,sessionAdminSubDiv2)
                    }
                });
        }

        /**
         * Update location in scope, session storage and refresh the google map marker and center
         * @param city : geoname object
         * @param countryCode : String
         * @param countryName : String
         * @param sessionAdminSubDiv1 : String
         * @param sessionAdminSubDiv2 : String
         */
        function _refreshLocation(city, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2, propName) {
            var locationProperties = {
                location: {},
                sessionCity: null,
                sessionCityId: null,
                sessionAdminSubDiv2: sessionAdminSubDiv2,
                sessionAdminSubDiv1: sessionAdminSubDiv1,
                sessionCountryCode: countryCode,
                sessionCountryName: countryName
            };
            if (city) {
                locationProperties.location = {lat: city.location.lat, lon: city.location.lon};
                locationProperties.sessionCity = city.name;
                locationProperties.sessionCityId = city.itemId;
            }
            else {
                var oCountry = _.find(vm.countries, function(country) {
                    return country.countryCode == countryCode;
                });
                locationProperties.location = {lat: oCountry.location.lat, lon: oCountry.location.lon};
            }
            //Refresh geo lists
            if (countryCode) {
                //If country updated,then update adm1, adm2 & country list if needed
                if (propName == 'sessionCountryName') {
                    if (!sessionAdminSubDiv1) {
                        delete vm.adm1List;
                        delete vm.adm2List;
                        _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                    } else {
                        _getAdm1(countryCode, function() {
                            if (sessionAdminSubDiv2) {
                                _getAdm2(countryCode, sessionAdminSubDiv1, function() {
                                    _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                                });
                            }
                            else {
                                delete vm.adm2List;
                                _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                            }
                        });
                    }
                }

                else if (propName == 'sessionAdminSubDiv1') {
                    if (!sessionAdminSubDiv2) {
                        _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                    } else {
                        _getAdm2(countryCode, sessionAdminSubDiv1, function() {
                            _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                        });
                    }
                }

                else if (propName == 'sessionAdminSubDiv2') {
                    _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                }
                else {
                    if (countryName) {
                        if (!sessionAdminSubDiv1) {
                            delete vm.adm1List;
                            delete vm.adm2List;
                            _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                        } else {
                            _getAdm1(countryCode, function() {
                                if (sessionAdminSubDiv2) {
                                    _getAdm2(countryCode, sessionAdminSubDiv1, function() {
                                        _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                                    });
                                }
                                else {
                                    delete vm.adm2List;
                                    _getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                                }
                            });
                        }
                    }
                }
            }
            _updateLocation(locationProperties.location, locationProperties.sessionCity, locationProperties.sessionCityId, locationProperties.sessionCountryCode, locationProperties.sessionCountryName, locationProperties.sessionAdminSubDiv1, locationProperties.sessionAdminSubDiv2);
        }

        /**
         * This function get the city to point the map on according to the given object and refreshes the map and the admin1, admin2 and cities lists
         * @param object : geoname object
         * @param propName : String
         */
        function _setLocation(object, propName) {
            var sessionAdminSubDiv1 = null;
            var sessionAdminSubDiv2 = null;
            var countryCode = null;
            var countryName = null;
            //Get the capital of the entity and set location parameters
            if (propName != "sessionCityId") {
                unomiGeonamesService.getCapital(object.itemId)
                    .then(function (response) {
                            var data = response.data;
                            //City feature code must contain the String "PPL"
                            var city = _.find(data, function(citySubDiv) {
                                return citySubDiv.featureCode.indexOf("PPL") > -1;
                            });
                            //feature codes for admin sub areas are exactly "ADM1" and "ADM2"
                            var admSub1 = _.find(data, function(citySubDiv) {
                                return citySubDiv.featureCode == "ADM1";
                            });
                            var admSub2 = _.find(data, function(citySubDiv) {
                                return citySubDiv.featureCode == "ADM2";
                            });
                            //Feature code for country must contain the string "PCL"
                            var country = _.find(data, function(citySubDiv) {
                                return citySubDiv.featureCode.indexOf("PCL") > -1;
                            });
                            //If we found a location we refresh the scope and the map with it
                            if (city) {
                                countryCode = country.countryCode;
                                countryName = country.name;
                                //Update country code in scope
                                //Getting the sub area 1 from the data
                                if (admSub1) {
                                    sessionAdminSubDiv1 = admSub1.itemId;
                                }
                                if (admSub2) {
                                    sessionAdminSubDiv2 = admSub2.itemId;
                                }
                            } else {
                                //if we didn't find a location we base our refresh on the country contained in the object
                                countryCode = object.countryCode;
                                countryName = object.name;
                            }
                            _refreshLocation(city, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2, propName);
                        },
                        function() {
                            //The capital city was not found
                            //reverse geocoding to get the cityname of the country center point
                            _reverseGeoCode(object.location.lat, object.location.lon);
                        });
            } else {
                countryCode = vm.sessionProperties.sessionCountryCode;
                countryName = vm.sessionProperties.sessionCountryName;
                sessionAdminSubDiv1 = vm.sessionProperties.sessionAdminSubDiv1;
                sessionAdminSubDiv2 = vm.sessionProperties.sessionAdminSubDiv2;
                _refreshLocation(object, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2, propName);
            }
        }

        function _getPersonaOverride() {
            var personaOverride = vm.personaOverrides[vm.selectedPersona];
            if (personaOverride == null) {
                personaOverride = {};
                personaOverride.personaId = vm.selectedPersona;
            }
            return personaOverride;
        }

        function _contextLoaded(digitalData) {
            $timeout(function() {
                //Getting personaId from brower session

                var personaId = null;
                if (_storageAvailable('sessionStorage')) {
                    personaId = sessionStorage.personaId;
                }
                if (personaId) {
                    vm.selectedPersona = personaId;
                } else {
                    vm.selectedPersona = "none";
                }
                vm.profileId = window.parent.cxs.profileId;
                if (!vm.hasOverridePersona()) {
                    vm.profileProperties = window.parent.cxs.profileProperties;
                    vm.sessionProperties = window.parent.cxs.sessionProperties;
                    vm.profileSegments = window.parent.cxs.profileSegments;
                }

                vm.filterTargets = window.parent.wem.getFilterTargets();

                if (!vm.profileProperties) {
                    vm.profileProperties = {}
                }
                if (!vm.profileProperties.interests) {
                    vm.profileProperties.interests = {}
                }
                if (!vm.profileProperties.pageViewCount) {
                    vm.profileProperties.pageViewCount = {}
                }
                if (!vm.sessionProperties) {
                    vm.sessionProperties = {}
                }
                if (!vm.scoringPlans) {
                    vm.scoringPlans = {}
                }

                // now let's copy the overrides into the currently loaded segment, profile and session properties.
                var personaOverride = _getPersonaOverride();
                if (!vm.profileSegments) {
                    vm.profileSegments = [];
                }
                if (personaOverride.segments) {
                    vm.profileSegments = personaOverride.segments;
                }

                if(personaOverride.profileProperties) {
                    vm.profileProperties = angular.copy(personaOverride.profileProperties);
                }

                angular.forEach(vm.profileProperties, function(value, key, obj) {
                    if(vm.availableProfilePropertiesIndexed && vm.availableProfilePropertiesIndexed[key] && vm.availableProfilePropertiesIndexed[key].type == "date") {
                        _watchDate("profileProperties." + key);
                    }
                });

                if(personaOverride.sessionProperties) {
                    vm.sessionProperties = angular.copy(personaOverride.sessionProperties);
                }

                angular.forEach(vm.sessionProperties, function(value, key, obj) {
                    if(vm.availableSessionPropertiesIndexed && vm.availableSessionPropertiesIndexed[key] && vm.availableSessionPropertiesIndexed[key].type == "date") {
                        _watchDate("sessionProperties." + key);
                    }
                });

                if(personaOverride.scoringPlans) {
                    vm.scoringPlans = angular.copy(personaOverride.scoringPlans);
                }

                //Initializing google Map if needed
                if (vm.selectedPersona != "none") {
                    if (_.size(vm.countries) == 0) {
                        _getCountries();
                    }
                    if (vm.sessionProperties.sessionAdminSubDiv1) {
                        _getAdm1(vm.sessionProperties.sessionCountryCode, function() {
                            if (vm.sessionProperties.sessionAdminSubDiv2) {
                                _getAdm2(vm.sessionProperties.sessionCountryCode, vm.sessionProperties.sessionAdminSubDiv1, function() {
                                    _getCities(vm.sessionProperties.sessionCountryCode, vm.sessionProperties.sessionAdminSubDiv1, vm.sessionProperties.sessionAdminSubDiv2)
                                });
                            }
                            else {
                                _getCities(vm.sessionProperties.sessionCountryCode, vm.sessionProperties.sessionAdminSubDiv1, vm.sessionProperties.sessionAdminSubDiv2);
                            }
                        });
                    }
                    else {
                        _getCities(vm.sessionProperties.sessionCountryCode, vm.sessionProperties.sessionAdminSubDiv1, vm.sessionProperties.sessionAdminSubDiv2);
                    }
                    if (vm.sessionProperties.hasOwnProperty("location")) {
                        vm.marker.coords = {
                            latitude: vm.sessionProperties.location.lat,
                            longitude: vm.sessionProperties.location.lon
                        };
                        if (vm.map.control.refresh) {
                            vm.map.control.refresh({
                                latitude: vm.sessionProperties.location.lat,
                                longitude: vm.sessionProperties.location.lon
                            });
                        } else {
                            $scope.$watch(angular.bind(vm, function () {
                                return vm.map.control.refresh;
                            }), function(newVal, oldVal) {
                                if (newVal && !oldVal) {
                                    vm.map.control.refresh({
                                        latitude: vm.sessionProperties.location.lat,
                                        longitude: vm.sessionProperties.location.lon
                                    });
                                }
                            });
                        }
                    }
                }
                vm.locationProperties = ["sessionCountryName", "sessionAdminSubDiv1", "sessionAdminSubDiv2", "sessionCity"];
            });
        }

        // Scope function call by other scope
        /**
         * This function is called each time a foldable input send a blur event
         * It gets the property type (profile or session) and save it to the browser session scope.
         * @param event
         */
        $scope.blurCallback = function(event) {
            vm.updatePersonaOverride();
        };
    }
})();
