(function() {
    'use strict';

    angular.module('wemPersonalizationDashboardApp', ['wemApp']);
})();
(function() {
    'use strict';

    angular.module('wemPersonalizationDashboardApp').controller('PersonalizationDashboardController', PersonalizationDashboardController);

    PersonalizationDashboardController.$inject = ['$scope', '$http', '$uibModal', 'i18nService'];

    function PersonalizationDashboardController($scope, $http, $uibModal, i18nService) {
        $scope.personalizedContents = [];
        $scope.personalizedContentsPath = {};
        $scope.limit = 50;
        $scope.currentOffset = 0;
        $scope.currentOrderBy = 'personalizationName';
        $scope.currentOrderDirection = 'ASC';
        $scope.limitPage = 50;
        $scope.currentOffsetPage = 0;
        $scope.currentOrderByPage = 'optimizationName';
        $scope.currentOrderDirectionPage = 'ASC';
        $scope.disabled = true;
        $scope.fetching = false;
        $scope.searchValue = '';

        var getData = function(orderBy, direction, offset, limit, increaseCurrentOffset) {
            $scope.fetching = true;
            $http.get(ManagersContext.baseEdit + ManagersContext.currentSitePath + ".personalizationDashboardAction.do?orderBy=" + orderBy + "&orderDirection=" + direction + "&offset=" + offset + "&limit=" + limit + "&orderByPages=" + orderBy + "&orderDirectionPages=" + direction + "&offsetPages=" + offset + "&limitPages=" + limit).then(function(response) {
                var data = response.data;
                $scope.fetching = false;
                var length = data.personalizedContents.length;
                if (length < $scope.limit) {
                    $scope.disabled = true;
                }

                if (increaseCurrentOffset) {
                    $scope.currentOffset += length;
                }

                $scope.personalizedPages = data.personalizedPages;

                angular.forEach(data.personalizedContents, function(personalizedContent) {
                    $scope.personalizedContents.push(personalizedContent);
                });
                angular.forEach(data.personalizedContentsPath, function (value, key) {
                    $scope.personalizedContentsPath[key] = value;
                });

            });
        };

        $scope.updateOrderBy = function(orderBy) {
            $scope.disabled = false;
            $scope.currentOrderBy = orderBy;
            $scope.currentOffset = 0;
            $scope.personalizedContents = [];
            $scope.personalizedContentsPath = {};
            getData($scope.currentOrderBy, $scope.currentOrderDirection, $scope.currentOffset, $scope.limit, true);
        };

        $scope.updateOrderDirection = function() {
            $scope.currentOrderDirection = $scope.currentOrderDirection=='ASC'?'DESC':'ASC';
            $scope.personalizedContents = [];
            $scope.personalizedContentsPath = {};
            // To not loose current data with use the current offset as a limit and we call from offset 0
            getData($scope.currentOrderBy, $scope.currentOrderDirection, 0, $scope.currentOffset, false);
        };

        $scope.getMoreData = function() {
            getData($scope.currentOrderBy, $scope.currentOrderDirection, $scope.currentOffset, $scope.limit, true);
        };

        $scope.getMoreDataForPages = function() {
            getData($scope.currentOrderByPage, $scope.currentOrderDirectionPage, $scope.currentOffsetPage, $scope.limitPage, true);
        };

        $scope.toggleCardVariants = function(personalizedContent) {
            personalizedContent.collapsed = !personalizedContent.collapsed;
        };

        $scope.getEditPageLink = function(node) {
            var editPageUrl =  ManagersContext.baseEdit + $scope.personalizedContentsPath[node.nodeIdentifier].pagePath + '.html';
            if(node.pageVariantNodeName) {
                editPageUrl += '?wemActiveVariant=' + node.pageVariantNodeName;
            }
            return editPageUrl;
        };

        $scope.buildPageEditUrl = function(nodePath) {
            return ManagersContext.baseEdit + nodePath + '.html';
        };

        $scope.getLocalizedDateTime = function(dateTime) {
            return moment(dateTime).locale(ManagersContext.jahiaUILocale).format('llll');
        };

        $scope.trimVariantName = function (variantName) {
            if (variantName.length > 25) {
                return variantName.substring(0, 25) + '...';
            } else {
                return variantName;
            }
        };

        $scope.previewVariant = function (persoIdentifier, variantIdentifier) {
            $uibModal.open({
                template: '<div class="modal-body"><iframe style="height:100%; width: 100%; display: block; border: none;" wem-iframe-onload="iframeLoadedCallBack" src="' + ManagersContext.basePreview + $scope.personalizedContentsPath[persoIdentifier][variantIdentifier] + '.html.ajax"></iframe></div><div class="modal-footer"><button class="btn btn-default pull-right" type="button" message-key="wemnt_serverSettingsPersonalizationDashboard.label.close" ng-click="closeModal()"></button></div>',
                size: 'lg',
                windowClass: 'perso-modal',
                controller: function ($scope, $uibModalInstance) {
                    $scope.closeModal = function() {
                        $uibModalInstance.close();
                    }
                }
            })
        };

        $scope.previewPageVariant = function (pageVariantPath, variantNodeName) {
            $uibModal.open({
                template: '<div class="modal-body"><iframe style="height:100%; width: 100%; display: block; border: none;" wem-iframe-onload="iframeLoadedCallBack" src="' + ManagersContext.basePreview + pageVariantPath + '.html?wemActiveVariant='+ variantNodeName +'&hidePersonaPanel=true"></iframe></div><div class="modal-footer"><button class="btn btn-default pull-right" type="button" message-key="wemnt_serverSettingsPersonalizationDashboard.label.close" ng-click="closeModal()"></button></div>',
                size: 'lg',
                windowClass: 'perso-modal',
                controller: function ($scope, $uibModalInstance) {
                    $scope.closeModal = function() {
                        $uibModalInstance.close();
                    }
                }
            })
        };


        $scope.getMessage = function (rbKey) {
            return i18nService.message(rbKey);
        };

        $scope.getMessageWithParams = function (rbKey, params) {
            return i18nService.format(rbKey, params);
        };

        var init = function() {
            getData($scope.currentOrderBy, $scope.currentOrderDirection, $scope.currentOffset, $scope.limit, true);
            $scope.disabled = false;
        };

        init()
    }
})();
