(function() {
    'use strict';

    angular.module('wemReferrersApp', ['wemApp', 'wemProfilesApp']);
})();

(function() {
    'use strict';

    angular.module('wemReferrersApp').run(wemReferrersAppRun);

    wemReferrersAppRun.$inject = ['$rootScope'];

    function wemReferrersAppRun($rootScope) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };
    }
})();

(function() {
    'use strict';

    angular.module('wemReferrersApp').controller('ReferrerDetailsController', ReferrerDetailsController);

    ReferrerDetailsController.$inject = ['$scope', '$routeParams', 'referrerService', 'i18nService', '$location', '$compile', 'dataTableService', 'dtPagerService'];

    function ReferrerDetailsController($scope, $routeParams, referrerService, i18nService, $location, $compile, dataTableService, dtPagerService) {
        $scope.referrerHost = $routeParams.referrerId;

        var columnsPlaceHolder = function (data, type, full, meta) {
            if (!data || data == '') {
                return i18nService.message('serverSettings.internalSearches.missing');
            }
            if (meta.col == 1) {
                return moment(data).locale(ManagersContext.jahiaUILocale).format('LLL');
            }
            if (meta.col == 4) {
                return i18nService.message('profilesProperty.gender.' + data);
            }
            return data;
        };

        var serverData = function (sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var offset = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                return ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData) + ":" + o.dir;
            }).join(",");

            referrerService.getReferrerDetails($scope.referrerHost, offset, length, order).then(function(response) {
                var data = response.data;
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": data.totalSize,
                    "iTotalDisplayRecords": data.totalSize,
                    "aaData": data.list
                };
                fnCallback(results);
                dtPagerService.refreshPager($scope);
            },
            function(response) {
                console.error(response);
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": 0,
                    "iTotalDisplayRecords": 0,
                    "aaData": []
                };
                fnCallback(results);
                dtPagerService.refreshPager($scope);
            });
        };

        $scope.dtOptions = dataTableService.generateDataTableOptions({
            pageLength: 25,
            processing: true,
            serverSide: true,
            sDom: 't',
            defaultOrder: [[1, 'desc']],
            rowCallback: function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                $compile(nRow)($scope);
            }
        });

        $scope.dtOptions.withFnServerData(serverData);

        var columns = [
            {name: 'properties.referrerQuery', bundleKey: 'serverSettings.referrers.searchedTerms', actionsHtml: columnsPlaceHolder},
            {name: 'timeStamp', bundleKey: 'serverSettings.referrers.date', actionsHtml: columnsPlaceHolder},
            {name: 'profile.properties.firstName', bundleKey: 'serverSettings.referrers.firstName', actionsHtml: columnsPlaceHolder},
            {name: 'profile.properties.lastName', bundleKey: 'serverSettings.referrers.lastName', actionsHtml: columnsPlaceHolder},
            {name: 'profile.properties.gender', bundleKey: 'serverSettings.referrers.gender', actionsHtml: columnsPlaceHolder},
            {
                name: null, bundleKey: 'serverSettings.referrers.actions',
                actionsHtml: function(data, type, full, meta) {
                    return '<button type="button" class="btn btn-icon command-dashboard" ng-click="goToEditProfile(\'' + data.profileId + '\')" title="' + i18nService.message('serverSettings.incomingSearches.editProfile') + '"><i class="material-icons">person</i></button>';
                },
                notSortable: true,
                options: [
                    ["width", "90px"],
                    ["contentPadding", "0"]
                ]
            }
        ];
        $scope.dtColumns = dataTableService.generateDataTableColumns(columns);

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        $scope.goToEditProfile = function (profileId) {
            $location.path('/referrer/' + $scope.referrerHost + '/profile/' + profileId);
        };
    }
})();
(function() {
    'use strict';

    angular.module('wemReferrersApp').controller('ReferrerQueriesController', ReferrerQueriesController);

    ReferrerQueriesController.$inject = ['$scope', '$routeParams', 'referrerService'];

    function ReferrerQueriesController($scope, $routeParams, referrerService) {
        $scope.referrerQueries = referrerService.getReferrerQueries($routeParams.referrerId);
    }
})();
(function() {
    'use strict';

    angular.module('wemReferrersApp').controller('ReferrersListController', ReferrersListController);

    ReferrersListController.$inject = ['$scope', 'referrerService', 'i18nService', 'uibDatepickerConfig', '$compile', 'dataTableService', '$location', 'dtPagerService'];

    function ReferrersListController($scope, referrerService, i18nService, uibDatepickerConfig, $compile, dataTableService, $location, dtPagerService) {
        $scope.referrerFrom = Date.now() + -30 * 24 * 3600 * 1000;
        $scope.referrerTo = Date.now();

        var init = function (from, to) {
            var promise = referrerService.getReferrers(from, to);
            dataTableService.setPromise(promise);
            $scope.dtOptions = dataTableService.generateDataTableOptions({
                defaultOrder: [[2, 'desc']],
                rowCallback: function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    $compile(nRow)($scope);
                }
            });

            var columns = [
                {name: 'host', bundleKey: 'serverSettings.referrers'},
                {name: 'occurrences', bundleKey: 'serverSettings.referrers.occurrences'},
                {name: 'importance', bundleKey: 'serverSettings.referrers.importance'},
                {
                    name: null, bundleKey: 'serverSettings.manageWem.listAction',
                    actionsHtml: function (data, type, full, meta) {
                        var html = "<button title=\"" + i18nService.message("serverSettings.referrers.referrerQueries") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"displayReferrerQueries(" + meta.row + ")\"><i class=\"material-icons\">search</i></button> ";
                        html +=  "<button title=\"" + i18nService.message("serverSettings.referrers.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails(" + meta.row + ")\"><i class=\"material-icons\">list</i></button> ";
                        return html;
                    },
                    notSortable: true,
                    options: [
                        ["width", "180px"],
                        ["contentPadding", "0"]
                    ]
                }
            ];
            $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
            promise.then(function () {
                dtPagerService.refreshPager($scope);
            });
        };

        init($scope.referrerFrom, $scope.referrerTo);

        // date config
        uibDatepickerConfig.startingDay = 1;
        uibDatepickerConfig.minMode = "day";
        uibDatepickerConfig.datepickerMode = "day";
        uibDatepickerConfig.showWeeks = false;
        $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
        $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
        $scope.clearText = i18nService.message('wem.label.clear');
        $scope.datePickers = {};

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();

            angular.forEach($scope.datePickers, function (value, key) {
                $scope.datePickers[key] = false;
            });
            $scope.datePickers[selector] = true;
        };

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        $scope.goToDetails = function (index) {
            var referrer = $scope.dtInstance.DataTable.row(index).data().host;
            referrerService.setSelectedReferrer(referrer);
            $location.path('/referrer/' + referrer);
        };

        $scope.displayReferrerQueries = function (index) {
            var referrer = $scope.dtInstance.DataTable.row(index).data().host;
            referrerService.setSelectedReferrer(referrer);
            $location.path('/referrer/queries/' + referrer);
        };

        $scope.updateReferrersList = function () {
            init(new Date($scope.referrerFrom).getTime(), new Date($scope.referrerTo).getTime());
            $scope.dtInstance.DataTable.draw();
        };

        $scope.getMessage = function (key) {
            return i18nService.message(key);
        };

    }
})();