(function() {
    'use strict';

    angular.module('wemRulesApp', ['wemApp']);
})();

(function() {
    'use strict';

    angular.module('wemRulesApp').controller('RuleAdvancedEditController', RuleAdvancedEditController);

    RuleAdvancedEditController.$inject = ['$location', '$resource', '$q', '$timeout', '$routeParams',
                                            'unomiRulesService', 'unomiDefinitionService',
                                            'i18nService', 'modalService', 'rulesService', 'loadingSpinnerService'];

    function RuleAdvancedEditController($location, $resource, $q, $timeout, $routeParams,
                                        unomiRulesService, unomiDefinitionService,
                                        i18nService, modalService, rulesService, loadingSpinnerService) {

        var vm = this;

        vm.saveRule = saveRule;
        vm.deleteRule = deleteRule;

        _init();

        // public functions

        function saveRule($event) {
            if (!vm.advancedRuleForm.$valid) {
                modalService.newInfoModal({
                    title: i18nService.message("serverSettings.manageRules.formErrorTitle"),
                    bodyMessage: i18nService.message("serverSettings.manageRules.formErrorBody")
                }, function (modalData) {
                    console.warn("Form not submitted");
                });
                $event.preventDefault();
                return;
            }
            if (vm.sharedRule) {
                vm.currentRule.metadata.scope = 'systemscope';
            }
            loadingSpinnerService.show();
            unomiRulesService.setRule(vm.currentRule).then(function() {
                $timeout(function () {$location.path("/rules")}, 2000);
            });
        }

        function deleteRule(id) {
            modalService.newAlertModal({title: i18nService.message("serverSettings.manageRules.deleteRule"), bodyMessage: i18nService.message("serverSettings.manageRules.deleteRuleConfirmBody")}, function (modalData) {
                console.info("Delete confirmed, deleting rule", vm.currentRule);
                loadingSpinnerService.show();
                unomiRulesService.deleteRule(id).then(function(response) {
                    $timeout(function () {$location.path("/rules")}, 2000);
                });
            }, function (modalData) {
                console.info("Delete cancelled");
            });
        }

        // private functions

        function _init() {
            var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions", {}, { 'get': {method:'GET', isArray : true  }});
            var actionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/actions", {}, { 'get': {method:'GET', isArray : true }});
            var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId : "profileTags" });

            vm.conditionTypes = [];
            vm.actionTypes = [];
            vm.tags = {};
            vm.isNewRule = $routeParams.ruleId == "newAdvancedRule";

            vm.currentRule = rulesService.loadOrCreateRule($routeParams.ruleId).then(function (data) {
                vm.currentRule = data;
                vm.sharedRule = vm.currentRule.metadata.scope != ManagersContext.dmfScope

                var profileTagsPromise = $q.defer();
                var conditionTypesPromise = $q.defer();
                var actionTypesPromise = $q.defer();

                vm.profileTags = tagResource.get({tagId : "profileTags"}, function(value, responseHeaders) {
                    console.info("rule-advanced-edit.controller.js : tags loaded from Apache Unomi.");

                    angular.forEach(value.subTags, function(v) { vm.tags[v.id] = v } );
                    profileTagsPromise.resolve(vm.tags);
                });

                vm.conditionTypes = conditionTypesResource.get(function (conditionTypes) {
                    console.info("rule-advanced-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
                    conditionTypesPromise.resolve(conditionTypes);
                });

                vm.actionTypes = actionTypesResource.get(function (actionTypes) {
                    console.info("rule-advanced-edit.controller.js: Loaded " + actionTypes.length + " action types");
                    actionTypesPromise.resolve(actionTypes);
                });

                $q.all([profileTagsPromise.promise, conditionTypesPromise.promise, actionTypesPromise.promise]).then(function (data) {
                    console.info("Tags, conditions and action types were loaded sucessfully.");
                    console.debug("tags:", vm.profileTags.subTags.length, " conditionTypes:", vm.conditionTypes.length, " actionTypes:", vm.actionTypes.length);
                    unomiDefinitionService.resolveProfileTags(vm.conditionTypes, vm.tags);
                    unomiDefinitionService.resolveProfileTags(vm.actionTypes, vm.tags);
                });
            });

        }

    }
})();

(function () {
    'use strict';

    angular.module('wemRulesApp').controller('RuleSimpleEditController', RuleSimpleEditController);

    RuleSimpleEditController.$inject = ['$location', '$resource', '$q', '$routeParams', '$timeout',
                                        'unomiRulesService', 'unomiDefinitionService', 'unomiScoringService', 'unomiGoalService', 'unomiSegmentService',
                                        'modalService', 'notificationService', 'i18nService', 'rulesService', 'loadingSpinnerService'];

    function RuleSimpleEditController($location, $resource, $q, $routeParams, $timeout,
                                      unomiRulesService, unomiDefinitionService, unomiScoringService, unomiGoalService, unomiSegmentService,
                                      modalService, notificationService, i18nService, rulesService, loadingSpinnerService) {
        
        var vm = this;

        vm.saveRule = saveRule;
        vm.deleteRule = deleteRule;
        vm.openScoringPlanSelectionDialog = openScoringPlanSelectionDialog;
        vm.openGoalSelectionDialog = openGoalSelectionDialog;
        vm.openSegmentSelectionDialog = openSegmentSelectionDialog;
        vm.toggleActionByType = toggleActionByType;
        vm.addAction = addAction;
        vm.removeAction = removeAction;

        _init();

        function saveRule($event) {
            if (!_isFormValid()) {
                notificationService.errorToast(i18nService.message("serverSettings.manageRules.formErrorBody"));
                $event.preventDefault();
                return false;
            }
            _updateRealRule();
            if (vm.sharedRule) {
                vm.currentRule.metadata.scope = 'systemscope';
            }
            loadingSpinnerService.show();
            unomiRulesService.setRule(vm.currentRule).then(function () {
                $timeout(function () { $location.path("/rules")}, 2000);
            });
        }

        function deleteRule($event) {
            // we should ask for a confirmation first
            modalService.newAlertModal({
                title: i18nService.message("serverSettings.manageRules.deleteRule"),
                bodyMessage: i18nService.message("serverSettings.manageRules.deleteRuleConfirmBody")
            }, function (modalData) {
                console.info("Delete confirmed, deleting rule", vm.currentRule);
                loadingSpinnerService.show();
                unomiRulesService.deleteRule(vm.currentRule.itemId).then(function (response) {
                    $timeout(function () { $location.path("/rules")}, 2000);
                });
            }, function (modalData) {
                console.info("Delete cancelled");
            });
        }

        // Scoring plan selection modal
        function openScoringPlanSelectionDialog(event) {
            unomiScoringService.query(unomiScoringService.constructQuery(150, 0, null)).then(function (response) {
                var scoringPlans = response.data;
                var modalData = {
                    scoringPlans: scoringPlans,
                    templateUrl: ManagersContext.wemAppPath + '/wem-rules/rule-scoring-plan-selection-modal.html'
                };
                var okCallback = function (modalData) {
                    console.info("Selected scoring plan", modalData.selectedScoringPlan);
                    vm.simplifiedRule.scoringPlan = modalData.selectedScoringPlan;
                };
                var cancelCallback = function (modalData) {

                };
                modalService.newCustomModal(modalData, event, okCallback, cancelCallback);
            });
        }

        // Goal plan selection modal
        function openGoalSelectionDialog(event) {
            unomiGoalService.getAllForSite().then(function (response) {
                var goals = response.data;
                var modalData = {
                    goals: goals,
                    templateUrl: ManagersContext.wemAppPath + '/wem-rules/rule-goal-selection-modal.html'
                };
                var okCallback = function (modalData) {
                    console.info("Selected goal", modalData.selectedGoal);
                    vm.simplifiedRule.goal = modalData.selectedGoal;
                };
                var cancelCallback = function (modalData) {

                };
                modalService.newCustomModal(modalData, event, okCallback, cancelCallback);
            });
        }

        // Segment plan selection modal
        function openSegmentSelectionDialog(event) {
            unomiSegmentService.query(unomiSegmentService.constructQuery(150, 0, null)).then(function (response) {
                var segments = response.data;
                var modalData = {
                    segments: segments,
                    templateUrl: ManagersContext.wemAppPath + '/wem-rules/rule-segment-selection-modal.html'
                };
                var okCallback = function (modalData) {
                    console.info("Selected segment", modalData.selectedSegment);
                    vm.simplifiedRule.segment = modalData.selectedSegment;
                };
                var cancelCallback = function (modalData) {

                };
                modalService.newCustomModal(modalData, event, okCallback, cancelCallback);
            });
        }

        function toggleActionByType(actionByType) {
            if (actionByType.activated) {
                if (actionByType.actions.length == 0) {
                    actionByType.actions.push({
                        type: actionByType.actionType.id,
                        parameterValues: {}
                    });
                }
            } else {
            }
        }

        function addAction(actionByType) {
            actionByType.actions.push({
                type: actionByType.actionType.id,
                parameterValues: {}
            });
        }

        function removeAction(actionByType, index) {
            actionByType.actions.splice(index, 1);
            if (actionByType.actions.length == 0) {
                actionByType.activated = false;
            }
        }

        // private functions

        function _init() {
            var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions", {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var actionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/actions", {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});
            vm.triggerTypes = [
                {
                    id: "score",
                    name: i18nService.message("serverSettings.manageRules.triggers.visitorReachesScore")
                },
                {
                    id: "goal",
                    name: i18nService.message("serverSettings.manageRules.triggers.visitorFulfilledGoal")
                },
                {
                    id: "sessionStarts",
                    name: i18nService.message("serverSettings.manageRules.triggers.visitorStartsSession")
                },
                {
                    id : "dxLogin",
                    name: i18nService.message("serverSettings.manageRules.triggers.visitorLogsIn")
                }
            ];

            vm.applyToTypes = [
                {
                    id: "all",
                    name: i18nService.message("serverSettings.manageRules.applyTo.applyToAllVisitors")
                },
                {
                    id: "segment",
                    name: i18nService.message("serverSettings.manageRules.applyTo.applyToSpecificSegment")
                }
            ];
            vm.conditionTypes = [];
            vm.actionTypes = [];
            vm.selectedScoringPlan = null;
            vm.selectedGoal = null;
            vm.tags = {};
            vm.isNewRule = $routeParams.ruleId == "newSimpleRule";

            vm.currentRule = rulesService.loadOrCreateRule($routeParams.ruleId).then(function (data) {
                vm.currentRule = data;
                vm.sharedRule = vm.currentRule.metadata.scope != ManagersContext.dmfScope
                var profileTagsPromise = $q.defer();
                var conditionTypesPromise = $q.defer();
                var actionTypesPromise = $q.defer();

                vm.profileTags = tagResource.get({tagId: "profileTags"}, function (value, responseHeaders) {
                    console.info("rule-simple-edit.controller.js : "+value.subTags.length+" profile tags loaded from Apache Unomi.");

                    angular.forEach(value.subTags, function (v) {
                        vm.tags[v.id] = v
                    });
                    profileTagsPromise.resolve(vm.tags);
                });

                vm.conditionTypes = conditionTypesResource.get(function (conditionTypes) {
                    console.info("rule-simple-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
                    conditionTypesPromise.resolve(conditionTypes);
                });

                vm.actionTypes = actionTypesResource.get(function (actionTypes) {
                    console.info("rule-simple-edit.controller.js: Loaded " + actionTypes.length + " action types");
                    _updateSimplifiedRule();
                    actionTypesPromise.resolve(actionTypes);
                });

                $q.all([profileTagsPromise.promise, conditionTypesPromise.promise, actionTypesPromise.promise]).then(function (data) {
                    console.info("Tags, conditions and action types were loaded sucessfully.");
                    console.debug("tags:", vm.profileTags.subTags.length, " conditionTypes:", vm.conditionTypes.length, " actionTypes:", vm.actionTypes.length);
                    unomiDefinitionService.resolveProfileTags(vm.conditionTypes, vm.tags);
                    unomiDefinitionService.resolveProfileTags(vm.actionTypes, vm.tags);
                    console.debug(vm.conditionTypes);
                    console.debug(vm.actionTypes);
                });
            });

        }

        function _updateSimplifiedRule() {
            vm.simplifiedRule = {
                trigger: null,
                applyTo: null,
                scoringPlan: null,
                scoringPlanMinimumScore: 0,
                goal: null,
                segment: null,
                actionsByType: []
            };
            if (vm.currentRule.condition.parameterValues.subConditions.length > 0) {
                for (var i = 0; i < vm.currentRule.condition.parameterValues.subConditions.length; i++) {
                    var subCondition = vm.currentRule.condition.parameterValues.subConditions[i];
                    if (subCondition.type == 'profileSegmentCondition') {
                        var ruleSegments = subCondition.parameterValues.segments;
                        if (ruleSegments && ruleSegments.length > 0) {
                            unomiSegmentService.getSegment(ruleSegments[0]).then(function (response) {
                                vm.simplifiedRule.segment = response.data.metadata
                            });
                        }
                        vm.simplifiedRule.applyTo = vm.applyToTypes[1];
                    } else if (subCondition.type == 'scoringCondition') {
                        unomiScoringService.getScoring(subCondition.parameterValues.scoringPlanId).then(function (response) {
                            vm.simplifiedRule.scoringPlan = response.data.metadata;
                        });
                        vm.simplifiedRule.scoringPlanMinimumScore = subCondition.parameterValues.scoreValue;
                        vm.simplifiedRule.trigger = vm.triggerTypes[0];
                    } else if (subCondition.type == 'goalMatchCondition') {
                        unomiGoalService.getGoal(subCondition.parameterValues.goalId).then(function (response) {
                            vm.simplifiedRule.goal = response.data.metadata;
                        });
                        vm.simplifiedRule.trigger = vm.triggerTypes[1];
                    } else if (subCondition.type == "sessionCreatedEventCondition") {
                        vm.simplifiedRule.trigger = vm.triggerTypes[2];
                    } else if (subCondition.type == "loginEventCondition") {
                        vm.simplifiedRule.trigger = vm.triggerTypes[3];
                    }
                }
            }
            if (vm.simplifiedRule.applyTo == null) {
                vm.simplifiedRule.applyTo = vm.applyToTypes[0];
            }

            for (var i = 0; i < vm.actionTypes.length; i++) {
                var actions = [];
                var actionType = vm.actionTypes[i];
                if (vm.currentRule.actions.length > 0) {
                    for (var k = 0; k < vm.currentRule.actions.length; k++) {
                        var action = vm.currentRule.actions[k];
                        if (action.type == 'setPropertyAction' && action.parameterValues.setPropertyName == 'systemProperties.scoringTriggersExecuted') {
                            // we ignore this action since it is a internal one.
                            continue;
                        }
                        if (action.type == actionType.id) {
                            actions.push(action);
                        }
                    }
                }
                var actionTypeName = i18nService.message('action.' + actionType.id + '.name');
                if (actionTypeName.indexOf("???") == -1 && _.contains(actionType.tags, 'hidden.availableToEndUser')) {
                    var activated = actions.length > 0;
                    vm.simplifiedRule.actionsByType.push({
                        activated: activated,
                        allowMultipleInstances: _.contains(actionType.tags, 'hidden.allowMultipleInstances'),
                        actionType: vm.actionTypes[i],
                        actionTypeName: actionTypeName,
                        actions: actions
                    });
                }
            }
        }

        function _updateRealRule() {
            // the real rule is composed of the following elements :
            // Conditions:
            // - a condition that matches the profileUpdated event
            // - a condition that matches the score target
            // - a condition that matches the reached goal
            // - a condition that matches the selected segment (or none)
            // Actions:
            // - an action to insert the user in a list
            // - multiple actions to update profile properties
            // - an action to create a lead in salesforce, if the action is available.
            var ruleTimestamp = + new Date();
            var segmentMatchCondition = null;
            if (vm.simplifiedRule.applyTo.id == 'segment') {
                segmentMatchCondition = {
                    type: "profileSegmentCondition",
                    parameterValues: {
                        segments: [vm.simplifiedRule.segment.id],
                        matchType: "all"
                    }
                };
            }
            var scoringTriggerExecutedAction = null;
            if (vm.simplifiedRule.trigger.id == 'score') {
                var scoreCondition = {
                    type: "scoringCondition",
                    parameterValues: {
                        scoringPlanId: vm.simplifiedRule.scoringPlan.id,
                        comparisonOperator: "greaterThanOrEqualTo",
                        scoreValue: vm.simplifiedRule.scoringPlanMinimumScore
                    }
                };
                var profileUpdatedEventCondition = {
                    type: "eventTypeCondition",
                    parameterValues: {
                        eventTypeId: "profileUpdated"
                    }
                };
                var scoringTriggerMissingCondition = {
                    type : "profilePropertyCondition",
                    parameterValues: {
                        propertyName : "systemProperties.scoringTriggersExecuted",
                        comparisonOperator: "missing"
                    }
                };
                var scoringTriggerNotFoundCondition = {
                    type: "profilePropertyCondition",
                    parameterValues: {
                        propertyName : "systemProperties.scoringTriggersExecuted",
                        comparisonOperator: "notContains",
                        propertyValue : vm.simplifiedRule.scoringPlan.id + "-" + ruleTimestamp
                    }
                };
                var ruleAlreadyExecutedCondition = {
                    type: "booleanCondition",
                    parameterValues : {
                        operator: "or",
                        subConditions: [
                            scoringTriggerMissingCondition,
                            scoringTriggerNotFoundCondition
                        ]
                    }
                };
                vm.currentRule.condition.parameterValues.subConditions = [
                    ruleAlreadyExecutedCondition,
                    profileUpdatedEventCondition,
                    scoreCondition
                ];
                scoringTriggerExecutedAction = {
                    type : "setPropertyAction",
                    parameterValues : {
                        setPropertyName : "systemProperties.scoringTriggersExecuted",
                        setPropertyValueMultiple : vm.simplifiedRule.scoringPlan.id + "-" + ruleTimestamp,
                        setPropertyStrategy : "addValue",
                        storeInSession: false
                    }
                }
            } else if (vm.simplifiedRule.trigger.id == 'goal') {
                var goalEventCondition = {
                    type: "eventTypeCondition",
                    parameterValues: {
                        eventTypeId: "goal"
                    }
                };
                var goalCondition = {
                    type: "goalMatchCondition",
                    parameterValues: {
                        comparisonOperator: "exists",
                        goalId: vm.simplifiedRule.goal.id
                    }
                };
                vm.currentRule.condition.parameterValues.subConditions = [
                    goalEventCondition,
                    goalCondition
                ];
            } else if (vm.simplifiedRule.trigger.id == 'sessionStarts') {
                var sessionStartsEventCondition = {
                    type: "sessionCreatedEventCondition",
                    parameterValues: {
                    }
                };
                vm.currentRule.condition.parameterValues.subConditions = [
                    sessionStartsEventCondition
                ];
            } else if (vm.simplifiedRule.trigger.id == 'dxLogin') {
                var loginEventCondition = {
                    type: "loginEventCondition",
                    parameterValues: {
                    }
                };
                vm.currentRule.condition.parameterValues.subConditions = [
                    loginEventCondition
                ];
            }
            if (segmentMatchCondition) {
                vm.currentRule.condition.parameterValues.subConditions.push(segmentMatchCondition);
            }
            var actions = [];
            if (scoringTriggerExecutedAction) {
                actions.push(scoringTriggerExecutedAction);
            }
            for (var i = 0; i < vm.simplifiedRule.actionsByType.length; i++) {
                var actionByType = vm.simplifiedRule.actionsByType[i];
                if (actionByType.activated) {
                    if (actionByType.actions && actionByType.actions.length > 0) {
                        actions = actions.concat(actionByType.actions);
                    }
                }
            }
            vm.currentRule.actions = actions;
        }

        function _isFormValid() {
            if (!vm.simpleRuleForm.$valid) {
                return false;
            }
            if (!vm.simplifiedRule.trigger) {
                return false;
            }
            if (vm.simplifiedRule.applyTo.id == 'segment') {
                if (!vm.simplifiedRule.segment || !vm.simplifiedRule.segment.id) {
                    return false;
                }
            }
            if (vm.simplifiedRule.trigger.id == 'score') {
                if (!vm.simplifiedRule.scoringPlan || !vm.simplifiedRule.scoringPlan.id) {
                    return false;
                }
            } else if (vm.simplifiedRule.trigger.id == 'goal') {
                if (!vm.simplifiedRule.goal || !vm.simplifiedRule.goal.id) {
                    return false;
                }
            }
            return true;
        }

    }
})();

(function() {
    'use strict';

    angular.module('wemRulesApp').controller('RulesListController', RulesListController);

    RulesListController.$inject = ['$location', 'unomiRulesService'];

    function RulesListController($location, unomiRulesService) {

        var vm = this;

        vm.refreshRules = refreshRules;
        vm.toggleHidden = toggleHidden;
        vm.createSimpleRule = createSimpleRule;
        vm.createAdvancedRule = createAdvancedRule;
        vm.editRule = editRule;
        vm.incrementExpertModeClicks = incrementExpertModeClicks;
        vm.getRuleExecutionCount = getRuleExecutionCount;

        _init();

        // public functions

        function refreshRules(event) {
            unomiRulesService.getAll(100, 0, null).then(function(response) {
                vm.rules = _.filter(response.data.list, function (rule) {
                    var simpleRule = false;
                    if (rule.tags && _.contains(rule.tags, 'simpleRule')) {
                        simpleRule = true;
                    }
                    if (vm.expertMode) {
                        if (vm.showHiddenRules) {
                            return filterByEnabled(rule);
                        } else {
                            return !rule.hidden && filterByEnabled(rule);
                        }
                    } else {
                        return !rule.hidden && simpleRule && filterByEnabled(rule);
                    }
                });
                unomiRulesService.getStatistics().then(function (response) {
                    vm.ruleStatistics = response.data;
                });
            });
        }

        function toggleHidden(event) {
            vm.showHiddenRules = !vm.showHiddenRules;
            vm.refreshRules(event);
        }

        function createSimpleRule($event) {
            $location.path("/rule/simple/newSimpleRule");
        }

        function createAdvancedRule($event) {
            $location.path("/rule/advanced/newAdvancedRule");
        }

        function editRule(rule) {
            if (rule.tags && _.contains(rule.tags, 'simpleRule')) {
                $location.path('/rule/simple/' + rule.id);
            } else {
                $location.path('/rule/advanced/' + rule.id);
            }
        }

        function incrementExpertModeClicks() {
            if (!vm.expertMode) {
                vm.expertModeClicks++;
                if (vm.expertModeClicks > 5) {
                    vm.expertMode = true;
                }
            }
        }

        function getRuleExecutionCount(ruleId) {
            if (vm.ruleStatistics && vm.ruleStatistics[ruleId]) {
                return vm.ruleStatistics[ruleId].executionCount;
            } else {
                return 0;
            }
        }

        // private functions

        function _init() {
            vm.expertModeClicks = 0;
            vm.expertMode = false;
            vm.showHiddenRules = false;
            vm.rulesToDisplay = "all";

            vm.refreshRules();
        }

        function filterByEnabled(rule) {
            if (vm.rulesToDisplay === "all") {
                return true;
            } else if (vm.rulesToDisplay === "active") {
                return rule.enabled;
            } else if (vm.rulesToDisplay === "inactive") {
                return !rule.enabled;
            }
            return true;
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemRulesApp').service('rulesService', rulesService);

    rulesService.$inject = ['unomiRulesService', '$q'];

    function rulesService(unomiRulesService, $q) {
        return {
            createSimpleRule : createSimpleRule,
            createAdvancedRule : createAdvancedRule,
            loadOrCreateRule: loadOrCreateRule
        };

        function createSimpleRule() {
            var newRuleId = ManagersUtils.generateId();
            return {
                metadata: {
                    id: newRuleId,
                    scope: ManagersContext.dmfScope,
                    tags: ['simpleRule'],
                    enabled: true
                },
                condition: {
                    "parameterValues": {
                        "subConditions": [],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "actions": []
            };
        }

        function createAdvancedRule() {
            var newRuleId = ManagersUtils.generateId();
            return {
                metadata: {
                    id: newRuleId,
                    scope: ManagersContext.dmfScope,
                    enabled: true
                },
                condition: {
                    "parameterValues": {
                        "subConditions": [],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "actions": []
            };
        }

        function loadOrCreateRule(ruleId) {
            var rule = $q.defer();
            if (ruleId == "newSimpleRule") {
                rule.resolve(createSimpleRule())
            } else if (ruleId == "newAdvancedRule") {
                rule.resolve(createAdvancedRule())
            } else {
                unomiRulesService.getRule(ruleId).then(function (response) {
                    rule.resolve(response.data);
                });
            }
            return rule.promise;
        }

    }


})();