(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemHtmlTagVisualPicker', wemHtmlTagVisualPicker);

    function wemHtmlTagVisualPicker() {
        return {
            restrict: "E",
            scope: {
                selectedTagId: '=ngModel',
                origin: '@',
                tag: '@',
                attrs: '@',
                displayAttr: '@',
                disabled: '@',
                callback: '@'
            },
            templateUrl: ManagersContext.wemAppPath + "/components/pickers/wem-html-tag-visual-picker.directive.html",
            controller: WemHtmlTagVisualPickerController
        };
    }

    WemHtmlTagVisualPickerController.$inject = ['$scope', '$uibModal'];

    function WemHtmlTagVisualPickerController($scope, $uibModal) {
        $scope.showVisualPicker = function () {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/components/pickers/wem-html-tag-visual-picker-modal.directive.html',
                controller: 'WemHtmlTagVisualPickerModalController',
                size: 'full',
                windowClass: 'tagPickerModal',
                backdrop: false,
                resolve: {
                    options: function () {
                        return {
                            selectedTagId: $scope.selectedTagId,
                            src: ManagersContext.baseLive + $scope.origin + ".html?skipWem=true",
                            tag: $scope.tag,
                            attrs: $scope.attrs ? $scope.attrs.split('|') : ["name", "id"],
                            displayAttr: $scope.displayAttr,
                            disabled: $scope.disabled,
                            callback: $scope.callback
                        };
                    }
                }
            });

            modalInstance.result.then(function (selectedTagId) {
                $scope.selectedTagId = selectedTagId;
            });
        }
    }

    angular.module('wemApp.components').controller('WemHtmlTagVisualPickerModalController', WemHtmlTagVisualPickerModalController);

    WemHtmlTagVisualPickerModalController.$inject = ['$rootScope', '$scope', '$uibModalInstance', 'options'];

    function WemHtmlTagVisualPickerModalController($rootScope, $scope, $uibModalInstance, options) {
        $scope.options = options;

        $scope.ok = function () {
            $uibModalInstance.close($scope.options.selectedTagId);
            $('.modal-content').css('height', $('#newGoalForm').height());
        };

        $scope.iframeLoadedCallBack = function (element) {
            ["keypress","submit","focus","blur","tab"].forEach(function(eventName){
                frames['visualPickerFrame'].addEventListener(eventName, function(e){
                    e.preventDefault();
                    e.stopPropagation();

                    // if an input is somehow focused, unfocus it
                    var target = e.target;
                    setTimeout(function(){
                        if (target.blur) target.blur();
                    }, 50);
                }, true);
            });

            $('body', frames['visualPickerFrame'].document).bind("click", function (event) {
                var $targetElement = $(event.target);
                if ($scope.options.callback) {
                    if ($rootScope[$scope.options.callback]) {
                        $rootScope[$scope.options.callback](event, $scope);
                    }
                } else {
                    var found = $targetElement.closest($scope.options.tag);
                    if (found && found.length > 0) {
                        for (var i = 0; i < $scope.options.attrs.length; i++) {
                            var attr = $scope.options.attrs[i];
                            if (found.is("[" + attr + "]")) {
                                $scope.$apply(function () {
                                    $scope.options.selectedTagId = found.attr(attr);
                                });
                                break;
                            }
                        }
                    }
                }

                // disable links that trigger navigation on other pages, keep ajax links start with # and empty href links that can be used to display hided contents
                if(($targetElement.is("a") && $targetElement.attr("href") && $targetElement.attr("href").length > 0 && $targetElement.attr("href")[0] != "#")){
                    event.preventDefault();
                    event.stopPropagation();
                    return false;
                }
            });
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
            $('.modal-content').css('height', $('#newGoalForm').height());
        };
    }
})();