(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemAction', wemActionDirective);

    function wemActionDirective() {
        return {
            restrict: "E",
            scope: {
                ref: '=',
                parentList: '=',
                parentIndex: '=',
                types: '='
            },
            templateUrl: ManagersContext.wemAppPath + "/components/wem-action.directive.html",
            controller: WemActionDirectiveController
        };
    }

    WemActionDirectiveController.$inject = ['$scope'];

    function WemActionDirectiveController($scope) {

        $scope.updateTemplate = function() {
            if( Object.prototype.toString.call( $scope.ref ) === '[object Array]' ) {
                $scope.actionTemplateUrl = ManagersContext.wemAppPath + "/components/wem-actions-list.directive.html";
            } else if ($scope.ref && $scope.ref.type != "empty") {
                $scope.actionType = $scope.getActionType($scope.ref);
                if ($scope.actionType != null) {
                    $scope.actionTemplateUrl = ManagersContext.wemAppPath + "/actions/templates/" + $scope.actionType.id + ".html";
                }
            }
        };

        $scope.getActionType = function (action) {
            for (var i = 0; i < $scope.types.length; i++) {
                var actionType = $scope.types[i];
                if (actionType.id == action.type) {
                    return actionType;
                }
            }
            return null;
        };

        $scope.addAction = function() {
            var actionType = $scope.added.addedActionType;
            $scope.added.addedActionType = null;
            var newAction = {
                "type" : actionType.id,
                "parameterValues" : {}
            };
            for (var i=0; i < actionType.parameters.length; i++) {
                var parameter = actionType.parameters[i];
                if (parameter.multivalued) {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null && parameter.defaultValue.length > 0) {
                        newAction.parameterValues[parameter.id] = parameter.defaultValue.split(",");
                    } else {
                        newAction.parameterValues[parameter.id] = [];
                    }
                } else {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null && parameter.defaultValue.length > 0) {
                        newAction.parameterValues[parameter.id] = parameter.defaultValue;
                    } else {
                        if (parameter.type == "string") {
                            newAction.parameterValues[parameter.id] = "";
                        } else {
                            newAction.parameterValues[parameter.id] = null;
                        }
                    }
                }
            }
            $scope.ref.push(newAction);
        };

        $scope.onRemove = function (index) {
            console.log("wem-action.directive.js : onRemove("+ index +") called");
            $scope.parentList.splice(index, 1);
        };

        if (!$scope.ref) {
            console.log("wem-action.directive.js: no action passed, creating empty one.");
            $scope.ref = {};
            $scope.ref.type = "empty";
        }

        console.log("wem-action.directive.js: Initializing action controller for action " + $scope.ref + " of type="+$scope.ref.type + " parentIndex=" + $scope.parentIndex);

        $scope.added = {};
        $scope.updateTemplate();

        $scope.$watch("ref", function (newValue, oldValue) {
            console.log("wem-action.directive.js : action changed, refreshing...");
            $scope.updateTemplate();
        });

        $scope.$watch("types.length", function (newValue, oldValue) {
            console.log("wem-action.directive.js : action type list changed, refreshing...");
            $scope.updateTemplate();
        });
    }
})();