(function() {
    'use strict';

    angular.module('wemApp.components').directive("wemCondition", wemConditionDirective);

    wemConditionDirective.$inject = ['RecursionHelper'];

    function wemConditionDirective(RecursionHelper) {
        return {
            restrict: "E",
            scope: {
                countlabel:'@',
                countlabeli18n:'@',
                disableEdition: '=',
                disableBoolean: '=',
                displayCount: '=',
                form: '=',
                itemType: '@',
                labelNew: '=',
                level: '@',
                parentIndex: '=',
                parentName: '=',
                parentRef: '=',
                ref: '=',
                showCount: '=',
                types: '='
            },
            templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? "/components/wem-condition-builder.directive.html" : "/components/wem-condition-builder.directive.legacy.html"),
            compile: function (element) {
                // Use the compile function from the RecursionHelper,
                // And return the linking function(s) which it returns
                return RecursionHelper.compile(element);
            },
            controller: WemConditionController
        };
    }

    WemConditionController.$inject = ['$scope', '$rootScope', '$injector', 'unomiQueryService', 'i18nService', 'modalService', '_', 'autocompleteService'];

    function WemConditionController($scope, $rootScope, $injector, unomiQueryService, i18nService, modalService, _, autocompleteService) {
        $scope.autoCompletedProperties = [];
        autocompleteService.getAutocompletedProperties().then(function(response){
            $scope.autoCompletedProperties = response.data;
        });

        function isMultivalued (propertyName){
            var propertyIndex = _.findIndex($scope.autoCompletedProperties, function(value) {
                return value.itemId == propertyName.replace("properties.", "") && value.multivalued == true;
            });
            return propertyIndex > -1;
        }
        //Set autocomplete options
        $scope.autocompleteValue = "on";
        $scope.autoCompleteOptions = {
            minimumChars: 1,
            data: function (term) {
                var selectedProperty = $scope.ref.parameterValues.propertyName;
                var propertyType = "";
                if(!$scope.currentItemType){
                    if($scope.ref.type == "profilePropertyCondition"){
                        propertyType = "profile"
                    }
                    if($scope.ref.type == "sessionPropertyCondition"){
                        propertyType = "session"
                    }
                } else {
                    propertyType = $scope.currentItemType;
                }
                if(autocompleteService.isAutocomplete(selectedProperty,$scope.autoCompletedProperties)){
                    $scope.autocompleteValue = "off";
                    var multivalued = isMultivalued(selectedProperty);
                    term = term.toUpperCase();
                    //Candidate values are calculated by the autocomplete service
                    return autocompleteService.completePropertyValue(selectedProperty,term,propertyType, multivalued);
                }
            },
            selectedTextAttr : 'value',
            itemTemplate : "{{item.value}} - {{item.count}}"
        };

        var $uibModal = null;
        if ($injector.has('$uibModal')) {
            $uibModal = $injector.get('$uibModal');
        }
        var uibDatepickerConfig = null;
        if ($injector.has('uibDatepickerConfig')) {
            uibDatepickerConfig = $injector.get('uibDatepickerConfig');
        }

        if(angular.isUndefined($scope.disableEdition)) {
            $scope.disableEdition = false;
        }
        if (angular.isUndefined($scope.disableBoolean)) {
            $scope.disableBoolean = false;
        }

        $scope.getCountLabel =  function(){
            if(!$scope.countlabeli18n){
                if(!$scope.countlabel) {
                    return "";
                } else {
                    return $scope.countlabel;
                }
            } else {
                return i18nService.message($scope.countlabeli18n);
            }
        };

        if(!$scope.level) {
            $scope.level = 0;
        }

        if (angular.isUndefined($scope.displayCount)) {
            $scope.displayCount = true;
        }

        if (angular.isUndefined($scope.labelNew)) {
            $scope.labelNew = "condition.new";
        }
        $scope.newConditionText = i18nService.message($scope.labelNew);
        if($scope.form && $scope.form.scoringId){
            $scope.scoringId = $scope.form.scoringId.$modelValue;
        }

        if (uibDatepickerConfig) {
            // date config
            uibDatepickerConfig.startingDay = 1;
            uibDatepickerConfig.minMode = "day";
            uibDatepickerConfig.datepickerMode = "day";
            uibDatepickerConfig.showWeeks = false;
        }
        $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
        $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
        $scope.clearText = i18nService.message('wem.label.clear');
        $scope.datePickers = {};

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();
            angular.forEach($scope.datePickers, function(value, key) {
                $scope.datePickers[key] = false;
            });
            $scope.datePickers[selector] = true;
        };

        $scope.hasError = function(form, propertyName) {
            return ManagersUtils.hasErrors($scope.form, form, propertyName);
        };

        $scope.getHumanizeTime = function (value, unit) {
            return moment.duration(value, unit).humanize();
        };

        $scope.updateTemplate = function () {
            if ($scope.ref && $scope.ref.type) {
                $scope.conditionType = $scope.getConditionType($scope.ref);
                if ($scope.conditionType !== null) {
                    var conditionTypeId = $scope.conditionType.id;
                    var conditionTemplatesDir = (wemAppIsNewUI) ? ManagersContext.wemAppPath + '/conditions/templates/' : ManagersContext.wemAppPath + '/conditions/templates/legacy/';
                    var propertyTemplatesDir = conditionTemplatesDir + 'property/';

                    // special handling of property templates
                    if (conditionTypeId.indexOf('Property') !== -1) {
                        $scope.conditionLabelKey = 'condition.propertyCondition.' + conditionTypeId;
                        $scope.conditionTemplateUrl = propertyTemplatesDir + 'propertyCondition.html';
                        $scope.conditionSelectTemplate = conditionTypeId == 'eventPropertyCondition' ? propertyTemplatesDir + conditionTypeId + 'Select.html'
                            : propertyTemplatesDir + 'defaultPropertyConditionSelect.html';
                    } else {
                        $scope.conditionTemplateUrl = conditionTemplatesDir + conditionTypeId + '.html';
                    }
                }
            }
        };

        $scope.getConditionType = function (condition) {
            function resolveConditionTypeIn(list, condition) {
                if (list) {
                    for (var i = 0; i < list.length; i++) {
                        var conditionType = list[i];
                        if (conditionType.id == condition.type) {
                            return conditionType;
                        }
                    }
                }
            }

            if (condition.conditionType) {
                return condition.conditionType;
            }
            var lists = [$scope.types, $scope.conditionTypes, $scope.eventConditionTypes];
            for (var i = 0; i < lists.length; i++) {
                var resolved = resolveConditionTypeIn(lists[i], condition);
                if (resolved) {
                    return resolved;
                }
            }
            return null;
        };

        $scope.addCondition = function (conditionType, targetName, wrapIn) {
            if (wrapIn) {
                var copyCurrentRef = angular.copy($scope.ref);
                targetName = "subConditions";
                $scope.ref = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": wrapIn,
                        "subConditions": [copyCurrentRef]
                    }
                };
                if ($scope.parentRef) {
                    $scope.parentRef.parameterValues[$scope.parentName][$scope.parentIndex] = $scope.ref;
                }
            }


            var newCondition = {
                "type": conditionType.id,
                "parameterValues": {}
            };
            for (var i = 0; i < conditionType.parameters.length; i++) {
                var parameter = conditionType.parameters[i];
                if (parameter.multivalued) {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null && parameter.defaultValue.length > 0) {
                        newCondition.parameterValues[parameter.id] = parameter.defaultValue.split(",");
                    } else {
                        newCondition.parameterValues[parameter.id] = [];
                    }
                } else {
                    if (parameter.defaultValue !== undefined && parameter.defaultValue != null) {
                        if (parameter.type == "integer") {
                            newCondition.parameterValues[parameter.id] = parseInt(parameter.defaultValue);
                        } else {
                            newCondition.parameterValues[parameter.id] = parameter.defaultValue;
                        }
                    }
                }
            }

            if (targetName) {
                if (Object.prototype.toString.call($scope.ref.parameterValues[targetName]) === '[object Array]') {
                    $scope.ref.parameterValues[targetName].push(newCondition);
                } else {
                    $scope.ref.parameterValues[targetName] = newCondition;
                }
            } else {
                $scope.ref = newCondition;
            }
        };

        $scope.onRemove = function (parentName, index) {
            console.log("wem-condition.directive.js : onRemove(" + parentName + "," + index + ") called");
            var parent = $scope.ref.parameterValues[parentName];
            if (parent instanceof Array) {
                parent.splice(index, 1);
                $scope.ref.parameterValues[parentName] = parent;
                if ($scope.ref.type == 'booleanCondition') {
                    if (parent.length == 1) {
                        $scope.ref = $scope.ref.parameterValues.subConditions[0]; // unwrap boolean condition
                        if ($scope.parentRef) {
                            $scope.parentRef.parameterValues[$scope.parentName][$scope.parentIndex] = $scope.ref;
                        }
                    } else if (parent.length == 0) {
                        // delete empty boolean condition
                        if ($scope.parentRef) {
                            parent = $scope.parentRef.parameterValues[$scope.parentName];
                            if (parent instanceof Array) {
                                parent.splice($scope.parentIndex, 1);
                                $scope.parentRef.parameterValues[$scope.parentName] = parent;
                            } else {
                                $scope.parentRef.parameterValues[$scope.parentName] = null;
                            }
                        } else {
                            $scope.ref = {};
                        }
                    }
                }
            } else {
                $scope.ref.parameterValues[parentName] = null;
            }
        };

        $scope.onRemoveRoot = function () {
            console.log("wem-condition.directive.js : onRemoveRoot() called");
            $scope.ref = {};
        };

        $scope.onAddValue = function (values, newVal, $event) {
            // if we're dealing with dates, convert the given value to a date first
            if (values == $scope.ref.parameterValues.propertyValuesDate && typeof newVal === 'string') {
                console.log("Condition builder : Dealing with date !");
                newVal = new Date(newVal);
            }
            values.push(newVal);

            $event.preventDefault();
        };

        $scope.onRemoveValue = function (values, pos, $event) {
            values.splice(pos, 1);

            $event.preventDefault();
        };

        $scope.canWrapBoolCondition = function (operator) {
            return (!$scope.disableBoolean && $scope.ref.type != 'booleanCondition'
            && ($scope.parentRef == null || $scope.parentRef.type != 'booleanCondition' || $scope.parentRef.parameterValues.operator != operator));
        };

        $scope.openAddConditionModal = function (targetName, wrapIn) {
            if($scope.form && $scope.form.scoringType){
                $scope.scoringType = $scope.form.scoringType.$viewValue;
            }
            var filteredTypes = $scope.types;
            if ($scope.scoringType) {
                if ($scope.scoringType !== 'st:all') {
                    filteredTypes = _.filter(filteredTypes, function (conditionType) {
                        return $scope.scoringType.split(':')[1] == conditionType.profileTag;
                    });
                } else {
                    filteredTypes = _.filter(filteredTypes, function (conditionType) {
                        return _.contains($rootScope.allowedTags, conditionType.profileTag);
                    });
                }
            }
            var conditionTypes = _.filter(filteredTypes, function (conditionType) {
                return (conditionType.id != "booleanCondition" && conditionType.id != "notCondition" &&
                conditionType.id != "matchAllCondition");
            });
            var conditionTypeGroups = [];
            if (conditionTypes) {
                for (var i = 0; i < conditionTypes.length; i++) {
                    var profileTag = conditionTypes[i].profileTag;
                    if (!_.contains(conditionTypeGroups, profileTag)) {
                        conditionTypeGroups.push(profileTag);
                    }
                }
            }

            var modalData = {
                conditionTypes : _.filter(conditionTypes, function (value) {
                    return i18nService.message('condition.' + value.id + '.name').indexOf("???") == -1;
                }),
                conditionTypeGroups : conditionTypeGroups,
                title : $scope.newConditionText,
                templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? '/components/wem-condition-add-modal.directive.html' : '/components/wem-condition-add-modal.directive.legacy.html')
            };
            var okCallback = function (modalData) {
                if (modalData.conditionType) {
                    $scope.addCondition(modalData.conditionType, targetName, wrapIn);
                }
            };
            var cancelCallback = function (modalData) {};
            modalService.newCustomModal(
                modalData,
                null,
                okCallback,
                cancelCallback);
        };

        // init phase
        if (!$scope.ref) {
            console.log("wem-condition.directive.js: no condition passed, creating empty one.");
            $scope.ref = {};
        }

        console.log("wem-condition.directive.js: Initializing condition controller for condition " + $scope.ref + " of type=" + $scope.ref.type + " parentIndex=" + $scope.parentIndex);

        $scope.updateTemplate();
        $scope.$watch("ref", function (newValue, oldValue) {
            if (newValue != oldValue) {
                console.log("wem-condition.directive.js : condition changed, refreshing...");
                $scope.updateTemplate();
            }
        });

        if ($scope.itemType) {
            $scope.currentItemType = $scope.itemType;
        } else {
            $scope.currentItemType = $scope.$parent.currentItemType;
        }

        if (!$scope.showCount) {
            $scope.showCount = $scope.$parent.showCount;
        }

        $scope.validateCondition = function(condition) {
            if (!condition || !condition.type || !condition.parameterValues) {
                return false;
            }

            for (var parameter in condition.parameterValues) {
                if (condition.parameterValues.hasOwnProperty(parameter)) {
                    if (condition.parameterValues[parameter] == undefined) {
                        return false;
                    } else if (condition.parameterValues[parameter].constructor == Array
                        && condition.parameterValues[parameter][0] == undefined) { // multiple parameters are initialized
                        return false;
                    }
                }
            }

            if (condition.type == 'booleanCondition') {
                if (condition.parameterValues.subConditions.length == 0) {
                    return false;
                }
                for (var c = 0; c < condition.parameterValues.subConditions.length; c++) {
                    var subCondition = condition.parameterValues.subConditions[c];
                    var result = $scope.validateCondition(subCondition);
                    if (!result) {
                        return false;
                    }
                }
            }

            if (condition.type == 'pastEventCondition') {
                var result = $scope.validateCondition(condition.parameterValues.eventCondition);
                if (!result) {
                    return false;
                }
            }

            if (condition.type == 'campaignEngagedCondition') {
                if (!condition.parameterValues.campaignId) {
                    return false
                }
            }

            if (condition.parameterValues.comparisonOperator == 'between') {
                if (condition.parameterValues.propertyValuesDate) {
                    if (!condition.parameterValues.propertyValuesDate[1]) {
                        return false;
                    } else {
                        condition.parameterValues.propertyValuesDate[1] = moment(condition.parameterValues.propertyValuesDate[1]).endOf('day').toDate();
                    }
                }
            }

            return true;
        };

        if ($scope.displayCount && $scope.currentItemType && $scope.showCount!="false") {
            $scope.$watch("ref.parameterValues", function (newValue, oldValue) {
                // Wait for other watchers to have been executed before running query on server
                window.setTimeout(function() {
                    if ($scope.validateCondition($scope.ref)) {

                        var condition = $scope.ref;
                        if($scope.ref.parameterValues.propertyValues){
                            $scope.ref.parameterValues.propertyValues = _.map($scope.ref.parameterValues.propertyValues, function (value) {
                                if(value == undefined || value == null){
                                    return "";
                                }else {
                                    return value;
                                }
                            });
                        }
                        if ($scope.currentItemType == 'profile') {
                            var subCondition = $scope.ref;
                            if ($scope.ref.parameterValues.comparisonOperator === 'notContains') {
                                subCondition = {
                                    "parameterValues": {
                                        "subCondition":
                                            {
                                                type: "profilePropertyCondition",
                                                parameterValues: {
                                                    propertyName: $scope.ref.parameterValues.propertyName,
                                                    propertyValue: $scope.ref.parameterValues.propertyValue,
                                                    comparisonOperator: "contains"
                                                }
                                            }
                                    },
                                    "type": "notCondition"
                                };
                            }

                            condition = {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            type: "profilePropertyCondition",
                                            parameterValues: {
                                                propertyName: "mergedWith",
                                                comparisonOperator: "missing"
                                            }
                                        },
                                        {
                                            type: "profilePropertyCondition",
                                            parameterValues: {
                                                propertyName: "systemProperties.isAnonymousProfile",
                                                comparisonOperator: "missing"
                                            }
                                        },
                                        subCondition
                                    ],
                                    "operator": "and"
                                },
                                "type": "booleanCondition"
                            };

                        }

                        unomiQueryService.getMatchingItemsCountForCondition($scope.currentItemType, condition).then(function (response) {
                            $scope.matching = response.data;
                            if($scope.countlabel || $scope.countlabeli18n){
                                $scope.matching = $scope.getCountLabel() + " " + $scope.matching;
                            } else {
                                $scope.countClass="subcount";
                            }
                        });
                    } else {
                        $scope.matching = 0;
                        if($scope.countlabel || $scope.countlabeli18n){
                            $scope.matching = $scope.getCountLabel() + " " + $scope.matching;
                        } else {
                            $scope.countClass="subcount";
                        }
                    }
                });
            }, true);
        }

        //Handle date conditions
        if ($scope.ref.parameterValues) {
            if ($scope.ref.parameterValues.propertyValuesDate) {
                $scope.ref.parameterValues.propertyValuesDate[0] = moment($scope.ref.parameterValues.propertyValuesDate[0]).toDate();
                $scope.ref.parameterValues.propertyValuesDate[1] = moment($scope.ref.parameterValues.propertyValuesDate[1]).endOf('day').toDate();
            } else if ($scope.ref.parameterValues.propertyValueDate) {
                $scope.ref.parameterValues.propertyValueDate = moment($scope.ref.parameterValues.propertyValueDate).toDate();
            }
        }

        $scope.$watch("types.length", function (newValue, oldValue) {
            console.log("wem-condition.directive.js : condition type list changed, refreshing...");
            $scope.updateTemplate();
        });
        $scope.alternate = !$scope.$parent.alternate;
        $scope.conditionTypes = $scope.$parent.conditionTypes;
        $scope.eventConditionTypes = $scope.$parent.eventConditionTypes;
    }
})();