(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemTimeline', wemTimeline);

    function wemTimeline() {
        return {
            restrict: 'E',
            templateUrl: ManagersContext.wemAppPath + '/components/wem-timeline.directive.html',
            controller: WemTimelineController,
            controllerAs: 'wemTimeLineCtrl',
            scope: {},
            bindToController: {
                data: '=ngModel',
                colours: '=',
                report: '=',
                preset: '@'
            },
            link: linkFunc
        };

        function linkFunc(scope, el, attrs, ctrl) {
            ctrl._init();
        }
    }

    WemTimelineController.$inject = ['i18nService'];

    function WemTimelineController(i18nService) {
        var vm = this;
        vm.currentText = i18nService.message('serverSettings.manageWem.datepicker.today');
        vm.closeText = i18nService.message('serverSettings.manageWem.datepicker.close');
        vm.clearText = i18nService.message('wem.label.clear');

        vm.toggleDatePicker = toggleDatePicker;
        vm.openDatePopup = openDatePopup;
        vm.updateFromDate = updateFromDate;
        vm.updateToDate = updateToDate;
        vm.updateInterval = updateInterval;
        vm.updatePresetDate = updatePresetDate;
        vm._init = _init;

        function openDatePopup($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();

            if (selector == 'start') {
                vm.startOpened = true;
                vm.endOpened = false;
            } else {
                vm.endOpened = true;
                vm.startOpened = false;
            }
        }

        function updateFromDate() {
            if (vm.data.interval.key == '1M') {
                var fromDateMoment = moment(vm.data.fromDate);
                vm.data.fromDate = fromDateMoment.startOf('month').toDate();
            }
            _validateDate('fromDate');
            _updateDatePickerOptions();
            _queryData();
        }

        function updateToDate() {
            if (vm.data.interval.key == '1M') {
                var toDateMoment = moment(vm.data.toDate);
                vm.data.toDate = toDateMoment.endOf('month').toDate();
            }
            _validateDate('toDate');
            _updateDatePickerOptions();
            _queryData();
        }

        // convert dates when interval change
        function updateInterval() {
            var fromDateMoment = moment(vm.data.fromDate);
            var toDateMoment = moment(vm.data.toDate);
            if (vm.data.interval.key == '1M') {
                fromDateMoment.startOf('month');
                toDateMoment.endOf('month');
            }
            vm.data.fromDate = fromDateMoment.toDate();
            vm.data.toDate = toDateMoment.toDate();

            _validateDate('toDate');
            _updateDatePickerOptions();
            _queryData();
        }

        function updatePresetDate(selection) {
            var toDateMoment = moment();
            var fromDateMoment;
            var intervalUnit;
            vm.selectedPreset = selection;
            switch (selection) {
                case '1week':
                    intervalUnit = 'day';
                    fromDateMoment = moment().subtract(7, 'days');
                    break;
                case '1month':
                    intervalUnit = 'day';
                    fromDateMoment = moment().subtract(1, 'months');
                    break;
                case '3month':
                    intervalUnit = 'day';
                    fromDateMoment = moment().subtract(3, 'months');
                    break;
                case '6month':
                    intervalUnit = 'month';
                    toDateMoment.endOf('month');
                    fromDateMoment = moment().subtract(6, 'months').startOf('month');
                    break;
                case '1year':
                    intervalUnit = 'month';
                    toDateMoment.endOf('month');
                    fromDateMoment = moment().subtract(1, 'years').startOf('month');
                    break;
            }

            if (vm.data.interval.unit != intervalUnit) {
                for (var interval in vm.intervals) {
                    if (vm.intervals[interval].unit == intervalUnit) {
                        vm.data.interval = vm.intervals[interval];
                    }
                }
            }
            vm.data.fromDate = fromDateMoment.toDate();
            vm.data.toDate = toDateMoment.toDate();

            _updateDatePickerOptions();
            _queryData();
        }

        function toggleDatePicker() {
            vm.showDatePicker = !vm.showDatePicker;
            if (vm.showDatePicker) {
                vm.selectedPreset = 'custom';
            }
        }

        // Private functions under this line
        function _init() {
            // compute interval
            vm.data.interval = vm.data.interval || ManagersUtils.supportedTimelineIntervales['1M'];
            vm.intervals = ManagersUtils.supportedTimelineIntervales;

            if (vm.preset) {
                vm.showDatePicker = false;
                updatePresetDate(vm.preset);
            } else {
                vm.showDatePicker = true;
                vm.selectedPreset = 'custom';

                _updateDatePickerOptions();
                _queryData();
            }
        }

        function _updateDatePickerOptions() {
            vm.firstPickerOptions = {
                datepickerMode: vm.data.interval.unit,
                // maxDate: toDateMoment.toDate(),
                maxMode: vm.data.interval.unit,
                minMode: vm.data.interval.unit
            };

            vm.secondPickerOptions = {
                datepickerMode: vm.data.interval.unit,
                maxMode: vm.data.interval.unit,
                // minDate: fromDateMoment.toDate(),
                minMode: vm.data.interval.unit
            };
        }

        function _queryData() {
            vm.data.loadData(function(datasets) {
                vm.datasets = datasets;
                _redrawGraph();
            });
        }

        function _validateDate(selected) {
            var fromDateMoment = moment(vm.data.fromDate);
            var toDateMoment = moment(vm.data.toDate);
            if (selected == 'fromDate') {
                var minToDate = moment(fromDateMoment);
                minToDate.add(vm.data.interval.minpoints, vm.data.interval.units);
                if (toDateMoment.isBefore(minToDate)) {
                    if (vm.data.interval.key == '1M') {
                        minToDate.endOf('month');
                    }
                    vm.data.toDate = minToDate.toDate();
                }
            }
            if (selected == 'toDate') {
                var minFromDate = moment(toDateMoment);
                minFromDate.subtract(vm.data.interval.minpoints, vm.data.interval.units);
                if (fromDateMoment.isAfter(minFromDate)) {
                    if (vm.data.interval.key == '1M') {
                        minFromDate.startOf('month');
                    }
                    vm.data.fromDate = minFromDate.toDate();
                }
            }
        }

        function _redrawGraph() {
            var filtered = {};
            var fromDateMoment = moment(vm.data.fromDate);
            var toDateMoment = moment(vm.data.toDate);
            var dates = [];

            // Fill empty points with 0
            if (fromDateMoment.isBefore(toDateMoment)) {
                var count = 0;
                var dif = toDateMoment.diff(fromDateMoment,  vm.data.interval.units);
                dates.push(fromDateMoment.format('YYYY-MM-DD'));
                while (count < dif) {
                    dates.push(fromDateMoment.add(vm.data.interval.unitcount,  vm.data.interval.units).format('YYYY-MM-DD'));
                    count++;
                }
            }

            filtered = [];
            angular.forEach(vm.datasets, function(d,i) {
                filtered[i] = [];
            });

            angular.forEach(dates, function (v, k) {
                angular.forEach(vm.datasets, function(d,i) {
                    filtered[i].push(vm.datasets[i][v] ? vm.datasets[i][v] : 0);
                })
            });
            var labels = _.map(dates, function(f) { return moment(f).format( vm.data.interval.scaleFormat) });
            vm.lineChartData = {
                labels: labels,
                datasets: filtered,
                series: vm.data.series,
                colours: vm.colours ? vm.colours : [
                        {
                            backgroundColor: 'rgba(220,220,220,0.2)',
                            borderColor: 'rgba(220,220,220,1)',
                            pointBackgroundColor: 'rgba(220,220,220,1)',
                            pointBorderColor: '#fff',
                            pointHoverBackgroundColor: '#fff',
                            pointHoverBorderColor: 'rgba(220,220,220,1)'
                        },
                        {
                            backgroundColor: 'rgba(0, 188, 212, 0.2)',
                            borderColor: '#00A0E3',
                            pointBackgroundColor: '#00A0E3',
                            pointBorderColor: '#fff',
                            pointHoverBackgroundColor: '#fff',
                            pointHoverBorderColor: '#00A0E3'
                        }
                    ],
                options: angular.extend({
                    scales: {
                        yAxes: [{
                            ticks: {
                                beginAtZero:true
                            }
                        }]
                    },
                    legend: {
                        display: true,
                        position: 'bottom',
                        labels: {
                            usePointStyle: true
                        }
                    },
                    maintainAspectRatio: false
                }, vm.data.chartOptions)
            };
        }
    }
})();
