var ManagersUtils = ManagersUtils || {};

ManagersUtils.cxsContext = '/cxs/';

ManagersUtils.formMappingActionStrategies = ['alwaysSet', 'setIfMissing'];

ManagersUtils.getQueryOrDefault = function (query) {
    var scopedAndSharedQuery = {
        "condition": {
            "parameterValues": {
                "subConditions": [
                    {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "metadata.scope",
                            "comparisonOperator": "equals",
                            "propertyValue": ManagersContext.dmfScope
                        }
                    },
                    {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "metadata.scope",
                            "comparisonOperator": "equals",
                            "propertyValue": "systemscope"
                        }
                    }
                ],
                "operator": "or"
            },
            "type": "booleanCondition"
        },
        "limit": 50,
        "offset": 0,
        "sortby": null,
        "forceRefresh": false
    };

    return query || scopedAndSharedQuery;
};

ManagersUtils.humanReadableMissingIfNeeded = function (value, i18nService) {
    if (value == "_missing") {
        // replace _missing by something more user friendly
        value = i18nService.message("serverSettings.internalSearches.missing");
    }

    return value;
};

ManagersUtils.AggregateDisplay = function (key, occurrences, i18nService) {
    var AggregateDetail = function (value, occurrences) {
        this.item = value;
        this.occurrences = occurrences;

        this.importance = function (totalOccurrences) {
            return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
        }
    };

    this.key = key;
    this.occurrences = occurrences;
    this.details = {};

    this.importance = function (totalOccurrences) {
        return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
    };

    this.setDetails = function (type, details) {
        if (details) {
            var detailsArray = [];
            var totalOccurrences = 0;
            for (var detail in details) {
                if (ManagersUtils.acceptProperty(details, detail)) {
                    occurrences = details[detail];

                    detail = ManagersUtils.humanReadableMissingIfNeeded(detail, i18nService);

                    totalOccurrences += occurrences;
                    detailsArray.push(new AggregateDetail(detail, occurrences));
                }
            }
            this.occurrences = totalOccurrences;

            this.details[type] = detailsArray;

            return detailsArray;
        }
    };
};

ManagersUtils.booleanAnd = function (firstCondition, secondCondition) {
    return {
        "type": "booleanCondition",
        "parameterValues": {
            "operator": "and",
            "subConditions": [
                firstCondition,
                secondCondition
            ]
        }
    }
};

ManagersUtils.booleanAndArray = function (subConditions) {
    return {
        "type": "booleanCondition",
        "parameterValues": {
            "operator": "and",
            "subConditions": subConditions
        }
    }
};

ManagersUtils.getDateRangeConditionBody = function (from, to) {
    return ManagersUtils.booleanAnd(
        {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": "scope",
                "comparisonOperator": "equals",
                "propertyValue": ManagersContext.dmfScope
            }
        },
        {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": "timeStamp",
                "comparisonOperator": "between",
                "propertyValues": [from, to]
            }
        });
};

ManagersUtils.parseErrorsInForm = function (form, parsedErrors, subFormName) {
    if (!parsedErrors) {
        parsedErrors = {};
    }

    angular.forEach(form.$error, function (value, key) {

        if (value.length) {
            angular.forEach(value, function (subForm) {
                ManagersUtils.parseErrorsInForm(subForm, parsedErrors, form.$name);
            });
        } else if (value && form.$name) {
            parsedErrors["wem_validation." + (subFormName.indexOf('_') != -1 ? subFormName.substr(0, subFormName.indexOf('_')) : subFormName) + "." + form.$name + "." + key] = true;
        }
    });

    return parsedErrors;
};

ManagersUtils.displayErrors = function (errorKeys, i18nService, notificationService) {
    angular.forEach(errorKeys, function (value, key) {
        notificationService.errorToast(i18nService.message(key));
    });
};

ManagersUtils.hasErrors = function (rootForm, form, propertyName) {
    if (rootForm) {
        if (!rootForm.showErrors) {
            return false;
        }

        if (form[propertyName]) {
            return form[propertyName].$invalid;
        }
    }
    return false;
};

ManagersUtils.goalTags = ["downloadGoal", "formGoal", "funnelGoal", "landingPageGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.campaingTags = ["downloadGoal", "formGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.optimizationTestGoalTags = ["downloadGoal", "formGoal", "pageVisitGoal", "videoGoal"];

ManagersUtils.buildTags = function (tags, i18nService) {
    var result = {};
    tags.forEach(function (tag) {
        result[tag] = {
            id: tag,
            name: i18nService.message("serverSettings.manageGoals.tag." + tag),
            description: i18nService.message("serverSettings.manageGoals.tag." + tag + ".description"),
            goals: []
        }
    });

    return result;
};

ManagersUtils.supportedTimelineIntervales = [
    {
        "key": "1M",
        "i18n": "serverSettings.manageGoals.interval.1M",
        "unitcount": 1,
        "units": "months",
        "unit": "month",
        minpoints: 1,
        maxpoints: 36,
        scaleFormat: "YYYY-MM",
        pickerFormat: "MMMM yyyy",
        aggregateFormat: "MMMM YYYY"
    },
    {
        "key": "1d",
        "i18n": "serverSettings.manageGoals.interval.1d",
        "unitcount": 1,
        "units": "days",
        "unit": "day",
        minpoints: 1,
        maxpoints: 1080,
        scaleFormat: "L",
        pickerFormat: "mediumDate",
        aggregateFormat: "LL"
    }
];

ManagersUtils.supportedTimelineIntervalesObj = {
    "1M": ManagersUtils.supportedTimelineIntervales[0],
    "1d": ManagersUtils.supportedTimelineIntervales[1]
};

ManagersUtils.generateId = function () {
    // Math.random should be unique because of its seeding algorithm.
    // Convert it to base 36 (numbers + letters), and grab the first 9 characters
    // after the decimal.
    return '_' + Math.random().toString(36).substr(2, 9);
};
/**
 * Utility method to read nested objects in a json obj without having to test all the object in the path
 * example before to access goal.metadata.type you have to check if goal and metadata are not undefined before trying access type, if not you will have
 * "cannot read property type of undefined"
 *
 * Now you can use ManagersUtils.getProperty(goal, "metadata.type") safely, can return undefined in case of undefined are found in the given path
 * @param obj
 * @param propPath
 * @returns {*}
 */
ManagersUtils.getProperty = function (obj, propPath) {
    props = propPath.split('.');
    if (props.length == 1) {
        return obj[props[0]]
    } else if (props.length > 1) {
        var prop = props.shift();
        var resolvedObj = obj[prop];
        if (resolvedObj) {
            return this.getProperty(resolvedObj, props.join("."))
        }
    }
    return undefined;
};

ManagersUtils.acceptProperty = function (data, prop) {
    return (prop && data.hasOwnProperty(prop) && prop != "_all" && prop != "_filtered");
};

ManagersUtils.getCurrentModuleResource = function (resourcePath) {
    if (ManagersContext.currentModuleURL) {
        return ManagersContext.currentModuleURL + resourcePath;
    } else {
        return "/modules/marketing-factory-core" + resourcePath;
    }
};

ManagersUtils.getPercentage = function (value) {
    return +((value * 100).toFixed(2)); // convert formatted string back to a number
};

ManagersUtils.downloadForm = function (url, data, method) {
    //url and data options required
    if (url && data) {
        //split params into form inputs
        var $query = jQuery('<input type="hidden" name="query"/>').val(data);
        //send request
        jQuery('<form action="' + url + '" method="' + (method || 'post') + '"></form>').append($query)
            .appendTo('body').submit().remove();
    }
};

ManagersUtils.maxInterestsInChart = 16;

ManagersUtils.getProfileDisplayName = function (i18nService, properties) {
    if (properties.firstName && properties.lastName) {
        return properties.firstName + ' ' + properties.lastName;
    } else if (properties.firstName) {
        return properties.firstName;
    } else if (properties.lastName) {
        return properties.lastName;
    } else {
        return i18nService.message("serverSettings.manageProfiles.profile.unnamed");
    }
};

ManagersUtils.campaignPropertiesForColName = {
    "id": "metadata.id",
    "name": "metadata.name",
    "description": "metadata.description",
    "startDate": "startDate",
    "endDate": "endDate"
};

ManagersUtils.formMappingPropertiesForColName = {
    "id": "metadata.id",
    "name": "metadata.name",
    "description": "metadata.description",
    "action": "action"
};

ManagersUtils.personaMap = {
    center: {
        latitude: 45,
        longitude: -73
    },
    zoom: 8,
    control: {},
    options:{
        draggable:true,
        styles:
            [
                {
                    "featureType": "administrative",
                    "elementType": "labels.text.fill",
                    "stylers": [
                        {
                            "color": "#444444"
                        }
                    ]
                },
                {
                    "featureType": "landscape",
                    "elementType": "all",
                    "stylers": [
                        {
                            "color": "#f2f2f2"
                        }
                    ]
                },
                {
                    "featureType": "poi",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "poi.park",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "road",
                    "elementType": "all",
                    "stylers": [
                        {
                            "saturation": -100
                        },
                        {
                            "lightness": 45
                        }
                    ]
                },
                {
                    "featureType": "road.highway",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "simplified"
                        }
                    ]
                },
                {
                    "featureType": "road.arterial",
                    "elementType": "labels.icon",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "transit",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "off"
                        }
                    ]
                },
                {
                    "featureType": "transit.line",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.airport",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.airport",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.bus",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.bus",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.rail",
                    "elementType": "all",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "transit.station.rail",
                    "elementType": "geometry",
                    "stylers": [
                        {
                            "visibility": "on"
                        }
                    ]
                },
                {
                    "featureType": "water",
                    "elementType": "all",
                    "stylers": [
                        {
                            "color": "#6e769e"
                        },
                        {
                            "visibility": "on"
                        }
                    ]
                }
            ]
    }
};


ManagersUtils.dateFormats = {
    '/y':'YYYY',
    '/M':'MMMM YYYY',
    '/d':'LL',
    '/h':'LL',
    '/m':'LL H:00',
    '/s':'LLL'
};

ManagersUtils.parseDateMath = function(f) {
    var result = /now(([+-]\d+[yMwdhms])+)(\/[yMwdhms])?/.exec(f);
    if (result) {
        var offsetRegexp = /([+-])(\d+)([yMwdhms])/g;
        f = moment();
        var offset;
        while (offset = offsetRegexp.exec(result[1])) {
            if (offset[1] == '+') {
                f = f.add(offset[2], offset[3])
            } else {
                f = f.subtract(offset[2], offset[3])
            }
        }

        var format = 'LLL';
        if (result[3]) {
            format = ManagersUtils.dateFormats[result[3]];
            moment().millisecond(0);
            var reset;
            if (reset = (result[3] == '/y')) {
                moment().month(0);
            }
            if (reset |= (result[3] == '/M')) {
                moment().date(0);
            }
            if (reset |= (result[3] == '/d')) {
                moment().hour(0);
            }
            if (reset |= (result[3] == '/h')) {
                moment().minute(0);
            }
            if (reset |= (result[3] == '/m')) {
                moment().second(0);
            }
            if (reset || (result[3] == '/s')) {
                moment().millisecond(0);
            }
        }
        f = f.format(format);
    }
    return f;
};
