(function () {
    'use strict';

    angular.module('wemApp.services').service('modalService', wemAppIsNewUI ? mdModalService : uibModalService);

    mdModalService.$inject = ['$mdDialog'];

    function mdModalService($mdDialog) {
        return {
            newAlertModal: newAlertModal,
            newInfoModal: newInfoModal,
            newCustomModal: newCustomModal
        };

        function newAlertModal(modalData, okCallback, cancelCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-alert.html',
                controller: 'ModalController',
                locals: {
                    modalData: modalData
                }
            });
            var modalInstance = $mdDialog.show(modalData);

            modalInstance.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function (modalData) {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }

        function newInfoModal(modalData, okCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-info.html',
                controller: 'ModalController',
                locals: {
                    modalData: modalData
                }
            });
            var modalInstance = $mdDialog.show(modalData);

            modalInstance.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            });
        }

        function newCustomModal(modalData, event, okCallback, cancelCallback) {
            angular.extend(modalData, {
                controller: 'ModalController',
                controllerAs: 'modalCtrl',
                parent: angular.element('section[wem-main-section]'),
                targetEvent: event,
                locals: {
                    modalData: modalData
                },
                clickOutsideToClose: true
            });
            var modalInstance = $mdDialog.show(modalData);

            modalInstance.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function () {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }
    }

    uibModalService.$inject = ['$uibModal'];

    function uibModalService($uibModal) {
        return {
            newAlertModal: newAlertModal,
            newInfoModal: newInfoModal,
            newCustomModal: newCustomModal
        };

        function newAlertModal(modalData, okCallback, cancelCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-alert.legacy.html',
                controller: 'ModalController',
                size: 'sm',
                resolve: {
                    "modalData": function () {
                        return modalData;
                    }
                }
            });
            var modalInstance = $uibModal.open(modalData);

            modalInstance.result.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function (modalData) {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }

        function newInfoModal(modalData, okCallback) {
            angular.extend(modalData, {
                templateUrl: ManagersContext.wemAppPath + '/services/modal-info.legacy.html',
                controller: 'ModalController',
                resolve: {
                    "modalData": function () {
                        return modalData;
                    }
                }
            });
            var modalInstance = $uibModal.open(modalData);

            modalInstance.result.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            });
        }

        function newCustomModal(modalData, event, okCallback, cancelCallback) {
            angular.extend(modalData, {
                controller: 'ModalController',
                controllerAs: 'modalCtrl',
                resolve: {
                    "modalData": function () {
                        return modalData;
                    }
                }
            });
            var modalInstance = $uibModal.open(modalData);

            modalInstance.result.then(function (modalData) {
                if (okCallback) {
                    okCallback(modalData);
                }
            }, function (modalData) {
                if (cancelCallback) {
                    cancelCallback(modalData);
                }
            });
        }
    }

    angular.module('wemApp.services').controller('ModalController', wemAppIsNewUI ? mdModalController : uibModalController);

    mdModalController.$inject = ['$scope', '$mdDialog', 'i18nService', 'modalData'];

    function mdModalController($scope, $mdDialog, i18nService, modalData) {
        $scope.modal = modalData;
        if ($scope.modal.init) {
            $scope.modal.init($scope, i18nService, modalData);
        }
        if (!$scope.modal.okMsg) {
            $scope.modal.okMsg = i18nService.message("wem.label.ok");
        }
        if (!$scope.modal.cancelMsg) {
            $scope.modal.cancelMsg = i18nService.message("wem.label.cancel");
        }

        $scope.ok = function () {
            $mdDialog.hide($scope.modal);
        };

        $scope.cancel = function () {
            $mdDialog.cancel($scope.modal);
        }
    }

    uibModalController.$inject = ['$scope', '$uibModalInstance', 'i18nService', 'modalData'];

    function uibModalController($scope, $uibModalInstance, i18nService, modalData) {
        $scope.modal = modalData;
        if ($scope.modal.init) {
            $scope.modal.init($scope, i18nService, modalData);
        }
        if (!$scope.modal.okMsg) {
            $scope.modal.okMsg = i18nService.message("wem.label.ok");
        }
        if (!$scope.modal.cancelMsg) {
            $scope.modal.cancelMsg = i18nService.message("wem.label.cancel");
        }

        $scope.ok = function () {
            $uibModalInstance.close($scope.modal);
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
        }
    }

})();