(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiCampaignService', unomiCampaignService);

    unomiCampaignService.$inject = ['$http', 'i18nService'];

    function unomiCampaignService($http, i18nService) {
        var _campaignsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'campaigns/';
        var _deletedCampaign;
        var _deletedEvent;
        var _newEvent;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            queryDetails: queryDetails,
            setCampaign: setCampaign,
            setCampaignEvent: setCampaignEvent,
            getCampaign: getCampaign,
            getCampaignDetail: getCampaignDetail,
            getLocalizedCampaignType: getLocalizedCampaignType,
            getLocalizedCampaignParam: getLocalizedCampaignParam,
            getTypeSpecificCampaignParameter: getTypeSpecificCampaignParameter,
            getCampaignEvents: getCampaignEvents,
            deleteCampaign: deleteCampaign,
            deleteCampaignEvent: deleteCampaignEvent,
            getCampaignType: getCampaignType,
            createCampaignEntryCondition: createCampaignEntryCondition
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (_deletedCampaign) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedCampaign
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedCampaign = undefined
            }

            return query;
        }

        function getAll() {
            return $http.post(_campaignsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function queryDetails(query) {
            return $http.post(_campaignsContext + "query/detailed", query);
        }

        function setCampaign(campaign) {
            return $http.post(_campaignsContext, campaign);
        }

        function setCampaignEvent(campaignEvent) {
            _newEvent = campaignEvent;
            return $http.post(_campaignsContext + "event", campaignEvent);
        }

        function getCampaign(id) {
            return $http.get(_campaignsContext + id);
        }

        function getCampaignDetail(id) {
            return $http.get(_campaignsContext + id + "/detailed");
        }

        function getLocalizedCampaignType(campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType);
        }

        function getLocalizedCampaignParam(campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType + '.paramName');
        }

        function getTypeSpecificCampaignParameter(campaign) {
            var campaignType = getCampaignType(campaign);
            switch (campaignType) {
                case 'landing':
                    return campaign.entryCondition.parameterValues.subConditions[1].parameterValues.pagePath;
                case 'referrer':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
                case 'parameter':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
            }
        }

        function getCampaignEvents(campaignId, length, start, order, additionalCondition) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaignId
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (additionalCondition) {
                query.condition.parameterValues.subConditions.push(additionalCondition);
            }

            if (_newEvent) {
                query.forceRefresh = true;
                _newEvent = undefined;
            }

            if (_deletedEvent) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedEvent
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedEvent = undefined;
            }
            return $http.post(_campaignsContext + "events/query", query);
        }

        function deleteCampaign(id) {
            _deletedCampaign = id;
            return $http.delete(_campaignsContext + id);
        }

        function deleteCampaignEvent(eventId) {
            _deletedEvent = eventId;
            return $http.delete(_campaignsContext + "event/" + eventId);
        }

        function getCampaignType(campaign) {
            return campaign.metadata.tags[0];
        }

        function createCampaignEntryCondition(type) {
            var entryCondition = {
                "parameterValues": {
                    "subConditions": [],
                    "operator": "and"
                },
                "type": "booleanCondition"
            };
            switch (type.value) {
                case 'landing':
                    entryCondition.parameterValues.subConditions.push({
                        "parameterValues": {
                            "propertyName": "duration",
                            "propertyValueInteger": 0,
                            "comparisonOperator": "equals"
                        },
                        "type": "sessionPropertyCondition"
                    });
                    entryCondition.parameterValues.subConditions.push({
                            "parameterValues": {
                                "pagePath": type.param
                            },
                            "type": "pageViewEventCondition"
                        }
                    );
                    break;
                case 'referrer':
                    entryCondition.parameterValues.subConditions.push({
                        "parameterValues": {
                            "propertyName": "duration",
                            "propertyValueInteger": 0,
                            "comparisonOperator": "equals"
                        },
                        "type": "sessionPropertyCondition"
                    });
                    entryCondition.parameterValues.subConditions.push({
                            "type": "pageViewEventCondition"
                        }
                    );
                    entryCondition.parameterValues.subConditions.push({
                            "parameterValues": {
                                "propertyName": "target.properties.pageInfo.referringURL",
                                "propertyValue": _escapeRegex(type.param),
                                "comparisonOperator": "matchesRegex"
                            },
                            "type": "eventPropertyCondition"

                        }
                    );
                    break;
                case 'parameter':
                    entryCondition.parameterValues.subConditions.push({
                        "parameterValues": {
                            "propertyName": "duration",
                            "propertyValueInteger": 0,
                            "comparisonOperator": "equals"
                        },
                        "type": "sessionPropertyCondition"
                    });
                    entryCondition.parameterValues.subConditions.push({
                            "type": "pageViewEventCondition"
                        }
                    );
                    entryCondition.parameterValues.subConditions.push({
                            "parameterValues": {
                                "propertyName": "target.properties.pageInfo.destinationURL",
                                "propertyValue": _escapeRegex(type.param),
                                "comparisonOperator": "matchesRegex"
                            },
                            "type": "eventPropertyCondition"

                        }
                    );
                    break;
            }
            return entryCondition;
        }

        // Private function under this line
        function _escapeRegex(regex) {
            var result = regex.replace(/[\\\.\[\]\{\}\(\)\+\-\?\^\$\|]/g, "\\$&").replace(/\*/g, ".*");
            if (result.substr(0, 2) != ".*") {
                result = ".*" + result;
            }
            if (result.substr(result.length - 2) != ".*") {
                result += ".*";
            }
            return result;
        }
    }
})();