(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiListService', unomiListService);

    unomiListService.$inject = ['$http'];

    function unomiListService($http) {
        var _listsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'lists/';
        var _newItem;
        var _deletedItem;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            query: query,
            setList: setList,
            getList: getList,
            deleteList: deleteList
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedItem = undefined;
            }

            return query;
        }

        function getAll() {
            return $http.get(_listsContext);
        }

        function query(query) {
            return $http.post(_listsContext + "query", ManagersUtils.getQueryOrDefault(query));
        }

        function setList(list, shared) {
            if (shared) {
                list.metadata.scope = 'systemscope';
            }
            _newItem = list;
            return $http.post(_listsContext, list);
        }

        function getList(id) {
            return $http.get(_listsContext + id);
        }

        function deleteList(id) {
            _deletedItem = id;
            return $http.delete(_listsContext + id);
        }
    }
})();