(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiQueryService', unomiQueryService);

    unomiQueryService.$inject = ['$http', 'i18nService', 'notificationService'];

    function unomiQueryService($http, i18nService, notificationService) {
        var _queryContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'query/';

        return {
            query: query,
            getMatchingItemsCountForCondition: getMatchingItemsCountForCondition,
            createWithCallbackIfDoesNotExist: createWithCallbackIfDoesNotExist,
            nbOfVisits: nbOfVisits
        };

        function query(type, property, aggregate) {
            if (!aggregate) {
                return $http.get(_queryContext + type + '/' + property);
            }
            return $http.post(_queryContext + type + '/' + property, aggregate);
        }

        function getMatchingItemsCountForCondition(type, condition, validate) {
            return $http.post(_queryContext + type + '/count' + (validate != undefined ? ("?validate="+validate) : ""), condition);
        }

        function createWithCallbackIfDoesNotExist(itemType, scope, name, typeId, errorMessageKey, errorMessageParams, callback, failCallback) {
            var condition = {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.scope",
                                comparisonOperator: "equals",
                                propertyValue: scope ? scope : ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.name",
                                comparisonOperator: "equals",
                                propertyValue: name
                            }
                        }
                    ],
                    operator: "and"
                }
            };

            if (typeId) {
                condition.parameterValues.subConditions.push({
                    type: "sessionPropertyCondition",
                    parameterValues: {
                        propertyName: "metadata.tags",
                        comparisonOperator: "contains",
                        propertyValue: typeId
                    }
                });
            }

            getMatchingItemsCountForCondition(itemType, condition)
                .then(function (response) {
                // if we have matching elements, display an error
                if (response.data > 0) {
                    if (failCallback) {
                        failCallback();
                    }
                    notificationService.errorToast(errorMessageParams ? i18nService.format(errorMessageKey, errorMessageParams) : i18nService.message(errorMessageKey));
                } else {
                    // otherwise perform the provide callback
                    callback();
                }
            });
        }

        // todo: move to unomiCampaignService?
        function nbOfVisits(campaignId, startDate, endDate, callback) {
            getMetricsForCondition("session", "profile.properties.nbOfVisits", ["sum", "avg"], {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "scope",
                                "comparisonOperator": "equals",
                                "propertyValue": ManagersContext.dmfScope
                            }
                        },
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "systemProperties.campaigns." + campaignId + "Engaged",
                                "comparisonOperator": "between",
                                "propertyValuesDate": [startDate, endDate]
                            }
                        }
                    ]
                }
            }).then(callback);
        }

        // Private functions under this line
        function getMetricsForCondition(type, property, metrics, condition) {
            return $http.post(_queryContext + type + '/' + property + '/' + ($.isArray(metrics) ? metrics.join('/') : metrics), condition);
        }
    }
})();