(function() {
    'use strict';

    angular.module('wemCampaignsApp').controller('CampaignEditController', CampaignEditController);

    CampaignEditController.$inject = ['$scope', '$location', 'unomiCampaignService', '$route', 'currentCampaign', 'isNew', 'i18nService'];

    function CampaignEditController($scope, $location, unomiCampaignService, $route, currentCampaign, isNew, i18nService) {
        $scope.currentCampaign = currentCampaign;
        $scope.isNew = isNew;

        $scope.dateOptions = {
            formatYear: 'yyyy',
            startingDay: 1
        };

        $scope.currentText = i18nService.message('serverSettings.manageWem.datepicker.today');
        $scope.closeText = i18nService.message('serverSettings.manageWem.datepicker.close');
        $scope.clearText = i18nService.message('wem.label.clear');

        $scope.format = 'dd-MMMM-yyyy';

        //Timepicker functions
        $scope.dates = {
            startDate : moment($scope.currentCampaign.startDate).toDate(),
            endDate : moment($scope.currentCampaign.endDate).toDate(),
            startTime: new Date($scope.currentCampaign.startDate),
            endTime : new Date($scope.currentCampaign.endDate)
        };
        $scope.dates.startTime.setHours(moment($scope.dates.startDate).tz($scope.currentCampaign.timezone).hours());
        $scope.dates.endTime.setHours(moment($scope.dates.endDate).tz($scope.currentCampaign.timezone).hours());

        // campaign type and localized names of types
        $scope.currentCampaignType = {value: unomiCampaignService.getCampaignType($scope.currentCampaign)};
        $scope.$watch("currentCampaignType.value", function (newValue, oldValue) {
            if (angular.isDefined(oldValue) && newValue != oldValue) {
                $scope.localizedCampaignType = unomiCampaignService.getLocalizedCampaignType($scope.currentCampaignType.value);
            }
        });
        $scope.localizedCampaignParam = unomiCampaignService.getLocalizedCampaignParam($scope.currentCampaignType.value);
        $scope.campaignIDTypes = [
            {name: unomiCampaignService.getLocalizedCampaignType('referrer'), value: 'referrer'},
            {name: unomiCampaignService.getLocalizedCampaignType('landing'), value: 'landing'},
            {name: unomiCampaignService.getLocalizedCampaignType('parameter'), value: 'parameter'}
        ];

        // Calendar functions
        $scope.open = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();
            if (selector == 'start')
                $scope.startopened = true;
            if (selector == 'end')
                $scope.endopened = true;
        };

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();
            if (selector == 'start') {
                if (typeof $scope.startDatePicker === 'undefined') {
                    $scope.startDatePicker = {};
                }
                $scope.startDatePicker.opened = true;
            }
            if (selector == 'end') {
                if (typeof $scope.endDatePicker === 'undefined') {
                    $scope.endDatePicker = {};
                }
                $scope.endDatePicker.opened = true;
            }
        };


        $scope.saveCampaign = function () {
            $scope.currentCampaign.metadata.tags = [$scope.currentCampaignType.value];
            $scope.currentCampaign.entryCondition = unomiCampaignService.createCampaignEntryCondition($scope.currentCampaignType);

            $scope.currentCampaign.startDate = moment($scope.dates.startDate).tz($scope.currentCampaign.timezone);
            $scope.currentCampaign.startDate.hours($scope.dates.startTime.getHours());
            $scope.currentCampaign.startDate.minutes($scope.dates.startTime.getMinutes());
            $scope.currentCampaign.startDate.seconds(0);

            $scope.currentCampaign.endDate = moment($scope.dates.endDate).tz($scope.currentCampaign.timezone);
            $scope.currentCampaign.endDate.hours($scope.dates.endTime.getHours());
            $scope.currentCampaign.endDate.minutes($scope.dates.endTime.getMinutes());
            $scope.currentCampaign.endDate.seconds(0);

            if (!$scope.isNew) {
                unomiCampaignService.setCampaign($scope.currentCampaign).then(function () {
                    unomiCampaignService.updatedCampaign = $scope.currentCampaign;
                    $location.path("/campaigndashboard/" + $scope.currentCampaign.metadata.id);
                });
            } else {
                unomiCampaignService.campaignToBeSaved = $scope.currentCampaign;
                $location.path("/campaign/" + $scope.currentCampaign.metadata.id + "/newGoal");
            }
        };

        $scope.getPreviousPage = function () {
            return $scope.isNew ? "#!/campaigns" : ("#!/campaigndashboard/" + $scope.currentCampaign.metadata.id);
        };

        $scope.resetCampaign = function () {
            $route.reload();
        };

        if ($scope.currentCampaign.entryCondition) {
            $scope.currentCampaignType.param = unomiCampaignService.getTypeSpecificCampaignParameter($scope.currentCampaign);

            $scope.hasCampaignStarted = moment().isAfter($scope.currentCampaign.startDate);
            $scope.hasCampaignClosed = moment().isAfter($scope.currentCampaign.endDate);
        }
    }
})();
