(function() {
    'use strict';

    angular.module('wemCampaignsApp').controller('CampaignGoalCreateController', CampaignGoalCreateController);

    CampaignGoalCreateController.$inject = [
        '$scope', '$location', '$route', '$timeout', '_', 'i18nService', 'unomiGoalService', 'unomiRulesService',
        'currentCampaign', '$controller', 'unomiCampaignService', 'notificationService', 'unomiQueryService',
        'loadingSpinnerService', 'unomiListService', 'goal', 'modalService'];

    function CampaignGoalCreateController($scope, $location, $route, $timeout, _, i18nService, unomiGoalService, unomiRulesService,
                                          currentCampaign, $controller, unomiCampaignService, notificationService, unomiQueryService,
                                          loadingSpinnerService, unomiListService, goal, modalService) {
        var tags = _.map(ManagersUtils.buildTags(ManagersUtils.campaingTags, i18nService), function (tag, key) {
            return tag;
        });
        $scope.goalTags = angular.copy(tags);

        angular.extend(this, $controller('GoalCreateController', {
            $scope: $scope,
            $location: $location,
            $route: $route,
            $timeout: $timeout,
            i18nService: i18nService,
            _: _,
            unomiGoalService: unomiGoalService,
            unomiQueryService: unomiQueryService,
            notificationService: notificationService,
            unomiListService: unomiListService,
            unomiRulesService: unomiRulesService,
            loadingSpinnerService: loadingSpinnerService,
            goal: goal,
            modalService: modalService
        }));

        $scope.goal.campaignId = currentCampaign.metadata.id;
        // $scope.goal.type = $scope.goalTags[0];

        // execute the save() from the callback to build the goal based on the specific logic
        $scope.saveGoal = function (createForm) {
            if (createForm.$invalid) {
                var parsedErrors = ManagersUtils.parseErrorsInForm(createForm);
                if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            if ($scope.newGoal) {
                $scope.callback.save(function() {
                    loadingSpinnerService.show();
                    $scope.goal.metadata.tags = [$scope.goal.type.id];
                    delete $scope.goal.type;
                    delete $scope.goal.startEvent;
                    var lists = $scope.goal.lists;
                    delete $scope.goal.lists;

                    unomiGoalService.setGoal($scope.goal).then(function () {
                        unomiRulesService.setRule(unomiGoalService.localToServerGoalRule($scope.goal, lists));
                        var newCampaign = unomiCampaignService.campaignToBeSaved;
                        if (newCampaign && newCampaign.metadata.id == $scope.goal.campaignId) {
                            newCampaign.primaryGoal = $scope.goal.metadata.id;
                            unomiCampaignService.setCampaign(newCampaign).then(function () {
                                $timeout(function () {
                                    loadingSpinnerService.hide();
                                    delete unomiCampaignService.campaignToBeSaved;
                                    $location.path("/campaigndashboard/" + $scope.goal.campaignId);
                                }, 1000);
                            });
                        } else {
                            $timeout(function () {
                                loadingSpinnerService.hide();
                                $location.path("/campaigndashboard/" + $scope.goal.campaignId);
                            }, 1000);
                        }
                    });
                });
            } else {
                var saveGoalIfConfirm = function () {
                    $scope.callback.save(function () {
                        loadingSpinnerService.show();
                        $scope.goal.metadata.tags = [$scope.goal.type.id];
                        delete $scope.goal.type;
                        delete $scope.goal.startEvent;
                        delete $scope.goal.lists;

                        unomiGoalService.setGoal($scope.goal).then(function () {
                            $timeout(function () {
                                loadingSpinnerService.hide();
                                $location.path("/campaigndashboard/" + $scope.goal.campaignId + "/goal-report/" + $scope.goal.itemId);
                            }, 1000);
                        });
                    });
                };

                modalService.newAlertModal({
                    title: i18nService.message('serverSettings.manageGoals.title.editConfirm'),
                    bodyMessage: i18nService.message('serverSettings.manageGoals.message.editConfirm')
                }, saveGoalIfConfirm);
            }
        };
    }
})();
