(function () {
    'use strict';

    angular.module('wemExportConfigsApp').controller('ExportConfigsListController', ExportConfigsListController);

    ExportConfigsListController.$injecet = ['$route', '$location', '$filter', 'unomiExportConfigService', 'unomiSegmentService'];

    function ExportConfigsListController($route, $location, $filter, unomiExportConfigService, unomiSegmentService) {
        var vm = this;

        vm.onChangeFilterByType = onChangeFilterByType;
        vm.newExportConfig = newExportConfig;
        vm.editExportConfig = editExportConfig;
        vm.getLastExecution = getLastExecution;
        vm.reload = reload;

        _init();

        function onChangeFilterByType() {
            if (vm.filterByType == 'all') {
                vm.exportConfigs = angular.copy(vm.exportConfigsAll);
            } else {
                vm.exportConfigs = R.filter(function (elt) {
                    return elt.configType == vm.filterByType;
                }, vm.exportConfigsAll);
            }
        }

        function newExportConfig() {
            $location.path('/export-configs/new');
        }

        function editExportConfig(exportConfigId) {
            $location.path('/export-configs/' + exportConfigId);
        }

        function getLastExecution(executions) {
            if (executions && executions.length) {
                return $filter('orderBy')(executions, 'date', true)[0];
            } else {
                return null;
            }

        }

        function reload() {
            $route.reload();
        }

        vm.segments = [];

        // Private functions
        function _init() {
            vm.filterByType = 'all';
            unomiSegmentService.getAll().then(
                function (segments) {
                    vm.segments = segments.data;
                    _getExportConfigs();
                },
                function () {
                    console.error("Error while retrieving segments list!");
                }
            );
        }

        function _getExportConfigs() {
            unomiExportConfigService.getAll().then(function (response) {
                vm.exportConfigsAll = response.data;
                R.forEach(function (exportConfig) {
                    var segment = R.find(R.propEq('id', exportConfig.properties.segment))(vm.segments);
                    exportConfig.segmentName = segment ? segment.name : '';
                }, vm.exportConfigsAll);
                vm.exportConfigs = angular.copy(vm.exportConfigsAll);
            });
        }
    }
})();
