(function() {
    'use strict';

    angular.module('wemFormMappingApp').controller('EditFormMappingController', EditFormMappingController);

    EditFormMappingController.$inject = ['$rootScope', '$scope', 'formMappingService', '$location', '_', 'unomiProfileService', 'unomiListService', 'formMapping', 'isNew', 'i18nService', 'notificationService', '$timeout', 'validationUtilsService'];

    function EditFormMappingController($rootScope, $scope, formMappingService, $location, _, unomiProfileService, unomiListService, formMapping, isNew, i18nService, notificationService, $timeout, validationUtilsService) {
        $scope.isNew = isNew;
        $scope.formMapping = formMapping;
        $scope.originalFormMapping = angular.copy($scope.formMapping);

        $scope.startPath = ManagersContext.currentSiteHomePagePath;
        $scope.strategies = ManagersUtils.formMappingActionStrategies;
        $scope.formAttrs = {};

        unomiListService.query().then(function (response) {
            $scope.userLists = response.data.list;
        });

        var findLabelForInput = function ($doc, $input, multiple) {
            if(multiple){
                var labelMultiple = $doc.find("label[for='" + $input.attr('name') + "']");
                if(labelMultiple.length) {
                    return labelMultiple.text();
                }
            } else if ($input[0].type != "radio" || $input[0].type != "checkbox" && $input.attr("id")) {
                var label = $doc.find("label[for='" + $input.attr('id') + "']");
                if (label.length > 0) {
                    return label.text();
                }
            }

            return $input.attr("name");
        };

        function extractFormFields($form) {
            var formFactoryId = $form.attr("data-form-id");
            if (formFactoryId) {
                // Form factory form
                var handleFormFactory = function(data) {
                    var items = _.flatten(_.values(data[formFactoryId]));
                    var keys = _.map(items, function (i) {
                        return i.inputId
                    });
                    var allEntries = _.object(keys, _.map(items, function (i) {
                        return {value: i.inputId, label: i.label}
                    }));
                    _.each(keys, function (entry) {
                        if (!$scope.formAttrs[entry] && !$scope.formMapping.actions[entry]) {
                            $scope.formAttrs[entry] = allEntries[entry];
                        }
                        if ($scope.formMapping.actions && $scope.formMapping.actions[entry]) {
                            $scope.formMapping.actions[entry].fieldNameLabel = allEntries[entry].label;
                        }
                        if ($scope.originalFormMapping.actions && $scope.originalFormMapping.actions[entry]) {
                            $scope.originalFormMapping.actions[entry].fieldNameLabel = allEntries[entry].label;
                        }
                    });
                };
                if ($scope["formFactory" + formFactoryId]) {
                    handleFormFactory($scope["formFactory" + formFactoryId])
                } else {
                    formMappingService.getFormFactory(formFactoryId).then(function (response) {
                        var data = response.data;
                        $scope["formFactory" + formFactoryId] = data;
                        handleFormFactory(data)
                    });
                }
            } else {
                // Standard form
                $form.find(":input").each(
                    function (index, entry) {
                        if (entry.type != 'hidden' && entry.type != 'submit' && !$scope.formAttrs[entry.name] && !$scope.formMapping.actions[entry.name]) {
                            if($form.find(":input[name='" + entry.name + "']").length>1){
                                $scope.formAttrs[entry.name] = {
                                    value: entry.name,
                                    label: findLabelForInput($form, $(entry), true)
                                };
                            }else{
                                $scope.formAttrs[entry.name] = {
                                    value: entry.name,
                                    label: findLabelForInput($form, $(entry))
                                };
                            }
                        }
                        if ($scope.formMapping.actions && $scope.formMapping.actions[entry.name]) {
                            $scope.formMapping.actions[entry.name].fieldNameLabel = findLabelForInput($form, $(entry));
                        }
                        if ($scope.originalFormMapping.actions && $scope.originalFormMapping.actions[entry.name]) {
                            $scope.originalFormMapping.actions[entry.name].fieldNameLabel = findLabelForInput($form, $(entry));
                        }
                    }
                );
            }
        }

        var initExistingForm = function () {
            formMappingService.serverToLocalFormMappingActions($scope.formMapping, $scope.propertyNameChoiceListValuesIndexById);
            validationUtilsService.getFormInPage($scope, $scope.formMapping.formId, $scope.formMapping.primaryPath, function ($form) {
                if ($form.length > 0) {
                    extractFormFields($form);
                } else {
                    // Unable to locate the form
                    notificationService.errorToast(i18nService.format("serverSettings.formMapping.unableToLoad", formMapping.formId + '|' + $scope.formMapping.primaryPath));
                }
            });
        };

        unomiProfileService.getPropertyTypesByTag("profileProperties", true).then(function (response) {
            var data = response.data;
            var filteredData = _.filter(data, function (profileProperty) {
                return _.indexOf(profileProperty.tags, "hiddenFromFormMappingProperties") == -1
                    && _.indexOf(profileProperty.tags, "systemProfileProperties") == -1;
            });

            filteredData.push({
                itemId: 'user-specified-property',
                metadata: {
                    id: 'user-specified-property',
                    name: i18nService.message('action.setPropertyAction.other')
                }, type: 'string', multivalued: false
            });
            $scope.propertyNameChoiceListValues = filteredData;
            $scope.propertyNameChoiceListValuesIndexById = _.indexBy(filteredData, "itemId");

            if ($scope.isNew) {
                // this callback will be used by the tag picker modal
                // it is defined in the rootScope in order to be accessed
                $rootScope.formInputCallback = function (event, remoteScope) {
                    var $targetElement = $(event.target);
                    var found = $targetElement.closest(remoteScope.options.tag);
                    if (found && found.length > 0) {
                        for (var i = 0; i < remoteScope.options.attrs.length; i++) {
                            var attr = remoteScope.options.attrs[i];
                            if (found.is("[" + attr + "]")) {
                                remoteScope.$apply(function () {
                                    remoteScope.options.selectedTagId = found.attr(attr);
                                });

                                // it's a new different form, reset attrs and mappings
                                if ($scope.formMapping.formId != found.attr(attr)) {
                                    $scope.formMapping.formId = found.attr(attr);
                                    // attributes retrieval handled in watch of $scope.formMapping.formId
                                    break;
                                } else {
                                    // it's the same form
                                    extractFormFields(found);
                                    break;
                                }
                            }
                        }
                    }
                };
            } else {
                initExistingForm();
            }
        });

        $scope.getNbOfActions = function () {
            var size = 0, key;
            for (key in $scope.formMapping.actions) {
                if ($scope.formMapping.actions.hasOwnProperty(key)) size++;
            }
            size += $scope.formMapping.lists != undefined ? $scope.formMapping.lists.length : 0;
            return size;
        };

        $scope.saveForm = function (mapping) {
            if (mapping.$invalid) {
                mapping.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm(mapping);
                if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            formMappingService.save(formMappingService.localToServerFormMapping($scope.formMapping)).then(function () {
                $location.path("/list");
            });
        };

        $scope.$watch("formMapping.primaryPath", function (newVal, oldVal) {
            if (newVal && $scope.isNew) {
                // the start changed, reset all the formMapping
                $scope.formMapping.actions = {};
                $scope.formAttrs = {};
                $scope.formMapping.formId = "";

                // hide pickers
                $scope.showNewStartPathPicker = false;
            }
        });

        var tempFormId = '';
        var formIdTimeout;
        if (isNew) {
            $scope.$watch('formMapping.formId', function (newVal, oldVal) {
                if (newVal) {
                    if (formIdTimeout) $timeout.cancel(formIdTimeout);

                    tempFormId = newVal;
                    formIdTimeout = $timeout(function () {
                        validationUtilsService.getFormInPage($scope, $scope.formMapping.formId, $scope.formMapping.primaryPath, function ($form) {
                            $scope.formMapping.actions = {};
                            $scope.formAttrs = {};
                            if ($form.length > 0) {
                                extractFormFields($form);
                            } else {
                                notificationService.errorToast(i18nService.format("serverSettings.formMapping.formNotFoundOnPage", newVal + "|" + $scope.formMapping.primaryPath))
                            }
                        });
                    }, 300);
                }
            });
        }

        $scope.$watch("newPagePath", function (newVal, oldVal) {
            if (newVal) {
                if ($scope.formMapping.primaryPath == newVal || _.contains($scope.formMapping.additionalPaths, newVal)) {
                    return;
                }
                validationUtilsService.getFormInPage($scope, $scope.formMapping.formId, newVal, function ($form) {
                    if ($form.length > 0) {
                        $scope.formMapping.additionalPaths.push(newVal);
                    } else {
                        notificationService.errorToast(i18nService.format("serverSettings.formMapping.formNotFoundOnPage", $scope.formMapping.formId + "|" + newVal))
                    }
                });
            }
        });

        $scope.addMapping = function () {
            if ($scope.newMapping && $scope.formAttrs[$scope.newMapping]) {
                $scope.formMapping.actions[$scope.newMapping] = {
                    fieldNameLabel: $scope.formAttrs[$scope.newMapping].label,
                    fieldName: $scope.newMapping,
                    strategy: ManagersUtils.formMappingActionStrategies[0]
                };
                delete $scope.formAttrs[$scope.newMapping];
                $scope.newMapping = "";
            }
        };

        $scope.removeMapping = function (action) {
            $scope.formAttrs[action.fieldName] = {value: action.fieldName, label: action.fieldNameLabel};
            delete $scope.formMapping.actions[action.fieldName];
        };

        $scope.removePath = function (path) {
            for (var i = $scope.formMapping.additionalPaths.length - 1; i >= 0; i--) {
                if ($scope.formMapping.additionalPaths[i] === path) {
                    $scope.formMapping.additionalPaths.splice(i, 1);
                    break;
                }
            }
        };

        $scope.reset = function () {
            $scope.formMapping = angular.copy($scope.originalFormMapping);
            if (!$scope.isNew) {
                initExistingForm();
            }
        }
    }
})();
