(function() {
    'use strict';

    angular.module('wemFormMappingApp').controller('ListFormMappingController', ListFormMappingController);

    ListFormMappingController.$inject = ['i18nService', '$scope', 'formMappingService', '_', 'DTOptionsBuilder', 'DTColumnBuilder', '$compile', 'modalService', '$location', 'dtPagerService'];

    function ListFormMappingController(i18nService, $scope, formMappingService, _, DTOptionsBuilder, DTColumnBuilder, $compile, modalService, $location, dtPagerService) {
        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                return ManagersUtils.formMappingPropertiesForColName[prop] + ":" + o.dir
            }).join(",");

            formMappingService.getAll(start, length, order).then(function (response) {
                var data = response.data;
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": data.totalSize,
                    "iTotalDisplayRecords": data.totalSize,
                    "aaData": data.list
                };
                fnCallback(results);
                dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            },
            function () {
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": 0,
                    "iTotalDisplayRecords": 0,
                    "aaData": []
                };
                fnCallback(results);
            });
        }

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('fnRowCallback',
                function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    $compile(nRow)($scope);
                })
            .withOption('lengthMenu', [25, 50, 100])
            .withOption('pageLength', 25)
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 'lt')
            .withFnServerData(serverData)
            .withOption('order', [[0, 'desc']]);

        $scope.dtColumns = [
            DTColumnBuilder
                .newColumn("name")
                .withTitle(i18nService.message("serverSettings.manageCampaigns.campaignName")),
            DTColumnBuilder
                .newColumn("description")
                .withTitle(i18nService.message("serverSettings.manageWem.description"))
                .withOption('defaultContent', ''),
            DTColumnBuilder
                .newColumn(null)
                .withTitle(i18nService.message("serverSettings.manageWem.listAction")).notSortable()
                .renderWith(function (data, type, full, meta) {
                    return "<button title=\"" + i18nService.message("serverSettings.manageFormMapping.edit") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + data.id + "')\"><i class=\"material-icons\">mode_edit</i></button> " +
                        "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"remove('" + data.id + "')\"><i class=\"material-icons\">delete</i></button>";
                })
                .withOption('width', '15%')
        ];

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
        };

        $scope.goToEdit = function (id) {
            $location.path("/edit/" + id);
        };

        $scope.remove = function (formMappingId) {
            modalService.newAlertModal({
                title: i18nService.message("serverSettings.manageFormMapping.delete"),
                bodyMessage: i18nService.message("serverSettings.manageFormMapping.delete.confirm")
            }, function () {
                formMappingService.remove(formMappingId).then(function () {
                    $scope.dtInstance.DataTable.draw()
                });
            })
        };
    }
})();
