(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalCreateController', GoalCreateController);

    GoalCreateController.$inject = ['$scope', '$location', '$route', '$timeout', '_', 'i18nService', 'unomiGoalService', 'unomiQueryService', 'notificationService', 'unomiListService', 'unomiRulesService', 'loadingSpinnerService', 'goal', 'modalService'];

    function GoalCreateController($scope, $location, $route, $timeout, _, i18nService, unomiGoalService, unomiQueryService, notificationService, unomiListService, unomiRulesService, loadingSpinnerService, goal, modalService) {
        _init();

        $scope.getEditTemplate = function () {
            if ($scope.goal.type) {
                $scope.template = ManagersContext.wemAppPath + "/wem-goals/goal-" + $scope.goal.type.id + "-edit.html";
            }

            return $scope.template
        };

        $scope.saveGoal = function (createForm) {
            // validate form
            if (createForm.$invalid) {
                var parsedErrors = ManagersUtils.parseErrorsInForm(createForm);
                if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            var goalMetadata = $scope.goal.metadata;
            var type = $scope.goal.type;
            if ($scope.newGoal) {
                // check that we don't already have a goal of this type and name
                unomiQueryService.createWithCallbackIfDoesNotExist('goal', ManagersContext.dmfScope, goalMetadata.name, type.id,
                    "serverSettings.manageGoals.modal.newGoal.alreadyExisting", type.name + "|" + goalMetadata.name, function () {
                        // perform save from specific callback (as defined in specific goal type controllers) to add what is needed for that particular goal type
                        $scope.callback.save(function () {
                            loadingSpinnerService.show();
                            goalMetadata.tags = [type.id];
                            delete $scope.goal.type;
                            var lists = $scope.goal.lists;
                            delete $scope.goal.lists;

                            unomiGoalService.setGoal($scope.goal).then(function () {
                                unomiRulesService.setRule(unomiGoalService.localToServerGoalRule($scope.goal, lists));
                                // Add timeout before reloading to make sure Unomi has time to create the new goal
                                $timeout(function () {
                                    loadingSpinnerService.hide();
                                    $location.path("/goals");
                                }, 1000);
                            });
                        });
                    });
            } else {

                var saveGoalIfConfirm = function () {
                    $scope.callback.save(function () {
                        loadingSpinnerService.show();
                        goalMetadata.tags = [type.id];
                        delete $scope.goal.type;
                        delete $scope.goal.lists;

                        unomiGoalService.setGoal($scope.goal).then(function () {
                            $timeout(function () {
                                loadingSpinnerService.hide();
                                $location.path("/goal-report/" + $scope.goal.itemId);
                            }, 1000);
                        });
                    })
                };

                modalService.newAlertModal({
                    title: i18nService.message('serverSettings.manageGoals.title.editConfirm'),
                    bodyMessage: i18nService.message('serverSettings.manageGoals.message.editConfirm')
                }, saveGoalIfConfirm);
            }
        };

        $scope.resetGoal = function () {
            if($route) {
                $route.reload();
            }
        };

        // Private functions under this line
        function _init() {
            unomiListService.query().then(function (response) {
                $scope.userLists = response.data.list;
            });

            if (!$scope.goalTags) {
                var tags = _.map(ManagersUtils.buildTags(ManagersUtils.goalTags, i18nService), function (tag, key) {
                    return tag;
                });
                $scope.goalTags = angular.copy(tags);
            }
            // this callback object will be inherited in the specific edit goal ctrl and complete by this one with a save() method
            // this will allow specific edit ctrl to do specific logic to build the goal
            $scope.callback = {};

            if (goal) {
                $scope.goal = goal;
                $scope.newGoal = false;
                angular.forEach($scope.goalTags, function(goalTag) {
                    if (goalTag.id == goal.metadata.tags[0]) {
                        $scope.goal.type = goalTag;
                    }
                });
            } else {
                $scope.newGoal = true;
                $scope.goal = {
                    metadata: {
                        id: ManagersUtils.generateId(),
                        name: '',
                        description: '',
                        enabled: false,
                        scope: ManagersContext.dmfScope
                    },
                    type: $scope.goalTags[0]
                };
            }
        }
    }
})();
