(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalReportController', GoalReportController);

    GoalReportController.$inject = ['$scope', '$route', '$routeParams', '$location', '$timeout', 'unomiGoalService', 'goal', 'i18nService', 'unomiProfileService', 'modalService', 'loadingSpinnerService'];

    function GoalReportController($scope, $route, $routeParams, $location, $timeout, unomiGoalService, goal, i18nService, unomiProfileService, modalService, loadingSpinnerService) {
        $scope.goalId = $routeParams.goalId;
        $scope.goal = goal;
        $scope.goalReportIntervales = ManagersUtils.supportedTimelineIntervales;

        // timeline data
        $scope.timelineData = {
            toDate: moment(moment().format("YYYY-MM"), "YYYY-MM").toDate(),
            fromDate: moment(moment().subtract(12, "months").format("YYYY-MM"), "YYYY-MM").toDate(),
            interval: $scope.goalReportIntervales[0],
            title: "serverSettings.manageGoals.report.timeline",
            goalId: $scope.goalId,
            goal: $scope.goal,
            series: [i18nService.message('serverSettings.manageGoals.report.startCountTimeline'), i18nService.message('serverSettings.manageGoals.report.targetCountTimeline')],
            loadData: function (success) {
                var query = {
                    aggregate: {
                        type: "date",
                        property: "timeStamp",
                        parameters: {
                            interval: $scope.timelineData.interval.key,
                            format: "yyyy-MM-dd"
                        }
                    }
                };
                unomiGoalService.getGoalReportAggregated($scope.timelineData.goalId, query).then(function (response) {
                    var data = response.data;
                    $scope.report = data;
                    $scope.report.globalStats.conversion = ManagersUtils.getPercentage($scope.report.globalStats.conversionRate);

                    var datasets = [{}, {}];

                    if (data.split && data.split.length > 0) {
                        angular.forEach(data.split, function (v, k) {
                            if (v.key) {
                                datasets[0][v.key] = v.startCount;
                                datasets[1][v.key] = v.targetCount;
                            }
                        });
                    }
                    success(datasets);
                });
            }
        };

        // get properties filter
        $scope.quantitativeProperties = [];
        $scope.qualitativeProperties = [];
        $scope.qualitativeProperties.push(
            {
                "type": 'string',
                "itemType": 'segments',
                "metadata": {
                    "id": 'segments',
                    "name": i18nService.message("serverSettings.manageGoals.report.table.properties.segments")
                },
                "itemTypeLabel": i18nService.message("serverSettings.manageGoals.report.table.properties.segments")
            });
        unomiProfileService.getPropertyTypes().then(function (response) {
            var data = response.data;
            if (data) {
                angular.forEach(data, function (v, k) {
                    if (v && v.length > 0) {
                        angular.forEach(v, function (p) {
                            p.itemType = k;
                            p.itemTypeLabel = i18nService.message("serverSettings.manageGoals.report.table.properties." + k);
                            if (p.metadata.id == "gender") {
                                $scope.qualitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": $scope.goalReportIntervales[0].key,
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": $scope.goalReportIntervales[0]
                                };
                                $scope.quantitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": $scope.goalReportIntervales[0].key,
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": $scope.goalReportIntervales[0]
                                };

                                $scope.$watch('timelineData', function (newVal, oldVal) {
                                    $scope.qualitativeReportData.fromDate = newVal.fromDate;
                                    $scope.qualitativeReportData.toDate = newVal.toDate;
                                    $scope.qualitativeReportData.timelineInterval = newVal.interval;
                                    $scope.quantitativeReportData.fromDate = newVal.fromDate;
                                    $scope.quantitativeReportData.toDate = newVal.toDate;
                                    $scope.quantitativeReportData.timelineInterval = newVal.interval;
                                }, true);
                            }
                            $scope.quantitativeProperties.push(p);
                            $scope.qualitativeProperties.push(p);
                        });
                    }
                });
            }
        });

        $scope.refresh = function () {
            $route.reload();
        };

        $scope.deleteGoal = function (id) {
            modalService.newAlertModal({
                title: i18nService.message("serverSettings.manageWem.delete"),
                bodyMessage: i18nService.message("serverSettings.manageGoals.delete.confirm")
            }, function () {
                loadingSpinnerService.show();
                unomiGoalService.deleteGoal(id).then(function (response) {
                    // added timeout before reloading to make sure Unomi has time to remove the goal
                    $timeout(function() {
                        loadingSpinnerService.hide();
                        $location.path('/goals');
                    }, 1000);
                });
            });
        };
    }
})();
