(function() {
    'use strict';

    angular.module('wemGoalsApp').directive('goalWidget', goalWidgetDirective);

    function goalWidgetDirective() {
        return {
            restrict: 'E',
            scope: {
                goal: '=',
                tagId: '=',
                campaignId: '@'
            },
            template: '<div ng-include="templateUrl"></div>',
            controller: GoalWidgetController
        };
    }

    GoalWidgetController.$inject = ['$rootScope', '$scope', 'unomiGoalService'];

    function GoalWidgetController($rootScope, $scope, unomiGoalService) {
        // get report
        var query = {
            aggregate: {
                type: "date",
                property: "timeStamp",
                parameters: {
                    interval: "1d"
                }
            }
        };
        unomiGoalService.getGoalReportAggregated($scope.goal.id, query).then(function (response) {
            var reportData = response.data;
            $scope.report = reportData.globalStats;
            $scope.report.conversion = ManagersUtils.getPercentage($scope.report.conversionRate);
            $scope.doughnutLabels = ['Success', 'Views'];
            $scope.doughnutData = [$scope.report.conversion, 100 - $scope.report.conversion];
            var doughnutColor = _getDoughnutColor($scope.report.conversion);
            $scope.doughnutColor = [doughnutColor, '#eeeeee'];
            $scope.doughnutOpts = {
                cutoutPercentage: 60,
                title: {
                    display: true,
                    text: $scope.report.conversion + '%',
                    fontColor: doughnutColor,
                    fontSize: 22,
                    fullWidth: false,
                    fontStyle: 'normal',
                    position: 'bottom',
                    fontFamily: "'Nunito Sans','Helvetica Neue',Helvetica,Arial,sans-serif"
                },
                animation: {
                    easing: 'easeOutBounce'
                }
            };

            // compute trend
            if (reportData.split && reportData.split.length > 0) {
                var currentWeek = moment().startOf('week');
                var weekOneBeforeCurrent = moment().startOf('week').subtract(1, 'weeks');
                var weekTwoBeforeCurrent = moment().startOf('week').subtract(2, 'weeks');
                var result = {
                    startCountBefore: 0,
                    startCountBeforeTwo: 0,
                    targetCountBefore: 0,
                    targetCountBeforeTwo: 0
                };
                angular.forEach(reportData.split, function (v, k) {
                    if (v && v.key) {
                        var thisMoment = moment(v.key);
                        if (thisMoment.isBetween(weekTwoBeforeCurrent, currentWeek)) {
                            if (thisMoment.isAfter(weekOneBeforeCurrent)) {
                                result.startCountBefore = result.startCountBefore + v.startCount;
                                result.targetCountBefore = result.targetCountBefore + v.targetCount;
                            } else {
                                result.startCountBeforeTwo = result.startCountBeforeTwo + v.startCount;
                                result.targetCountBeforeTwo = result.targetCountBeforeTwo + v.targetCount;
                            }
                        }
                    }
                });
                result.conversionBefore = result.startCountBefore > 0 ? result.targetCountBefore / result.startCountBefore : 0;
                result.conversionBeforeTwo = result.startCountBeforeTwo > 0 ? result.targetCountBeforeTwo / result.startCountBeforeTwo : 0;
                if (result.conversionBefore > result.conversionBeforeTwo) {
                    $scope.progressionArrow = 'trending_up';
                } else if (result.conversionBefore == result.conversionBeforeTwo) {
                    $scope.progressionArrow = 'trending_flat';
                } else {
                    $scope.progressionArrow = 'trending_down';
                }
            }
        });

        // resolve template
        $scope.templateUrl = ManagersContext.wemAppPath + '/wem-goals/goal-widget-' + $scope.tagId + '.directive.html';

        $scope.goalLinksTmpl = $rootScope.getGoalLinksView();

        function _getDoughnutColor(percentage) {
            if (percentage > 90) {
                return '#8454c6';
            }
            if (percentage > 20) {
                return '#4BE45F';
            }
            if (percentage > 5) {
                return '#ffc500';
            }
            return '#E44759';
        }
    }
})();
