(function() {
    'use strict';

    angular.module('wemIncomingSearchesApp').controller('IncomingSearchesListController', IncomingSearchesListController);

    IncomingSearchesListController.$inject = ['$scope', 'referrerService', 'i18nService', 'uibDatepickerConfig', '$compile', 'dataTableService', '$location', 'dtPagerService'];

    function IncomingSearchesListController($scope, referrerService, i18nService, uibDatepickerConfig, $compile, dataTableService, $location, dtPagerService) {
        $scope.incomingSearchFrom = Date.now() + -30 * 24 * 3600 * 1000;
        $scope.incomingSearchTo = Date.now();

        var actionsHtml = function (data, type, full, meta) {
            var html = "<button title=\"" + i18nService.message("serverSettings.internalSearches.byReferrers") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"displaySearchByReffers(" + meta.row + ")\"><i class=\"material-icons\">http</i></button> ";
            html += "<button title=\"" + i18nService.message("serverSettings.incomingSearches.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails(" + meta.row + ")\"><i class=\"material-icons\">list</i></button> ";
            return html;
        };

        var handleMissing = function (data, type, full, meta) {
            return ManagersUtils.humanReadableMissingIfNeeded(data, i18nService);
        };

        //No global controller where to put this function, it must be redefined for each app
        var rowCallback = function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
            $compile(nRow)($scope);
        };

        var init = function (from, to) {
            var promise = referrerService.getQueries(from, to);
            dataTableService.setPromise(promise);
            $scope.dtOptions = dataTableService.generateDataTableOptions({
                pageLength: 25,
                defaultOrder: [[2, 'desc']],
                rowCallback: rowCallback
            });

            var columns = [
                {name: 'referrerQuery', bundleKey: 'serverSettings.incomingSearches.searchedTerms', actionsHtml: handleMissing},
                {name: 'occurrences', bundleKey: 'serverSettings.incomingSearches.occurrences'},
                {name: 'importance', bundleKey: 'serverSettings.incomingSearches.importance'},
                {
                    name: null, bundleKey: 'serverSettings.manageWem.listAction',
                    actionsHtml: actionsHtml,
                    notSortable: true,
                    options: [
                        ["width", "180px"],
                        ["contentPadding", "0"]
                    ]
                }
            ];
            $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
            promise.then(function () {
                dtPagerService.refreshPager($scope);
            });
        };

        init($scope.incomingSearchFrom, $scope.incomingSearchTo);

        // date config
        uibDatepickerConfig.startingDay = 1;
        uibDatepickerConfig.minMode = "day";
        uibDatepickerConfig.datepickerMode = "day";
        uibDatepickerConfig.showWeeks = false;
        $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
        $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
        $scope.clearText = i18nService.message('wem.label.clear');
        $scope.datePickers = {};

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();

            angular.forEach($scope.datePickers, function (value, key) {
                $scope.datePickers[key] = false;
            });
            $scope.datePickers[selector] = true;
        };

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        $scope.displaySearchByReffers = function (index) {
            var query = $scope.dtInstance.DataTable.row(index).data().referrerQuery;
            var encodedQuery = referrerService.setSelectedQuery(query);
            $location.path('/incoming-search/by-referrers/' + encodedQuery);
        };

        $scope.goToDetails = function (index) {
            var query = $scope.dtInstance.DataTable.row(index).data().referrerQuery;
            var encodedQuery = referrerService.setSelectedQuery(query);
            $location.path('/incoming-search/' + encodedQuery);
        };

        $scope.updateIncomingSearchesList = function () {
            init(new Date($scope.incomingSearchFrom).getTime(), new Date($scope.incomingSearchTo).getTime());
            $scope.dtInstance.DataTable.draw();
        };

        $scope.getMessage = function (key) {
            return i18nService.message(key);
        };
    }
})();