(function() {
    'use strict';

    angular.module('wemMostVisitedPagesApp').controller('MostVisitedPagesListController', MostVisitedPagesListController);

    MostVisitedPagesListController.$inject = ['$scope', '$compile', '$location', 'uibDatepickerConfig', 'i18nService', 'dataTableService', 'internalSearchService', 'dtPagerService'];

    function MostVisitedPagesListController($scope, $compile, $location, uibDatepickerConfig, i18nService, dataTableService, internalSearchService, dtPagerService) {
        $scope.mostVisitedFrom = Date.now() + -30*24*3600*1000;
        $scope.mostVisitedTo = Date.now();
        $scope.selectedLanguage = {id: "_all", name: "all"};
        internalSearchService.getMostVisitedLanguages().then(function (result) {
            $scope.availableLanguages = result;
        });

        var actionsHtml = function (data, type, full, meta) {
            return "<button title=\"" + i18nService.message("serverSettings.internalSearches.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails('" + data.url + "')\"><i class=\"material-icons\">search</i></button> ";
        };

        //No global controller where to put this function, it must be redefined for each app
        var rowCallback = function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
            $compile(nRow)($scope);
        };

        var init = function (from, to, language) {
            var promise = internalSearchService.getMostVisitedList(null, from, to, language);
            dataTableService.setPromise(promise);

            $scope.dtOptions = dataTableService.generateDataTableOptions(
                {defaultOrder: [[1, 'desc']], rowCallback: rowCallback}
            );


            var columns = [
                {name: 'url', bundleKey: 'serverSettings.mostVisitedPages.url'},
                {name: 'occurrences', bundleKey: 'serverSettings.mostVisitedPages.occurrences'},
                {name: null, bundleKey: 'serverSettings.manageWem.listAction',
                    actionsHtml: actionsHtml,
                    notSortable: true,
                    options: [
                        ["width", "180px"],
                        ["contentPadding", "0"]
                    ]
                }
            ];

            $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
            promise.then(function(){
                dtPagerService.refreshPager($scope);
            });
        };

        init($scope.mostVisitedFrom, $scope.mostVisitedTo, $scope.selectedLanguage.id);

        // date config
        uibDatepickerConfig.startingDay = 1;
        uibDatepickerConfig.minMode = "day";
        uibDatepickerConfig.datepickerMode = "day";
        uibDatepickerConfig.showWeeks = false;
        $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
        $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
        $scope.clearText = i18nService.message('wem.label.clear');
        $scope.selectedLanguageLabel = i18nService.message("serverSettings.manageWem.selectedLanguage");
        $scope.datePickers = {};

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();

            angular.forEach($scope.datePickers, function(value, key) {
                $scope.datePickers[key] = false;
            });
            $scope.datePickers[selector] = true;
        };

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        $scope.goToDetails = function (url) {
            internalSearchService.setSelectedUrl(url);
            // the redirect to search details view using a fake route parameter so that we can properly find the previous page in profiles controllers
            $location.path('/mostvisited/selected');
        };

        $scope.updatePagesViewsList = function () {
            init(new Date($scope.mostVisitedFrom).getTime(), new Date($scope.mostVisitedTo).getTime(), $scope.selectedLanguage.id);
            $scope.dtInstance.DataTable.draw();
        };

        $scope.getMessage = function (key) {
            return i18nService.message(key);
        };
    }
})();
