(function() {
    'use strict';

    angular.module('wemManageOptimizationTestApp').controller('ManageOptimizationTestController', ManageOptimizationTestController);

    ManageOptimizationTestController.$inject = [
        '$scope', '$location', '$controller', '$timeout', '_',
        'i18nService', 'unomiGoalService', 'unomiProfileService', 'unomiListService',
        'unomiRulesService', 'loadingSpinnerService', 'modalService', 'unomiQueryService', 'notificationService'];

    function ManageOptimizationTestController($scope, $location, $controller, $timeout, _,
                                              i18nService, unomiGoalService, unomiProfileService, unomiListService,
                                              unomiRulesService, loadingSpinnerService, modalService, unomiQueryService, notificationService) {
        // init (goalId come from the jsp)
        $scope.init = function() {
            if (goalId) {
                unomiGoalService.getGoal(goalId).then(function(response) {
                    var goal = response.data;
                    if (goal) {
                        if (goal.startEvent.parameterValues.subConditions[1].type == "eventPropertyCondition" && currentDMFNodeUUID != goal.startEvent.parameterValues.subConditions[1].parameterValues.propertyValue) {
                            goal.startEvent.parameterValues.subConditions[1] = {
                                "type": "booleanCondition",
                                "parameterValues": {
                                    "operator": "or",
                                    "subConditions": [
                                        goal.startEvent.parameterValues.subConditions[1],
                                        {
                                            "parameterValues": {
                                                "propertyName": "target.itemId",
                                                "propertyValue": currentDMFNodeUUID,
                                                "comparisonOperator": "equals"
                                            },
                                            "type": "eventPropertyCondition"
                                        }
                                    ]
                                }
                            };
                            unomiGoalService.setGoal(goal);
                        } else if (goal.startEvent.parameterValues.subConditions[1].type == "booleanCondition" && !_.find(goal.startEvent.parameterValues.subConditions[1].parameterValues.subConditions, function (cond) {
                                return cond.parameterValues.propertyValue == currentDMFNodeUUID;
                            })) {
                            goal.startEvent.parameterValues.subConditions[1].parameterValues.subConditions.push(
                                {
                                    "parameterValues": {
                                        "propertyName": "target.itemId",
                                        "propertyValue": currentDMFNodeUUID,
                                        "comparisonOperator": "equals"
                                    },
                                    "type": "eventPropertyCondition"
                                }
                            );
                            unomiGoalService.setGoal(goal);
                        }

                        $scope.goalExist = true;
                        $scope.showGoalForm = false;
                        $scope.goal = goal;
                        $scope.goal.metadata.name = i18nService.message("serverSettings.manageGoals.optimizationTestFor") + " " + currentDMFNodeDisplayName;
                        delete $scope.goal.metadata.description;
                        var tag = goal.metadata.tags[0];
                        $scope.goalTemplateView = ManagersContext.wemAppPath + "/wem-goals/goal-" + tag + "-view.html";
                        $scope.goalTitle = i18nService.message("serverSettings.manageGoals.tag." + tag + ".description") + " : " + goal.metadata.name;


                        // get properties filter
                        $scope.properties = [];
                        $scope.properties.push(
                            {
                                "metadata": {
                                    "id": "segments",
                                    "name": i18nService.message("serverSettings.manageGoals.report.table.properties.segments")
                                },
                                "type": 'string',
                                "itemType": 'segments',
                                "itemTypeLabel": i18nService.message("serverSettings.manageGoals.report.table.properties.segments")
                            });
                        var variantProperty = {
                            "metadata": {
                                "id": "optimizationTest_" + currentDMFNodeUUID,
                                "name": i18nService.message("serverSettings.manageGoals.report.table.properties.variant")
                            },
                            "type": 'string',
                            "itemType": 'sessions',
                            "itemTypeLabel": i18nService.message("serverSettings.manageGoals.report.table.properties.sessions")
                        };
                        $scope.properties.push(variantProperty);
                        unomiProfileService.getPropertyTypes().then(function (response) {
                            var data = response.data;
                            if (data) {
                                angular.forEach(data, function (v, k) {
                                    if (v && v.length > 0) {
                                        angular.forEach(v, function (p) {
                                            p.itemType = k;
                                            p.itemTypeLabel = i18nService.message("serverSettings.manageGoals.report.table.properties." + k);
                                            $scope.properties.push(p);
                                        });
                                    }
                                });
                                $scope.qualitativeReportData = {
                                    "selectedProperty": variantProperty,
                                    "useRanges": true,
                                    "intervals": ManagersUtils.supportedTimelineIntervales,
                                    "selectedInterval": ManagersUtils.supportedTimelineIntervales[0]
                                };
                            }
                        });
                    } else {
                        $scope._showGoalForm();
                    }
                });
            } else {
                $scope._showGoalForm();
            }
        };

        /**
         * Triggered by the goal report directive when a query is done, giving access to the result, in the current case we look for the control variant row to store locally this result
         * @param result query result
         * @param data the directive data (selectedProperty, properties, date ranges, ...)
         */
        $scope.reportTableQueryHook = function (result, data) {
            if(data && data.selectedProperty.metadata.id == ("optimizationTest_" + currentDMFNodeUUID) && controlVariantId) {
                angular.forEach(result.split, function (v, k) {
                    if (v.key == controlVariantId) {
                        $scope.controlVariantRow = v;
                    }
                });
            }
        };

        /**
         * Triggered by the goal report directive when iterate on the query rows, giving access to the row, allowing to add/remove/change properties
         * @param row query row
         * @param data the directive data (selectedProperty, properties, date ranges, ...)
         */
        $scope.reportTableQueryRowHook = function (row, data) {
            // set lift data for extra lift column
            if(data && data.selectedProperty.metadata.id == ("optimizationTest_" + currentDMFNodeUUID) && $scope.controlVariantRow && $scope.controlVariantRow.conversionRate != 0) {
                if (row == $scope.controlVariantRow) {
                    row.lift = "";
                } else {
                    row.lift = (((row.conversionRate / $scope.controlVariantRow.conversionRate) - 1) * 100).toFixed(2) + " %";
                }
            } else {
                row.lift = "";
            }

            // display the node name instead of the id
            if(data && data.selectedProperty.metadata.id == ("optimizationTest_" + currentDMFNodeUUID)) {
                if(variants[row.key]) {
                    row.displayableKey = variants[row.key];
                } else {
                    row.displayableKey = i18nService.message("label.engineTab.dmfOptimization.deletedVariant");
                }
            }
        };

        /**
         * Triggered by the goal report directive when creating the table columns, call each time the data change in the directive
         * allow to add/remove/update columns
         * @param columns default columns build by the directive
         * @param DTColumnBuilder Utility provide by angular datatables to build columns
         * @param data the directive data (selectedProperty, properties, date ranges, ...)
         */
        $scope.reportTableColumnsHook = function (columns, DTColumnBuilder, data) {
            if(data && data.selectedProperty.metadata.id == ("optimizationTest_" + currentDMFNodeUUID) && $scope.controlVariantRow) {
                columns.push(DTColumnBuilder.newColumn('lift').withTitle(i18nService.message("serverSettings.manageGoals.report.table.column.variantDif")));
            } else {
                columns.push(DTColumnBuilder.newColumn('lift').notVisible());
            }
        };

        $scope.validate = function (gwtValidation) {
            if($scope.goalExist || !$scope.callback || !$scope.showGoalForm) {
                return;
            }

            // $scope.optimizationGoal is the form in the jsp with the name="optimizationGoal"
            console.log("wem optimization test: goal validating...");
            if($scope.goalOptimizationForm && !$scope.goalOptimizationForm.$valid) {
                console.log("wem optimization test: goal not valid");
                gwtValidation.addValidationError(i18nService.message("optimizationGoalForm.invalid"));
            }

            console.log("wem optimization test: goal validation end");
        };

        $scope.save = function (gwtNode) {
            if ($scope.goalExist || !$scope.callback) {
                return;
            }

            if ($scope.goalToDisabled) {
                console.log("wem optimization test: disabling old goal...");
                $scope.goalToDisabled.metadata.enabled = false;
                unomiGoalService.setGoal($scope.goalToDisabled).then(function() {
                    console.log("wem optimization test: old goal disabled");
                    delete $scope.goalToDisabled;
                })
            }

            if (!$scope.showGoalForm) {
                gwtNode.setProperty("wem:goalId", "", 1);
                gwtNode.setProperty("wem:hits", "0", 3);
                return;
            }

            // execut the save() from the callback to build the goal based on the specific logic
            console.log("wem optimization test: goal saving...");
            $scope.callback.save(function() {
                $scope.goal.metadata.tags = [$scope.goal.type.id];

                // set specific start event for optimization test
                var goal = angular.copy($scope.goal);
                delete goal.type;
                goal.metadata.id = ("optimizationTestGoal_" + goal.metadata.id);
                goal.metadata.name = (i18nService.message('manageGoals.name.optimizationTestFor') + currentDMFNodeName);
                goal.metadata.description = (i18nService.message('manageGoals.description.optimizationTestPath') + currentDMFNodePath);
                goal.metadata.hidden = true;
                goal.startEvent = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "eventTypeId": "optimizationTestEvent"
                                },
                                "type": "eventTypeCondition"
                            },
                            {
                                "parameterValues": {
                                    "propertyName": "target.itemId",
                                    "propertyValue": currentDMFNodeUUID,
                                    "comparisonOperator": "equals"
                                },
                                "type": "eventPropertyCondition"
                            }
                        ]
                    }
                };

                // set goal id on the node before the AJAX request, because in the callback the edit engine will be closed
                gwtNode.setProperty("wem:goalId", goal.metadata.id, 1);
                gwtNode.setProperty("wem:hits", "0", 3);

                // do save
                unomiGoalService.setGoal(goal).then(function() {
                    console.log("wem optimization test: goal saved");
                })
            });
        };

        $scope.reset = function () {
            if(!$scope.goalExist) {
                return;
            }

            modalService.newAlertModal({
                title: i18nService.message("wemnt_optimizationTest.reset.modal.title"),
                bodyMessage: i18nService.message("wemnt_optimizationTest.reset.modal.body")
            }, function() {
                $scope.goalToDisabled = angular.copy($scope.goal);
                $scope._showGoalForm();
            })
        };

        $scope._showGoalForm = function() {
            // extend CreateModalCtrl because it's the one that hold all the logic of goal creation
            $scope.goalTags = ManagersUtils.buildTags(ManagersUtils.optimizationTestGoalTags, i18nService);
            angular.extend(this, $controller('GoalCreateController', {
                $scope:$scope,
                $location:$location,
                $route: null,
                $timeout: $timeout,
                i18nService: i18nService,
                _:_,
                unomiGoalService:unomiGoalService,
                unomiQueryService: unomiQueryService,
                notificationService:notificationService,
                unomiListService: unomiListService,
                unomiRulesService: unomiRulesService,
                loadingSpinnerService: loadingSpinnerService,
                goal: undefined,
                modalService: modalService
            }));
            $scope.goalExist = false;
            $scope.goal.type = $scope.goalTags["formGoal"];
        }
    }
})();
