(function() {
    'use strict';

    angular.module('wemProfilesApp').controller('AddPropertyModalController', AddPropertyModalController);

    AddPropertyModalController.$inject = ['$scope', '$uibModalInstance', '$resource', 'unomiProfileService', 'i18nService', 'notificationService', 'propertiesByTag', 'tagId'];

    function AddPropertyModalController($scope, $uibModalInstance, $resource, unomiProfileService, i18nService, notificationService, propertiesByTag, tagId) {
        $scope.type = "string";
        $scope.valueTypes = [];
        var valueTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/values', {}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });
        valueTypesResource.get(function (valueTypes) {
            console.log("add-property-modal.controller.js: Loaded " + valueTypes.length + " value types from Apache Unomi.");
            for (var i = 0; i < valueTypes.length; i++) {
                $scope.valueTypes.push(valueTypes[i].id);
            }
        });

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
        };

        $scope.saveProperty = function () {
            $scope.addProfilePropertyForm.showErrors = true;

            var parsedErrors = ManagersUtils.parseErrorsInForm($scope.addProfilePropertyForm);

            if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                return;
            }

            var tagProperties = propertiesByTag[tagId];
            var propertyType = {
                "defaultValue": "",
                "itemId": $scope.id,
                "itemType": "propertyType",
                "metadata": {
                    "id": $scope.id,
                    "name": $scope.name
                },
                "rank": tagProperties[tagProperties.length - 1].rank + 1,
                "tags": [tagId],
                "target": "profiles",
                "type": $scope.type,
                "multivalued": $scope.isMultivalued
            };
            if($scope.isAutocompleted){
                propertyType.tags.push("autocompleted")
            }
            unomiProfileService.createPropertyType(propertyType).then(function (response) {
                var result = response.data;
                if (result) {
                    tagProperties.push(propertyType);
                    notificationService.successToast(i18nService.format("serverSettings.manageProfiles.property.add.success", $scope.id));
                    $uibModalInstance.close();
                } else {
                    notificationService.errorToast(i18nService.format("serverSettings.manageProfiles.property.add.failed", $scope.id));
                }
            });
        };

        $scope.hasError = function (propertyName) {
            return ManagersUtils.hasErrors($scope.addProfilePropertyForm, $scope.addProfilePropertyForm, propertyName);
        };
    }
})();