(function() {
    'use strict';

    angular.module('wemProfilesApp').controller('ProfileSessionController', ProfileSessionController);

    ProfileSessionController.$inject = ['$scope', '$routeParams', '$resource', 'i18nService', 'DTOptionsBuilder', 'DTColumnBuilder', '$http', 'dtPagerService'];

    function ProfileSessionController($scope, $routeParams, $resource, i18nService, DTOptionsBuilder, DTColumnBuilder, $http, dtPagerService) {
        // Scroll to the top
        window.scrollTo(0, 0);

        $scope.sessionId = $routeParams.sessionId;
        $scope.profileId = $routeParams.profileId;

        $scope.loaded = true;
        $scope.propertyTypesLoaded = false;
        $scope.propertiesLoaded = false;

        $scope.currentSession = {};

        var sessionResourceURL = '/cxs/profiles/sessions/:sessionId';

        var sessionResource = $resource(ManagersContext.proxyServletUrl + sessionResourceURL);
        var valueTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/values', {}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });
        var propertyTypeRootTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/tags/sessionProperties', {}, {
            'get': {
                method: 'GET',
                isArray: false
            }
        });
        var allPropertyTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/properties/sessions', {}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });
        var propertyTypesByTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/properties/tags/:tagId', {}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });

        $scope.getMomentDate = function (date) {
            return moment(date).format('LL');
        };

        $scope.getMomentDuration = function (duration) {
            return moment.duration(duration).humanize();
        };

        $scope.resetSession = function () {
            $scope.currentSession = sessionResource.get({sessionId: $scope.sessionId}, function (profile) {
                console.log("profile-session.controller.js: Loaded profile " + profile.itemId + " succesfully from Apache Unomi.");
                $scope.propertiesLoaded = true;
                $scope.loaded = $scope.propertiesLoaded && $scope.propertyTypesLoaded;

                if ($scope.currentSession.properties.hasOwnProperty('userAgentName')) {
                    $scope.browserImage = $scope.getCurrentModuleResource('/images/browsers/' + $scope.currentSession.properties.userAgentName.toLowerCase().replace(" ", "-") + '.png');
                }

                if ($scope.currentSession.properties.hasOwnProperty('deviceCategory')) {
                    var deviceCategory = ($scope.currentSession.properties.deviceCategory || '').toLowerCase();
                    if (deviceCategory == "personal computer") {
                        $scope.device = "desktop";
                    } else if (deviceCategory == "smartphone") {
                        $scope.device = "mobile";
                    } else if (deviceCategory == "tablet") {
                        $scope.device = "tablet";
                    } else {
                        $scope.device = "others";
                    }
                    $scope.deviceLabel = i18nService.message("serverSettings.manageSessions.device." + $scope.device);
                }
            });
        };

        $scope.saveSession = function () {
            $scope.currentSession.$save({sessionId: $scope.sessionId},
                function () {
                    notificationService.successToast(i18nService.message("serverSettings.manageSessions.save.success"));
                }
            );
        };

        $scope.valueTypes = valueTypesResource.get(function (valueTypes) {
            console.log("profile-session.controller.js: Loaded " + valueTypes.length + " value types from Apache Unomi.");

            $scope.propertyTypeRootTag = propertyTypeRootTagResource.get(function (rootTag) {
                console.log("profile-session.controller.js: Loaded " + rootTag.subTags.length + " property type tags from Apache Unomi.");
                var loadedNewTranslationParts = false;
                for (var i = 0; i < rootTag.subTags.length; i++) {
                    var propertyTag = rootTag.subTags[i];
                    propertyTag.propertyTypes = propertyTypesByTagResource.get({tagId: propertyTag.id}, function (propertyTypes) {
                        console.log("profile-session.controller.js: Loaded " + propertyTypes.length + " property types from Apache Unomi.");
                        var loadedNewTranslationParts = false;
                        for (var j = 0; j < propertyTypes.length; j++) {
                            var propertyType = propertyTypes[j];
                        }
                    })
                }
            });

            $scope.existingPropertyTypes = allPropertyTypesResource.get(function (propertyTypes) {
                console.log("profile-session.controller.js: Loaded " + propertyTypes.length + " propertyTypes from Apache Unomi.");
                $scope.propertyTypesLoaded = true;
                $scope.loaded = $scope.propertiesLoaded && $scope.propertyTypesLoaded;
            });

        });

        function infoColumn(data, type, full, meta) {
            if (data == "null") {
                return "";
            }
            if (full.eventType == 'view') {
                return i18nService.message("serverSettings.manageSessions.event.viewedPage") + " : <a href=\"" + data + "\" target=\"_blank\">" + data + "</a>";
            }
            // event.eventType == 'goal'
            return i18nService.message("serverSettings.manageSessions.event.reachedGoal") + " : " + data;
        }

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 't')
            .withFnServerData(serverData)
            .withPaginationType('full_numbers')
            .withOption("order", [[0, 'asc']]);

        $scope.dtColumns = [
            DTColumnBuilder.newColumn('timeStamp').withTitle(i18nService.message('event.time')).renderWith(function (data) {
                if (data == $scope.currentSession.timeStamp) {
                    return ""
                }
                return i18nService.message('event.timeAfter') + " " + moment.duration(moment(data).diff(moment($scope.currentSession.timeStamp))).humanize()
            }),
            DTColumnBuilder.newColumn('info').withTitle(i18nService.message('serverSettings.manageSessions.event.info')).renderWith(infoColumn).notSortable(),
        ];

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                return "timeStamp:" + o.dir
            }).join(",");

            var search = _.find(aoData, function (x) {
                return x.name == "search"
            }).value.value;
            if (search != '') {
                search = '*' + search.trim() + '*';
            }

            var config = {
                url: ManagersContext.proxyServletUrl + "/cxs/profiles/sessions/" + $scope.sessionId + "/events",
                method: 'get',
                params: {
                    eventTypes: ["view", "goal"],
                    q: search,
                    offset: start,
                    size: length,
                    sort: order
                }
            };

            $http(config)
                .then(function (response) {
                    var data = response.data;
                    console.log("profile-session.controller.js: " + data.list.length + " events loaded out of a total " + data.totalSize + " events.");
                    // copy the event information to a format that can be used by the table component we use
                    var eventTable = [];
                    for (var i = 0; i < data.list.length; i++) {
                        var eventTableRow = {};
                        var event = data.list[i];
                        eventTableRow.timeStamp = event.timeStamp;
                        if (event.eventType == 'view') {
                            eventTableRow.info = event.target.properties.pageInfo.destinationURL;
                        } else { // event.eventType == 'goal'
                            eventTableRow.info = event.target.metadata.name;
                        }
                        eventTableRow.eventType = event.eventType;
                        eventTable.push(eventTableRow);
                    }

                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": data.totalSize,
                        "iTotalDisplayRecords": data.totalSize,
                        "aaData": eventTable
                    };
                    fnCallback(results);
                    dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                },
                function () {
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                });
        }

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
        };

        $scope.getPreviousPage = function () {
            // if we have search id, route to the search
            if ($routeParams.searchId) {
                return '#!/search/' + $routeParams.searchId + '/profile/' + $routeParams.profileId;
            } else if ($routeParams.segmentId) {
                return '#!/segment/' + $routeParams.segmentId + '/profile/' + $routeParams.profileId;
            } else if ($routeParams.campaignId) {
                var previousPage = '#!/campaign/' + $routeParams.campaignId;
                if ($routeParams.goalId) {
                    previousPage += "/converted/" + $routeParams.goalId;
                }
                previousPage += '/profile/' + $routeParams.profileId
                return previousPage;
            } else if ($routeParams.listId) {
                return '#!/list/' + $routeParams.listId + '/profile/' + $routeParams.profileId;
            } else if ($routeParams.scoringId) {
                return '#!/scoring/' + $routeParams.scoringId + '/profile/' + $routeParams.profileId;
            } else if ($routeParams.queryID) {
                return '#!/incoming-search/' + $routeParams.queryID + '/profile/' + $routeParams.profileId;
            } else if ($routeParams.referrerId) {
                return '#!/referrer/' + $routeParams.referrerId + '/profile/' + $routeParams.profileId;
            }

            if ($routeParams.profileId) {
                return '#!/profile/' + $routeParams.profileId;
            }

            return '#!/profiles';
        };

        $scope.resetSession();
    }
})();