(function() {
    'use strict';

    angular.module('wemProfilesApp').controller('ProfilesListController', ProfilesListController);

    ProfilesListController.$injecet = ['$scope', '$uibModal', 'unomiSegmentService', 'unomiProfileService', 'unomiQueryService'];

    function ProfilesListController($scope, $uibModal, unomiSegmentService, unomiProfileService, unomiQueryService) {
        if (!unomiProfileService.getSelectedTab()) {
            unomiProfileService.setSelectedTab({
                type: 'visitors'
            });
        }
        $scope.selectedTab = unomiProfileService.getSelectedTab().type;

        $scope.someData = [];
        $scope.visitorCount = -1;
        $scope.contactCount = -1;
        $scope.leadCount = -1;

        unomiProfileService.getCount().then(function (response) {
            $scope.visitorsCount = parseInt(response.data);
        });

        unomiSegmentService.getSegmentCount('contacts').then(function (response) {
            $scope.contactsCount = parseInt(response.data);
        });

        unomiSegmentService.getSegmentCount('leads').then(function (response) {
            $scope.leadsCount = parseInt(response.data);
        });

        unomiSegmentService.getSegment('contacts').then(function (response) {
            $scope.contactsSegment = response.data;
        });

        unomiSegmentService.getSegment('leads').then(function (response) {
            $scope.leadsSegment = response.data;
        });

        function getQuery(propertyName, operator, value) {
            return {
                "aggregate": {
                    "type": "date",
                    "parameters": {
                        "interval": "1d",
                        "format": "yyyy-MM-dd"
                    }
                },
                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": propertyName,
                                    "comparisonOperator": operator,
                                    "propertyValueInteger": value
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "timeStamp",
                                    "comparisonOperator": "between",
                                    "propertyValues": ["now-60d", "now"]
                                }
                            }
                        ]
                    }
                }
            };
        }

        function parse(data) {
            var subtract = moment().subtract(30, 'days');
            var res = [0, 0];
            angular.forEach(data, function (value, key) {
                if (!(key.indexOf('_') == 0)) {
                    if (moment(key).isAfter(subtract)) {
                        res[1] += value
                    } else {
                        res[0] += value
                    }
                }
            });
            if (res[0] == 0) {
                res[2] = 0;
                res[3] = "-"
            } else {
                res[2] = (res[1] * 100 / res[0]) - 100;
                res[3] = Math.abs(res[2]).toFixed(2);
            }
            return res;
        }

        // get all sessions
        unomiQueryService.query("session", "timeStamp", getQuery("profile.properties.nbOfVisits", "greaterThan", 1)).then(function (response) {
            $scope.returningVisitors = parse(response.data);
            // get new sessions:
        });
        unomiQueryService.query("session", "timeStamp", getQuery("profile.properties.nbOfVisits", "equals", 1)).then(function (response) {
            $scope.newVisitors = parse(response.data)
        });
        unomiQueryService.query("event", "timeStamp", getQuery("eventType", "equals", "view")).then(function (response) {
            $scope.pageViews = parse(response.data)
        });

        $scope.openSearchModal = function () {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/search-profiles-modal.html',
                controller: 'SearchProfilesModalController',
                size: 'vlg'
            });

            modalInstance.result.finally(function () {
                if (unomiProfileService.getSelectedTab() && unomiProfileService.getSelectedTab().dtInstance) {
                    unomiProfileService.getSelectedTab().dtInstance.DataTable.draw();
                }
            });
        };
    }
})();
