(function() {
    'use strict';

    angular.module('wemProfilesApp').directive("wemProfilesTable", wemProfilesTableDirective);

    function wemProfilesTableDirective() {
        return {
            restrict: "E",
            scope: {
                profileType: '@'
            },
            templateUrl: function (elem, attrs) {
                if (attrs.profileType == "visitors") {
                    return ManagersContext.wemAppPath + "/wem-profiles/wem-profiles-table-visitors.directive.html";
                } else if (attrs.profileType == "personas") {
                    return ManagersContext.wemAppPath + "/wem-profiles/wem-profiles-table-personas.directive.html";
                } else {
                    return ManagersContext.wemAppPath + "/wem-profiles/wem-profiles-table-contacts.directive.html";
                }

            },
            controller: "WemProfilesTableController"
        }
    }

    angular.module('wemProfilesApp').controller('WemProfilesTableController', WemProfilesTableController);

    WemProfilesTableController.$inject = ['$rootScope', '$scope', '$resource', '$location', '$routeParams', 'i18nService', 'DTOptionsBuilder', 'DTColumnBuilder', '$uibModal', 'unomiDefinitionService', '$compile', 'unomiProfileService', 'modalService', 'unomiListService', 'dtPagerService'];

    function WemProfilesTableController($rootScope, $scope, $resource, $location, $routeParams, i18nService, DTOptionsBuilder, DTColumnBuilder, $uibModal, unomiDefinitionService, $compile, unomiProfileService, modalService, unomiListService, dtPagerService) {
        if (!unomiProfileService.getSelectedTab()) {
            unomiProfileService.setSelectedTab({
                searchCondition: {},
                type: $scope.profileType,
                deleteProfileId: $routeParams.deletedProfileId
            });
        } else {
            unomiProfileService.setSelectedTabType($scope.profileType);
            unomiProfileService.setSelectedTabDeletedProfileId($routeParams.deletedProfileId);
        }
        if (!$scope.pageLength) {
            $scope.pageLength = 25;
        }

        $scope.fnSelectedColumnsInTab = function (column) {
            if ($rootScope.selectedColumns[unomiProfileService.getSelectedTab().type] == undefined) {
                $rootScope.selectedColumns[unomiProfileService.getSelectedTab().type] = angular.copy($rootScope.selectedColumns.defaultValues);
            }
            if (!column.bVisible) {
                for (var i = 0; i < $rootScope.selectedColumns[unomiProfileService.getSelectedTab().type].length; i++) {
                    var j = $rootScope.selectedColumns[unomiProfileService.getSelectedTab().type].indexOf(column.mData);
                    if (j != -1) {
                        $rootScope.selectedColumns[unomiProfileService.getSelectedTab().type].splice(j, 1);
                    }
                }
            } else {
                $rootScope.selectedColumns[unomiProfileService.getSelectedTab().type].push(column.mData);
            }
            if (typeof(Storage) !== "undefined") {
                localStorage.selectedColumns = JSON.stringify($rootScope.selectedColumns);
            }
        };

        // LOAD condition and tags for condition builder for advanced search
        if (!unomiProfileService.getSelectedTab().conditionTypes) {
            var tags = [];
            unomiDefinitionService.getTag("profileTags").then(function (response) {
                var tagsData = response.data;
                angular.forEach(tagsData.subTags, function (v) {
                    tags[v.id] = v
                });

                unomiDefinitionService.getConditionTypesByTag("profileCondition").then(function (response) {
                    var data = response.data;
                    angular.forEach(data, function (condition) {
                        if (ManagersContext.conditionsTags && ManagersContext.conditionsTags[condition.id]) {
                            condition.profileTag = ManagersContext.conditionsTags[condition.id];
                        } else {
                            angular.forEach(condition.tags, function (tag) {
                                if (tags[tag]) {
                                    condition.profileTag = tag;
                                }
                            });
                        }
                    });

                    unomiProfileService.setSelectedTabConditionTypes(data);
                });

                unomiDefinitionService.getConditionTypesByTag("usableInPastEventCondition").then(function (response) {
                    var data = response.data;
                    angular.forEach(data, function (condition) {
                        if (ManagersContext.conditionsTags && ManagersContext.conditionsTags[condition.id]) {
                            condition.profileTag = ManagersContext.conditionsTags[condition.id];
                        } else {
                            angular.forEach(condition.tags, function (tag) {
                                if (tags[tag]) {
                                    condition.profileTag = tag;
                                }
                            });
                        }
                    });

                    unomiProfileService.setSelectedTabEventConditionTypes(data);
                });
            });
        }

        unomiProfileService.deleteSelectedTabBaseCondition();
        if (unomiProfileService.getSelectedTab().type == "contacts") {
            unomiProfileService.setSelectedTabBaseCondition({
                parameterValues: {
                    matchType: "all",
                    segments: ["contacts"]
                },
                type: "profileSegmentCondition"
            });
        } else if (unomiProfileService.getSelectedTab().type == 'leads') {
            unomiProfileService.setSelectedTabBaseCondition({
                parameterValues: {
                    matchType: "all",
                    segments: ["leads"]
                },
                type: "profileSegmentCondition"
            });
        } else if ($routeParams.segmentId) {
            unomiProfileService.setSelectedTabBaseCondition({
                parameterValues: {
                    matchType: "all",
                    segments: [$routeParams.segmentId]
                },
                type: "profileSegmentCondition"
            });
        } else if ($routeParams.campaignId) {
            var conditionPropertyName;
            if ($routeParams.goalId) {
                conditionPropertyName = "systemProperties.goals." + $routeParams.goalId + "TargetReached";
            } else {
                conditionPropertyName = "systemProperties.campaigns." + $routeParams.campaignId + "Engaged";
            }
            unomiProfileService.setSelectedTabBaseCondition({
                parameterValues: {
                    propertyName: conditionPropertyName,
                    comparisonOperator: "exists"
                },
                type: "profilePropertyCondition"
            });
        } else if ($routeParams.listId) {
            unomiProfileService.setSelectedTabBaseCondition({
                parameterValues: {
                    propertyName: "systemProperties.lists",
                    comparisonOperator: "equals",
                    propertyValue: $routeParams.listId
                },
                type: "profilePropertyCondition"
            });
        } else if ($routeParams.scoringId) {
            unomiProfileService.setSelectedTabBaseCondition({
                "parameterValues": {
                    "subConditions": [
                        {
                            parameterValues: {
                                propertyName: "scores." + $routeParams.scoringId,
                                comparisonOperator: "exists"
                            },
                            type: "profilePropertyCondition"
                        },
                        {
                            parameterValues: {
                                propertyName: "scores." + $routeParams.scoringId,
                                "comparisonOperator": "greaterThan",
                                "propertyValueInteger": 0
                            },
                            type: "profilePropertyCondition"
                        }
                    ],
                    "operator": "and"
                },
                "type": "booleanCondition"
            });
        }

        var existingProfilePropertyTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/existingProperties?tagId=profileProperties&itemType=' + ($scope.profileType == 'personas' ? "persona" : "profile"), {}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });

        $scope.loading = true;

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var aoDataCopy = angular.copy(aoData);
            // shift for remove the draw object, it's just a counter increment each time the table is draw
            aoDataCopy.shift();
            if ($scope.aoData && angular.equals($scope.aoData, aoDataCopy) && $scope.previousResult) {
                fnCallback($scope.previousResult);
                dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            }
            $scope.aoData = aoDataCopy;
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                if (prop == "id") {
                    prop = "itemId";
                } else if (prop == "scores" && $routeParams.scoringId) {
                    prop = "scores." + $routeParams.scoringId;
                } else {
                    prop = "properties." + prop;
                }
                return prop + ":" + o.dir
            }).join(",");

            var search = _.find(aoData, function (x) {
                return x.name == "search"
            }).value.value;
            if (search != '') {
                search = '*' + search.trim() + '*';
            }else {
                search = "***";
            }

            unomiProfileService.search(unomiProfileService.constructQueryOnCurrentSelectedTab(search, start, length, order), unomiProfileService.getSelectedTab().type).then(function (response) {
                var data = response.data;
                $scope.profiles = {};

                console.log("wem-profiles-table.directive.js: " + data.list.length + " profiles loaded out of a total " + data.totalSize + " profiles.");
                // copy the profile information to a format that can be used by the table component we use
                var profileTable = [];
                for (var i = 0; i < data.list.length; i++) {
                    var profileTableRow = {};
                    var currentProfile = data.list[i];
                    $scope.profiles[currentProfile.itemId] = currentProfile;
                    profileTableRow.id = currentProfile.itemId;
                    for (var j = 0; j < $scope.existingPropertyTypes.length; j++) {
                        var existingPropertyType = $scope.existingPropertyTypes[j];
                        var propertyId = existingPropertyType.metadata.id;
                        if (currentProfile.properties.hasOwnProperty(propertyId)) {
                            if (existingPropertyType.type == "date") {
                                profileTableRow[propertyId] = new Date(currentProfile.properties[propertyId]);
                            } else {
                                profileTableRow[propertyId] = currentProfile.properties[propertyId];
                            }
                        } else {
                            profileTableRow[propertyId] = "";
                        }
                    }
                    if ($routeParams.scoringId) {
                        profileTableRow['scores'] = currentProfile.scores[$routeParams.scoringId];
                    }
                    profileTable.push(profileTableRow);
                }

                var results = {
                    "sEcho": draw,
                    "iTotalRecords": data.totalSize,
                    "iTotalDisplayRecords": data.totalSize,
                    "aaData": profileTable
                };
                $scope.searchError = false;
                $scope.previousResult = results;
                fnCallback(results);
                $scope.loading = false;
                dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            },
            function () {
                $scope.searchError = true;
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": 0,
                    "iTotalDisplayRecords": 0,
                    "aaData": []
                };
                fnCallback(results);
            });
        }

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('pageLength', 25)
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 't')
            .withOption('createdRow', createdRow)
            .withFnServerData(serverData);

        function createdRow(row, data, dataIndex) {
            console.log("Row is created: " + row);
            // Recompiling so we can bind Angular directive to the DT
            $compile(angular.element(row).contents())($scope);
        }

        $scope.existingPropertyTypes = existingProfilePropertyTypesResource.get({}, function (propertyTypes) {
            console.log("wem-profiles-table.directive.js: " + propertyTypes.length + " propertyTypes loaded.");

            $scope.dtColumns = [DTColumnBuilder.newColumn('id').withTitle('ID').renderWith(function (data, type, properties, meta) {
                return ManagersUtils.getProfileDisplayName(i18nService, properties);
            })];
            if (unomiProfileService.getSelectedTab().type == 'contacts') {
                $scope.dtOptions.withOption("order", [[0, 'asc']]);
            }

            var selectedColumns = $rootScope.selectedColumns.defaultValues;
            if ($rootScope.selectedColumns[unomiProfileService.getSelectedTab().type] != undefined) {
                selectedColumns = $rootScope.selectedColumns[unomiProfileService.getSelectedTab().type]
            }

            for (var i = 0; i < propertyTypes.length; i++) {
                var propertyType = propertyTypes[i];
                var propTypeId = propertyType.metadata.id;
                var columnTitle = propertyType.metadata.name;
                var col = DTColumnBuilder.newColumn(propTypeId).withTitle(propertyType.metadata.name);
                col.bVisible = true;
                col.excludedFromSelection = false;

                if (propTypeId == "lastVisit") {
                    $scope.dtOptions.withOption("order", [[i + 1, 'desc']]);
                    col.renderWith(function (data, type, full, meta) {
                        return moment(data).fromNow();
                    })
                }

                if (!( _.contains(selectedColumns, propTypeId))) {
                    col.notVisible();
                }

                if (unomiProfileService.getSelectedTab().type == 'visitors') {
                    if (propTypeId == "lastVisit") {
                        $scope.dtOptions.withOption("order", [[i + 1, 'desc']]);
                        col.renderWith(function (data, type, full, meta) {
                            return moment(data).fromNow();
                        })
                    }
                } else if (unomiProfileService.getSelectedTab().type == 'leads') {
                    if (propTypeId == "leadAssignedTo") {
                        $scope.dtOptions.withOption("order", [[i + 1, 'asc']]);
                    }
                } else if (unomiProfileService.getSelectedTab().type == 'personas') {
                    if (propTypeId == "lastName") {
                        $scope.dtOptions.withOption("order", [[i + 1, 'asc']]);
                    }
                }
                $scope.dtColumns.push(col);
            }

            if ($routeParams.scoringId) {
                $scope.dtColumns.push(DTColumnBuilder.newColumn('scores').withTitle(i18nService.message('serverSettings.manageProfiles.score')));
                $scope.dtOptions.withOption("order", [[$scope.dtColumns.length - 1, 'desc']]);
            }

            $scope.dtColumns.push(DTColumnBuilder.newColumn(null)
                .withTitle(i18nService.message('serverSettings.manageWem.listAction'))
                .notSortable()
                .renderWith(actionsHtml));
            $scope.loading = false;
        });

        $scope.isAdvanceSearchActive = function () {
            return unomiProfileService.getSelectedTab() && unomiProfileService.getSelectedTab().searchCondition && unomiProfileService.getSelectedTab().searchCondition.type;
        };

        $scope.deleteProfile = function (profileId) {
            modalService.newAlertModal({
                title: i18nService.message("serverSettings.manageWem.delete"),
                bodyMessage: i18nService.message("serverSettings.manageProfiles.delete.confirm")
            }, function () {
                unomiProfileService.deleteProfile(profileId, unomiProfileService.getSelectedTab().type).then(function (response) {
                    unomiProfileService.setSelectedTabDeletedProfileId(profileId);
                    $scope.updateDatatable();
                });
            });
        };

        function actionsHtml(data, type, full, meta) {
            return "<button title=\"" + i18nService.message("serverSettings.manageProfiles.gotoProfile") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + full.id + "')\"><i class=\"material-icons\">mode_edit</i></button> " +
                (($scope.profileType == "personas") ? "" : "<button title=\"" + i18nService.message("serverSettings.manageLists") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"openListsModal('" + full.id + "')\" ><i class=\"material-icons\">list</i></button>") +
                "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteProfile('" + full.id + "')\"><i class=\"material-icons\">delete</i></button>";
        }

        $scope.goToEdit = function (id) {
            if ($routeParams.segmentId) {
                $location.path("/segment/" + $routeParams.segmentId + "/profile/" + id);
            } else if ($routeParams.campaignId) {
                var path = "/campaign/" + $routeParams.campaignId;
                if ($routeParams.goalId) {
                    path += "/converted/" + $routeParams.goalId;
                }
                path += "/profile/" + id;
                $location.path(path);
            } else if ($routeParams.listId) {
                $location.path("/list/" + $routeParams.listId + "/profile/" + id);
            } else if ($routeParams.scoringId) {
                $location.path("/scoring/" + $routeParams.scoringId + "/profile/" + id);
            } else if ($scope.profileType == 'personas') {
                $location.path("/persona/" + id);
            } else {
                $location.path("/profile/" + id);
            }
        };

        $scope.updateSearch = function(){
            if ($scope.dtInstance) {
                $scope.dtInstance.DataTable.search($scope.searchField).draw();
            }
            $scope.emptySearchField = ($scope.searchField == '')
        };

        $scope.$watch("pageLength", function (newValue, oldValue) {
            if (newValue && parseInt(newValue) != oldValue && oldValue && $scope.dtInstance) {
                $scope.dtInstance.DataTable.page.len(newValue).draw();
            }
        });

        $scope.updateDatatable = function () {
            $scope.dtInstance.DataTable.draw();
        };

        $scope.openListsModal = function (profileId) {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/add-to-lists-modal.html',
                controller: 'AddToListsModalController',
                size: 'sm',
                resolve: {
                    "lists": function ($q) {
                        var lists = $q.defer();
                        unomiListService.query().then(function (response) {
                            lists.resolve(response.data.list);
                        });
                        return lists.promise;
                    },
                    "selectedProfile": function () {
                        return $scope.profiles[profileId];
                    }
                }
            });
            modalInstance.result.then(function (profile) {
                // refresh table in case we are in the list members and list have been removed for the given profile
                if ($routeParams.listId && $.inArray($routeParams.listId, profile.systemProperties.lists) == -1) {
                    unomiProfileService.setSelectedTabDeletedProfileId(profileId);
                    $scope.updateDatatable();
                }
            })
        };

        $scope.openSearchModal = function () {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/search-profiles-modal.html',
                controller: 'SearchProfilesModalController',
                size: 'vlg'
            });

            modalInstance.result.finally(function () {
                if (unomiProfileService.getSelectedTab() && unomiProfileService.getSelectedTab().dtInstance) {
                    unomiProfileService.getSelectedTab().dtInstance.DataTable.draw()
                }
            });
        };

        $scope.openSaveAsModal = function (type) {
            $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/save-as-modal-' + type + '.html',
                controller: 'SaveAsModalController',
                size: 'lg',
                resolve: {
                    "type": function () {
                        return type;
                    }
                }
            });
        };

        $scope.cancelSearch = function () {
            unomiProfileService.setSelectedTabSearchCondition({});
            $scope.updateDatatable();
        };

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            unomiProfileService.getSelectedTab().dtInstance = instance;
        };
    }
})();