(function() {
    'use strict';

    angular.module('wemRulesApp').controller('RulesListController', RulesListController);

    RulesListController.$inject = ['$location', 'unomiRulesService'];

    function RulesListController($location, unomiRulesService) {

        var vm = this;

        vm.refreshRules = refreshRules;
        vm.toggleHidden = toggleHidden;
        vm.createSimpleRule = createSimpleRule;
        vm.createAdvancedRule = createAdvancedRule;
        vm.editRule = editRule;
        vm.incrementExpertModeClicks = incrementExpertModeClicks;
        vm.getRuleExecutionCount = getRuleExecutionCount;

        _init();

        // public functions

        function refreshRules(event) {
            unomiRulesService.getAll(100, 0, null).then(function(response) {
                vm.rules = _.filter(response.data.list, function (rule) {
                    var simpleRule = false;
                    if (rule.tags && _.contains(rule.tags, 'simpleRule')) {
                        simpleRule = true;
                    }
                    if (vm.expertMode) {
                        if (vm.showHiddenRules) {
                            return filterByEnabled(rule);
                        } else {
                            return !rule.hidden && filterByEnabled(rule);
                        }
                    } else {
                        return !rule.hidden && simpleRule && filterByEnabled(rule);
                    }
                });
                unomiRulesService.getStatistics().then(function (response) {
                    vm.ruleStatistics = response.data;
                });
            });
        }

        function toggleHidden(event) {
            vm.showHiddenRules = !vm.showHiddenRules;
            vm.refreshRules(event);
        }

        function createSimpleRule($event) {
            $location.path("/rule/simple/newSimpleRule");
        }

        function createAdvancedRule($event) {
            $location.path("/rule/advanced/newAdvancedRule");
        }

        function editRule(rule) {
            if (rule.tags && _.contains(rule.tags, 'simpleRule')) {
                $location.path('/rule/simple/' + rule.id);
            } else {
                $location.path('/rule/advanced/' + rule.id);
            }
        }

        function incrementExpertModeClicks() {
            if (!vm.expertMode) {
                vm.expertModeClicks++;
                if (vm.expertModeClicks > 5) {
                    vm.expertMode = true;
                }
            }
        }

        function getRuleExecutionCount(ruleId) {
            if (vm.ruleStatistics && vm.ruleStatistics[ruleId]) {
                return vm.ruleStatistics[ruleId].executionCount;
            } else {
                return 0;
            }
        }

        // private functions

        function _init() {
            vm.expertModeClicks = 0;
            vm.expertMode = false;
            vm.showHiddenRules = false;
            vm.rulesToDisplay = "all";

            vm.refreshRules();
        }

        function filterByEnabled(rule) {
            if (vm.rulesToDisplay === "all") {
                return true;
            } else if (vm.rulesToDisplay === "active") {
                return rule.enabled;
            } else if (vm.rulesToDisplay === "inactive") {
                return !rule.enabled;
            }
            return true;
        }
    }
})();
