(function() {
    'use strict';

    angular.module('wemScoringApp').controller('ScoringEditController', ScoringEditController);

    ScoringEditController.$inject = ['$scope', '$rootScope', '$resource', '$location', '$route', '$q','unomiProfileService', 'unomiRulesService', 'unomiScoringService','currentScoring', 'newScoring', 'unomiDefinitionService', 'unomiQueryService', 'i18nService', 'notificationService', 'loadingSpinnerService'];

    function ScoringEditController($scope, $rootScope, $resource, $location, $route, $q, unomiProfileService, unomiRulesService, unomiScoringService, currentScoring, newScoring, unomiDefinitionService, unomiQueryService, i18nService, notificationService, loadingSpinnerService) {
        $scope.currentScoring = currentScoring;
        $scope.conditionTypes = [];
        $scope.sharedScoring = currentScoring.metadata.scope != ManagersContext.dmfScope;
        $scope.newScoring = newScoring || false;
        $scope.cannotShareTooltip = i18nService.message('serverSettings.manageScoring.sharedCannotEdit');

        var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition"}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });
        var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

        $scope.tags = {};

        $rootScope.allowedTags = ['aggregated', 'demographic', 'event', 'commerceFactory'];

        $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value) {
            console.log("scoring-edit.controller.js : tags loaded from Apache Unomi.");

            angular.forEach(value.subTags, function (v) {
                $scope.tags[v.id] = v;
            });
            unomiDefinitionService.fillTags("conditionTypes", $scope);
            unomiDefinitionService.fillTags("eventConditionTypes", $scope);
            $scope.allowedConditionTypes = getAllowedConditions();
        });

        var getAllowedConditions = function () {
            $scope.$watch("conditionTypes.length", function (newVal, oldVal, scope) {
                if(!$scope.allowedConditionTypes){
                    $scope.allowedConditionTypes = [];
                }
                angular.forEach($scope.conditionTypes, function (ct) {
                    if ($rootScope.allowedTags.indexOf(ct.profileTag) !== -1) {
                        ct.name_tr = i18nService.message('condition.' + ct.id + '.name');
                        ct.profileTag_tr = i18nService.message('tag.' + ct.profileTag + '.name');
                        $scope.allowedConditionTypes.push(ct);
                    }
                });
            });
        };

        $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition"}, function (conditionTypes) {
            console.log("scoring-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
        });

        $scope.scoringType = _.find($scope.currentScoring.metadata.tags, function (tag) {
                                return tag.indexOf('st:') !== -1;
                            });
        /*if(!$scope.scoringType){
            $scope.scoringType = "st:all";
        }*/

        $scope.scoringTypeChange = function (){
            $scope.currentScoring.metadata.tags = _.filter($scope.currentScoring.metadata.tags, function (tag) {
                return tag && tag.indexOf(':') === -1;
            });
            $scope.currentScoring.metadata.tags.push($scope.scoringType);
        };
        $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"}, function (conditionTypes) {
            console.log("scoring-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
        });

        unomiScoringService.getImpacted(currentScoring.metadata.id).then(function(response){
            $scope.dependents = response.data;
        });

        $scope.resetScoring = function ($event) {
            unomiScoringService.getScoring(currentScoring.metadata.id).then(function (response) {
                currentScoring = response.data;
                $route.reload();
            });
        };

        $scope.copyScoring = function ($event) {
            unomiScoringService.scoringToCopy = angular.copy(currentScoring);
            $location.path("/newScoring");
        };

        $scope.hasError = function(propertyName) {
            return ManagersUtils.hasErrors($scope.scoringForm, $scope.scoringForm, propertyName);
        };

        $scope.saveScoring = function (scoringForm) {
            if(scoringForm.$invalid){
                scoringForm.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm(scoringForm);
                if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            var save = function () {
                if ($scope.newScoring) {
                    unomiScoringService.setScoring($scope.currentScoring, $scope.sharedScoring).then(function () {
                        $location.path("/scoring");
                    });
                } else {
                    unomiScoringService.setScoring($scope.currentScoring, $scope.sharedScoring).then(function () {
                        if (!$scope.newScoring) {
                            var update = unomiScoringService.buildUpdateBatchRemoveScoring($scope.currentScoring.metadata.id);
                            unomiProfileService.batchProfilesUpdate(update);
                        }
                        $location.path("/scoring");
                    });
                }
            };

            loadingSpinnerService.show();
            if ($scope.newScoring) {
                unomiQueryService.createWithCallbackIfDoesNotExist('scoring', ManagersContext.dmfScope, $scope.currentScoring.metadata.name, null,
                    "serverSettings.manageScoring.create.alreadyExisting", $scope.currentScoring.metadata.name, save, function() {
                        loadingSpinnerService.hide();
                    });
            } else {
                save();
            }
        };

        $scope.deleteCurrent = function () {
            var id = $scope.currentScoring.metadata.id;
            var rulePromise = unomiRulesService.deleteRule("sendEmailRulePrefix_" + $scope.currentScoring.metadata.id).then(function(response){
                var update = unomiScoringService.buildUpdateBatchRemoveScoring($scope.currentScoring.metadata.id);
                unomiProfileService.batchProfilesUpdate(update);
            });
            $q.all([rulePromise]).then(function () {
                $scope.tryDeleteItem($scope.currentScoring.metadata, function() {
                    unomiScoringService.deletedItem = id;
                    $location.path('/scoring');
                });
            });
        };

        $scope.deleteScoringElement = function (index) {
            $scope.currentScoring.elements.splice(index, 1);
        };

        $scope.duplicateScoringElement = function (index) {
            $scope.currentScoring.elements.splice(index + 1, 0, angular.copy($scope.currentScoring.elements[index]));
        };

        $scope.addScoringElement = function () {
            if (!$scope.currentScoring.elements) {
                $scope.currentScoring.elements = [];
            }
            $scope.currentScoring.elements.push({condition: {}, value: 1});
        };
    }
})();
