(function() {
    'use strict';

    angular.module('wemScoringApp').controller('ScoringListController', ScoringListController);

    ScoringListController.$inject = ['$scope', 'unomiScoringService', 'unomiQueryService', '$location', 'DTOptionsBuilder', 'DTColumnBuilder', 'i18nService', '$compile', 'dtPagerService', 'loadingSpinnerService'];

    function ScoringListController($scope, unomiScoringService, unomiQueryService, $location, DTOptionsBuilder, DTColumnBuilder, i18nService, $compile, dtPagerService, loadingSpinnerService) {
        $scope.items = {};
        $scope.stopPolling = true;

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                if (prop == "scoringType") {
                    prop = "metadata.tags";
                } else {
                    prop = "metadata." + prop;
                }
                return prop + ":" + o.dir
            }).join(",");

            unomiScoringService.query(unomiScoringService.constructQuery(length, start, order)).then(function (response) {
                fnCallback(getResult(draw, response.data, unomiScoringService));
                dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            },
            function () {
                fnCallback(getErrorResult());
            });
        }

        var getResult = function(draw, data, service) {
            angular.forEach(data.list, processItem);

            $scope.stopPolling = false;

            return {
                "sEcho": draw,
                "iTotalRecords": data.totalSize,
                "iTotalDisplayRecords": data.totalSize,
                "aaData": data.list
            };
        };

        var getErrorResult = function (draw) {
            $scope.searchError = true;
            return {
                "sEcho": draw,
                "iTotalRecords": 0,
                "iTotalDisplayRecords": 0,
                "aaData": []
            };
        };

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('lengthMenu', [ 25, 50, 100 ])
            .withOption('pageLength', 25)
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 'lt')
            .withOption('aaSorting', [])
            .withOption('createdRow', createdRow)
            .withFnServerData(serverData)
        ;

        $scope.dtColumns = [
            DTColumnBuilder
                .newColumn("name")
                .withTitle(i18nService.message("serverSettings.manageScoring.scoringName"))
                .renderWith(renderNameCol)
                .withOption("width","35%"),
            DTColumnBuilder
                .newColumn("scoringType")
                .withTitle(i18nService.message("serverSettings.manageScoring.type"))
                .withOption("width","10%"),
            DTColumnBuilder
                .newColumn("description")
                .withTitle(i18nService.message("serverSettings.manageWem.description"))
                .withOption("width","40%"),
            DTColumnBuilder
                .newColumn(null)
                .withTitle(i18nService.message("serverSettings.manageWem.listAction"))
                .notSortable()
                .renderWith(actionsHtml)
                .withOption("width","15%")
                .withOption("contentPadding","0")
        ];

        $scope.setDtInstance = function(instance) {
            $scope.dtInstance = instance;
        };

        function createdRow(row, data, dataIndex) {
            console.log("Row is created: "+row);
            // Recompiling so we can bind Angular directive to the DT
            $compile(angular.element(row).contents())($scope);
        }

        function actionsHtml(data, type, full, meta) {
            return "<button title=\"" + i18nService.message("serverSettings.manageScoring.editScoring") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + data.id + "')\"><i class=\"material-icons\">mode_edit</i></button> " +
                "<button title=\"" + i18nService.message("serverSettings.manageScoring.manageMembers") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToProfiles('" + data.id + "')\" ><i class=\"material-icons\">person</i></button>" +
                "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteItem('" + data.id + "')\"><i class=\"material-icons\">delete</i></button>";
        }

        $scope.goToEdit = function(id) {
            $scope.stopPolling = true;
            $location.path( "/scoring/" +id );
        };

        $scope.goToProfiles = function(id) {
            $scope.stopPolling = true;
            $location.path( "/scoring/" +id + "/profiles" );
        };

        function renderNameCol(data, type, full, meta) {
            var result = data + ' <span class="badge deep-purple-bg">{{items["' + full.id + '"].count}}</span>';
            if(full.isShared) {
                result += ' <span class="badge  mf-pink-bg">' + i18nService.message("serverSettings.manageScoring.shared") + '</span>'
            }
            if(!full.enabled) {
                result += ' <span> [' + i18nService.message("serverSettings.manageWem.disabled") + ']</span>'
            }
            return result;
        }

        function getCount (scoringId) {
            var scoringCondition = {
                "parameterValues": {
                    "subConditions": [
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "mergedWith",
                                comparisonOperator: "missing"
                            }
                        },
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "systemProperties.isAnonymousProfile",
                                comparisonOperator: "missing"
                            }
                        },
                        {
                            parameterValues: {
                                propertyName:"scores." + scoringId,
                                comparisonOperator: "exists"
                            },
                            type: "profilePropertyCondition"
                        },
                        {
                            parameterValues: {
                                propertyName:"scores." + scoringId,
                                comparisonOperator: "greaterThan",
                                propertyValueInteger: 0
                            },
                            type: "profilePropertyCondition"
                        }
                    ],
                    "operator": "and"
                },
                "type": "booleanCondition"
            };

            var observable =Rx.Observable.create( function(o){
                unomiQueryService.getMatchingItemsCountForCondition('profile', scoringCondition)
                .then(function(response){
                    o.onNext(response.data)
                    o.onCompleted();
                })
            });

            return observable.merge(
                    Rx.Observable
                        .interval(5000)
                        .flatMapLatest(observable)
                        .takeWhile(function (x) { return !$scope.stopPolling; })
                );
        }

        var processItem = function (itemMetadata) {
            $scope.items[itemMetadata.id] = itemMetadata;
            itemMetadata.isShared = itemMetadata.scope != ManagersContext.dmfScope;
            itemMetadata.count = 0;
            if(_.isEmpty(itemMetadata.tags)){
                itemMetadata.scoringType = i18nService.message("serverSettings.manageScoring.type.all");
            }else{
                //tag.demographicProfileProperties.name
                var type = _.find(itemMetadata.tags, function (tag) {
                    if(tag.indexOf('st:')>=0){
                        return true;
                    }
                }).split(':')[1];
                if(type){
                    itemMetadata.scoringType = i18nService.message("tag."+type+".name");
                }else{
                    itemMetadata.scoringType = "";
                }
            }
            $scope.items[itemMetadata.id].observable = getCount(itemMetadata.id).subscribe(function(count){
                $scope.items[itemMetadata.id].count = count;
            });
        };

        $scope.deleteItem = function (id) {
            $scope.tryDeleteItem($scope.items[id], function() {
                unomiScoringService.deletedItem = id;
                $scope.dtInstance.DataTable.draw();
            });
        };
    }
})();
