(function() {
    'use strict';

    angular.module('wemScoringApp').config(wemScoringAppConfig);

    wemScoringAppConfig.$inject = ['$routeProvider'];

    function wemScoringAppConfig($routeProvider) {
        $routeProvider
            .when('/scoring', {
                templateUrl: ManagersContext.wemAppPath + '/wem-scoring/scoring-list.html',
                controller: 'ScoringListController',
                reloadOnSearch: false
            })
            .when('/scoring/:scoringId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-scoring/scoring-edit.html',
                controller: 'ScoringEditController',
                resolve: {
                    "currentScoring": function ($q, $route, unomiScoringService) {
                        var scoring = $q.defer();
                        unomiScoringService.getScoring($route.current.params.scoringId).then(function (response) {
                            scoring.resolve(response.data);
                        });
                        return scoring.promise;
                    },
                    "newScoring": function () {
                        return false;
                    }
                }
            })
            .when('/newScoring', {
                templateUrl: ManagersContext.wemAppPath + '/wem-scoring/scoring-edit.html',
                controller: 'ScoringEditController',
                resolve: {
                    "currentNotifRule": function () {
                        return {};
                    },
                    "currentScoring": function ($route, unomiScoringService, i18nService) {
                        var scoring = {};
                        if (unomiScoringService.scoringToCopy) {
                            scoring = unomiScoringService.scoringToCopy;
                            scoring.metadata.id = ManagersUtils.generateId();
                            scoring.metadata.name = i18nService.format('serverSettings.manageScoring.duplicate.name', unomiScoringService.scoringToCopy.metadata.name);
                            scoring.metadata.tags = [];
                            delete unomiScoringService.scoringToCopy;
                        } else {
                            scoring = {
                                metadata: {
                                    id: ManagersUtils.generateId(),
                                    scope: ManagersContext.dmfScope,
                                    enabled: true
                                },
                                elements: [
                                    {
                                        condition: {},
                                        value: 1
                                    }
                                ]
                            };
                        }
                        return scoring;
                    },
                    "newScoring": function () {
                        return true;
                    }
                }
            })
            .when('/scoring/:scoringId/profiles', {
                templateUrl: ManagersContext.wemAppPath + '/wem-scoring/profiles-list.html',
                controller: function ($scope) {
                    $scope.profileType = 'scoring';
                },
                resolve: {
                    // resolve scoring here to avoid async issue using deferred API
                    "scoring": function ($q, $route, unomiScoringService) {
                        var scoring = $q.defer();
                        unomiScoringService.getScoring($route.current.params.scoringId).then(function (response) {
                            scoring.resolve(response.data);
                        });
                        return scoring.promise;
                    }
                }
            })
            .when('/scoring/:scoringId/profile/:profileId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/manage-profile.html',
                controller: 'ManageProfileController',
                resolve: {
                    // resolve profile here to avoid async issue using deferred API
                    "profile": function ($q, $route, unomiProfileService) {
                        var profile = $q.defer();
                        unomiProfileService.getProfile($route.current.params.profileId).then(function (response) {
                            profile.resolve(response.data);
                        });
                        return profile.promise;
                    }
                }
            })
            .when('/scoring/:scoringId/profile/:profileId/:sessionId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/profile-session.html',
                controller: 'ProfileSessionController'
            })
            .otherwise({
                redirectTo: '/scoring'
            });
    }
})();
