(function() {
    'use strict';

    angular.module('wemSegmentsApp').controller('SegmentEditController', SegmentEditController);

    SegmentEditController.$inject = ['$scope', '$resource', '$location', '$route', 'unomiSegmentService', 'unomiListService', 'currentSegment', 'newSegment', 'unomiDefinitionService', 'unomiQueryService', 'i18nService', 'notificationService', 'loadingSpinnerService', '_'];

    function SegmentEditController($scope, $resource, $location, $route, unomiSegmentService, unomiListService, currentSegment, newSegment, unomiDefinitionService, unomiQueryService, i18nService, notificationService, loadingSpinnerService, _) {
        $scope.isStaticList = ManagersContext.isStaticList != undefined ? ManagersContext.isStaticList : false;
        $scope.currentSegment = currentSegment;
        $scope.deleteAllowed = !_.contains(['contacts', 'leads'], currentSegment.metadata.id);
        $scope.conditionTypes = [];
        $scope.sharedSegment = currentSegment.metadata.scope != ManagersContext.dmfScope;
        $scope.newSegment = newSegment || false;
        $scope.cannotShareTooltip = i18nService.message('serverSettings.manageSegments.sharedCannotEdit');

        if(!$scope.isStaticList) {
            var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition"}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

            $scope.tags = {};

            $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value, responseHeaders) {
                console.log("segment-edit.controller.js : tags loaded from Apache Unomi.");

                angular.forEach(value.subTags, function (v) {
                    $scope.tags[v.id] = v
                });
                unomiDefinitionService.fillTags("conditionTypes", $scope);
                unomiDefinitionService.fillTags("eventConditionTypes", $scope);
            });

            $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition"}, function (conditionTypes) {
                console.log("segment-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
            });
            $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"}, function (conditionTypes) {
                console.log("segment-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
            });

            unomiSegmentService.getImpacted(currentSegment.metadata.id).then(function(data){
                $scope.dependents = data;
            });

            $scope.resetSegment = function ($event) {
                unomiSegmentService.getSegment(currentSegment.metadata.id).then(function (segmentData) {
                    currentSegment = segmentData;
                    $route.reload();
                });
            };

            $scope.copySegment = function ($event) {
                unomiSegmentService.segmentToCopy = angular.copy(currentSegment);
                $location.path("/newSegment");
            };

            $scope.hasError = function(propertyName) {
                return ManagersUtils.hasErrors($scope.segmentForm, $scope.segmentForm, propertyName);
            };
        }

        $scope.saveSegment = function (segmentForm) {
            if(segmentForm.$invalid){
                segmentForm.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm(segmentForm);
                if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            var type = $scope.isStaticList ? 'userList' : 'segment';
            var errorKey = $scope.isStaticList ? "serverSettings.manageLists.create.alreadyExisting" : "serverSettings.manageSegments.create.alreadyExisting";

            var save = function () {
                if ($scope.isStaticList) {
                    unomiListService.setList($scope.currentSegment, $scope.sharedSegment).then(function () {
                        $location.path("/lists")
                    });
                } else {
                    unomiSegmentService.setSegment($scope.currentSegment, $scope.sharedSegment).then(function () {
                        $location.path("/segments")
                    });
                }
            };

            loadingSpinnerService.show();
            if ($scope.newSegment) {
                unomiQueryService.createWithCallbackIfDoesNotExist(type, ManagersContext.dmfScope, $scope.currentSegment.metadata.name, null,
                    errorKey, $scope.currentSegment.metadata.name, save, function() {
                        loadingSpinnerService.hide();
                    });
            } else {
                save();
            }
        };

        $scope.deleteCurrent = function () {
            var id = $scope.currentSegment.metadata.id;
            $scope.tryDeleteItem($scope.currentSegment.metadata, $scope.isStaticList,
            function() {
                $location.path('/lists');
            }, function() {
                unomiSegmentService.deletedItem = id;
                $location.path('/segments');
            });
        };
    }
})();
