(function() {
    'use strict';

    angular.module('wemSegmentsApp').config(wemSegmentsAppConfig);

    wemSegmentsAppConfig.$inject = ['$routeProvider'];

    function wemSegmentsAppConfig($routeProvider) {
        $routeProvider
            .when('/segments', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segments-list.html',
                controller: 'SegmentsListController',
                reloadOnSearch: false
            })
            .when('/segment/:segmentId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segment-edit.html',
                controller: 'SegmentEditController',
                resolve: {
                    // resolve rule here to avoid async issue using deferred API
                    "currentSegment": function( $q, $route, unomiSegmentService) {
                        var segment = $q.defer();
                        unomiSegmentService.getSegment($route.current.params.segmentId).then(function (response) {
                            segment.resolve(response.data);
                        });
                        return segment.promise;
                    },
                    "newSegment": function () {
                        return false;
                    }
                }
            })
            .when('/newSegment', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segment-edit.html',
                controller: 'SegmentEditController',
                resolve: {
                    "currentSegment": function ($route, unomiSegmentService, i18nService) {
                        var segment = {};
                        if(unomiSegmentService.segmentToCopy){
                            segment = unomiSegmentService.segmentToCopy;
                            segment.metadata.id = ManagersUtils.generateId();
                            segment.metadata.name = i18nService.format('serverSettings.manageSegments.duplicate.name', unomiSegmentService.segmentToCopy.metadata.name);
                            delete unomiSegmentService.segmentToCopy;
                        } else {
                            segment = {
                                metadata: {
                                    id: ManagersUtils.generateId(),
                                    scope: ManagersContext.dmfScope,
                                    enabled: true
                                },
                                condition: {}
                            }
                        }
                        return segment;
                    },
                    "newSegment": function () {
                        return true;
                    }
                }
            })
            .when('/segment/:segmentId/profiles', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segment-profiles-list.html',
                controller: function($scope, unomiProfileService) {
                    $scope.profileType='segment';
                }
            })
            .when('/segment/:segmentId/profile/:profileId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/manage-profile.html',
                controller: 'ManageProfileController',
                resolve: {
                    // resolve profile here to avoid async issue using deferred API
                    "profile": function ($q, $route, unomiProfileService) {
                        var profile = $q.defer();
                        unomiProfileService.getProfile($route.current.params.profileId).then(function (response) {
                            profile.resolve(response.data);
                        });
                        return profile.promise;
                    }
                }
            })
            .when('/segment/:segmentId/profile/:profileId/:sessionId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/profile-session.html',
                controller: 'ProfileSessionController'
            })
            .otherwise({
                redirectTo: '/segments'
            });
    }
})();
