(function() {
    'use strict';

    angular.module('wemSegmentsApp').run(wemSegmentsAppRun);

    wemSegmentsAppRun.$inject = ['$rootScope', '$timeout', 'i18nService', 'modalService', 'unomiListService', 'unomiSegmentService', 'loadingSpinnerService'];

    function wemSegmentsAppRun($rootScope, $timeout, i18nService, modalService, unomiListService, unomiSegmentService, loadingSpinnerService) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.tryDeleteItem = function (segmentOrList, isStaticList, staticListCB, segmentCB) {
            // if the segment we're trying to delete is shared, display a modal to ask confirmation
            if (segmentOrList.scope != ManagersContext.dmfScope) {
                modalService.newAlertModal({
                    title: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.sharedWarning.title" : "serverSettings.manageSegments.delete.sharedWarning.title", segmentOrList.name),
                    bodyMessage: i18nService.message(isStaticList ? "serverSettings.manageLists.delete.sharedWarning.message" : "serverSettings.manageSegments.delete.sharedWarning.message")
                }, function () {
                    if(isStaticList) {
                        deleteList(segmentOrList, staticListCB);
                    } else {
                        deleteSegment(segmentOrList, segmentCB);
                    }
                });
            } else {
                modalService.newAlertModal({
                    title: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.title" : "serverSettings.manageSegments.delete.title", segmentOrList.name),
                    bodyMessage: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.message" : "serverSettings.manageSegments.delete.message", segmentOrList.name)
                }, function () {
                    if(isStaticList) {
                        deleteList(segmentOrList, staticListCB);
                    } else {
                        deleteSegment(segmentOrList, segmentCB);
                    }
                });
            }
        };

        var deleteList = function (list, callback) {
            loadingSpinnerService.show();
            unomiListService.deleteList(list.id);
            $timeout(function () {
                loadingSpinnerService.hide();
                if (callback) {
                    callback();
                }
            }, 1000);
        };

        var deleteSegment = function(segment, callback) {
            loadingSpinnerService.show();
            unomiSegmentService.deleteSegment(segment.id, true).then(function (response) {
                var data = response.data;
                // if data isn't empty, it contains the list of the segments that use the segment we're trying to delete as a condition
                if (data.segments.length > 0 || data.scorings.length > 0) {
                    loadingSpinnerService.hide();
                    // display the list of dependent segments and ask what to do
                    modalService.newAlertModal({
                        title: i18nService.format("serverSettings.manageSegments.delete.dependentsModal.title", segment.name),
                        bodyTemplateUrl: ManagersContext.wemAppPath + "/wem-segments/segment-delete-dependents-modal-body.html",
                        segmentName: segment.name,
                        dependents: data
                    }, function () {
                        loadingSpinnerService.show();
                        unomiSegmentService.deleteSegment(segment.id, false);
                        $timeout(function () {
                            loadingSpinnerService.hide();
                            if (callback) {
                                callback();
                            }
                        }, 1000);
                    });
                } else {
                    $timeout(function () {
                        loadingSpinnerService.hide();
                        if (callback) {
                            callback();
                        }
                    }, 1000);
                }
            }, function() {
                loadingSpinnerService.hide();
                if (callback) {
                    callback();
                }
            });
        }
    }
})();
