(function() {
    'use strict';

    angular.module('wemSettingsApp').service('wemSettingsService', wemSettingsService);

    wemSettingsService.$inject = ['$http', 'i18nService', 'notificationService'];

    function wemSettingsService($http, i18nService, notificationService) {
        var _settingsUrlAction = ManagersContext.basePreview + ManagersContext.currentSiteHomePagePath + '.wemSettings.do';

        return {
            getSettings: getSettings,
            saveSettings: saveSettings,
            deleteSettings: deleteSettings,
            tryPurgeScope: tryPurgeScope,
            handleErrors: handleErrors
        };

        function getSettings() {
            return $http.get(_settingsUrlAction);
        }

        function saveSettings(settings) {
            return $http.post(_settingsUrlAction, settings);
        }

        function deleteSettings() {
            return $http.post(ManagersContext.basePreview + ManagersContext.currentSiteHomePagePath + '.removeWemSettings.do');
        }

        function tryPurgeScope(contextServerScope) {
            return $http.delete(ManagersContext.proxyServletUrl + '/cxs/cluster/' + contextServerScope);
        }

        function handleErrors(errorMsg, data, contextServerURL, contextServerUsername) {
            var exceptionToKey = {
                UnknownHostException: {
                    key: 'serverSettings.manageSettings.save.error.unknownHost',
                    param: contextServerURL
                },
                '401': {
                    key: 'serverSettings.manageSettings.save.error.unauthorized',
                    param: contextServerUsername ? contextServerUsername : '__void__'
                },
                SSLHandshakeException: {
                    key: 'serverSettings.manageSettings.save.error.ssl',
                    param: undefined
                }
            };

            // check if we do special handling of the returned exception
            var detail = exceptionToKey[data.type];
            var detailMsg;
            if (detail) {
                detailMsg = ': ' + i18nService.format(detail.key, detail.param);
            } else {
                // if we have an error append it to the default error message
                detailMsg = (data.error ? ': ' + data.error : '');
            }
            errorMsg = errorMsg + detailMsg;
            notificationService.errorToast(errorMsg);
        }
    }
})();