<%@ taglib prefix="jcr" uri="http://www.jahia.org/tags/jcr" %>
<%@ taglib prefix="fmt" uri="http://java.sun.com/jsp/jstl/fmt" %>
<%@ taglib prefix="c" uri="http://java.sun.com/jsp/jstl/core" %>
<%@ taglib prefix="utility" uri="http://www.jahia.org/tags/utilityLib" %>
<%@ taglib prefix="template" uri="http://www.jahia.org/tags/templateLib" %>
<%@ taglib prefix="fn" uri="http://java.sun.com/jsp/jstl/functions" %>
<%@ taglib prefix="functions" uri="http://www.jahia.org/tags/functions" %>
<%@ taglib prefix="wem" uri="http://www.jahia.org/tags/wem" %>
<%--@elvariable id="currentNode" type="org.jahia.services.content.JCRNodeWrapper"--%>
<%--@elvariable id="out" type="java.io.PrintWriter"--%>
<%--@elvariable id="script" type="org.jahia.services.render.scripting.Script"--%>
<%--@elvariable id="scriptInfo" type="java.lang.String"--%>
<%--@elvariable id="workspace" type="java.lang.String"--%>
<%--@elvariable id="renderContext" type="org.jahia.services.render.RenderContext"--%>
<%--@elvariable id="currentResource" type="org.jahia.services.render.Resource"--%>
<%--@elvariable id="url" type="org.jahia.services.render.URLGenerator"--%>
<%--@elvariable id="subchild" type="org.jahia.services.content.JCRNodeWrapper"--%>


<template:include view="editToolbar">
    <template:param name="arrowColor" value="green"/>
    <template:param name="editableVariants" value="${editableModule}"/>
</template:include>

<c:if test="${renderContext.liveMode or renderContext.previewMode}">
    <c:if test="${not empty currentNode.properties['wem:controlVariant'].string}">
        <%-- Fallback variant is explicitly specified --%>
        <c:set var="controlVariant" value="${currentNode.properties['wem:controlVariant'].node}"/>
    </c:if>
    <c:set var="optimizationActive" value="${wem:isOptimizationActive(currentNode)}"/>
    <c:set var="ajaxRender" value="${empty currentNode.properties['wem:ajaxRender'] or currentNode.properties['wem:ajaxRender'].boolean}"/>

    <c:if test="${optimizationActive}">
        <template:addResources type="css" resources="marketing-factory/optimized-content.css"/>
        <c:set var="elementID" value="wemoptimized-${currentNode.identifier}" />
        <c:set var="parsedId" value="${fn:replace(currentNode.identifier,'-','_')}"/>

        <c:choose>
            <c:when test="${not empty currentNode.properties['wem:variantsTraffic'].string}">
                <c:set var="variantsTraffic" value="${currentNode.properties['wem:variantsTraffic'].string}"/>
            </c:when>
            <c:otherwise>
                <c:set var="variantsTraffic" value="undefined"/>
            </c:otherwise>
        </c:choose>

        <div id="${elementID}">
            <c:if test="${not ajaxRender}">
                <c:forEach items="${moduleMap.variants}" var="subchild" varStatus="st">
                    <div style="display: none" id="wem${parsedId}${st.index}">
                        <template:module node="${subchild}" nodeTypes="${requestScope[constraints] == 'jmix:droppableContent' ? '' : requestScope[constraints]}" />
                    </div>
                </c:forEach>
            </c:if>
        </div>

        <script type="text/javascript" id="${currentNode.identifier}">

            if(!optimizedContentAreas){
                var optimizedContentAreas = {};
            }

            (function(){
                optimizedContentAreas['${currentNode.identifier}'] = { 'name': '${currentNode.displayableName}', 'variants': {
                    <c:forEach items="${moduleMap.variants}" var="subchild" varStatus="varStatus">
                    '${subchild.identifier}' : {
                        "variantWrapper": "${currentNode.identifier}",
                        "variantWrapperPath": "${currentNode.path}",
                        "variantPath": "${subchild.path}",
                        "name": "${subchild.name}",
                        "pagePath": "${jcr:getParentOfType(currentNode,"jnt:page").path}",
                        <c:if test="${not ajaxRender}">
                        "content": "wem${parsedId}${varStatus.index}"
                        </c:if>
                        <c:if test="${ajaxRender}">
                        "content": "<c:url value="${url.base}${functions:escapePath(subchild.path)}.${functions:default(subchild.properties['j:view'].string, 'default')}.html.ajax"/>"
                        </c:if>
                    }${!varStatus.last ? ',' : ''}
                    </c:forEach>
                }};

                if(window.wem) {
                    window.wem.registerOptimizationTest("${currentNode.identifier}", "${currentNode.properties['wem:goalId'].string}", "${elementID}", optimizedContentAreas['${currentNode.identifier}'].variants, ${ajaxRender}, ${variantsTraffic});
                } else {
                    <c:if test="${not ajaxRender}">
                        document.getElementById(optimizedContentAreas['${currentNode.identifier}'].variants['${controlVariant.identifier}'].content).style.display = '';
                    </c:if>
                    <c:if test="${ajaxRender}">
                        var xhr = new XMLHttpRequest();
                        var finished = false;
                        xhr.onabort = xhr.onerror = function xhrError() {
                            finished = true;
                        };

                        xhr.onreadystatechange = function xhrStateChange() {
                            if (xhr.readyState === 4 && !finished) {
                                finished = true;
                                try {
                                    document.getElementById('${elementID}').innerHTML = xhr.responseXML.getElementsByTagName('body')[0].innerHTML;
                                } catch (e) {
                                    console.log(e);
                                }
                            }
                        };

                        xhr.open('GET', '<c:url value="${url.base}${functions:escapePath(controlVariant.path)}.${functions:default(controlVariant.properties['j:view'].string, 'default')}.html.ajax"/>');
                        xhr.responseType = 'document';
                        xhr.send();
                    </c:if>
                    console.log("No wem available in page, can't register optimization.")
                }
            })();
        </script>
    </c:if>
    <c:if test="${not optimizationActive}">
        <c:if test="${not empty controlVariant}">
            <template:module node="${controlVariant}"/>
        </c:if>
    </c:if>
</c:if>